@@ -59,15 +59,16 @@ Ray::Ray()
 //--------------------------------------------------------------------------------------------------
 /// 
 //--------------------------------------------------------------------------------------------------
-Ray::Ray(const Ray& other) : Object()
+Ray::Ray(const Ray& other)
+    : Object{}
+    , m_origin(other.origin())
+    , m_direction(other.direction())
+    , m_minDistance(other.m_minDistance)
+    , m_maxDistance(other.m_maxDistance)
+    , m_minDistanceSquared(other.m_minDistanceSquared)
+    , m_maxDistanceSquared(other.m_maxDistanceSquared)
+    , m_distanceLimitedRay(other.m_distanceLimitedRay)
 {
-    m_origin = other.origin();
-    m_direction = other.direction();
-    m_minDistance = other.m_minDistance;
-    m_maxDistance = other.m_maxDistance;
-    m_minDistanceSquared = other.m_minDistanceSquared;
-    m_maxDistanceSquared = other.m_maxDistanceSquared;
-    m_distanceLimitedRay = other.m_distanceLimitedRay;
 }
 
 
@@ -164,7 +165,7 @@ bool Ray::triangleIntersect(const Vec3d& v1, const Vec3d& v2, const Vec3d& v3, V
     }
 
     Vec3d fp = origin() + direction()*t;
-    Vec3d pts[] = { v1, v2, v3, v1 };
+    const Vec3d pts[] = { v1, v2, v3, v1 };
     
     int i;
     for(i = 0; i <  3; i++)
@@ -229,7 +230,7 @@ bool Ray::quadIntersect(const Vec3d& v1, const Vec3d& v2, const Vec3d& v3, const
     }
 
     Vec3d fp = origin() + direction()*t;
-    Vec3d pts[] = { v1, v2, v3, v4, v1 };
+    const Vec3d pts[] = { v1, v2, v3, v4, v1 };
 
     int i;
     for(i = 0; i <  4; i++)
