@@ -33,6 +33,7 @@
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
 
 #include <opm/input/eclipse/Schedule/Action/ActionContext.hpp>
+#include <opm/input/eclipse/Schedule/Action/ActionResult.hpp>
 #include <opm/input/eclipse/Schedule/Action/Actions.hpp>
 #include <opm/input/eclipse/Schedule/Action/SimulatorUpdate.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
@@ -71,6 +72,8 @@ namespace {
 
 namespace Opm {
 
+std::shared_ptr<Python> msim::python = std::make_shared<Python>();
+
 msim::msim(const EclipseState& state_arg, const Schedule& schedule_arg)
     : state   (state_arg)
     , schedule(schedule_arg)
@@ -86,7 +89,6 @@ void msim::run(EclipseIO& io, bool report_only)
     data::Solution sol;
     UDQState udq_state(this->schedule.getUDQConfig(0).params().undefinedValue());
     WellTestState wtest_state;
-    Python python;
 
     io.writeInitial();
     for (size_t report_step = 1; report_step < schedule.size(); report_step++) {
@@ -115,8 +117,9 @@ void msim::run(EclipseIO& io, bool report_only)
 }
 
 
-UDAValue msim::uda_val() {
- return UDAValue();
+UDAValue msim::uda_val()
+{
+    return UDAValue();
 }
 
 
@@ -131,12 +134,14 @@ void msim::post_step(data::Solution& /* sol */,
         return;
     }
 
-    Action::Context context(this->st, this->schedule[report_step].wlist_manager.get());
+    const auto context = Action::Context {
+        this->st, this->schedule[report_step].wlist_manager.get()
+    };
 
     for (const auto& action : actions.pending(this->action_state, std::chrono::system_clock::to_time_t(sim_time))) {
-        auto result = action->eval(context);
-        if (result) {
-            this->schedule.applyAction(report_step, *action, result.wells(),
+        const auto result = action->eval(context);
+        if (result.conditionSatisfied()) {
+            this->schedule.applyAction(report_step, *action, result.matches(),
                                        std::unordered_map<std::string,double>{});
         }
     }
@@ -195,8 +200,8 @@ void msim::run_step(const WellTestState& wtest_state,
 
         this->schedule.getUDQConfig(report_step - 1)
             .eval(report_step,
-                  this->schedule,
                   this->schedule.wellMatcher(report_step),
+                  this->schedule[report_step].group_order(),
                   this->schedule.segmentMatcherFactory(report_step),
                   createRegionSetMatcherFactory(this->state),
                   this->st,
