@@ -23,10 +23,12 @@
 #include <opm/input/eclipse/EclipseState/Aquifer/Aquancon.hpp>
 #include <opm/input/eclipse/EclipseState/Aquifer/Aquifetp.hpp>
 #include <opm/input/eclipse/EclipseState/Aquifer/AquiferCT.hpp>
+#include <opm/input/eclipse/EclipseState/Aquifer/AquiferFlux.hpp>
 #include <opm/input/eclipse/EclipseState/Aquifer/NumericalAquifer/NumericalAquifers.hpp>
 
 #include <cstddef>
 #include <vector>
+#include <unordered_set>
 
 namespace Opm {
     class TableManager;
@@ -43,21 +45,30 @@ namespace Opm {
 
 class AquiferConfig {
 public:
+
     AquiferConfig() = default;
     AquiferConfig(const TableManager& tables, const EclipseGrid& grid,
                   const Deck& deck, const FieldPropsManager& field_props);
-    AquiferConfig(const Aquifetp& fetp, const AquiferCT& ct, const Aquancon& conn);
+    AquiferConfig(const Aquifetp& fetp, const AquiferCT& ct, const AquiferFlux& aqufluxs, const Aquancon& conn);
     void load_connections(const Deck& deck, const EclipseGrid& grid);
 
     void pruneDeactivatedAquiferConnections(const std::vector<std::size_t>& deactivated_cells);
     void loadFromRestart(const RestartIO::RstAquifer& aquifers,
                          const TableManager&          tables);
 
+    // there might be some aquifers (AQUFLUX only for now) are opened through
+    // SCHEDULE section while not specified in the SOLUTION section.
+    // We create dummy aquifers in the AquiferConfig to make sure we are aware of them
+    // when we handle the SUMMARY section.
+    // Since those aquifers are not active, basically we only need the id information
+    void appendAqufluxSchedule(const std::unordered_set<int>& ids);
+
     static AquiferConfig serializationTestObject();
 
     bool active() const;
     const AquiferCT& ct() const;
     const Aquifetp& fetp() const;
+    const AquiferFlux& aquflux() const;
     const Aquancon& connections() const;
     bool operator==(const AquiferConfig& other) const;
     bool hasAquifer(const int aquID) const;
@@ -74,12 +85,14 @@ public:
         serializer(aquifetp);
         serializer(aquiferct);
         serializer(aqconn);
+        serializer(aquiferflux);
         serializer(numerical_aquifers);
     }
 
 private:
     Aquifetp aquifetp{};
     AquiferCT aquiferct{};
+    AquiferFlux aquiferflux{};
     mutable NumericalAquifers numerical_aquifers{};
     Aquancon aqconn{};
 };
