@@ -15,23 +15,27 @@
 
   You should have received a copy of the GNU General Public License
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
- */
+*/
 #ifndef OPM_AQUIFERHELPERS_HPP
 #define OPM_AQUIFERHELPERS_HPP
 
 #include <opm/input/eclipse/EclipseState/Grid/FaceDir.hpp>
 
-#include <vector>
+#include <cstddef>
 #include <optional>
 #include <unordered_set>
+#include <vector>
 
 namespace Opm {
     class EclipseGrid;
-    namespace AquiferHelpers {
-        bool neighborCellInsideReservoirAndActive(const EclipseGrid &grid, int i, int j, int k,
-                                                  FaceDir::DirEnum faceDir, const std::vector<int>& actnum,
-                                                  const std::optional<const std::unordered_set<std::size_t>>& numerical_aquifer_cells = std::nullopt);
-    }
-}
+} // namespace Opm
 
-#endif //OPM_AQUIFERHELPERS_HPP
+namespace Opm::AquiferHelpers {
+    bool neighborCellInsideReservoirAndActive(const EclipseGrid& grid,
+                                              int i, int j, int k,
+                                              FaceDir::DirEnum faceDir,
+                                              const std::vector<int>& actnum,
+                                              const std::optional<const std::unordered_set<std::size_t>>& numerical_aquifer_cells = std::nullopt);
+} // namespace Opm::AquiferHelpers
+
+#endif // OPM_AQUIFERHELPERS_HPP
