@@ -18,7 +18,6 @@
  */
 
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
-
 #include <opm/common/ErrorMacros.hpp>
 #include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/common/OpmLog/InfoLogger.hpp>
@@ -55,6 +54,7 @@
 #include <opm/input/eclipse/Parser/ParserKeywords/T.hpp>
 
 #include <fmt/format.h>
+#include <fmt/ranges.h>
 
 #include <cstddef>
 #include <filesystem>
@@ -144,16 +144,17 @@ namespace Opm {
         , compositional_config(deck, m_runspec)
         , m_transMult(         GridDims(deck), deck, field_props)
         , tracer_config(       m_deckUnitSystem, deck)
-        , m_micppara(          deck)
         , wag_hyst_config(     deck)
         , co2_store_config(    deck)
     {
         this->assignRunTitle(deck);
         this->reportNumberOfActivePhases();
 
-        this->conveyNumericalAquiferEffects();
         if (field_props.has_double("MINPVV")) {
             this->m_inputGrid.setMINPVV(field_props.get_global_double("MINPVV"));
+        }
+        this->conveyNumericalAquiferEffects();
+        if (field_props.has_double("MINPVV")) {
             field_props.deleteMINPVV();
         }
         this->initLgrs(deck);
@@ -276,10 +277,6 @@ namespace Opm {
         return m_lgrs;
     }
 
-    const MICPpara& EclipseState::getMICPpara() const {
-        return m_micppara;
-    }
-
     const WagHysteresisConfig& EclipseState::getWagHysteresis() const {
         return wag_hyst_config;
     }
@@ -338,7 +335,7 @@ namespace Opm {
         const GRIDSection gridSection ( deck );
 
         m_lgrs = LgrCollection(gridSection, m_inputGrid);
-
+        m_inputGrid.init_lgr_cells(m_lgrs);
     }
 
 
@@ -425,7 +422,9 @@ namespace Opm {
             return;
         }
 
-        const auto& numerical_aquifer = this->aquifer_config.numericalAquifers();
+        auto& numerical_aquifer = this->aquifer_config.mutableNumericalAquifers();
+
+        numerical_aquifer.applyMinPV(this->m_inputGrid);
 
         // Update field_props for numerical aquifer cells and set the
         // transmissiblity related to aquifer cells to zero.
