@@ -17,9 +17,10 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+
+
 #ifndef OPM_PARSER_ECLIPSE_GRID_HPP
 #define OPM_PARSER_ECLIPSE_GRID_HPP
-
 #include <opm/input/eclipse/EclipseState/Grid/GridDims.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/MapAxes.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/MinpvMode.hpp>
@@ -29,18 +30,24 @@
 #include <memory>
 #include <optional>
 #include <stdexcept>
+#include <string>
 #include <unordered_set>
 #include <vector>
 #include <map>
 
+
 namespace Opm {
 
     class Deck;
-    namespace EclIO { class EclFile; }
+    namespace EclIO {
+        class EclFile;
+        class EclOutput;
+    }
     struct NNCdata;
     class UnitSystem;
     class ZcornMapper;
-
+    class EclipseGridLGR;
+    class LgrCollection;
     /**
        About cell information and dimension: The actual grid
        information is held in a pointer to an ERT ecl_grid_type
@@ -74,6 +81,7 @@ namespace Opm {
                     const std::vector<double>& zcorn ,
                     const int * actnum = nullptr);
 
+        virtual ~EclipseGrid() = default;
 
         /// EclipseGrid ignores ACTNUM in Deck, and therefore needs ACTNUM
         /// explicitly.  If a null pointer is passed, every cell is active.
@@ -91,10 +99,42 @@ namespace Opm {
         size_t activeIndex(size_t i, size_t j, size_t k) const;
         size_t activeIndex(size_t globalIndex) const;
 
+        size_t getTotalActiveLGR() const;
+        size_t getActiveIndexLGR(const std::string& label, size_t i, size_t j, size_t k) const;
+        size_t getActiveIndexLGR(const std::string& label, size_t localIndex) const;
+
+        size_t activeIndexLGR(const std::string& label, size_t i, size_t j, size_t k) const;
+        size_t activeIndexLGR(const std::string& label, size_t localIndex) const;
+
         size_t getActiveIndex(size_t i, size_t j, size_t k) const {
             return activeIndex(i, j, k);
         }
 
+        size_t getActiveIndex(size_t globalIndex) const {
+            return activeIndex(globalIndex);
+        }
+
+        std::vector<std::string> get_all_lgr_labels() const
+        {
+            return  {this->all_lgr_labels.begin() + 1, this->all_lgr_labels.end()};
+        }
+
+        const std::vector<std::string>& get_all_labels() const
+        {
+            return this->all_lgr_labels;
+        }
+        
+        std::string get_lgr_tag() const {
+            return this->lgr_label;
+        }
+
+        std::vector<GridDims> get_lgr_children_gridim() const;
+      
+        
+        void assertIndexLGR(size_t localIndex) const;
+
+        void assertLabelLGR(const std::string& label) const;
+
         void save(const std::string& filename, bool formatted, const std::vector<Opm::NNCdata>& nnc, const Opm::UnitSystem& units) const;
         /*
           Observe that the there is a getGlobalIndex(i,j,k)
@@ -102,6 +142,10 @@ namespace Opm {
           from an active index to a global index must be implemented
           in the current class.
         */
+        void init_children_host_cells(bool logical = true);
+        void init_children_host_cells_logical(void);
+        void init_children_host_cells_geometrical(void);
+
         using GridDims::getGlobalIndex;
         size_t getGlobalIndex(size_t active_index) const;
 
@@ -111,7 +155,6 @@ namespace Opm {
           applied in the 'THETA' direction; this will only apply if
           the theta keywords entered sum up to exactly 360 degrees!
         */
-
         bool circle() const;
         bool isPinchActive() const;
         double getPinchThresholdThickness() const;
@@ -119,7 +162,6 @@ namespace Opm {
         PinchMode getMultzOption() const;
         PinchMode getPinchGapMode() const;
         double getPinchMaxEmptyGap() const;
-
         MinpvMode getMinpvMode() const;
         const std::vector<double>& getMinpvVector( ) const;
 
@@ -160,6 +202,9 @@ namespace Opm {
         /// Will return a vector a length num_active; where the value
         /// of each element is the corresponding global index.
         const std::vector<int>& getActiveMap() const;
+
+        void init_lgr_cells(const LgrCollection& lgr_input);
+        void create_lgr_cells_tree(const LgrCollection& );
         /// \brief get cell center, and center and normal of bottom face
         std::tuple<std::array<double, 3>,std::array<double, 3>,std::array<double, 3>>
         getCellAndBottomCenterNormal(size_t globalIndex) const;
@@ -176,10 +221,16 @@ namespace Opm {
         bool cellActive( size_t globalIndex ) const;
         bool cellActive( size_t i , size_t j, size_t k ) const;
         bool cellActiveAfterMINPV( size_t i , size_t j , size_t k, double cell_porv ) const;
-
+        bool is_lgr() const {return lgr_grid;};
         std::array<double, 3> getCellDimensions(size_t i, size_t j, size_t k) const {
             return getCellDims(i, j, k);
         }
+        std::array<double,3> getCellDimensionsLGR(const std::size_t  i, 
+                                                  const std::size_t  j, 
+                                                  const std::size_t  k, 
+                                                  const std::string& lgr_tag) const;
+        double getCellDepthLGR(size_t i, size_t j, size_t k, const std::string& lgr_tag) const;
+
 
         bool isCellActive(size_t i, size_t j, size_t k) const {
             return cellActive(i, j, k);
@@ -201,6 +252,7 @@ namespace Opm {
         const std::vector<double>& getZCORN() const;
         const std::vector<int>& getACTNUM( ) const;
 
+        const std::optional<MapAxes>& getMapAxes() const;
 
         const std::map<size_t, std::array<int,2>>& getAquiferCellTabnums() const;
 
@@ -234,11 +286,43 @@ namespace Opm {
 
         static bool hasEqualDVDEPTHZ(const Deck&);
         static bool allEqual(const std::vector<double> &v);
+        EclipseGridLGR& getLGRCell(std::size_t index);
+        const EclipseGridLGR& getLGRCell(const std::string& lgr_tag) const;
+        int getLGR_global_father(std::size_t global_index,  const std::string& lgr_tag) const;
+
+        std::vector<EclipseGridLGR> lgr_children_cells;
+        /**
+        * @brief Sets Local Grid Refinement for the EclipseGrid.
+        *
+        * @param lgr_tag The string that contains the name of a given LGR cell.
+        * @param coords The coordinates of a given LGR cell in  CPG COORDSformat.
+        * @param zcorn The z-coordinates values of a given LGR cell in CPG ZCORN format.
+        */
+        virtual void set_lgr_refinement(const std::string& lgr_tag, const std::vector<double>& coords, const std::vector<double>& zcorn);
+
+    protected:
+        std::size_t lgr_global_counter = 0;
+        std::string lgr_label = "GLOBAL";
+        int lgr_level = 0;
+        int lgr_level_father = 0;
+        std::vector<std::string> lgr_children_labels;
+        std::vector<std::size_t> lgr_active_index;
+        std::vector<std::size_t> lgr_level_active_map;
+        std::vector<std::string> all_lgr_labels;
+        std::map<std::vector<std::size_t>, std::size_t> num_lgr_children_cells;
+        std::vector<double> m_zcorn;
+        std::vector<double> m_coord;
+        std::vector<int> m_actnum;
+        std::vector<std::size_t> m_print_order_lgr_cells;
+        // Input grid data.
+        mutable std::optional<std::vector<double>> m_input_zcorn;
+        mutable std::optional<std::vector<double>> m_input_coord;
 
     private:
         std::vector<double> m_minpvVector;
         MinpvMode m_minpvMode;
         std::optional<double> m_pinch;
+
         // Option 4 of PINCH (TOPBOT/ALL), how to calculate TRANS
         PinchMode m_pinchoutMode;
         // Option 5 of PINCH (TOP/ALL), how to apply MULTZ
@@ -246,23 +330,13 @@ namespace Opm {
         // Option 2 of PINCH (GAP/NOGAP)
         PinchMode m_pinchGapMode;
         double    m_pinchMaxEmptyGap;
-
+        bool lgr_grid = false;
         mutable std::optional<std::vector<double>> active_volume;
 
         bool m_circle = false;
-
         size_t zcorn_fixed = 0;
         bool m_useActnumFromGdfile = false;
 
-        // Input grid data.
-        mutable std::optional<std::vector<double>> m_input_zcorn;
-        mutable std::optional<std::vector<double>> m_input_coord;
-
-        std::vector<double> m_zcorn;
-        std::vector<double> m_coord;
-
-
-        std::vector<int> m_actnum;
         std::optional<MapAxes> m_mapaxes;
 
         // Mapping to/from active cells.
@@ -278,7 +352,10 @@ namespace Opm {
         // Radial grids need this for volume calculations.
         std::optional<std::vector<double>> m_thetav;
         std::optional<std::vector<double>> m_rv;
-
+        void parseGlobalReferenceToChildren(void);
+        int initializeLGRObjectIndices(int);
+        void initializeLGRTreeIndices(void);
+        void propagateParentIndicesToLGRChildren(int);
         void updateNumericalAquiferCells(const Deck&);
         double computeCellGeometricDepth(size_t globalIndex) const;
 
@@ -303,7 +380,7 @@ namespace Opm {
         void initGrid(const Deck&, const int* actnum);
         void initCornerPointGrid(const Deck&);
         void assertCornerPointKeywords(const Deck&);
-
+        void save_all_lgr_labels(const LgrCollection& );
         static bool hasDTOPSKeywords(const Deck&);
         static void assertVectorSize(const std::vector<double>& vector, size_t expectedSize, const std::string& msg);
 
@@ -322,8 +399,82 @@ namespace Opm {
                             std::array<double,8>& X,
                             std::array<double,8>& Y,
                             std::array<double,8>& Z) const;
+        std::array<int,3> getCellSubdivisionRatioLGR(const std::string&  lgr_tag, 
+                                                     std::array<int,3>   acum = {1,1,1}) const;
+    
+    
+    };
+
+    /// Specialized Class to describe LGR refined cells.
+    class EclipseGridLGR : public EclipseGrid
+    {
+    public:
+        using vec_size_t = std::vector<std::size_t>;
+        EclipseGridLGR() = default;
+        EclipseGridLGR(const std::string& self_label,
+                       const std::string& father_label_,
+                       size_t nx,
+                       size_t ny,
+                       size_t nz,
+                       const vec_size_t& father_lgr_index,
+                       const std::array<int, 3>& low_fatherIJK_,
+                       const std::array<int, 3>& up_fatherIJK_);
+        const vec_size_t& getFatherGlobalID() const;
+        void save(Opm::EclIO::EclOutput&, const std::vector<Opm::NNCdata>&, const Opm::UnitSystem&) const;
+        void save_nnc(Opm::EclIO::EclOutput&) const;
+        void set_lgr_global_counter(std::size_t counter)
+        {
+            lgr_global_counter = counter;
+        }
+        const vec_size_t& get_father_global() const
+        {
+            return father_global;
+        }
+        std::optional<std::reference_wrapper<EclipseGridLGR>>
+        get_child_LGR_cell(const std::string& lgr_tag) const;
+        std::vector<int> save_hostnum(void) const;
+        int get_hostnum(std::size_t global_index) const {return(m_hostnum[global_index]);};
+        void get_label_child_to_top_father(std::vector<std::reference_wrapper<const std::string>>& list) const;
+        void get_global_index_child_to_top_father(std::vector<std::size_t> & list, 
+                                                  std::size_t i, std::size_t j, std::size_t k) const;
+        void get_global_index_child_to_top_father(std::vector<std::size_t> & list, std::size_t global_ind) const;
+  
+        void set_hostnum(const std::vector<int>&);
+        const std::array<int,3>& get_low_fatherIJK() const{
+          return low_fatherIJK;
+        }
+        const std::string& get_father_label() const{
+          return father_label;
+        }
+
+        const std::array<int,3>& get_up_fatherIJK() const{
+          return up_fatherIJK;
+        }
+
+
+        /**
+         * @brief Sets Local Grid Refinement for the EclipseGridLGR.
+         *
+         * @param lgr_tag The string that contains the name of a given LGR cell.
+         * @param coords The coordinates of a given LGR cell in  CPG COORDSformat.
+         * @param zcorn The z-coordinates values of a given LGR cell in CPG ZCORN format.
+         */
+        void set_lgr_refinement(const std::string& lgr_tag,
+                                const std::vector<double>& coord,
+                                const std::vector<double>& zcorn) override;
+
+        void set_lgr_refinement(const std::vector<double>&, const std::vector<double>&);
+
+    private:
+        void init_father_global();
+        std::string father_label;
+        // references global on the father label
+        vec_size_t father_global;
+        std::array<int, 3> low_fatherIJK {};
+        std::array<int, 3> up_fatherIJK {};
+        std::vector<int> m_hostnum;
+    };
 
-   };
 
     class CoordMapper {
     public:
@@ -374,5 +525,4 @@ namespace Opm {
         std::array<size_t,8> cell_shift;
     };
 }
-
 #endif // OPM_PARSER_ECLIPSE_GRID_HPP
