@@ -508,6 +508,8 @@ public:
     bool h2Storage() const noexcept;
     bool micp() const noexcept;
     bool mech() const noexcept;
+    bool temp() const noexcept;
+    bool compositional() const noexcept;
 
     bool operator==(const Runspec& data) const;
     static bool rst_cmp(const Runspec& full_state, const Runspec& rst_state);
@@ -537,31 +539,33 @@ public:
         serializer(m_h2storage);
         serializer(m_micp);
         serializer(m_mech);
+        serializer(m_temp);
     }
 
 private:
-    std::time_t m_start_time;
-    Phases active_phases;
-    Tabdims m_tabdims;
-    Regdims m_regdims;
-    EndpointScaling endscale;
-    Welldims welldims;
-    WellSegmentDims wsegdims;
-    NetworkDims netwrkdims;
-    AquiferDimensions aquiferdims;
-    UDQParams udq_params;
-    EclHysterConfig hystpar;
-    Actdims m_actdims;
-    SatFuncControls m_sfuncctrl;
-    Nupcol m_nupcol;
-    Tracers m_tracers;
+    std::time_t m_start_time{};
+    Phases active_phases{};
+    Tabdims m_tabdims{};
+    Regdims m_regdims{};
+    EndpointScaling endscale{};
+    Welldims welldims{};
+    WellSegmentDims wsegdims{};
+    NetworkDims netwrkdims{};
+    AquiferDimensions aquiferdims{};
+    UDQParams udq_params{};
+    EclHysterConfig hystpar{};
+    Actdims m_actdims{};
+    SatFuncControls m_sfuncctrl{};
+    Nupcol m_nupcol{};
+    Tracers m_tracers{};
     size_t m_comps = 0;
-    bool m_co2storage;
-    bool m_co2sol;
-    bool m_h2sol;
-    bool m_h2storage;
-    bool m_micp;
-    bool m_mech;
+    bool m_co2storage{false};
+    bool m_co2sol{false};
+    bool m_h2sol{false};
+    bool m_h2storage{false};
+    bool m_micp{false};
+    bool m_mech{false};
+    bool m_temp{false};
 };
 
 std::size_t declaredMaxRegionID(const Runspec& rspec);
