@@ -57,6 +57,7 @@ namespace Opm {
         bool hasVAPOIL() const;
         bool hasVAPWAT() const;
         bool isThermal() const;
+        bool useEnthalpy() const;
         bool isDiffusive() const;
         bool hasPRECSALT() const;
 
@@ -78,6 +79,7 @@ namespace Opm {
             serializer(m_VAPOIL);
             serializer(m_VAPWAT);
             serializer(m_isThermal);
+            serializer(m_useEnthalpy);
             serializer(m_diffuse);
             serializer(m_PRECSALT);
         }
@@ -96,6 +98,7 @@ namespace Opm {
         bool m_VAPOIL{false};
         bool m_VAPWAT{false};
         bool m_isThermal{false};
+        bool m_useEnthalpy{false};
         bool m_diffuse{false};
         bool m_PRECSALT{false};
     };
