@@ -167,6 +167,11 @@ struct SummaryConfigContext {
          {"GMUS",     {"RGMUS", "FGMUS"}},
          {"GKTR",     {"RGKTR", "FGKTR"}},
          {"GKMO",     {"RGKMO", "FGKMO"}},
+         {"MMIP",     {"RMMIP", "FMMIP"}},
+         {"MOIP",     {"RMOIP", "FMOIP"}},
+         {"MUIP",     {"RMUIP", "FMUIP"}},
+         {"MBIP",     {"RMBIP", "FMBIP"}},
+         {"MCIP",     {"RMCIP", "FMCIP"}},
     };
 
     using keyword_set = std::unordered_set<std::string>;
@@ -237,6 +242,7 @@ struct SummaryConfigContext {
     bool is_rate(const std::string& keyword) {
         static const keyword_set ratekw {
             "OPR", "GPR", "WPR", "GLIR", "LPR", "NPR", "CPR", "VPR", "TPR", "TPC",
+            "GMPR",
 
             "OFR", "OFRF", "OFRS", "OFR+", "OFR-", "TFR",
             "GFR", "GFRF", "GFRS", "GFR+", "GFR-",
@@ -256,6 +262,8 @@ struct SummaryConfigContext {
             "OPI", "OPP", "GPI", "GPP", "WPI", "WPP",
 
             "AQR", "AQRG", "NQR",
+
+            "MMIR", "MOIR", "MUIR", "MMPR", "MOPR", "MUPR",
         };
 
         return is_in_set(ratekw, keyword.substr(1))
@@ -278,7 +286,7 @@ struct SummaryConfigContext {
             "OPT", "GPT", "WPT", "LPT", "NPT", "CPT",
             "VPT", "TPT", "OVPT", "GVPT", "WVPT",
             "WPTH", "OPTH", "GPTH", "LPTH",
-            "GPTS", "OPTS", "GPTF", "OPTF",
+            "GPTS", "OPTS", "GPTF", "OPTF", "GMPT",
 
             "OFT", "OFT+", "OFT-", "OFTL", "OFTG",
             "GFT", "GFT+", "GFT-", "GFTL", "GFTG",
@@ -288,6 +296,8 @@ struct SummaryConfigContext {
             "WITH", "OITH", "GITH", "WVIT", "OVIT", "GVIT", "GMIT",
 
             "AQT", "AQTG", "NQT",
+
+            "MMIT", "MOIT", "MUIT", "MMPT", "MOPT", "MUPT",
         };
 
         return is_in_set(totalkw, keyword.substr(1))
@@ -394,7 +404,7 @@ struct SummaryConfigContext {
     bool is_well_comp(const std::string& keyword)
     {
         static const auto well_comp_kw = keyword_set {
-             "WXMF", "WYMF", "WZMF", "WCGMR", "WCOMR"
+             "WAMF", "WXMF", "WYMF", "WZMF", "WCGMR", "WCOMR",
         };
 
         return is_in_set(well_comp_kw, keyword);
