@@ -85,6 +85,8 @@
 #include <opm/input/eclipse/EclipseState/Tables/RvwvdTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/PcfactTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/PermfactTable.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/BiofilmTable.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/DiffMICPTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/SaltvdTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/SaltpvdTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/SaltSolubilityTable.hpp>
@@ -364,8 +366,9 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
     }
 
 
-    void TableManager::addTables( const std::string& tableName , size_t numTables) {
-        m_simpleTables.emplace(std::make_pair(tableName , TableContainer( numTables )));
+    void TableManager::addTables(const std::string& tableName, size_t numTables)
+    {
+        this->m_simpleTables.try_emplace(tableName, numTables);
     }
 
 
@@ -473,8 +476,10 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
         addTables( "SALTVD", m_eqldims.getNumEquilRegions());
         addTables( "SALTPVD", m_eqldims.getNumEquilRegions());
         addTables( "SALTSOL", m_tabdims.getNumPVTTables());
-        addTables( "PERMFACT",  m_tabdims.getNumPVTTables());
+        addTables( "PERMFACT",  m_tabdims.getNumSatTables());
         addTables( "PCFACT",  m_tabdims.getNumSatTables());
+        addTables( "BIOFPARA", m_tabdims.getNumSatTables());
+        addTables( "DIFFMICP", m_tabdims.getNumPVTTables());
 
         addTables( "AQUTAB", m_aqudims.getNumInfluenceTablesCT());
         {
@@ -543,8 +548,10 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
         initSimpleTableContainer<SaltpvdTable>(deck, "SALTPVD" , m_eqldims.getNumEquilRegions());
         initSimpleTableContainer<SaltvdTable>(deck, "SALTVD" , m_eqldims.getNumEquilRegions());
         initSimpleTableContainer<SaltsolTable>(deck, "SALTSOL" , m_tabdims.getNumPVTTables());
-        initSimpleTableContainer<PermfactTable>(deck, "PERMFACT" , m_tabdims.getNumPVTTables());
+        initSimpleTableContainer<PermfactTable>(deck, "PERMFACT" , m_tabdims.getNumSatTables());
         initSimpleTableContainer<PcfactTable>(deck, "PCFACT" , m_tabdims.getNumSatTables());
+        initSimpleTableContainer<BiofilmTable>(deck, "BIOFPARA" , m_tabdims.getNumSatTables());
+        initSimpleTableContainer<DiffMICPTable>(deck, "DIFFMICP" , m_tabdims.getNumPVTTables());
         initSimpleTableContainer<AqutabTable>(deck, "AQUTAB" , m_aqudims.getNumInfluenceTablesCT());
         {
             size_t numEndScaleTables = ParserKeywords::ENDSCALE::NTENDP::defaultValue;
@@ -952,7 +959,7 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
         return getTables("SALTPVD");
     }
 
-   const TableContainer& TableManager::getSaltsolTables() const {
+    const TableContainer& TableManager::getSaltsolTables() const {
         return getTables("SALTSOL");
     }
 
@@ -964,6 +971,14 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
         return getTables("PERMFACT");
     }
 
+    const TableContainer& TableManager::getBiofilmTables() const {
+        return getTables("BIOFPARA");
+    }
+
+    const TableContainer& TableManager::getDiffMICPTables() const {
+        return getTables("DIFFMICP");
+    }
+
     const TableContainer& TableManager::getEnkrvdTables() const {
         return getTables("ENKRVD");
     }
@@ -1553,19 +1568,31 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
             return;
         }
 
+        auto lastComplete = 0 * numTables;
         const auto& tableKeyword = deck[keywordName].back();
         for (size_t tableIdx = 0; tableIdx < tableKeyword.size(); ++tableIdx) {
             const auto& dataItem = tableKeyword.getRecord( tableIdx ).getItem("DATA");
             if (dataItem.data_size() > 0) {
                 try {
-                    std::shared_ptr<TableType> table = std::make_shared<TableType>(dataItem, useJFunc(), tableIdx );
+                    std::shared_ptr<TableType> table = std::make_shared<TableType>( dataItem, useJFunc(), tableIdx );
                     container.addTable( tableIdx , table );
+                    lastComplete = tableIdx;
                 } catch (const std::runtime_error& err) {
                     throw OpmInputError(err, tableKeyword.location());
                 } catch (const std::invalid_argument& err) {
                     throw OpmInputError(err, tableKeyword.location());
                 }
             }
+            else if (tableIdx > static_cast<size_t>(0)) {
+                const auto& item = tableKeyword.getRecord(lastComplete).getItem("DATA");
+                container.addTable(tableIdx, std::make_shared<TableType>(item, useJFunc(), tableIdx));
+            }
+            else {
+                throw OpmInputError {
+                    fmt::format("Cannot default region {}'s table data", tableIdx + 1),
+                    tableKeyword.location()
+                };
+            }
         }
     }
 
@@ -1573,29 +1600,33 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
     void TableManager::initSimpleTableContainer(const Deck& deck,
                                                 const std::string& keywordName,
                                                 const std::string& tableName,
-                                                size_t numTables) {
-        if (!deck.hasKeyword(keywordName))
+                                                size_t numTables)
+    {
+        if (!deck.hasKeyword(keywordName)) {
             return; // the table is not featured by the deck...
-
-        auto& container = forceGetTables(tableName , numTables);
+        }
 
         if (deck.count(keywordName) > 1) {
             complainAboutAmbiguousKeyword(deck, keywordName);
             return;
         }
 
+        auto& container = forceGetTables(tableName, numTables);
+
         auto lastComplete = 0 * numTables;
         const auto& tableKeyword = deck[keywordName].back();
         for (size_t tableIdx = 0; tableIdx < tableKeyword.size(); ++tableIdx) {
-            const auto& dataItem = tableKeyword.getRecord( tableIdx ).getItem("DATA");
+            const auto& dataItem = tableKeyword.getRecord(tableIdx).getItem("DATA");
+
             if (dataItem.data_size() > 0) {
                 try {
-                    std::shared_ptr<TableType> table = std::make_shared<TableType>( dataItem, tableIdx );
-                    container.addTable( tableIdx , table );
+                    container.addTable(tableIdx, std::make_shared<TableType>(dataItem, tableIdx));
                     lastComplete = tableIdx;
-                } catch (const std::runtime_error& err) {
+                }
+                catch (const std::runtime_error& err) {
                     throw OpmInputError(err, tableKeyword.location());
-                } catch (const std::invalid_argument& err) {
+                }
+                catch (const std::invalid_argument& err) {
                     throw OpmInputError(err, tableKeyword.location());
                 }
             }
