@@ -67,6 +67,7 @@
 #include <stack>
 #include <stdexcept>
 #include <string>
+#include <string_view>
 #include <regex>
 #include <utility>
 #include <vector>
@@ -214,7 +215,7 @@ inline Itr find_terminator( Itr begin, Itr end, Term terminator ) {
 static inline std::string_view strip_comments( std::string_view str ) {
     auto terminator = find_terminator( str.begin(), str.end(), find_comment() );
     std::size_t size = std::distance(str.begin(), terminator);
-    return { str.begin(), size };
+    return str.substr(0, size);
 }
 
 template< typename Itr >
@@ -234,8 +235,9 @@ inline Itr trim_right( Itr begin, Itr end ) {
 inline std::string_view trim( std::string_view str ) {
     auto fst = trim_left( str.begin(), str.end() );
     auto lst = trim_right( fst, str.end() );
-    std::size_t size = std::distance(fst, lst);
-    return { fst, size };
+    const std::size_t start = std::distance(str.begin(), fst);
+    const std::size_t size = std::distance(fst, lst);
+    return str.substr(start, size);
 }
 
 inline std::string_view del_after_first_slash( std::string_view view ) {
@@ -249,32 +251,26 @@ inline std::string_view del_after_first_slash( std::string_view view ) {
     auto slash = find_terminator( begin, end, term );
 
     /* we want to preserve terminating slashes */
-    if( slash != end ) ++slash;
-    std::size_t size = std::distance(begin, slash);
-    return { begin, size };
+    if (slash != end) {
+        ++slash;
+    }
+    const std::size_t size = std::distance(begin, slash);
+    return view.substr(0, size);
 }
 
 inline std::string_view del_after_last_slash( std::string_view view ) {
-  auto begin = view.begin();
-  auto end = view.end();
-  auto slash = end;
+    if (view.empty()) {
+        return view;
+    }
+    auto slash = view.find_last_of('/');
+    if (slash == std::string_view::npos) {
+        slash = view.size();
+    }
+    else {
+        ++slash;
+    }
 
-  while (true) {
-      if (slash == begin)
-          break;
-
-      if (*slash == '/')
-          break;
-
-      slash--;
-  }
-  if (slash == begin && *slash != '/')
-      slash = end;
-
-  /* we want to preserve terminating slashes */
-  if( slash != end ) ++slash;
-  std::size_t size = std::distance(begin, slash);
-  return { begin, size };
+    return view.substr(0, slash);
 }
 
 inline std::string_view del_after_slash(std::string_view view, bool raw_strings) {
@@ -286,18 +282,20 @@ inline std::string_view del_after_slash(std::string_view view, bool raw_strings)
 
 
 inline bool getline( std::string_view& input, std::string_view& line ) {
-    if( input.empty() ) return false;
-
-    auto end = std::find( input.begin(), input.end(), '\n' );
-
-    line = std::string_view( input.begin(), end - input.begin() );
-    input = std::string_view( end + 1, input.end() - (end + 1));
-    return true;
+    if (input.empty()) {
+        return false;
+    }
 
     /* we know that we always append a newline onto the input string, so we can
-     * safely assume that end+1 will either be end-of-input (i.e. empty range)
+     * safely assume that pos+1 will either be end-of-input (i.e. empty range)
      * or the start of the next line
      */
+
+    const auto pos = input.find_first_of('\n');
+    line = input.substr(0, pos);
+    input = input.substr(pos+1);
+
+    return true;
 }
 
 /*
@@ -373,14 +371,14 @@ inline std::string clean( const std::vector<std::pair<std::string, std::string>>
                     if (end_pos == std::string::npos) {
                         std::copy(input.begin(), input.end(), dsti);
                         dsti += std::distance( input.begin(), input.end() );
-                        input = std::string_view(input.end(), 0);
+                        input = {};
                         break;
                     } else {
                         end_pos += end_string.size();
                         std::copy(input.begin(), input.begin() + end_pos, dsti);
                         dsti += end_pos;
                         *dsti++ = '\n';
-                        input = std::string_view(input.begin() + end_pos + 1, input.end() - (input.begin() + end_pos + 1));
+                        input.remove_prefix(end_pos + 1);
                         break;
                     }
                 }
@@ -410,12 +408,18 @@ inline std::string make_deck_name(const std::string_view& str) {
 }
 
 
-inline std::string_view update_record_buffer(const std::string_view& record_buffer, const std::string_view& line) {
-    if (record_buffer.empty())
+inline std::string_view update_record_buffer(const std::string_view& record_buffer,
+                                             const std::string_view& line) {
+    // If line.data() + line.size() - record_buffer.data() < 0 something is wrong with the
+    // record_buffer, possibly caused by reading in too much data due to an incorreclty
+    // terminated keyword. In this case, the simulation might run into a segmentation
+    // fault, so instead: return the line and let the error be handled later.
+    if (record_buffer.empty() || line.data() + line.size() - record_buffer.data() < 0)
         return line;
     else {
-        std::size_t size = std::distance(record_buffer.begin(), line.end());
-        return { record_buffer.begin(), size };
+        const std::size_t size = line.data() + line.size() - record_buffer.data();
+        // intentionally not using substr since that will clamp the size
+        return {record_buffer.data(), size};
     }
 }
 
@@ -525,7 +529,7 @@ std::string_view ParserState::getline() {
 
     str::getline( this->input_stack.top().input, ln );
     this->input_stack.top().lineNR++;
-    
+
     return ln;
 }
 
@@ -533,10 +537,13 @@ std::string_view ParserState::getline() {
 
 void ParserState::ungetline(const std::string_view& line) {
     auto& file_view = this->input_stack.top().input;
-    if (line.end() + 1 != file_view.begin())
+    if (line.data() + line.size() + 1 != file_view.data())
         throw std::invalid_argument("line view does not immediately proceed file_view");
 
-    file_view = std::string_view(line.begin(), file_view.end() - line.begin());
+    const auto* end = file_view.data() + file_view.size();
+    const std::size_t size = std::distance(line.data(), end);
+    // intentionally not using substr since that will clamp the size
+    file_view = {line.data(), size};
     this->input_stack.top().lineNR--;
 }
 
@@ -621,10 +628,10 @@ void ParserState::loadString(const std::string& input) {
 void ParserState::loadFile(const std::filesystem::path& inputFile) {
 
     const auto closer = []( std::FILE* f ) { std::fclose( f ); };
-    std::unique_ptr< std::FILE, decltype( closer ) > ufp(
-            std::fopen( inputFile.c_str(), "rb" ),
-            closer
-            );
+    std::unique_ptr<std::FILE, decltype(closer)> ufp{
+        std::fopen( inputFile.generic_string().c_str(), "rb" ),
+        closer
+    };
 
     // make sure the file we'd like to parse is readable
     if( !ufp ) {
@@ -659,27 +666,31 @@ void ParserState::loadFile(const std::filesystem::path& inputFile) {
  * of the data section of any keyword.
  */
 
-void ParserState::handleRandomText(const std::string_view& keywordString ) const {
-    std::string errorKey;
-    std::string trimmedCopy = std::string( keywordString );
-    std::string msg;
-    KeywordLocation location{lastKeyWord, this->current_path(), this->line()};
+void ParserState::handleRandomText(const std::string_view& keywordString) const
+{
+    const std::string trimmedCopy { keywordString };
+    const KeywordLocation location {
+        lastKeyWord, this->current_path().generic_string(), this->line()
+    };
 
-    if (trimmedCopy == "/") {
+    std::string errorKey{};
+    std::string msg{};
+    if (trimmedCopy[0] == '/') {
         errorKey = ParseContext::PARSE_RANDOM_SLASH;
-        msg = "Extra '/' detected in {file} line {line}";
+        msg = "Extra line starting with '/' detected in {file} line {line}";
     }
     else if (lastSizeType == OTHER_KEYWORD_IN_DECK) {
-      errorKey = ParseContext::PARSE_EXTRA_RECORDS;
-      msg = "Too many records in keyword {keyword}\n"
-            "In {} line {}";
+        errorKey = ParseContext::PARSE_EXTRA_RECORDS;
+        msg = "Too many records in keyword {keyword}\n"
+            "In {file} line {line}";
     }
     else {
         errorKey = ParseContext::PARSE_RANDOM_TEXT;
         msg = fmt::format("String {} not formatted as valid keyword\n"
                           "In {{file}} line {{line}}.", keywordString);
     }
-    parseContext.handleError( errorKey , msg, location, errors );
+
+    parseContext.handleError(errorKey, msg, location, errors);
 }
 
 
@@ -750,8 +761,13 @@ newRawKeyword(const ParserKeyword& parserKeyword,
                 .parseContext
                 .handleError(
                     ParseContext::PARSE_INVALID_KEYWORD_COMBINATION,
-                    fmt::format("Incompatible keyword combination: {} declared when {} is already present.", keywordString, keyword),
-                    KeywordLocation { keywordString, parserState.current_path(), parserState.line() } ,
+                    fmt::format("Incompatible keyword combination: {} declared "
+                                "when {} is already present.", keywordString, keyword),
+                    KeywordLocation{
+                        keywordString,
+                        parserState.current_path().generic_string(),
+                        parserState.line()
+                    },
                     parserState.errors
                 );
         }
@@ -763,8 +779,13 @@ newRawKeyword(const ParserKeyword& parserKeyword,
                 .parseContext
                 .handleError(
                     ParseContext::PARSE_INVALID_KEYWORD_COMBINATION,
-                    fmt::format("Incompatible keyword combination: {} declared, but {} is missing.", keywordString, keyword),
-                    KeywordLocation { keywordString, parserState.current_path(), parserState.line() } ,
+                    fmt::format("Incompatible keyword combination: {} declared, "
+                                "but {} is missing.", keywordString, keyword),
+                    KeywordLocation{
+                        keywordString,
+                        parserState.current_path().generic_string(),
+                        parserState.line()
+                    },
                     parserState.errors
                 );
         }
@@ -1020,14 +1041,14 @@ std::unique_ptr<RawKeyword> tryParseKeyword( ParserState& parserState, const Par
             if (rawKeyword->getSizeType() == Raw::CODE) {
                 auto end_pos = line.find(parserKeyword->codeEnd());
                 if (end_pos != std::string::npos) {
-                    std::string_view line_content = { line.begin(), end_pos};
-                    record_buffer = str::update_record_buffer( record_buffer, line_content );
+                    std::string_view line_content = line.substr(0, end_pos);
+                    record_buffer = str::update_record_buffer(record_buffer, line_content);
 
                     RawRecord record(record_buffer, rawKeyword->location(), true);
                     rawKeyword->addRecord(record);
                     return rawKeyword;
                 } else
-                    record_buffer = str::update_record_buffer( record_buffer.begin(), line );
+                    record_buffer = str::update_record_buffer(record_buffer, line);
 
                 continue;
             }
@@ -1061,8 +1082,7 @@ std::unique_ptr<RawKeyword> tryParseKeyword( ParserState& parserState, const Par
                     RawRecord record("opm/flow simulation", rawKeyword->location());
                     rawKeyword->addRecord(record);
                 } else {
-                    std::size_t size = std::distance(record_buffer.begin(),record_buffer.end());
-                    RawRecord record( std::string_view{ record_buffer.begin(), size }, rawKeyword->location());
+                    RawRecord record(record_buffer, rawKeyword->location());
                     rawKeyword->addRecord(record);
                 }
                 return rawKeyword;
@@ -1076,8 +1096,8 @@ std::unique_ptr<RawKeyword> tryParseKeyword( ParserState& parserState, const Par
 
 
             if (str::isTerminatedRecordString(record_buffer)) {
-                std::size_t size = std::distance(record_buffer.begin(), record_buffer.end()) - 1;
-                RawRecord record( std::string_view{ record_buffer.begin(), size }, rawKeyword->location());
+                const std::size_t size = record_buffer.size() - 1;
+                RawRecord record(record_buffer.substr(0, size), rawKeyword->location());
                 if (rawKeyword->addRecord(record))
                     return rawKeyword;
 
@@ -1103,7 +1123,7 @@ std::unique_ptr<RawKeyword> tryParseKeyword( ParserState& parserState, const Par
 
 std::string_view advance_parser_state( ParserState& parserState, const std::string& to_keyw )
 {
-    
+
     auto line = parserState.getline();
 
     while (line != to_keyw) {
@@ -1120,7 +1140,7 @@ void addSectionKeyword(ParserState& parserState, const std::string& keyw)
 
         Opm::ParserKeyword section_keyw(keyw);
         Opm::DeckKeyword dk_keyw(section_keyw);
-                
+
         parserState.deck.addKeyword(dk_keyw);
     }
 }
@@ -1135,37 +1155,37 @@ void cleanup_deck_keyword_list(ParserState& parserState, const std::set<Opm::Ecl
     bool ignore_solution = ignore.find(Opm::Ecl::SOLUTION) !=ignore.end()  ? true : false;
     bool ignore_summary = ignore.find(Opm::Ecl::SUMMARY) !=ignore.end()  ? true : false;
     bool ignore_schedule = ignore.find(Opm::Ecl::SCHEDULE) !=ignore.end()  ? true : false;
-    
+
     std::vector<std::string> keyw_names;
     keyw_names.reserve(parserState.deck.size());
 
     std::transform(parserState.deck.begin(), parserState.deck.end(),
                    std::back_inserter(keyw_names),
                    [](const auto& dk_keyw) { return dk_keyw.name(); });
-    
+
     if (ignore_runspec){
 
-        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "RUNSPEC");            
-        auto iter_to = std::find(keyw_names.begin(), keyw_names.end(), "GRID");            
-        
+        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "RUNSPEC");
+        auto iter_to = std::find(keyw_names.begin(), keyw_names.end(), "GRID");
+
         auto n1 = std::distance(keyw_names.begin(), iter_from);
         auto n2 = std::distance(keyw_names.begin(), iter_to);
-        
+
         parserState.deck.remove_keywords(n1, n2);
         keyw_names.erase(keyw_names.begin() + n1, keyw_names.begin() + n2);
     }
 
     if (ignore_grid){
 
-        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "GRID");            
+        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "GRID");
         auto iter_to = std::find(keyw_names.begin(), keyw_names.end(), "EDIT");
-        
+
         if (iter_to == keyw_names.end())
             iter_to = std::find(keyw_names.begin(), keyw_names.end(), "PROPS");
-        
+
         auto n1 = std::distance(keyw_names.begin(), iter_from);
         auto n2 = std::distance(keyw_names.begin(), iter_to);
-        
+
         parserState.deck.remove_keywords(n1, n2);
         keyw_names.erase(keyw_names.begin() + n1, keyw_names.begin() + n2);
     }
@@ -1173,43 +1193,43 @@ void cleanup_deck_keyword_list(ParserState& parserState, const std::set<Opm::Ecl
     if (ignore_edit){
 
         auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "EDIT");
-        
+
         if (iter_from != keyw_names.end()){
             auto iter_to = std::find(keyw_names.begin(), keyw_names.end(), "PROPS");
             auto n1 = std::distance(keyw_names.begin(), iter_from);
             auto n2 = std::distance(keyw_names.begin(), iter_to);
-        
+
             parserState.deck.remove_keywords(n1, n2);
             keyw_names.erase(keyw_names.begin() + n1, keyw_names.begin() + n2);
-            
+
         }
     }
 
     if (ignore_props){
 
-        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "PROPS");            
+        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "PROPS");
         auto iter_to = std::find(keyw_names.begin(), keyw_names.end(), "REGIONS");
-        
+
         if (iter_to == keyw_names.end())
             iter_to = std::find(keyw_names.begin(), keyw_names.end(), "SOLUTION");
-        
+
         auto n1 = std::distance(keyw_names.begin(), iter_from);
         auto n2 = std::distance(keyw_names.begin(), iter_to);
-        
+
         parserState.deck.remove_keywords(n1, n2);
         keyw_names.erase(keyw_names.begin() + n1, keyw_names.begin() + n2);
     }
 
     if (ignore_regions){
 
-        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "REGIONS");            
+        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "REGIONS");
 
         if (iter_from != keyw_names.end()){
             auto iter_to = std::find(keyw_names.begin(), keyw_names.end(), "SOLUTION");
-        
+
             auto n1 = std::distance(keyw_names.begin(), iter_from);
             auto n2 = std::distance(keyw_names.begin(), iter_to);
-        
+
             parserState.deck.remove_keywords(n1, n2);
             keyw_names.erase(keyw_names.begin() + n1, keyw_names.begin() + n2);
         }
@@ -1217,15 +1237,15 @@ void cleanup_deck_keyword_list(ParserState& parserState, const std::set<Opm::Ecl
 
     if (ignore_solution){
 
-        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "SOLUTION");            
+        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "SOLUTION");
         auto iter_to = std::find(keyw_names.begin(), keyw_names.end(), "SUMMARY");
-        
+
         if (iter_to == keyw_names.end())
             iter_to = std::find(keyw_names.begin(), keyw_names.end(), "SCHEDULE");
-        
+
         auto n1 = std::distance(keyw_names.begin(), iter_from);
         auto n2 = std::distance(keyw_names.begin(), iter_to);
-        
+
         parserState.deck.remove_keywords(n1, n2);
         keyw_names.erase(keyw_names.begin() + n1, keyw_names.begin() + n2);
     }
@@ -1233,12 +1253,12 @@ void cleanup_deck_keyword_list(ParserState& parserState, const std::set<Opm::Ecl
     if (ignore_summary){
 
         auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "SUMMARY");
-        
+
         if (iter_from != keyw_names.end()){
             auto iter_to = std::find(keyw_names.begin(), keyw_names.end(), "SCHEDULE");
             auto n1 = std::distance(keyw_names.begin(), iter_from);
             auto n2 = std::distance(keyw_names.begin(), iter_to);
-        
+
             parserState.deck.remove_keywords(n1, n2);
             keyw_names.erase(keyw_names.begin() + n1, keyw_names.begin() + n2);
         }
@@ -1246,11 +1266,11 @@ void cleanup_deck_keyword_list(ParserState& parserState, const std::set<Opm::Ecl
 
     if (ignore_schedule){
 
-        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "SCHEDULE");            
-        
+        auto iter_from = std::find(keyw_names.begin(), keyw_names.end(), "SCHEDULE");
+
         auto n1 = std::distance(keyw_names.begin(), iter_from);
         auto n2 = std::distance(keyw_names.begin(), keyw_names.end());
-        
+
         parserState.deck.remove_keywords(n1, n2);
         keyw_names.erase(keyw_names.begin() + n1, keyw_names.begin() + n2);
     }
@@ -1261,11 +1281,11 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
     std::string filename = parserState.current_path().string();
 
     auto ignore = parserState.get_ignore();
-    
+
     bool has_edit = true;
     bool has_regions = true;
     bool has_summary = true;
-     
+
     if (ignore.size() > 0)
         if (!parserState.check_section_keywords(has_edit, has_regions, has_summary))
             throw std::runtime_error("Parsing individual sections not possible when section keywords in root input file");
@@ -1277,7 +1297,7 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
     bool ignore_solution = ignore.find(Opm::Ecl::SOLUTION) !=ignore.end()  ? true : false;
     bool ignore_summary = ignore.find(Opm::Ecl::SUMMARY) !=ignore.end()  ? true : false;
     bool ignore_schedule = ignore.find(Opm::Ecl::SCHEDULE) !=ignore.end()  ? true : false;
-    
+
     if ((ignore_grid) && (!has_edit) && (!ignore_edit))
         ignore_grid = false;
 
@@ -1286,21 +1306,21 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
 
     if ((ignore_solution) && (!has_summary) && (!ignore_summary))
         ignore_solution = false;
-    
+
     while( !parserState.done() ) {
-        
+
         auto rawKeyword = tryParseKeyword( parserState, parser);
         bool do_not_add = false;
-         
+
         if( !rawKeyword )
             continue;
 
         std::string_view keyw = rawKeyword->getKeywordName();
         if ((ignore_grid) && (keyw == "GRID")){
-            
+
             do_not_add = true;
             addSectionKeyword(parserState, "GRID");
-            
+
             if (has_edit){
                 keyw = advance_parser_state( parserState, "EDIT" );
                 addSectionKeyword(parserState, "EDIT");
@@ -1309,14 +1329,14 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
                 addSectionKeyword(parserState, "PROPS");
             }
         }
-          
+
         if ((ignore_edit) && (keyw=="EDIT")){
             do_not_add = true;
             addSectionKeyword(parserState, "EDIT");
             keyw = advance_parser_state( parserState, "PROPS" );
             addSectionKeyword(parserState, "PROPS");
         }
-          
+
         if ((ignore_props) && (keyw=="PROPS")){
             do_not_add = true;
             addSectionKeyword(parserState, "PROPS");
@@ -1328,16 +1348,16 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
                 keyw = advance_parser_state( parserState, "SOLUTION" );
                 addSectionKeyword(parserState, "SOLUTION");
             }
-            
+
         }
-        
+
         if ((ignore_regions) && (keyw=="REGIONS")){
             do_not_add = true;
             addSectionKeyword(parserState, "REGIONS");
             keyw = advance_parser_state( parserState, "SOLUTION" );
             addSectionKeyword(parserState, "SOLUTION");
         }
-        
+
         if ((ignore_solution) && (keyw=="SOLUTION")){
             do_not_add = true;
             addSectionKeyword(parserState, "SOLUTION");
@@ -1350,19 +1370,19 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
                 addSectionKeyword(parserState, "SCHEDULE");
             }
         }
-        
+
         if ((ignore_summary) && (keyw=="SUMMARY")){
             do_not_add = true;
             addSectionKeyword(parserState, "SUMMARY");
             keyw = advance_parser_state( parserState, "SCHEDULE" );
             addSectionKeyword(parserState, "SCHEDULE");
         }
-            
+
         if ((ignore_schedule) && (keyw=="SCHEDULE")){
             addSectionKeyword(parserState, "SCHEDULE");
             return true;
         }
-        
+
         if (rawKeyword->getKeywordName() == Opm::RawConsts::end)
             return true;
 
@@ -1388,8 +1408,9 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
 
             if (includeFile.has_value()) {
                 auto& deck_tree = parserState.deck.tree();
-                deck_tree.add_include(std::filesystem::absolute(parserState.current_path()), includeFile.value() );
-                parserState.loadFile( includeFile.value() );
+                deck_tree.add_include(std::filesystem::absolute(parserState.current_path()).generic_string(),
+                                      includeFile.value().generic_string());
+                parserState.loadFile(includeFile.value());
             }
             continue;
         }
@@ -1534,13 +1555,13 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
     Deck Parser::parseFile(const std::string &dataFileName, const ParseContext& parseContext,
                            ErrorGuard& errors, const std::vector<Opm::Ecl::SectionType>& sections) const {
 
-        
+
         std::set<Opm::Ecl::SectionType> ignore_sections;
 
         if (sections.size() > 0) {
 
             std::set<Opm::Ecl::SectionType> all_sections;
-            all_sections = {Opm::Ecl::RUNSPEC, Opm::Ecl::GRID, Opm::Ecl::EDIT, Opm::Ecl::PROPS, Opm::Ecl::REGIONS, 
+            all_sections = {Opm::Ecl::RUNSPEC, Opm::Ecl::GRID, Opm::Ecl::EDIT, Opm::Ecl::PROPS, Opm::Ecl::REGIONS,
                             Opm::Ecl::SOLUTION, Opm::Ecl::SUMMARY, Opm::Ecl::SCHEDULE};
 
             std::set<Opm::Ecl::SectionType> read_sections;
@@ -1561,18 +1582,18 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
 
            2. The relative/abolute status of the path is retained.
         */
-        
+
         std::string data_file;
         if (dataFileName[0] == '/')
-            data_file = std::filesystem::canonical(dataFileName).string();
+            data_file = std::filesystem::canonical(dataFileName).generic_string();
         else
-            data_file = std::filesystem::proximate( std::filesystem::canonical(dataFileName) );
+            data_file = std::filesystem::proximate(std::filesystem::canonical(dataFileName)).generic_string();
 
         ParserState parserState( this->codeKeywords(), parseContext, errors, data_file, ignore_sections);
         parseState( parserState, *this );
-        
+
         auto ignore = parserState.get_ignore();
-        
+
         if (ignore.size() > 0)
             cleanup_deck_keyword_list(parserState, ignore);
 
@@ -1830,7 +1851,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
 
         bool validKwIdx = curKwIdx < deck.size();
         if( !validKwIdx || deck[curKwIdx].name() != "RUNSPEC" ) {
-            std::string msg = "The first keyword of a valid deck must be RUNSPEC (is {})\n";
+            constexpr std::string_view msg = "The first keyword of a valid deck must be RUNSPEC (is {})\n";
             auto curKeyword = deck[0];
             topLevelErrors.push_back(Log::fileMessage(curKeyword.location(),
                                                       fmt::format(msg,
@@ -1848,7 +1869,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
                     const auto& parserKeyword =
                         parser.getParserKeywordFromDeckName( curKeywordName );
                     if (ensureKeywordSectionAffiliation && !parserKeyword.isValidSection(curSectionName)) {
-                        std::string msg =
+                        constexpr std::string_view msg =
                             "The keyword '{}' is located in the '{}' section where it is invalid";
                         errorGuard.addError(errorKey,
                                             Log::fileMessage(location,
