@@ -34,18 +34,18 @@
 #include <utility>
 
 namespace {
-    bool has_limit(const double x)
+    bool is_defined(const double x)
     {
-        return std::abs(x) > 0.0;
+        return  x < (Opm::RestartIO::RstGroup::UNDEFINED_VALUE / 2);
     }
 
     struct ProductionLimits
     {
         explicit ProductionLimits(const Opm::RestartIO::RstGroup& rst_group)
-            : oil { has_limit(rst_group.oil_rate_limit)    }
-            , gas { has_limit(rst_group.gas_rate_limit)    }
-            , wat { has_limit(rst_group.water_rate_limit)  }
-            , liq { has_limit(rst_group.liquid_rate_limit) }
+            : oil { is_defined(rst_group.oil_rate_limit)    }
+            , gas { is_defined(rst_group.gas_rate_limit)    }
+            , wat { is_defined(rst_group.water_rate_limit)  }
+            , liq { is_defined(rst_group.liquid_rate_limit) }
         {}
 
         bool oil { false };
@@ -57,10 +57,10 @@ namespace {
     struct GasInjectionLimits
     {
         explicit GasInjectionLimits(const Opm::RestartIO::RstGroup& rst_group)
-            : rate { has_limit(rst_group.gas_surface_limit)   }
-            , resv { has_limit(rst_group.gas_reservoir_limit) }
-            , rein { has_limit(rst_group.gas_reinject_limit)  }
-            , vrep { has_limit(rst_group.gas_voidage_limit)   }
+            : rate { is_defined(rst_group.gas_surface_limit)   }
+            , resv { is_defined(rst_group.gas_reservoir_limit) }
+            , rein { is_defined(rst_group.gas_reinject_limit)  }
+            , vrep { is_defined(rst_group.gas_voidage_limit)   }
         {}
 
         bool rate { false };
@@ -72,10 +72,10 @@ namespace {
     struct WaterInjectionLimits
     {
         explicit WaterInjectionLimits(const Opm::RestartIO::RstGroup& rst_group)
-            : rate { has_limit(rst_group.water_surface_limit)   }
-            , resv { has_limit(rst_group.water_reservoir_limit) }
-            , rein { has_limit(rst_group.water_reinject_limit)  }
-            , vrep { has_limit(rst_group.water_voidage_limit)   }
+            : rate { is_defined(rst_group.water_surface_limit)   }
+            , resv { is_defined(rst_group.water_reservoir_limit) }
+            , rein { is_defined(rst_group.water_reinject_limit)  }
+            , vrep { is_defined(rst_group.water_voidage_limit)   }
         {}
 
         bool rate { false };
@@ -125,10 +125,16 @@ namespace {
     {
         auto production = Opm::Group::GroupProductionProperties { unit_system, rst_group.name };
 
-        production.oil_target.update(rst_group.oil_rate_limit);
-        production.gas_target.update(rst_group.gas_rate_limit);
-        production.water_target.update(rst_group.water_rate_limit);
-        production.liquid_target.update(rst_group.liquid_rate_limit);
+        auto update_if_defined = [](Opm::UDAValue& target, const double value) {
+                if (is_defined(value))
+                    target.update(value);
+            };
+
+        update_if_defined(production.oil_target, rst_group.oil_rate_limit);
+        update_if_defined(production.gas_target, rst_group.gas_rate_limit);
+        update_if_defined(production.water_target, rst_group.water_rate_limit);
+        update_if_defined(production.liquid_target, rst_group.liquid_rate_limit);
+
         production.cmode = Opm::Group::ProductionCModeFromInt(rst_group.prod_cmode);
         production.group_limit_action.allRates = Opm::Group::ExceedActionFromInt(rst_group.exceed_action);
         production.guide_rate_def = Opm::Group::GuideRateProdTargetFromInt(rst_group.prod_guide_rate_def);
@@ -156,14 +162,20 @@ namespace {
     {
         auto injection = Opm::Group::GroupInjectionProperties { rst_group.name };
 
-        injection.surface_max_rate.update(rst_group.gas_surface_limit);
-        injection.resv_max_rate.update(rst_group.gas_reservoir_limit);
-        injection.target_reinj_fraction.update(rst_group.gas_reinject_limit);
-        injection.target_void_fraction.update(rst_group.gas_voidage_limit);
+        auto update_if_defined = [](Opm::UDAValue& target, const double value) {
+                if (is_defined(value))
+                    target.update(value);
+            };
+
+        update_if_defined(injection.surface_max_rate, rst_group.gas_surface_limit);
+        update_if_defined(injection.resv_max_rate, rst_group.gas_reservoir_limit);
+        update_if_defined(injection.target_reinj_fraction, rst_group.gas_reinject_limit);
+        update_if_defined(injection.target_void_fraction, rst_group.gas_voidage_limit);
+
         injection.phase = Opm::Phase::GAS;
         injection.cmode = Opm::Group::InjectionCModeFromInt(rst_group.ginj_cmode);
         injection.guide_rate_def = Opm::Group::GuideRateInjTargetFromInt(rst_group.inj_gas_guide_rate_def);
-        injection.guide_rate = rst_group.inj_gas_guide_rate;
+        injection.guide_rate = is_defined(rst_group.inj_gas_guide_rate) ? rst_group.inj_gas_guide_rate : 0.0;
 
         assign_injection_controls(active, injection);
 
@@ -176,14 +188,23 @@ namespace {
     {
         auto injection = Opm::Group::GroupInjectionProperties { rst_group.name };
 
-        injection.surface_max_rate.update(rst_group.water_surface_limit);
-        injection.resv_max_rate.update(rst_group.water_reservoir_limit);
-        injection.target_reinj_fraction.update(rst_group.water_reinject_limit);
-        injection.target_void_fraction.update(rst_group.water_voidage_limit);
+        auto update_if_defined = [](Opm::UDAValue& target, const double value) {
+                if (is_defined(value))
+                    target.update(value);
+            };
+
+        update_if_defined(injection.surface_max_rate, rst_group.water_surface_limit);
+        update_if_defined(injection.resv_max_rate, rst_group.water_reservoir_limit);
+        update_if_defined(injection.target_reinj_fraction, rst_group.water_reinject_limit);
+        update_if_defined(injection.target_void_fraction, rst_group.water_voidage_limit);
+
         injection.phase = Opm::Phase::WATER;
         injection.cmode = Opm::Group::InjectionCModeFromInt(rst_group.winj_cmode);
         injection.guide_rate_def = Opm::Group::GuideRateInjTargetFromInt(rst_group.inj_water_guide_rate_def);
-        injection.guide_rate = rst_group.inj_water_guide_rate;
+        injection.guide_rate = is_defined(rst_group.inj_water_guide_rate) ? rst_group.inj_water_guide_rate : 0;
+
+        //Note: InjectionProperties::available_group_control is not recovered during the RESTART reading
+        //      when InjectionProperties::cmode is FLD, this will not be recovered during the RESTART reading
 
         assign_injection_controls(active, injection);
 
@@ -209,7 +230,7 @@ Group::Group(const std::string& name, std::size_t insert_index_arg, double udq_u
     unit_system(unit_system_arg),
     group_type(GroupType::NONE),
     gefac(1),
-    transfer_gefac(true),
+    use_efficiency_in_network(true),
     production_properties(unit_system, name)
 {
     // All groups are initially created as children of the "FIELD" group.
@@ -230,11 +251,11 @@ Group::Group(const RestartIO::RstGroup& rst_group, std::size_t insert_index_arg,
         this->updateProduction(make_production_properties(rst_group, prod_limits, unit_system_arg));
     }
 
-    if ((rst_group.ginj_cmode != 0) || has_active(gas_inj_limits)) {
+    if ((rst_group.ginj_cmode != 0) || has_active(gas_inj_limits) || (rst_group.inj_gas_guide_rate_def != 0)) {
         this->updateInjection(make_injection_properties(rst_group, gas_inj_limits));
     }
 
-    if ((rst_group.winj_cmode != 0) || has_active(water_inj_limits)) {
+    if ((rst_group.winj_cmode != 0) || has_active(water_inj_limits) || (rst_group.inj_water_guide_rate_def != 0)) {
         this->updateInjection(make_injection_properties(rst_group, water_inj_limits));
     }
 }
@@ -249,7 +270,7 @@ Group Group::serializationTestObject()
     result.unit_system = UnitSystem::serializationTestObject();
     result.group_type = GroupType::PRODUCTION;
     result.gefac = 4.0;
-    result.transfer_gefac = true;
+    result.use_efficiency_in_network = true;
     result.parent_group = "test2";
     result.m_wells = {{"test3", "test4"}, {"test5", "test6"}};
     result.m_groups = {{"test7", "test8"}, {"test9", "test10"}};
@@ -680,27 +701,31 @@ void Group::delGroup(const std::string& group_name) {
         };
 }
 
-bool Group::update_gefac(double gf, bool transfer_gf) {
+bool Group::update_gefac(double gf, bool use_efficiency_in_network_arg) {
     bool update = false;
     if (this->gefac != gf) {
         this->gefac = gf;
         update = true;
     }
 
-    if (this->transfer_gefac != transfer_gf) {
-        this->transfer_gefac = transfer_gf;
+    if (this->use_efficiency_in_network != use_efficiency_in_network_arg) {
+        this->use_efficiency_in_network = use_efficiency_in_network_arg;
         update = true;
     }
 
     return update;
 }
 
-double Group::getGroupEfficiencyFactor() const {
+double Group::getGroupEfficiencyFactor(bool network) const {
+    if (network && !(this->use_efficiency_in_network)) {
+        return 1.0;
+    }
+
     return this->gefac;
 }
 
-bool Group::getTransferGroupEfficiencyFactor() const {
-    return this->transfer_gefac;
+bool Group::useEfficiencyInNetwork() const {
+    return this->use_efficiency_in_network;
 }
 
 const std::string& Group::parent() const {
@@ -865,6 +890,14 @@ bool Group::has_gpmaint_control(ProductionCMode control) const {
 
 }
 
+bool Group::as_choke() const {
+    return this->m_choke_group.has_value();
+}
+
+void Group::as_choke(const std::string& group) {
+    this->m_choke_group = group;
+}
+
 const std::string Group::ExceedAction2String( ExceedAction enumValue ) {
     switch(enumValue) {
     case ExceedAction::NONE:
@@ -905,7 +938,7 @@ Group::ExceedAction Group::ExceedActionFromString( const std::string& stringValu
 
 Group::ExceedAction Group::ExceedActionFromInt( const int value ) {
 
-    if (value < 0) return ExceedAction::NONE;
+    if (value <= 0) return ExceedAction::NONE;
     if (value == 4) return ExceedAction::RATE;
 
     throw std::invalid_argument(fmt::format("Unknown ExceedAction state integer: {}", value));
@@ -1197,7 +1230,7 @@ bool Group::operator==(const Group& data) const
            this->unit_system == data.unit_system &&
            this->group_type == data.group_type &&
            this->getGroupEfficiencyFactor() == data.getGroupEfficiencyFactor() &&
-           this->getTransferGroupEfficiencyFactor() == data.getTransferGroupEfficiencyFactor() &&
+           this->useEfficiencyInNetwork() == data.useEfficiencyInNetwork() &&
            this->parent() == data.parent() &&
            this->m_wells == data.m_wells &&
            this->m_groups == data.m_groups &&
