@@ -22,10 +22,11 @@
 
 #include <opm/input/eclipse/Deck/UDAValue.hpp>
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
-#include <opm/input/eclipse/Schedule/Well/Well.hpp>
 
 namespace Opm {
 
+enum class WellGuideRateTarget;
+
 class GuideRateModel {
 public:
 
@@ -82,7 +83,7 @@ public:
     double getE() const;
     double getF() const;
 
-    static Target convert_target(Well::GuideRateTarget well_target);
+    static Target convert_target(WellGuideRateTarget well_target);
     static Target convert_target(Group::GuideRateProdTarget group_target);
     static Target convert_target(Phase injection_phase);
     static double pot(Target target, double oil_pot, double gas_pot, double wat_pot);
