@@ -51,6 +51,7 @@
 #include "MixingRateControlKeywordHandlers.hpp"
 #include "MSW/MSWKeywordHandlers.hpp"
 #include "Network/NetworkKeywordHandlers.hpp"
+#include "ResCoup/ReservoirCouplingKeywordHandlers.hpp"
 #include "RXXKeywordHandlers.hpp"
 #include "UDQ/UDQKeywordHandlers.hpp"
 #include "Well/WellCompletionKeywordHandlers.hpp"
@@ -69,12 +70,12 @@ namespace Opm {
 
 namespace {
 
-void handleAQUCT(HandlerContext& handlerContext)
+void handleAQUCT(const HandlerContext& handlerContext)
 {
     throw OpmInputError("AQUCT is not supported as SCHEDULE keyword", handlerContext.keyword.location());
 }
 
-void handleAQUFETP(HandlerContext& handlerContext)
+void handleAQUFETP(const HandlerContext& handlerContext)
 {
     throw OpmInputError("AQUFETP is not supported as SCHEDULE keyword", handlerContext.keyword.location());
 }
@@ -107,15 +108,13 @@ void handleSource(HandlerContext& handlerContext)
 
 void handleEXIT(HandlerContext& handlerContext)
 {
-    if (handlerContext.actionx_mode) {
-        using ES = ParserKeywords::EXIT;
-        int status = handlerContext.keyword.getRecord(0).getItem<ES::STATUS_CODE>().get<int>(0);
-        OpmLog::info("Simulation exit with status: " +
-                     std::to_string(status) +
-                     " requested as part of ACTIONX at report_step: " +
-                     std::to_string(handlerContext.currentStep));
-        handlerContext.setExitCode(status);
-    }
+    using ES = ParserKeywords::EXIT;
+    int status = handlerContext.keyword.getRecord(0).getItem<ES::STATUS_CODE>().get<int>(0);
+    OpmLog::info("Simulation exit with status: " +
+                 std::to_string(status) +
+                 " requested by an action keyword at report_step: " +
+                 std::to_string(handlerContext.currentStep));
+    handlerContext.setExitCode(status);
 }
 
 void handleFBHPDEF(HandlerContext& handlerContext)
@@ -329,8 +328,7 @@ const KeywordHandlers& KeywordHandlers::getInstance()
 }
 
 KeywordHandlers::KeywordHandlers()
-{
-    handler_functions = {
+    : handler_functions{
         { "AQUCT",    &handleAQUCT      },
         { "AQUFETP",  &handleAQUFETP    },
         { "AQUFLUX",  &handleAQUFLUX    },
@@ -364,8 +362,8 @@ KeywordHandlers::KeywordHandlers()
         { "TUNING"  , &handleTUNING     },
         { "VFPINJ"  , &handleVFPINJ     },
         { "VFPPROD" , &handleVFPPROD    },
-    };
-
+    }
+{
     for (const auto& handlerFactory : {getGasLiftOptHandlers,
                                        getGroupHandlers,
                                        getGuideRateHandlers,
@@ -373,6 +371,7 @@ KeywordHandlers::KeywordHandlers()
                                        getMSWHandlers,
                                        getNetworkHandlers,
                                        getUDQHandlers,
+                                       getReservoirCouplingHandlers,
                                        getRXXHandlers,
                                        getWellCompletionHandlers,
                                        getWellHandlers,
