@@ -32,6 +32,7 @@
 #include <utility>
 #include <vector>
 
+#include <opm/input/eclipse/Schedule/Action/ActionResult.hpp>
 #include <opm/input/eclipse/Schedule/Action/SimulatorUpdate.hpp>
 #include <opm/input/eclipse/Schedule/Action/WGNames.hpp>
 #include <opm/input/eclipse/Schedule/CompletedCells.hpp>
@@ -41,16 +42,11 @@
 #include <opm/input/eclipse/Schedule/ScheduleStatic.hpp>
 #include <opm/input/eclipse/Schedule/Well/PAvg.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
+#include <opm/input/eclipse/Schedule/Well/Connection.hpp>
 #include <opm/input/eclipse/Schedule/WriteRestartFileEvents.hpp>
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
-namespace Opm
-{
-    namespace Action {
-        class ActionX;
-        class PyAction;
-        class State;
-    }
+namespace Opm {
     class ActiveGridCells;
     class Deck;
     class DeckKeyword;
@@ -66,6 +62,7 @@ namespace Opm
     class GuideRateModel;
     class HandlerContext;
     enum class InputErrorAction;
+    class NumericalAquifers;
     class ParseContext;
     class Python;
     class Runspec;
@@ -83,10 +80,25 @@ namespace Opm
     enum class WellStatus;
     class WelSegsSet;
     class WellTestConfig;
+} // namespace Opm
 
-    namespace RestartIO { struct RstState; }
+namespace Opm::ReservoirCoupling {
+    class CouplingInfo;
+} // namespace Opm::ReservoirCoupling
 
-    class Schedule {
+namespace Opm::Action {
+    class ActionX;
+    class PyAction;
+    class State;
+} // namespace Opm::Action
+
+namespace Opm::RestartIO {
+    struct RstState;
+} // namespace Opm::RestartIO
+
+namespace Opm {
+    class Schedule
+    {
     public:
         Schedule() = default;
 
@@ -100,6 +112,7 @@ namespace Opm
          *  \param errors Error configuration
          *  \param python Python interpreter to use
          *  \param lowActionParsingStrictness Reduce parsing strictness for actions
+         *  \param slave_mode Slave mode flag
          *  \param keepKeywords Keep the schdule keywords even if there are no actions
          *  \param output_interval Output interval to use
          *  \param rst Restart state to use
@@ -108,11 +121,13 @@ namespace Opm
         Schedule(const Deck& deck,
                  const EclipseGrid& grid,
                  const FieldPropsManager& fp,
+                 const NumericalAquifers& numAquifers,
                  const Runspec &runspec,
                  const ParseContext& parseContext,
                  ErrorGuard& errors,
                  std::shared_ptr<const Python> python,
                  const bool lowActionParsingStrictness = false,
+                 const bool slave_mode = false,
                  const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr,
@@ -122,11 +137,13 @@ namespace Opm
         Schedule(const Deck& deck,
                  const EclipseGrid& grid,
                  const FieldPropsManager& fp,
+                 const NumericalAquifers& numAquifers,
                  const Runspec &runspec,
                  const ParseContext& parseContext,
                  T&& errors,
                  std::shared_ptr<const Python> python,
                  const bool lowActionParsingStrictness = false,
+                 const bool slave_mode = false,
                  const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr,
@@ -135,9 +152,11 @@ namespace Opm
         Schedule(const Deck& deck,
                  const EclipseGrid& grid,
                  const FieldPropsManager& fp,
+                 const NumericalAquifers& numAquifers,
                  const Runspec &runspec,
                  std::shared_ptr<const Python> python,
                  const bool lowActionParsingStrictness = false,
+                 const bool slave_mode = false,
                  const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr,
@@ -149,6 +168,7 @@ namespace Opm
                  ErrorGuard& errors,
                  std::shared_ptr<const Python> python,
                  const bool lowActionParsingStrictness = false,
+                 const bool slave_mode = false,
                  const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr);
@@ -160,6 +180,7 @@ namespace Opm
                  T&& errors,
                  std::shared_ptr<const Python> python,
                  const bool lowActionParsingStrictness = false,
+                 const bool slave_mode = false,
                  const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr);
@@ -168,6 +189,7 @@ namespace Opm
                  const EclipseState& es,
                  std::shared_ptr<const Python> python,
                  const bool lowActionParsingStrictness = false,
+                 const bool slave_mode = false,
                  const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr);
@@ -207,24 +229,69 @@ namespace Opm
         std::vector<std::string> wellNames(const std::string& pattern) const;
         std::vector<std::string> wellNames(std::size_t timeStep) const;
         std::vector<std::string> wellNames() const;
-
+        /// Query for group existence at particular time
+        ///
+        /// \param[in] groupName Fully specified group name.
+        ///
+        /// \param[in] timeStep Zero-based report step index.
+        ///
+        /// \return Whether or not group \p groupName exists at time \p
+        /// timeStep.
         bool hasGroup(const std::string& groupName, std::size_t timeStep) const;
-        std::vector<std::string> groupNames(const std::string& pattern, std::size_t timeStep) const;
-        std::vector<std::string> groupNames(std::size_t timeStep) const;
-        std::vector<std::string> groupNames(const std::string& pattern) const;
-        std::vector<std::string> groupNames() const;
-        /*
-          The restart_groups function returns a vector of groups pointers which
-          is organized as follows:
 
-            1. The number of elements is WELLDIMS::MAXGROUPS + 1
-            2. The elements are sorted according to group.insert_index().
-            3. If there are less than WELLDIMS::MAXGROUPS nullptr is used.
-            4. The very last element corresponds to the FIELD group.
-        */
+        /// Retrieve names of all groups at particular time whose names
+        /// match a pattern.
+        ///
+        /// \param[in] pattern Group name pattern.  Expected to be a group
+        /// name root such as 'PR*' or a fully specified group name such as
+        /// 'PROD'.
+        ///
+        /// \param[in] timeStep Zero-based report step index.
+        ///
+        /// \return Names of all groups at \p timeStep whose names match the
+        /// \p pattern.
+        std::vector<std::string>
+        groupNames(const std::string& pattern, std::size_t timeStep) const;
+
+        /// Retrieve names of all groups at a particular time.
+        ///
+        /// \param[in] timeStep Zero-based report step index.
+        ///
+        /// \return Names of all groups in simulation which are active at
+        /// time \p timeStep.
+        const std::vector<std::string>& groupNames(std::size_t timeStep) const;
+
+        /// Retrieve names of all groups matching a pattern
+        ///
+        /// \param[in] pattern Group name pattern.  Expected to be a group
+        /// name root such as 'PR*' or a fully specified group name such as
+        /// 'PROD'.
+        ///
+        /// \return Names of all groups in simulation run matching the \p
+        /// pattern.
+        std::vector<std::string> groupNames(const std::string& pattern) const;
+
+        /// Retrieve names of all groups in model
+        ///
+        /// Includes FIELD group.
+        const std::vector<std::string>& groupNames() const;
+
+        /// Retrieve collection of group objects suiteable for restart file output
+        ///
+        /// The collection is sized and ordered as follows
+        ///
+        ///  -# The number of elements is WELLDIMS::MAXGROUPS + 1
+        ///  -# The elements are sorted according to group.insert_index().
+        ///  -# Nullptr represents an unused slot.
+        ///  -# The very last element corresponds to the FIELD group.
+        ///
+        /// \param[in] timeStep Zero-based report step index.
         std::vector<const Group*> restart_groups(std::size_t timeStep) const;
 
-        std::vector<std::string> changed_wells(std::size_t reportStep) const;
+        std::vector<std::string>
+        changed_wells(std::size_t reportStep,
+                      std::size_t initialStep = 0) const;
+
         const Well& getWell(std::size_t well_index, std::size_t timeStep) const;
         const Well& getWell(const std::string& wellName, std::size_t timeStep) const;
         const Well& getWellatEnd(const std::string& well_name) const;
@@ -233,7 +300,14 @@ namespace Opm
         std::vector<Well> getWells(std::size_t timeStep) const;
         std::vector<Well> getWellsatEnd() const;
 
-        const std::unordered_map<std::string, std::set<int>>& getPossibleFutureConnections() const;
+        // Get wells that have been active any time during simulation
+        std::vector<Well> getActiveWellsAtEnd() const;
+
+        // Get well names of wells that have never been active
+        std::vector<std::string> getInactiveWellNamesAtEnd() const;
+
+        const std::unordered_map<std::string, std::set<int>>&
+        getPossibleFutureConnections() const;
 
         void shut_well(const std::string& well_name, std::size_t report_step);
         void shut_well(const std::string& well_name);
@@ -242,10 +316,12 @@ namespace Opm
         void open_well(const std::string& well_name, std::size_t report_step);
         void open_well(const std::string& well_name);
         void clear_event(ScheduleEvents::Events, std::size_t report_step);
+        void add_event(ScheduleEvents::Events, std::size_t report_step);
         void applyWellProdIndexScaling(const std::string& well_name, const std::size_t reportStep, const double scalingFactor);
 
-        std::vector<const Group*> getChildGroups2(const std::string& group_name, std::size_t timeStep) const;
-        std::vector<Well> getChildWells2(const std::string& group_name, std::size_t timeStep) const;
+        //! \brief Clear out all registered events at a given report step.
+        void clearEvents(const std::size_t report_step);
+
         WellProducerCMode getGlobalWhistctlMmode(std::size_t timestep) const;
 
         const UDQConfig& getUDQConfig(std::size_t timeStep) const;
@@ -266,21 +342,19 @@ namespace Opm
         bool write_rst_file(std::size_t report_step) const;
         const std::map< std::string, int >& rst_keywords( size_t timestep ) const;
 
-        /*
-          The applyAction() is invoked from the simulator *after* an ACTIONX has
-          evaluated to true. The return value is a small structure with
-          'information' which the simulator should take into account when
-          updating internal datastructures after the ACTIONX keywords have been
-          applied.
-        */
+        // The applyAction() member function is invoked from the simulator
+        // *after* an ACTIONX has triggered.  Its return value is a small
+        // structure with 'information' which the simulator should take into
+        // account when updating internal datastructures after the ACTIONX
+        // keywords have been applied.
         SimulatorUpdate applyAction(std::size_t reportStep,
                                     const Action::ActionX& action,
-                                    const std::vector<std::string>& matching_wells,
+                                    const Action::Result::MatchingEntities& matches,
                                     const std::unordered_map<std::string, double>& wellpi);
 
         SimulatorUpdate applyAction(std::size_t reportStep,
                                     const Action::ActionX& action,
-                                    const std::vector<std::string>& matching_wells,
+                                    const Action::Result::MatchingEntities& matches,
                                     const std::unordered_map<std::string, float>& wellpi);
         /*
           The runPyAction() will run the Python script in a PYACTION keyword. In
@@ -289,8 +363,28 @@ namespace Opm
           Python code. he return value from runPyAction() comes from such a
           internal ACTIONX.
         */
-        SimulatorUpdate runPyAction(std::size_t reportStep, const Action::PyAction& pyaction, Action::State& action_state, EclipseState& ecl_state, SummaryState& summary_state);
+        SimulatorUpdate runPyAction(std::size_t reportStep,
+                                    const Action::PyAction& pyaction,
+                                    Action::State& action_state,
+                                    EclipseState& ecl_state,
+                                    SummaryState& summary_state);
 
+        SimulatorUpdate runPyAction(std::size_t reportStep,
+                                    const Action::PyAction& pyaction,
+                                    Action::State& action_state,
+                                    EclipseState& ecl_state,
+                                    SummaryState& summary_state,
+                                    const std::unordered_map<std::string, double>& target_wellpi);
+
+        SimulatorUpdate runPyAction(std::size_t reportStep,
+                                    const Action::PyAction& pyaction,
+                                    Action::State& action_state,
+                                    EclipseState& ecl_state,
+                                    SummaryState& summary_state,
+                                    const std::unordered_map<std::string, float>& target_wellpi);
+
+        SimulatorUpdate modifyCompletions(const std::size_t reportStep,
+                                          const std::map<std::string, std::vector<Connection>>& extraConns);
 
         const GasLiftOpt& glo(std::size_t report_step) const;
 
@@ -315,8 +409,8 @@ namespace Opm
           for the schedule instances created by loading a restart file.
         */
         static bool cmp(const Schedule& sched1, const Schedule& sched2, std::size_t report_step);
-        void applyKeywords(std::vector<std::unique_ptr<DeckKeyword>>& keywords, std::size_t report_step);
-        void applyKeywords(std::vector<std::unique_ptr<DeckKeyword>>& keywords);
+        void applyKeywords(std::vector<std::unique_ptr<DeckKeyword>>& keywords, std::unordered_map<std::string, double>& target_wellpi, bool action_mode, std::size_t report_step);
+        void applyKeywords(std::vector<std::unique_ptr<DeckKeyword>>& keywords, std::unordered_map<std::string, double>& target_wellpi, bool action_mode);
 
         template<class Serializer>
         void serializeOp(Serializer& serializer)
@@ -324,41 +418,18 @@ namespace Opm
             serializer(this->m_static);
             serializer(this->m_sched_deck);
             serializer(this->action_wgnames);
+            serializer(this->potential_wellopen_patterns);
             serializer(this->exit_status);
             serializer(this->snapshots);
             serializer(this->restart_output);
             serializer(this->completed_cells);
+            serializer(this->completed_cells_lgr);
+            serializer(this->completed_cells_lgr_map);
             serializer(this->m_treat_critical_as_non_critical);
             serializer(this->current_report_step);
             serializer(this->m_lowActionParsingStrictness);
             serializer(this->simUpdateFromPython);
 
-            this->template pack_unpack<PAvg>(serializer);
-            this->template pack_unpack<WellTestConfig>(serializer);
-            this->template pack_unpack<GConSale>(serializer);
-            this->template pack_unpack<GConSump>(serializer);
-            this->template pack_unpack<GroupEconProductionLimits>(serializer);
-            this->template pack_unpack<WListManager>(serializer);
-            this->template pack_unpack<Network::ExtNetwork>(serializer);
-            this->template pack_unpack<Network::Balance>(serializer);
-            this->template pack_unpack<RPTConfig>(serializer);
-            this->template pack_unpack<Action::Actions>(serializer);
-            this->template pack_unpack<UDQActive>(serializer);
-            this->template pack_unpack<UDQConfig>(serializer);
-            this->template pack_unpack<NameOrder>(serializer);
-            this->template pack_unpack<GroupOrder>(serializer);
-            this->template pack_unpack<GuideRateConfig>(serializer);
-            this->template pack_unpack<GasLiftOpt>(serializer);
-            this->template pack_unpack<RFTConfig>(serializer);
-            this->template pack_unpack<RSTConfig>(serializer);
-            this->template pack_unpack<ScheduleState::BHPDefaults>(serializer);
-            this->template pack_unpack<Source>(serializer);
-
-            this->template pack_unpack_map<int, VFPProdTable>(serializer);
-            this->template pack_unpack_map<int, VFPInjTable>(serializer);
-            this->template pack_unpack_map<std::string, Group>(serializer);
-            this->template pack_unpack_map<std::string, Well>(serializer);
-
             // If we are deserializing we need to setup the pointer to the
             // unit system since this is process specific. This is safe
             // because we set the same value in all well instances.
@@ -374,23 +445,9 @@ namespace Opm
             }
         }
 
-        template <typename T, class Serializer>
-        void pack_unpack(Serializer& serializer) {
-            std::vector<T> value_list;
-            std::vector<std::size_t> index_list;
-
-            if (serializer.isSerializing())
-                this->template pack_state<T>(value_list, index_list);
-
-            serializer(value_list);
-            serializer(index_list);
-
-            if (!serializer.isSerializing())
-                this->template unpack_state<T>(value_list, index_list);
-        }
-
         template <typename T>
-        std::vector<std::pair<std::size_t,  T>> unique() const {
+        std::vector<std::pair<std::size_t,  T>> unique() const
+        {
             std::vector<std::pair<std::size_t, T>> values;
             for (std::size_t index = 0; index < this->snapshots.size(); index++) {
                 const auto& member = this->snapshots[index].get<T>();
@@ -401,110 +458,6 @@ namespace Opm
             return values;
         }
 
-
-        template <typename T>
-        void pack_state(std::vector<T>& value_list, std::vector<std::size_t>& index_list) const {
-            auto unique_values = this->template unique<T>();
-            for (auto& [index, value] : unique_values) {
-                value_list.push_back( std::move(value) );
-                index_list.push_back( index );
-            }
-        }
-
-
-        template <typename T>
-        void unpack_state(const std::vector<T>& value_list, const std::vector<std::size_t>& index_list) {
-            std::size_t unique_index = 0;
-            while (unique_index < value_list.size()) {
-                const auto& value = value_list[unique_index];
-                const auto& first_index = index_list[unique_index];
-                auto last_index = this->snapshots.size();
-                if (unique_index < (value_list.size() - 1))
-                    last_index = index_list[unique_index + 1];
-
-                auto& target_state = this->snapshots[first_index];
-                target_state.get<T>().update( std::move(value) );
-                for (std::size_t index=first_index + 1; index < last_index; index++)
-                    this->snapshots[index].get<T>().update( target_state.get<T>() );
-
-                unique_index++;
-            }
-        }
-
-
-        template <typename K, typename T, class Serializer>
-        void pack_unpack_map(Serializer& serializer) {
-            std::vector<T> value_list;
-            std::vector<std::size_t> index_list;
-
-            if (serializer.isSerializing())
-                pack_map<K,T>(value_list, index_list);
-
-            serializer(value_list);
-            serializer(index_list);
-
-            if (!serializer.isSerializing())
-                unpack_map<K,T>(value_list, index_list);
-        }
-
-
-        template <typename K, typename T>
-        void pack_map(std::vector<T>& value_list,
-                      std::vector<std::size_t>& index_list) {
-
-            const auto& last_map = this->snapshots.back().get_map<K,T>();
-            std::vector<K> key_list{ last_map.keys() };
-            std::unordered_map<K,T> current_value;
-
-            for (std::size_t index = 0; index < this->snapshots.size(); index++) {
-                auto& state = this->snapshots[index];
-                const auto& current_map = state.template get_map<K,T>();
-                for (const auto& key : key_list) {
-                    auto& value = current_map.get_ptr(key);
-                    if (value) {
-                        auto it = current_value.find(key);
-                        if (it == current_value.end() || !(*value == it->second)) {
-                            value_list.push_back( *value );
-                            index_list.push_back( index );
-
-                            current_value[key] = *value;
-                        }
-                    }
-                }
-            }
-        }
-
-
-        template <typename K, typename T>
-        void unpack_map(const std::vector<T>& value_list,
-                        const std::vector<std::size_t>& index_list) {
-
-            std::unordered_map<K, std::vector<std::pair<std::size_t, T>>> storage;
-            for (std::size_t storage_index = 0; storage_index < value_list.size(); storage_index++) {
-                const auto& value = value_list[storage_index];
-                const auto& time_index = index_list[storage_index];
-
-                storage[ value.name() ].emplace_back( time_index, value );
-            }
-
-            for (const auto& [key, values] : storage) {
-                for (std::size_t unique_index = 0; unique_index < values.size(); unique_index++) {
-                    const auto& [time_index, value] = values[unique_index];
-                    auto last_index = this->snapshots.size();
-                    if (unique_index < (values.size() - 1))
-                        last_index = values[unique_index + 1].first;
-
-                    auto& map_value = this->snapshots[time_index].template get_map<K,T>();
-                    map_value.update(std::move(value));
-
-                    for (std::size_t index=time_index + 1; index < last_index; index++) {
-                        auto& forward_map = this->snapshots[index].template get_map<K,T>();
-                        forward_map.update( key, map_value );
-                    }
-                }
-            }
-        }
-
         friend std::ostream& operator<<(std::ostream& os, const Schedule& sched);
         void dump_deck(std::ostream& os) const;
 
@@ -520,11 +473,13 @@ namespace Opm
         ScheduleStatic m_static{};
         ScheduleDeck m_sched_deck{};
         Action::WGNames action_wgnames{};
+        std::unordered_set<std::string> potential_wellopen_patterns{}; // Set of well name patterns that potentially can open
         std::optional<int> exit_status{};
         std::vector<ScheduleState> snapshots{};
         WriteRestartFileEvents restart_output{};
         CompletedCells completed_cells{};
-
+        std::vector<CompletedCells> completed_cells_lgr{};
+        std::unordered_map<std::string, std::size_t> completed_cells_lgr_map;
         // Boolean indicating the strictness of parsing process for ActionX and PyAction.
         // If lowActionParsingStrictness is true, the simulator tries to apply unsupported
         // keywords, if lowActionParsingStrictness is false, the simulator only applies
@@ -546,6 +501,9 @@ namespace Opm
         // The copy constructor is needed for creating a mocked simulator (msim).
         std::shared_ptr<SimulatorUpdate> simUpdateFromPython{};
 
+        void init_completed_cells_lgr(const EclipseGrid& ecl_grid);
+        void init_completed_cells_lgr_map(const EclipseGrid& ecl_grid);
+
         void load_rst(const RestartIO::RstState& rst,
                       const TracerConfig& tracer_config,
                       const ScheduleGrid& grid,
@@ -597,8 +555,8 @@ namespace Opm
                            const ParseContext& parseContext,
                            ErrorGuard& errors,
                            const ScheduleGrid& grid,
-                           const std::vector<std::string>& matching_wells,
-                           bool actionx_mode,
+                           const Action::Result::MatchingEntities& matches,
+                           bool action_mode,
                            SimulatorUpdate* sim_update,
                            const std::unordered_map<std::string, double>* target_wellpi,
                            std::unordered_map<std::string, double>& wpimult_global_factor,
@@ -606,7 +564,8 @@ namespace Opm
                            std::set<std::string>* compsegs_wells = nullptr);
 
         void internalWELLSTATUSACTIONXFromPYACTION(const std::string& well_name, std::size_t report_step, const std::string& wellStatus);
-        void prefetchPossibleFutureConnections(const ScheduleGrid& grid, const DeckKeyword& keyword);
+        void prefetchPossibleFutureConnections(const ScheduleGrid& grid, const DeckKeyword& keyword,
+                                               const ParseContext& parseContext, ErrorGuard& errors);
         void store_wgnames(const DeckKeyword& keyword);
         std::vector<std::string> wellNames(const std::string& pattern,
                                            const HandlerContext& context,
