@@ -21,6 +21,7 @@
 #define SCHEDULE_TSTEP_HPP
 
 #include <opm/input/eclipse/Deck/DeckKeyword.hpp>
+#include <opm/common/utility/gpuDecorators.hpp>
 #include <opm/common/utility/TimeService.hpp>
 
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
@@ -32,6 +33,7 @@
 #include <opm/input/eclipse/Schedule/BCProp.hpp>
 #include <opm/input/eclipse/Schedule/Source.hpp>
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
+#include <opm/input/eclipse/Schedule/Well/WCYCLE.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellEnums.hpp>
 #include <opm/input/eclipse/Schedule/MessageLimits.hpp>
 #include <opm/input/eclipse/Schedule/VFPProdTable.hpp>
@@ -65,6 +67,7 @@ namespace Opm {
     class GasLiftOpt;
     class GConSale;
     class GConSump;
+    class GSatProd;
     class GroupEconProductionLimits;
     class GroupOrder;
     class GuideRateConfig;
@@ -143,6 +146,12 @@ namespace Opm {
                 return *this->m_data;
             }
 
+            template<class Serializer>
+            void serializeOp(Serializer& serializer)
+            {
+                serializer(m_data);
+            }
+
         private:
             std::shared_ptr<T> m_data;
         };
@@ -193,6 +202,9 @@ namespace Opm {
                 return (ptr != nullptr);
             }
 
+            void update(const K& key, std::shared_ptr<T> value) {
+                this->m_data.insert_or_assign(key, std::move(value));
+            }
 
             void update(T object) {
                 auto key = object.name();
@@ -277,6 +289,11 @@ namespace Opm {
                 return map_object;
             }
 
+            template<class Serializer>
+            void serializeOp(Serializer& serializer)
+            {
+                serializer(m_data);
+            }
 
         private:
             std::unordered_map<K, std::shared_ptr<T>> m_data;
@@ -385,6 +402,7 @@ namespace Opm {
 
         ptr_member<GConSale> gconsale;
         ptr_member<GConSump> gconsump;
+        ptr_member<GSatProd> gsatprod;
         ptr_member<GroupEconProductionLimits> gecon;
         ptr_member<GuideRateConfig> guide_rate;
 
@@ -409,6 +427,7 @@ namespace Opm {
 
         ptr_member<BHPDefaults> bhp_defaults;
         ptr_member<Source> source;
+        ptr_member<WCYCLE> wcycle;
 
         template <typename T>
         ptr_member<T>& get() {
@@ -428,6 +447,8 @@ namespace Opm {
                                   return this->gconsale;
             else if constexpr ( std::is_same_v<T, GConSump> )
                                   return this->gconsump;
+            else if constexpr ( std::is_same_v<T, GSatProd> )
+                                  return this->gsatprod;
             else if constexpr ( std::is_same_v<T, GroupEconProductionLimits> )
                                   return this->gecon;
             else if constexpr ( std::is_same_v<T, WListManager> )
@@ -436,6 +457,8 @@ namespace Opm {
                                   return this->network;
             else if constexpr ( std::is_same_v<T, Network::Balance> )
                                   return this->network_balance;
+            else if constexpr ( std::is_same_v<T, ReservoirCoupling::CouplingInfo> )
+                                  return this->rescoup;
             else if constexpr ( std::is_same_v<T, RPTConfig> )
                                   return this->rpt_config;
             else if constexpr ( std::is_same_v<T, Action::Actions> )
@@ -460,9 +483,13 @@ namespace Opm {
                                   return this->bhp_defaults;
             else if constexpr ( std::is_same_v<T, Source> )
                                   return this->source;
-
-            else
+            else if constexpr ( std::is_same_v<T, WCYCLE> )
+                                  return this->wcycle;
+            else {
+                #if !OPM_IS_COMPILING_WITH_GPU_COMPILER // NVCC evaluates this branch for some reason
                 static_assert(always_false1::value, "Template type <T> not supported in get()");
+                #endif
+            }
         }
 
 
@@ -478,8 +505,11 @@ namespace Opm {
                              return this->groups;
             else if constexpr ( std::is_same_v<T, Well> )
                                   return this->wells;
-            else
+            else {
+                #if !OPM_IS_COMPILING_WITH_GPU_COMPILER // NVCC evaluates this branch for some reason
                 static_assert(always_false2::value, "Template type <K,T> not supported in get_map()");
+                #endif
+            }
         }
 
         map_member<int, VFPProdTable> vfpprod;
@@ -489,13 +519,49 @@ namespace Opm {
         // constant flux aquifers
         std::unordered_map<int, SingleAquiferFlux> aqufluxs;
         BCProp bcprop;
+        // injection streams for compostional STREAM injection using WINJGAS 
+        map_member<std::string, std::vector<double>> inj_streams;
+
         std::unordered_map<std::string, double> target_wellpi;
         std::optional<NextStep> next_tstep;
 
 
         using WellPIMapType = std::unordered_map<std::string, double>;
         template<class Serializer>
-        void serializeOp(Serializer& serializer) {
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(gconsale);
+            serializer(gconsump);
+            serializer(gsatprod);
+            serializer(gecon);
+            serializer(guide_rate);
+            serializer(wlist_manager);
+            serializer(well_order);
+            serializer(group_order);
+            serializer(actions);
+            serializer(udq);
+            serializer(udq_active);
+            serializer(pavg);
+            serializer(wtest_config);
+            serializer(glo);
+            serializer(network);
+            serializer(network_balance);
+            serializer(rescoup);
+            serializer(rpt_config);
+            serializer(rft_config);
+            serializer(rst_config);
+            serializer(bhp_defaults);
+            serializer(source);
+            serializer(wcycle);
+            serializer(vfpprod);
+            serializer(vfpinj);
+            serializer(groups);
+            serializer(wells);
+            serializer(aqufluxs);
+            serializer(bcprop);
+            serializer(inj_streams);
+            serializer(target_wellpi);
+            serializer(this->next_tstep);
             serializer(m_start_time);
             serializer(m_end_time);
             serializer(m_sim_step);
@@ -504,9 +570,6 @@ namespace Opm {
             serializer(m_first_in_year);
             serializer(m_first_in_month);
             serializer(m_save_step);
-            serializer(m_sumthin);
-            serializer(this->m_rptonly);
-            serializer(this->next_tstep);
             serializer(m_tuning);
             serializer(m_nupcol);
             serializer(m_oilvap);
@@ -515,32 +578,30 @@ namespace Opm {
             serializer(m_geo_keywords);
             serializer(m_message_limits);
             serializer(m_whistctl_mode);
-            serializer(target_wellpi);
-            serializer(aqufluxs);
-            serializer(bcprop);
+            serializer(m_sumthin);
+            serializer(this->m_rptonly);
         }
 
-
     private:
-        time_point m_start_time;
-        std::optional<time_point> m_end_time;
+        time_point m_start_time{};
+        std::optional<time_point> m_end_time{};
 
         std::size_t m_sim_step = 0;
         std::size_t m_month_num = 0;
         std::size_t m_year_num = 0;
-        bool m_first_in_month;
-        bool m_first_in_year;
+        bool m_first_in_month{false};
+        bool m_first_in_year{false};
         bool m_save_step{false};
 
-        Tuning m_tuning;
-        Nupcol m_nupcol;
-        OilVaporizationProperties m_oilvap;
-        Events m_events;
-        WellGroupEvents m_wellgroup_events;
-        std::vector<DeckKeyword> m_geo_keywords;
-        MessageLimits m_message_limits;
+        Tuning m_tuning{};
+        Nupcol m_nupcol{};
+        OilVaporizationProperties m_oilvap{};
+        Events m_events{};
+        WellGroupEvents m_wellgroup_events{};
+        std::vector<DeckKeyword> m_geo_keywords{};
+        MessageLimits m_message_limits{};
         WellProducerCMode m_whistctl_mode = WellProducerCMode::CMODE_UNDEFINED;
-        std::optional<double> m_sumthin;
+        std::optional<double> m_sumthin{};
         bool m_rptonly{false};
     };
 }
