@@ -20,11 +20,14 @@
 #include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
 
 #include <opm/io/eclipse/rst/state.hpp>
+#include <opm/io/eclipse/rst/udq.hpp>
 
 #include <opm/common/OpmLog/KeywordLocation.hpp>
 #include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/common/utility/OpmInputError.hpp>
 
+#include <opm/input/eclipse/Utility/Typetools.hpp>
+
 #include <opm/input/eclipse/Schedule/MSW/SegmentMatcher.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
@@ -32,6 +35,8 @@
 #include <opm/input/eclipse/Schedule/UDQ/UDQInput.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQSet.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQState.hpp>
+#include <opm/input/eclipse/Schedule/Well/NameOrder.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellMatcher.hpp>
 
 #include <opm/input/eclipse/Parser/ParserKeywords/U.hpp> // UDQ
 
@@ -53,17 +58,35 @@
 namespace {
     std::string strip_quotes(const std::string& s)
     {
-        if (s[0] == '\'') {
-            return s.substr(1, s.size() - 2);
-        }
-        else {
+        if (s.front() != '\'') {
             return s;
         }
+
+        return s.substr(1, s.size() - 2);
     }
 
+    std::vector<std::string>
+    extractInputAssignmentSelector(const std::vector<std::string>& assignRhs)
+    {
+        auto selector = std::vector<std::string>{};
+
+        if (assignRhs.size() >= 2) {
+            selector.reserve(assignRhs.size() - 1);
+
+            std::transform(assignRhs.begin(),
+                           assignRhs.end() - 1,
+                           std::back_inserter(selector),
+                           strip_quotes);
+        }
+
+        return selector;
+    }
+
+    /// Callback for affecting a UDQ assignment
     class EvalAssign
     {
     public:
+        /// Construct callback for FIELD level UDQs
         static EvalAssign field()
         {
             return { []() {
@@ -71,30 +94,67 @@ namespace {
             }};
         }
 
-        static EvalAssign group(const std::size_t    report_step,
-                                const Opm::Schedule& sched)
+        /// Construct callback for group level UDQs
+        ///
+        /// \param[in] context Summary vectors and model's entities
+        ///
+        /// \return UDQ assignment callback
+        static EvalAssign group(const Opm::UDQContext& context)
         {
-            return { [report_step, &sched]() {
-                return [groups = sched.groupNames(report_step)]
-                    (const auto& assign)
-                {
-                    return assign.eval(groups);
+            // Return value is an EvalAssign object, created from
+            //    -> a lambda (convertible to EvalAssign::Create), that when called returns
+            //    -> a lambda (convertible to EvalAssign::Eval), that when called returns
+            //    -> a UDQSet made from UDQAssign::eval(vector, wgNameMatcher), with the
+            //    -> wgNameMatcher defined by the innermost lambda.
+            return { [&context]() {
+                return [&context](const Opm::UDQAssign& assign) {
+                    return assign.eval(context.nonFieldGroups(), [&context]
+                                       (const std::vector<std::string>& pattern)
+                    {
+                        return pattern.empty()
+                            ? context.nonFieldGroups() // No element selection => all non-FIELD groups.
+                            : context.groups(pattern.front());
+                    });
                 };
             }};
         }
 
-        static EvalAssign well(const std::size_t    report_step,
-                               const Opm::Schedule& sched)
+        /// Construct callback for well level UDQs
+        ///
+        /// \param[in] context Summary vectors and model's entities
+        ///
+        /// \return UDQ assignment callback
+        static EvalAssign well(const Opm::UDQContext& context)
         {
-            return { [report_step, &sched]() {
-                return [wells = sched.wellNames(report_step)]
-                    (const auto& assign)
-                {
-                    return assign.eval(wells);
+            // Return value is an EvalAssign object, created from
+            //    -> a lambda (convertible to EvalAssign::Create), that when called returns
+            //    -> a lambda (convertible to EvalAssign::Eval), that when called returns
+            //    -> a UDQSet made from UDQAssign::eval(vector, wgNameMatcher), with the
+            //    -> wgNameMatcher defined by the innermost lambda.
+            return { [&context]() {
+                return [&context](const Opm::UDQAssign& assign) {
+                    return assign.eval(context.wells(), [&context]
+                                       (const std::vector<std::string>& pattern)
+                    {
+                        if (pattern.size() > 1) {
+                            // Dynamic selection from "?" in an ACTIONX
+                            // block.  Selection is pattern itself.
+                            return pattern;
+                        }
+
+                        return !pattern.empty()
+                            ? context.wells(pattern.front())
+                            : context.wells(); // No element selection => assign all wells.
+                    });
                 };
             }};
         }
 
+        /// Construct callback for segment level UDQs
+        ///
+        /// \param[in] context Summary vectors and model's entities
+        ///
+        /// \return UDQ assignment callback
         static EvalAssign segment(const Opm::UDQContext& context)
         {
             return { [&context]() {
@@ -109,6 +169,11 @@ namespace {
             }};
         }
 
+        /// Affect UDQ assignment
+        ///
+        /// \param[in] assign UDQ assignment statement.
+        ///
+        /// \return UDQ Set populated by assignment.
         Opm::UDQSet operator()(const Opm::UDQAssign& assign) const
         {
             if (! this->eval_.has_value()) {
@@ -129,12 +194,27 @@ namespace {
         }
 
     private:
+        /// Evaluation function type.
+        ///
+        /// Populates a UDQ set from an assignment statement
         using Eval = std::function<Opm::UDQSet(const Opm::UDQAssign&)>;
+
+        /// Evaluation function factory type.
+        ///
+        /// Enables deferred initialisation.
         using Create = std::function<Eval()>;
 
+        /// Evaluation function factory.
         Create create_;
+
+        /// Assignment callback for populating UDQ sets.
         mutable std::optional<Eval> eval_;
 
+        /// Constructor.
+        ///
+        /// Used by static object creation functions only.
+        ///
+        /// \param[in] create Evaluation function factory.
         EvalAssign(Create create) : create_{ std::move(create) } {}
     };
 } // Anonymous namespace
@@ -142,35 +222,25 @@ namespace {
 namespace Opm {
 
     UDQConfig::UDQConfig(const UDQParams& params)
-        : udq_params(params)
-        , udqft(this->udq_params)
+        : udq_params { params }
+        , udqft      { udq_params }
     {}
 
-    UDQConfig::UDQConfig(const UDQParams& params, const RestartIO::RstState& rst_state)
-        : UDQConfig(params)
+    UDQConfig::UDQConfig(const UDQParams&           params,
+                         const RestartIO::RstState& rst_state)
+        : UDQConfig { params }
     {
-        for (const auto& rst_udq : rst_state.udqs) {
-            if (rst_udq.is_define()) {
-                KeywordLocation location("UDQ", "Restart file", 0);
-                this->add_define(rst_udq.name,
-                                 location,
-                                 { rst_udq.expression() },
-                                 rst_state.header.report_step);
-
-                auto pos = this->m_definitions.find(rst_udq.name);
-                assert (pos != this->m_definitions.end());
-
-                pos->second.update_status(rst_udq.updateStatus(),
-                                          rst_state.header.report_step);
+        for (const auto& udq : rst_state.udqs) {
+            if (udq.isDefine()) {
+                this->add_define(udq, rst_state.header.report_step);
             }
             else {
-                this->add_assign(rst_udq.name,
-                                 rst_udq.assign_selector(),
-                                 rst_udq.assign_value(),
-                                 rst_state.header.report_step);
+                this->add_assign(udq, rst_state.header.report_step);
             }
 
-            this->add_unit(rst_udq.name, rst_udq.unit);
+            if (!udq.unit.empty()) {
+                this->add_unit(udq.name, udq.unit);
+            }
         }
     }
 
@@ -190,89 +260,63 @@ namespace Opm {
         return result;
     }
 
-    const UDQParams& UDQConfig::params() const
+    const std::string& UDQConfig::unit(const std::string& key) const
     {
-        return this->udq_params;
+        const auto pair_ptr = this->units.find(key);
+        if (pair_ptr == this->units.end()) {
+            throw std::invalid_argument("No such UDQ quantity: " + key);
+        }
+
+        return pair_ptr->second;
     }
 
-    void UDQConfig::add_node(const std::string& quantity, const UDQAction action)
+    bool UDQConfig::has_unit(const std::string& keyword) const
     {
-        auto index_iter = this->input_index.find(quantity);
-        if (this->input_index.find(quantity) == this->input_index.end()) {
-            auto var_type = UDQ::varType(quantity);
-            auto insert_index = this->input_index.size();
+        return this->units.find(keyword) != this->units.end();
+    }
 
-            this->type_count[var_type] += 1;
-            this->input_index[quantity] = UDQIndex(insert_index, this->type_count[var_type], action, var_type);
+    bool UDQConfig::has_keyword(const std::string& keyword) const
+    {
+        return (this->m_assignments.find(keyword) != this->m_assignments.end())
+            || (this->m_definitions.find(keyword) != this->m_definitions.end());
+    }
+
+    void UDQConfig::add_record(SegmentMatcherFactory                 create_segment_matcher,
+                               const DeckRecord&                     record,
+                               const KeywordLocation&                location,
+                               const std::size_t                     report_step,
+                               const std::optional<DynamicSelector>& dynamic_selector)
+    {
+        using KW = ParserKeywords::UDQ;
+
+        const auto action = UDQ::actionType(record.getItem<KW::ACTION>().get<RawString>(0));
+        const auto& quantity = record.getItem<KW::QUANTITY>().get<std::string>(0);
+        const auto data = RawString::strings(record.getItem<KW::DATA>().getData<RawString>());
+
+        if (action == UDQAction::UPDATE) {
+            this->add_update(quantity, report_step, location, data);
+        }
+        else if (action == UDQAction::UNITS) {
+            this->add_unit(quantity, data.front());
+        }
+        else if (action == UDQAction::ASSIGN) {
+            this->add_assign(quantity,
+                             std::move(create_segment_matcher),
+                             extractInputAssignmentSelector(data),
+                             std::stod(data.back()),
+                             report_step,
+                             dynamic_selector);
+        }
+        else if (action == UDQAction::DEFINE) {
+            this->add_define(quantity, location, data, report_step);
         }
         else {
-            index_iter->second.action = action;
+            throw std::runtime_error {
+                "Unknown UDQ Operation " + std::to_string(static_cast<int>(action))
+            };
         }
     }
 
-    void UDQConfig::add_assign(const std::string&              quantity,
-                               SegmentMatcherFactory           create_segment_matcher,
-                               const std::vector<std::string>& selector,
-                               const double                    value,
-                               const std::size_t               report_step)
-    {
-        this->add_node(quantity, UDQAction::ASSIGN);
-
-        switch (UDQ::varType(quantity)) {
-        case UDQVarType::SEGMENT_VAR:
-            this->add_enumerated_assign(quantity,
-                                        std::move(create_segment_matcher),
-                                        selector, value, report_step);
-            break;
-
-        default:
-            this->add_named_assign(quantity, selector, value, report_step);
-            break;
-        }
-
-        if (this->m_assignments.find(quantity) != this->m_assignments.end()) {
-            this->pending_assignments_.push_back(quantity);
-        }
-    }
-
-    void UDQConfig::add_assign(const std::string&                     quantity,
-                               const std::unordered_set<std::string>& selector,
-                               const double                           value,
-                               const std::size_t                      report_step)
-    {
-        this->add_node(quantity, UDQAction::ASSIGN);
-
-        auto assignment = this->m_assignments.find(quantity);
-        if (assignment == this->m_assignments.end()) {
-            this->m_assignments.emplace(std::piecewise_construct,
-                                        std::forward_as_tuple(quantity),
-                                        std::forward_as_tuple(quantity, selector,
-                                                              value, report_step));
-        }
-        else {
-            assignment->second.add_record(selector, value, report_step);
-        }
-    }
-
-    void UDQConfig::add_define(const std::string&              quantity,
-                               const KeywordLocation&          location,
-                               const std::vector<std::string>& expression,
-                               const std::size_t               report_step)
-    {
-        this->add_node(quantity, UDQAction::DEFINE);
-
-        this->m_definitions.insert_or_assign(quantity,
-                                             UDQDefine {
-                                                 this->udq_params,
-                                                 quantity,
-                                                 report_step,
-                                                 location,
-                                                 expression
-                                             });
-
-        this->define_order.insert(quantity);
-    }
-
     void UDQConfig::add_unit(const std::string& keyword, const std::string& quoted_unit)
     {
         const auto Unit = strip_quotes(quoted_unit);
@@ -318,41 +362,57 @@ namespace Opm {
         def.update_status(update_status, report_step);
     }
 
-    void UDQConfig::add_record(SegmentMatcherFactory  create_segment_matcher,
-                               const DeckRecord&      record,
-                               const KeywordLocation& location,
-                               const std::size_t      report_step)
+    void UDQConfig::add_assign(const std::string&                    quantity,
+                               SegmentMatcherFactory                 create_segment_matcher,
+                               const std::vector<std::string>&       selector,
+                               const double                          value,
+                               const std::size_t                     report_step,
+                               const std::optional<DynamicSelector>& dynamic_selector)
     {
-        using KW = ParserKeywords::UDQ;
+        this->add_node(quantity, UDQAction::ASSIGN);
 
-        const auto action = UDQ::actionType(record.getItem<KW::ACTION>().get<RawString>(0));
-        const auto& quantity = record.getItem<KW::QUANTITY>().get<std::string>(0);
-        const auto data = RawString::strings(record.getItem<KW::DATA>().getData<RawString>());
+        switch (UDQ::varType(quantity)) {
+        case UDQVarType::SEGMENT_VAR:
+            this->add_enumerated_assign(quantity,
+                                        std::move(create_segment_matcher),
+                                        selector, value, report_step);
+            break;
 
-        if (action == UDQAction::UPDATE) {
-            this->add_update(quantity, report_step, location, data);
+        case UDQVarType::WELL_VAR:
+            this->add_assign_wellvar(quantity, selector,
+                                     value, report_step,
+                                     dynamic_selector);
+            break;
+
+        default:
+            this->add_assign_impl(quantity, selector, value, report_step);
+            break;
         }
-        else if (action == UDQAction::UNITS) {
-            this->add_unit(quantity, data.front());
-        }
-        else if (action == UDQAction::ASSIGN) {
-            auto selector = std::vector<std::string>(data.begin(), data.end() - 1);
-            std::transform(selector.cbegin(), selector.cend(), selector.begin(), strip_quotes);
-            const auto value = std::stod(data.back());
-            this->add_assign(quantity,
-                             std::move(create_segment_matcher),
-                             selector, value, report_step);
-        }
-        else if (action == UDQAction::DEFINE) {
-            this->add_define(quantity, location, data, report_step);
-        }
-        else {
-            throw std::runtime_error {
-                "Unknown UDQ Operation " + std::to_string(static_cast<int>(action))
-            };
+
+        if (this->m_assignments.find(quantity) != this->m_assignments.end()) {
+            this->pending_assignments_.push_back(quantity);
         }
     }
 
+    void UDQConfig::add_define(const std::string&              quantity,
+                               const KeywordLocation&          location,
+                               const std::vector<std::string>& expression,
+                               const std::size_t               report_step)
+    {
+        this->add_node(quantity, UDQAction::DEFINE);
+
+        this->m_definitions.insert_or_assign(quantity,
+                                             UDQDefine {
+                                                 this->udq_params,
+                                                 quantity,
+                                                 report_step,
+                                                 location,
+                                                 expression
+                                             });
+
+        this->define_order.insert(quantity);
+    }
+
     void UDQConfig::add_table(const std::string& name, UDT udt)
     {
         m_tables.emplace(name, std::move(udt));
@@ -361,14 +421,48 @@ namespace Opm {
     bool UDQConfig::clear_pending_assignments()
     {
         const auto update = ! this->pending_assignments_.empty();
+
         this->pending_assignments_.clear();
 
         return update;
     }
 
-    const UDQAssign& UDQConfig::assign(const std::string& key) const
+    void UDQConfig::eval_assign(const WellMatcher&    wm,
+                                const GroupOrder&     go,
+                                SegmentMatcherFactory create_segment_matcher,
+                                SummaryState&         st,
+                                UDQState&             udq_state) const
     {
-        return this->m_assignments.at(key);
+        auto factories = UDQContext::MatcherFactories{};
+        factories.segments = std::move(create_segment_matcher);
+
+        auto context = UDQContext {
+            this->function_table(), wm, go, this->m_tables,
+            std::move(factories), st, udq_state
+        };
+
+        this->eval_assign(context);
+    }
+
+    void UDQConfig::eval(const std::size_t       report_step,
+                         const WellMatcher&      wm,
+                         const GroupOrder&       go,
+                         SegmentMatcherFactory   create_segment_matcher,
+                         RegionSetMatcherFactory create_region_matcher,
+                         SummaryState&           st,
+                         UDQState&               udq_state) const
+    {
+        auto factories = UDQContext::MatcherFactories {};
+        factories.segments = std::move(create_segment_matcher);
+        factories.regions  = std::move(create_region_matcher);
+
+        auto context = UDQContext {
+            this->function_table(), wm, go, this->m_tables,
+            std::move(factories), st, udq_state
+        };
+
+        this->eval_assign(context);
+        this->eval_define(report_step, udq_state, context);
     }
 
     const UDQDefine& UDQConfig::define(const std::string& key) const
@@ -376,10 +470,9 @@ namespace Opm {
         return this->m_definitions.at(key);
     }
 
-    UDQAction UDQConfig::action_type(const std::string& udq_key) const
+    const UDQAssign& UDQConfig::assign(const std::string& key) const
     {
-        auto action_iter = this->input_index.find(udq_key);
-        return action_iter->second.action;
+        return this->m_assignments.at(key);
     }
 
     std::vector<UDQDefine> UDQConfig::definitions() const
@@ -430,6 +523,15 @@ namespace Opm {
         return res;
     }
 
+    void UDQConfig::exportTypeCount(std::array<int, static_cast<std::size_t>(UDQVarType::NumTypes)>& count) const
+    {
+        count.fill(0);
+
+        for (const auto& [type, cnt] : this->type_count) {
+            count[static_cast<std::size_t>(type)] = cnt;
+        }
+    }
+
     std::size_t UDQConfig::size() const
     {
         return std::count_if(this->input_index.begin(), this->input_index.end(),
@@ -442,56 +544,6 @@ namespace Opm {
                              });
     }
 
-    std::vector<UDQAssign> UDQConfig::assignments() const
-    {
-        std::vector<UDQAssign> ret;
-
-        for (const auto& [key, Input] : this->input_index) {
-            if (Input.action == UDQAction::ASSIGN) {
-                ret.push_back(this->m_assignments.at(key));
-            }
-        }
-
-        return ret;
-    }
-
-    std::vector<UDQAssign> UDQConfig::assignments(const UDQVarType var_type) const
-    {
-        std::vector<UDQAssign> filtered_assigns;
-
-        for (const auto& index_pair : this->input_index) {
-            auto assign_iter = this->m_assignments.find(index_pair.first);
-            if ((assign_iter != this->m_assignments.end()) &&
-                (assign_iter->second.var_type() == var_type))
-            {
-                filtered_assigns.push_back(assign_iter->second);
-            }
-        }
-
-        return filtered_assigns;
-    }
-
-    const std::string& UDQConfig::unit(const std::string& key) const
-    {
-        const auto pair_ptr = this->units.find(key);
-        if (pair_ptr == this->units.end()) {
-            throw std::invalid_argument("No such UDQ quantity: " + key);
-        }
-
-        return pair_ptr->second;
-    }
-
-    bool UDQConfig::has_unit(const std::string& keyword) const
-    {
-        return this->units.find(keyword) != this->units.end();
-    }
-
-    bool UDQConfig::has_keyword(const std::string& keyword) const
-    {
-        return (this->m_assignments.find(keyword) != this->m_assignments.end())
-            || (this->m_definitions.find(keyword) != this->m_definitions.end());
-    }
-
     UDQInput UDQConfig::operator[](const std::string& keyword) const
     {
         auto index_iter = this->input_index.find(keyword);
@@ -541,6 +593,40 @@ namespace Opm {
         throw std::logic_error("Internal error - should not be here");
     }
 
+    std::vector<UDQAssign> UDQConfig::assignments() const
+    {
+        std::vector<UDQAssign> ret;
+
+        for (const auto& [key, Input] : this->input_index) {
+            if (Input.action == UDQAction::ASSIGN) {
+                ret.push_back(this->m_assignments.at(key));
+            }
+        }
+
+        return ret;
+    }
+
+    std::vector<UDQAssign> UDQConfig::assignments(const UDQVarType var_type) const
+    {
+        std::vector<UDQAssign> filtered_assigns;
+
+        for (const auto& index_pair : this->input_index) {
+            auto assign_iter = this->m_assignments.find(index_pair.first);
+            if ((assign_iter != this->m_assignments.end()) &&
+                (assign_iter->second.var_type() == var_type))
+            {
+                filtered_assigns.push_back(assign_iter->second);
+            }
+        }
+
+        return filtered_assigns;
+    }
+
+    const UDQParams& UDQConfig::params() const
+    {
+        return this->udq_params;
+    }
+
     const UDQFunctionTable& UDQConfig::function_table() const
     {
         return this->udqft;
@@ -566,19 +652,64 @@ namespace Opm {
             ;
     }
 
-    void UDQConfig::eval_assign(const std::size_t report_step,
-                                const Schedule&   sched,
-                                UDQContext&       context) const
+    void UDQConfig::required_summary(std::unordered_set<std::string>& summary_keys) const
+    {
+        for (const auto& def_pair : this->m_definitions) {
+            def_pair.second.required_summary(summary_keys);
+        }
+    }
+
+    // ===========================================================================
+    // Private member functions below separator
+    // ===========================================================================
+
+    void UDQConfig::add_node(const std::string& quantity, const UDQAction action)
+    {
+        auto index_iter = this->input_index.find(quantity);
+        if (this->input_index.find(quantity) == this->input_index.end()) {
+            auto var_type = UDQ::varType(quantity);
+            auto insert_index = this->input_index.size();
+
+            this->type_count[var_type] += 1;
+            this->input_index[quantity] = UDQIndex(insert_index, this->type_count[var_type], action, var_type);
+        }
+        else {
+            index_iter->second.action = action;
+        }
+    }
+
+    void UDQConfig::add_assign(const RestartIO::RstUDQ& udq,
+                               const std::size_t        report_step)
+    {
+        this->add_node(udq.name, UDQAction::ASSIGN);
+
+        this->add_assign_impl(udq.name, udq, report_step);
+    }
+
+    void UDQConfig::add_define(const RestartIO::RstUDQ& udq,
+                               const std::size_t        report_step)
+    {
+        this->add_define(udq.name, KeywordLocation { "UDQ", "Restart file", 0 },
+                         std::vector { std::string { udq.definingExpression() } },
+                         report_step);
+
+        auto pos = this->m_definitions.find(udq.name);
+        assert (pos != this->m_definitions.end());
+
+        pos->second.update_status(udq.currentUpdateStatus(), report_step);
+    }
+
+    void UDQConfig::eval_assign(UDQContext& context) const
     {
         if (this->pending_assignments_.empty()) {
             return;             // Nothing to do
         }
 
         const auto handlers = std::map<UDQVarType, EvalAssign> {
-            { UDQVarType::FIELD_VAR  , EvalAssign::field()                   },
-            { UDQVarType::GROUP_VAR  , EvalAssign::group(report_step, sched) },
-            { UDQVarType::WELL_VAR   , EvalAssign::well(report_step, sched)  },
-            { UDQVarType::SEGMENT_VAR, EvalAssign::segment(context)          },
+            { UDQVarType::FIELD_VAR  , EvalAssign::field() },
+            { UDQVarType::GROUP_VAR  , EvalAssign::group(context) },
+            { UDQVarType::WELL_VAR   , EvalAssign::well(context) },
+            { UDQVarType::SEGMENT_VAR, EvalAssign::segment(context) },
         };
 
         // Recall: pending_assignments_ is mutable.
@@ -638,73 +769,13 @@ namespace Opm {
 
             const auto& def = def_pos->second;
             if (((select_var_type & var_type_bit(def.var_type())) == 0) || // Unwanted Var Type
-                ! udq_state.define(keyword, def.status())) // UDQ def not applicable now
+                ! udq_state.define(def.status())) // UDQ def not applicable now
             {
                 continue;
             }
 
             context.update_define(report_step, keyword, def.eval(context));
-        }
-    }
-
-    void UDQConfig::eval(const std::size_t       report_step,
-                         const Schedule&         sched,
-                         const WellMatcher&      wm,
-                         SegmentMatcherFactory   create_segment_matcher,
-                         RegionSetMatcherFactory create_region_matcher,
-                         SummaryState&           st,
-                         UDQState&               udq_state) const
-    {
-        auto factories = UDQContext::MatcherFactories {};
-        factories.segments = std::move(create_segment_matcher);
-        factories.regions  = std::move(create_region_matcher);
-
-        auto context = UDQContext {
-            this->function_table(), wm, this->m_tables,
-            std::move(factories), st, udq_state
-        };
-
-        this->eval_assign(report_step, sched, context);
-        this->eval_define(report_step, udq_state, context);
-    }
-
-    void UDQConfig::eval_assign(const std::size_t     report_step,
-                                const Schedule&       sched,
-                                const WellMatcher&    wm,
-                                SegmentMatcherFactory create_segment_matcher,
-                                SummaryState&         st,
-                                UDQState&             udq_state) const
-    {
-        auto factories = UDQContext::MatcherFactories{};
-        factories.segments = std::move(create_segment_matcher);
-
-        auto context = UDQContext {
-            this->function_table(), wm, this->m_tables,
-            std::move(factories), st, udq_state
-        };
-
-        this->eval_assign(report_step, sched, context);
-    }
-
-    void UDQConfig::required_summary(std::unordered_set<std::string>& summary_keys) const
-    {
-        for (const auto& def_pair : this->m_definitions) {
-            def_pair.second.required_summary(summary_keys);
-        }
-    }
-
-    void UDQConfig::add_named_assign(const std::string&              quantity,
-                                     const std::vector<std::string>& selector,
-                                     const double                    value,
-                                     const std::size_t               report_step)
-    {
-        auto [asgnPos, inserted] = this->m_assignments
-            .emplace(std::piecewise_construct,
-                     std::forward_as_tuple(quantity),
-                     std::forward_as_tuple(quantity, selector, value, report_step));
-
-        if (! inserted) {
-            asgnPos->second.add_record(selector, value, report_step);
+            def.clear_next();
         }
     }
 
@@ -727,14 +798,49 @@ namespace Opm {
         auto items = UDQSet::
             enumerateItems(segmentMatcher->findSegments(setDescriptor));
 
-        auto [asgnPos, inserted] = this->m_assignments
-            .emplace(std::piecewise_construct,
-                     std::forward_as_tuple(quantity),
-                     std::forward_as_tuple(quantity, items, value, report_step));
+        this->add_assign_impl(quantity, std::move(items), value, report_step);
+    }
 
-        if (! inserted) {
-            asgnPos->second.add_record(std::move(items), value, report_step);
+    void UDQConfig::add_assign_wellvar(const std::string&                    quantity,
+                                       const std::vector<std::string>&       selector,
+                                       const double                          value,
+                                       const std::size_t                     report_step,
+                                       const std::optional<DynamicSelector>& dynamic_selector)
+    {
+        if ((selector.size() != 1) || (selector.front() != "?")) {
+            // Normal well UDQ assignment.  Common case.  Selector is either
+            // empty (representing all wells), a well name, a well template,
+            // a well list, or a well list template.
+            this->add_assign_impl(quantity, selector, value, report_step);
+            return;
         }
+
+        // If we get here, then 'selector' is the one-element string vector
+        // { "?" } and we must define an assignment based on the set of
+        // triggering/matching dynamic wells from an ACTIONX condition.
+
+        if (! dynamic_selector.has_value()) {
+            throw std::logic_error {
+                fmt::format("UDQ assignment for {} does not "
+                            "have an associate match set", quantity)
+            };
+        }
+
+        const auto dynWells = dynamic_selector->wells();
+        if (! dynWells.has_value()) {
+            // No matching wells.  Nothing to do.
+            return;
+        }
+
+        const auto wellSet = dynWells->asVector();
+        if (wellSet.empty()) {
+            // No matching wells.  Nothing to do.
+            return;
+        }
+
+        // If we get here, then 'wellSet' is a non-empty set of dynamic well
+        // names.  Generate an assignment record for these wells.
+        this->add_assign_impl(quantity, wellSet, value, report_step);
     }
 
 } // namespace Opm
