@@ -42,7 +42,8 @@ class WellConnections;
 
 namespace Opm {
 
-template<class Scalar> class PAvgCalculator;
+template <typename Scalar>
+class PAvgCalculatorResult;
 
 /// Form linear combination of WBP result objects.
 ///
@@ -61,10 +62,73 @@ template<class Scalar> class PAvgCalculator;
 /// \param[in] y Second WBP result.  Typically the PV-weighted WBP result.
 ///
 /// \return \code alpha*x + beta*y \endcode.
-template<class Scalar>
-typename PAvgCalculator<Scalar>::Result
-linearCombination(const Scalar alpha, typename PAvgCalculator<Scalar>::Result        x,
-                  const Scalar beta , const typename PAvgCalculator<Scalar>::Result& y);
+template <typename Scalar>
+PAvgCalculatorResult<Scalar>
+linearCombination(const Scalar alpha, PAvgCalculatorResult<Scalar>        x,
+                  const Scalar beta , const PAvgCalculatorResult<Scalar>& y);
+
+/// Result of block-averaging well pressure procedure
+template <typename Scalar>
+class PAvgCalculatorResult
+{
+private:
+    /// Grant internal data member access to combination function.
+    template <typename T>
+    friend PAvgCalculatorResult<T>
+    linearCombination(const T alpha, PAvgCalculatorResult<T> x,
+                      const T beta , const PAvgCalculatorResult<T>& y);
+
+public:
+    /// Kind of block-averaged well pressure
+    enum class WBPMode
+    {
+        WBP,  //< Connecting cells
+        WBP4, //< Immediate neighbours
+        WBP5, //< Connecting cells and immediate neighbours
+        WBP9, //< Connecting cells, immediate, and diagonal neighbours
+    };
+
+    /// Assign single block-averaged pressure result.
+    ///
+    /// \param[in] type Block-averaged pressure kind.
+    /// \param[in] wbp Block-averaged pressure value.
+    /// \return \code *this \endcode.
+    PAvgCalculatorResult& set(const WBPMode type, const Scalar wbp)
+    {
+        this->wbp_[this->index(type)] = wbp;
+
+        return *this;
+    }
+
+    /// Retrieve numerical value of specific block-averaged well pressure.
+    ///
+    /// \param[in] type Block-averaged pressure kind.
+    /// \return Block-averaged pressure.
+    Scalar value(const WBPMode type) const
+    {
+        return this->wbp_[this->index(type)];
+    }
+
+private:
+    /// Number of block-averaged pressure kinds/modes.
+    static constexpr auto NumModes =
+        static_cast<std::size_t>(WBPMode::WBP9) + 1;
+
+    /// Storage type for block-averaged well pressure results.
+    using WBPStore = std::array<Scalar, NumModes>;
+
+    /// Block-averaged well pressure results.
+    WBPStore wbp_{};
+
+    /// Convert block-averaged pressure kind to linear index
+    ///
+    /// \param[in] mode Block-averaged pressure kind.
+    /// \return Linear index corresponding to \p mode.
+    constexpr typename WBPStore::size_type index(const WBPMode mode) const
+    {
+        return static_cast<typename WBPStore::size_type>(mode);
+    }
+};
 
 /// Facility for deriving well-level pressure values from selected
 /// block-averaging procedures.  Applicable to stopped wells which don't
@@ -76,71 +140,6 @@ protected:
     class Accumulator;
 
 public:
-    /// Result of block-averaging well pressure procedure
-    class Result
-    {
-    private:
-        /// Enclosing type's accumulator object can access internal data
-        /// members.
-        friend class Accumulator;
-
-        /// Grant internal data member access to combination function.
-        friend Result
-        linearCombination<Scalar>(const Scalar alpha, Result x,
-                                  const Scalar beta , const Result& y);
-
-    public:
-        /// Kind of block-averaged well pressure
-        enum class WBPMode
-        {
-            WBP,  //< Connecting cells
-            WBP4, //< Immediate neighbours
-            WBP5, //< Connecting cells and immediate neighbours
-            WBP9, //< Connecting cells, immediate, and diagonal neighbours
-        };
-
-        /// Retrieve numerical value of specific block-averaged well pressure.
-        ///
-        /// \param[in] type Block-averaged pressure kind.
-        /// \return Block-averaged pressure.
-        Scalar value(const WBPMode type) const
-        {
-            return this->wbp_[this->index(type)];
-        }
-
-    private:
-        /// Number of block-averaged pressure kinds/modes.
-        static constexpr auto NumModes =
-            static_cast<std::size_t>(WBPMode::WBP9) + 1;
-
-        /// Storage type for block-averaged well pressure results.
-        using WBPStore = std::array<Scalar, NumModes>;
-
-        /// Block-averaged well pressure results.
-        WBPStore wbp_{};
-
-        /// Assign single block-averaged pressure result.
-        ///
-        /// \param[in] type Block-averaged pressure kind.
-        /// \param[in] wbp Block-averaged pressure value.
-        /// \return \code *this \endcode.
-        Result& set(const WBPMode type, const Scalar wbp)
-        {
-            this->wbp_[this->index(type)] = wbp;
-
-            return *this;
-        }
-
-        /// Convert block-averaged pressure kind to linear index
-        ///
-        /// \param[in] mode Block-averaged pressure kind.
-        /// \return Linear index corresponding to \p mode.
-        constexpr typename WBPStore::size_type index(const WBPMode mode) const
-        {
-            return static_cast<typename WBPStore::size_type>(mode);
-        }
-    };
-
     /// References to source contributions owned by other party
     class Sources
     {
@@ -252,7 +251,7 @@ public:
     /// \param[in] mode Source cell selection.
     ///
     /// \return Block-average pressure
-    const Result& averagePressures() const
+    const PAvgCalculatorResult<Scalar>& averagePressures() const
     {
         return this->averagePressures_;
     }
@@ -376,7 +375,7 @@ protected:
         /// Calculate final WBP results from individual contributions
         ///
         /// \return New result object.
-        Result getFinalResult() const;
+        PAvgCalculatorResult<Scalar> getFinalResult() const;
 
     private:
         /// Implementation class
@@ -418,26 +417,21 @@ private:
     {
         /// Constructor.
         ///
-        /// \param[in] ctf_arg Connection's transmissiblity factor
+        /// Mostly for convenience to enable using vector<>::emplace_back().
         ///
-        /// \param[in] depth_arg Connection's depth
+        /// \param[in] ctf_arg Connection's transmissiblity factor
         ///
         /// \param[in] cell_arg Connection's connecting cell.  Enumerated
         ///   local contributing cell.
         PAvgConnection(const Scalar         ctf_arg,
-                       const Scalar         depth_arg,
                        const ContrIndexType cell_arg)
-            : ctf  (ctf_arg)
-            , depth(depth_arg)
-            , cell (cell_arg)
+            : ctf  { ctf_arg }
+            , cell { cell_arg }
         {}
 
         /// Connection transmissiblity factor.
         Scalar ctf{};
 
-        /// Connection's depth.
-        Scalar depth{};
-
         /// Index into \c contributingCells_ of connection's cell.
         ContrIndexType cell{};
 
@@ -471,8 +465,7 @@ private:
     /// connection \code inputConn_[i] \endcode.
     ///
     /// Needed to handle connections to inactive cells.  See
-    /// pruneInactiveConnections() and connectionPressureOffsetWell() for
-    /// details.
+    /// pruneInactiveConnections() and connectionDensityWell() for details.
     std::vector<typename std::vector<PAvgConnection>::size_type> inputConn_{};
 
     /// Collection of all (global) cell indices that potentially contribute
@@ -482,7 +475,7 @@ private:
     /// Well level pressure values derived from block-averaging procedures.
     ///
     /// Cached end result from \code inferBlockAveragePressures() \endcode.
-    Result averagePressures_{};
+    PAvgCalculatorResult<Scalar> averagePressures_{};
 
     /// Include reservoir connection and all direction-dependent level 1 and
     /// level 2 neighbours of connection's connecting cell into known cell
@@ -646,8 +639,14 @@ private:
     ///
     /// \param[in] controls Averaging procedure controls.
     ///
-    /// \param[in] connDP Pressure correction term for each reservoir
-    ///   connection.
+    /// \param[in] gravity Strength of gravity in SI units [m/s^2].
+    ///
+    /// \param[in] refDepth Well's reference depth for block-average
+    ///   pressure calculation.  Often, but not always, equal to the well's
+    ///   bottom-hole pressure reference depth.
+    ///
+    /// \param[in] connDensity Mixture density for pressure correction term
+    ///   for each reservoir connection.
     ///
     /// \param[in] connIndex Translation method from active connection index
     ///   to index into all known reservoir connections.
@@ -657,7 +656,9 @@ private:
     template <typename ConnIndexMap, typename CTFPressureWeightFunction>
     void accumulateLocalContributions(const Sources&             sources,
                                       const PAvg&                controls,
-                                      const std::vector<Scalar>& connDP,
+                                      const Scalar               gravity,
+                                      const Scalar               refDepth,
+                                      const std::vector<Scalar>& connDensity,
                                       ConnIndexMap               connIndex,
                                       CTFPressureWeightFunction  ctfPressWeight);
 
@@ -687,15 +688,23 @@ private:
     ///
     /// \param[in] controls Averaging procedure controls.
     ///
-    /// \param[in] connDP Pressure correction term for each reservoir
-    ///   connection.
+    /// \param[in] gravity Strength of gravity in SI units [m/s^2].
+    ///
+    /// \param[in] refDepth Well's reference depth for block-average
+    ///   pressure calculation.  Often, but not always, equal to the well's
+    ///   bottom-hole pressure reference depth.
+    ///
+    /// \param[in] connDensity Mixture density for pressure correction term
+    ///   for each reservoir connection.
     ///
     /// \param[in] connIndex Translation method from active connection index
     ///   to index into all known reservoir connections.
     template <typename ConnIndexMap>
     void accumulateLocalContributions(const Sources&             sources,
                                       const PAvg&                controls,
-                                      const std::vector<Scalar>& connDP,
+                                      const Scalar               gravity,
+                                      const Scalar               refDepth,
+                                      const std::vector<Scalar>& connDensity,
                                       ConnIndexMap&&             connIndex);
 
     /// First dispatch level before going to calculation routine which
@@ -707,10 +716,18 @@ private:
     ///
     /// \param[in] controls Averaging procedure controls.
     ///
-    /// \param[in] connDP Pressure correction term for each reservoir
-    ///   connection.
+    /// \param[in] gravity Strength of gravity in SI units [m/s^2].
+    ///
+    /// \param[in] refDepth Well's reference depth for block-average
+    ///   pressure calculation.  Often, but not always, equal to the well's
+    ///   bottom-hole pressure reference depth.
+    ///
+    /// \param[in] connDensity Mixture density for pressure correction term
+    ///   for each reservoir connection.
     void accumulateLocalContribOpen(const Sources&             sources,
                                     const PAvg&                controls,
+                                    const Scalar               gravity,
+                                    const Scalar               refDepth,
                                     const std::vector<Scalar>& connDP);
 
     /// First dispatch level before going to calculation routine which
@@ -722,13 +739,21 @@ private:
     ///
     /// \param[in] controls Averaging procedure controls.
     ///
-    /// \param[in] connDP Pressure correction term for each reservoir
-    ///   connection.
+    /// \param[in] gravity Strength of gravity in SI units [m/s^2].
+    ///
+    /// \param[in] refDepth Well's reference depth for block-average
+    ///   pressure calculation.  Often, but not always, equal to the well's
+    ///   bottom-hole pressure reference depth.
+    ///
+    /// \param[in] connDensity Mixture density for pressure correction term
+    ///   for each reservoir connection.
     void accumulateLocalContribAll(const Sources&             sources,
                                    const PAvg&                controls,
-                                   const std::vector<Scalar>& connDP);
+                                   const Scalar               gravity,
+                                   const Scalar               refDepth,
+                                   const std::vector<Scalar>& connDensity);
 
-    /// Compute pressure correction term/offset using Well method
+    /// Compute connection level mixture density using Well method
     ///
     /// Uses mixture density from well bore.
     ///
@@ -748,25 +773,18 @@ private:
     ///
     /// \param[in] sources Connection and cell-level raw data.
     ///
-    /// \param[in] gravity Strength of gravity in SI units [m/s^2].
-    ///
-    /// \param[in] refDepth Well's reference depth for block-average
-    ///   pressure calculation.  Often, but not always, equal to the well's
-    ///   bottom-hole pressure reference depth.
-    ///
     /// \param[in] connIndex Translation method from active connection index
     ///   to index into all known reservoir connections.
     ///
-    /// \return Pressure correction term for each active connection.
+    /// \return Mixture density for pressure correction term for each
+    ///   reservoir connection.
     template <typename ConnIndexMap>
     std::vector<Scalar>
-    connectionPressureOffsetWell(const std::size_t nconn,
-                                 const Sources&    sources,
-                                 const Scalar      gravity,
-                                 const Scalar      refDepth,
-                                 ConnIndexMap      connIndex) const;
+    connectionDensityWell(const std::size_t nconn,
+                          const Sources&    sources,
+                          ConnIndexMap      connIndex) const;
 
-    /// Compute pressure correction term/offset using Reservoir method
+    /// Compute connection level mixture density using Reservoir method
     ///
     /// Uses pore-volume weighted mixture density from connecting cell and
     /// its level 1 and level 2 neighbours.
@@ -787,47 +805,35 @@ private:
     ///
     /// \param[in] sources Connection and cell-level raw data.
     ///
-    /// \param[in] gravity Strength of gravity in SI units [m/s^2].
-    ///
-    /// \param[in] refDepth Well's reference depth for block-average
-    ///   pressure calculation.  Often, but not always, equal to the well's
-    ///   bottom-hole pressure reference depth.
-    ///
     /// \param[in] connIndex Translation method from active connection index
     ///   to index into all known reservoir connections.
     ///
-    /// \return Pressure correction term for each active connection.
+    /// \return Mixture density for pressure correction term for each
+    ///   reservoir connection.
     template <typename ConnIndexMap>
     std::vector<Scalar>
-    connectionPressureOffsetRes(const std::size_t nconn,
-                                const Sources&    sources,
-                                const Scalar      gravity,
-                                const Scalar      refDepth,
-                                ConnIndexMap      connIndex) const;
+    connectionDensityRes(const std::size_t nconn,
+                         const Sources&    sources,
+                         ConnIndexMap      connIndex) const;
 
-    /// Top-level entry point for computing the pressure correction
-    /// term/offset of each active reservoir connection
+    /// Top-level entry point for computing connection level mixture
+    /// densities for the pressure correction term/offset of each active
+    /// reservoir connection
     ///
     /// Will dispatch to lower level calculation routines based on algorithm
     /// selection parameter in procedure controls.
     ///
     /// \param[in] sources Connection and cell-level raw data.
     ///
-    /// \param[in] controls Averaging procedure controls.  This function uses
-    ///   the depth correction and open connections flags.
+    /// \param[in] controls Averaging procedure controls.  This function
+    ///   uses the depth correction and open connections flags.
     ///
-    /// \param[in] gravity Strength of gravity in SI units [m/s^2].
-    ///
-    /// \param[in] refDepth Well's reference depth for block-average
-    ///   pressure calculation.  Often, but not always, equal to the well's
-    ///   bottom-hole pressure reference depth.
-    ///
-    /// \return Pressure correction term for each active connection.
+    /// \return Mixture density for pressure correction term for each
+    ///   reservoir connection.
     std::vector<Scalar>
-    connectionPressureOffset(const Sources& sources,
-                             const PAvg&    controls,
-                             const Scalar   gravity,
-                             const Scalar   refDepth) const;
+    connectionDensity(const Sources& sources,
+                      const PAvg&    controls,
+                      const Scalar   gravity) const;
 };
 
 } // namespace Opm
