@@ -104,6 +104,10 @@ public:
 
     using GasInflowEquation = WellGasInflowEquation;
 
+    void flag_lgr_well();
+    void set_lgr_well_tag(const std::string& lgr_tag_name);
+    bool is_lgr_well() const;
+    std::optional<std::string> get_lgr_well_tag() const;
     struct WellGuideRate {
         bool available;
         double guide_rate;
@@ -160,6 +164,9 @@ public:
 
         double rsRvInj;
 
+        // injection stream compostion for compositional simulation
+        std::optional<std::vector<double>> gas_inj_composition{};
+
         bool operator==(const WellInjectionProperties& other) const;
         bool operator!=(const WellInjectionProperties& other) const;
 
@@ -226,6 +233,9 @@ public:
         void update_uda(const UDQConfig& udq_config, UDQActive& udq_active, UDAControl control, const UDAValue& value);
         void handleWTMULT(Well::WELTARGCMode cmode, double factor);
 
+        void setGasInjComposition(const std::vector<double>& composition);
+        const std::vector<double>& gasInjComposition() const;
+
         template<class Serializer>
         void serializeOp(Serializer& serializer)
         {
@@ -244,6 +254,7 @@ public:
             serializer(injectorType);
             serializer(controlMode);
             serializer(rsRvInj);
+            serializer(gas_inj_composition);
         }
     };
 
@@ -370,7 +381,7 @@ public:
         void init_history(const DeckRecord& record);
         void init_vfp(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type, const int vfp_table_nr, const UnitSystem& unit_system, const DeckRecord& record);
 
-        WellProductionProperties(const DeckRecord& record);
+        explicit WellProductionProperties(const DeckRecord& record);
 
         double getBHPLimit() const;
     };
@@ -400,14 +411,16 @@ public:
          bool allow_xflow,
          bool auto_shutin,
          int pvt_table,
-         GasInflowEquation inflow_eq);
+         GasInflowEquation inflow_eq,
+         bool temp_option = false);
 
     Well(const RestartIO::RstWell& rst_well,
          int report_step,
          int rst_whistctl_cmode,
          const TracerConfig& tracer_config,
          const UnitSystem& unit_system,
-         double udq_undefined);
+         double udq_undefined,
+         const std::optional<VFPProdTable::ALQ_TYPE>& alq_type);
 
     static Well serializationTestObject();
 
@@ -438,7 +451,7 @@ public:
     bool hasRefDepth() const;
     double getRefDepth() const;
     double getDrainageRadius() const;
-    double getEfficiencyFactor() const;
+    double getEfficiencyFactor(bool network = false) const;
     double getSolventFraction() const;
     Status getStatus() const;
     const std::string& groupName() const;
@@ -448,8 +461,9 @@ public:
     bool aciveWellInjMult() const;
 
     bool hasConnections() const;
-    const std::vector<const Connection *> getConnections(int completion) const;
+    std::vector<const Connection *> getConnections(int completion) const;
     const WellConnections& getConnections() const;
+    WellConnections& getConnections();
     const WellSegments& getSegments() const;
     int maxSegmentID() const;
     int maxBranchID() const;
@@ -514,7 +528,7 @@ public:
     bool updateWellGuideRate(bool available, double guide_rate, GuideRateTarget guide_phase, double scale_factor);
     bool updateWellGuideRate(double guide_rate);
     bool updateAvailableForGroupControl(bool available);
-    bool updateEfficiencyFactor(double efficiency_factor);
+    bool updateEfficiencyFactor(double efficiency_factor, bool use_efficiency_in_network);
 
     bool updateSolventFraction(double solvent_fraction);
     bool updateTracer(std::shared_ptr<WellTracerProperties> tracer_properties);
@@ -530,12 +544,13 @@ public:
     bool updateWSEGVALV(const std::vector<std::pair<int, Valve> >& valve_pairs);
     bool updateWSEGAICD(const std::vector<std::pair<int, AutoICD> >& aicd_pairs, const KeywordLocation& location);
     bool updateWPAVE(const PAvg& pavg);
+  
+
     void updateWPaveRefDepth(double ref_depth);
     bool updateWVFPDP(std::shared_ptr<WVFPDP> wvfpdp);
     bool updateWVFPEXP(std::shared_ptr<WVFPEXP> wvfpexp);
     bool updateWDFAC(std::shared_ptr<WDFAC> wdfac);
 
-
     bool handleWELSEGS(const DeckKeyword& keyword);
     bool handleCOMPSEGS(const DeckKeyword& keyword, const ScheduleGrid& grid, const ParseContext& parseContext, ErrorGuard& errors);
     bool handleWELOPENConnections(const DeckRecord& record, Connection::State status);
@@ -596,8 +611,11 @@ public:
         serializer(pvt_table);
         serializer(gas_inflow);
         serializer(wtype);
+        serializer(ref_type);
+        serializer(lgr_tag);
         serializer(guide_rate);
         serializer(efficiency_factor);
+        serializer(use_efficiency_in_network);
         serializer(solvent_fraction);
         serializer(has_produced);
         serializer(has_injected);
@@ -618,12 +636,18 @@ public:
         serializer(wvfpexp);
         serializer(m_pavg);
         serializer(well_inj_temperature);
+        serializer(default_well_inj_temperature);
         serializer(inj_mult_mode);
         serializer(well_inj_mult);
         serializer(m_filter_concentration);
     }
 
 private:
+    enum class WellRefinementType {
+        STANDARD,
+        LGR,
+        MIXED,
+    };
     void switchToInjector();
     void switchToProducer();
 
@@ -631,6 +655,7 @@ private:
 
     std::string wname{};
     std::string group_name{};
+
     std::size_t init_step{};
     std::size_t insert_index{};
     int headI{};
@@ -641,12 +666,19 @@ private:
     bool allow_cross_flow{false};
     bool automatic_shutin{false};
     int pvt_table{};
-    GasInflowEquation gas_inflow = GasInflowEquation::STD;  // Will NOT be loaded/assigned from restart file
+
+
+    // Will NOT be loaded/assigned from restart file
+    GasInflowEquation gas_inflow = GasInflowEquation::STD;
+
     const UnitSystem* unit_system{nullptr};
     double udq_undefined{};
     WellType wtype{};
+    WellRefinementType ref_type{WellRefinementType::STANDARD};
+    std::string lgr_tag{};
     WellGuideRate guide_rate{};
     double efficiency_factor{};
+    bool use_efficiency_in_network{};
     double solvent_fraction{};
     bool has_produced = false;
     bool has_injected = false;
@@ -659,7 +691,11 @@ private:
     std::shared_ptr<WellMICPProperties> micp_properties{};
     std::shared_ptr<WellBrineProperties> brine_properties{};
     std::shared_ptr<WellTracerProperties> tracer_properties{};
-    std::shared_ptr<WellConnections> connections{}; // The WellConnections object cannot be const because of WELPI and the filterConnections method
+
+    // The WellConnections object cannot be const because of WELPI and the
+    // filterConnections method
+    std::shared_ptr<WellConnections> connections{};
+
     std::shared_ptr<WellProductionProperties> production{};
     std::shared_ptr<WellInjectionProperties> injection{};
     std::shared_ptr<WellSegments> segments{};
@@ -670,6 +706,7 @@ private:
     Status status{Status::AUTO};
     PAvg m_pavg{};
     std::optional<double> well_inj_temperature{};
+    std::optional<double> default_well_inj_temperature{std::nullopt};
     InjMultMode inj_mult_mode = InjMultMode::NONE;
     std::optional<InjMult> well_inj_mult{};
     UDAValue m_filter_concentration{};
