@@ -25,6 +25,7 @@
 #include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/common/utility/ActiveGridCells.hpp>
 #include <opm/common/utility/numeric/linearInterpolation.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/W.hpp>
 
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
@@ -51,6 +52,7 @@
 #include <cmath>
 #include <cstddef>
 #include <limits>
+#include <optional>
 #include <stdexcept>
 #include <string>
 #include <utility>
@@ -323,6 +325,7 @@ namespace Opm {
                                         const Connection::Direction      direction,
                                         const Connection::CTFKind        ctf_kind,
                                         const std::size_t                seqIndex,
+                                        const int                        lgr_grid_number,
                                         const bool                       defaultSatTabId)
     {
         const int conn_i = (i < 0) ? this->headI : i;
@@ -330,7 +333,7 @@ namespace Opm {
 
         this->m_connections.emplace_back(conn_i, conn_j, k, global_index, complnum,
                                          state, direction, ctf_kind, satTableId,
-                                         depth, ctf_props, seqIndex, defaultSatTabId);
+                                         depth, ctf_props, seqIndex, defaultSatTabId, lgr_grid_number);
     }
 
     void WellConnections::addConnection(const int i, const int j, const int k,
@@ -342,6 +345,7 @@ namespace Opm {
                                         const Connection::Direction      direction,
                                         const Connection::CTFKind        ctf_kind,
                                         const std::size_t                seqIndex,
+                                        int                              lgr_grid_number,
                                         const bool                       defaultSatTabId)
     {
         const auto complnum = static_cast<int>(this->m_connections.size()) + 1;
@@ -349,14 +353,15 @@ namespace Opm {
         this->addConnection(i, j, k, global_index, complnum, state,
                             depth, ctf_props,
                             satTableId, direction, ctf_kind,
-                            seqIndex, defaultSatTabId);
+                            seqIndex, lgr_grid_number, defaultSatTabId);
     }
 
-    void WellConnections::loadCOMPDAT(const DeckRecord&      record,
-                                      const ScheduleGrid&    grid,
-                                      const std::string&     wname,
-                                      const WDFAC&           wdfac,
-                                      const KeywordLocation& location)
+    void WellConnections::loadCOMPDATX(const DeckRecord&      record,
+                                       const ScheduleGrid&    grid,
+                                       const std::string&     wname,
+                                       const WDFAC&           wdfac,
+                                       const KeywordLocation& location,
+                                       std::optional<std::string> lgr_label = std::nullopt)
     {
         const auto& itemI = record.getItem("I");
         const auto defaulted_I = itemI.defaultApplied(0) || (itemI.get<int>(0) == 0);
@@ -379,7 +384,8 @@ namespace Opm {
 
         const auto skin_factor = record.getItem("SKIN").getSIDouble(0);
         const auto d_factor = record.getItem("D_FACTOR").getSIDouble(0);
-
+        const int lgr_grid_number = grid.get_lgr_grid_number(lgr_label);
+        
         int satTableId = -1;
         bool defaultSatTable = true;
         if (satTableIdItem.hasValue(0) && (satTableIdItem.get<int>(0) > 0)) {
@@ -403,9 +409,9 @@ namespace Opm {
         const auto angle = 6.2831853071795864769252867665590057683943387987502116419498;
 
         for (int k = K1; k <= K2; ++k) {
-            const auto& cell = grid.get_cell(I, J, k);
+            const auto& cell = grid.get_cell(I, J, k, lgr_label);
             if (!cell.is_active()) {
-                auto msg = fmt::format(R"(Problem with COMPDAT keyword
+                auto msg = fmt::format(R"(Problem with COMPDATX keyword
 In {} line {}
 The cell ({},{},{}) in well {} is not active and the connection will be ignored)",
                                        location.filename, location.lineno,
@@ -523,8 +529,8 @@ The cell ({},{},{}) in well {} is not active and the connection will be ignored)
                 const std::size_t noConn = this->m_connections.size();
                 this->addConnection(I, J, k, cell.global_index, state,
                                     cell.depth, ctf_props, satTableId,
-                                    direction, ctf_kind,
-                                    noConn, defaultSatTable);
+                                    direction, ctf_kind, noConn, 
+                                    lgr_grid_number, defaultSatTable);
             }
             else {
                 const auto compl_num = prev->complnum();
@@ -536,7 +542,7 @@ The cell ({},{},{}) in well {} is not active and the connection will be ignored)
                     I, J, k, cell.global_index, compl_num,
                     state, direction, ctf_kind, satTableId,
                     cell.depth, ctf_props,
-                    css_ind, defaultSatTable
+                    css_ind, defaultSatTable, lgr_grid_number
                 };
 
                 prev->updateSegment(conSegNo, cell.depth, css_ind, perf_range);
@@ -544,6 +550,27 @@ The cell ({},{},{}) in well {} is not active and the connection will be ignored)
         }
     }
 
+
+    void WellConnections:: loadCOMPDAT(const DeckRecord&     record,
+                                      const ScheduleGrid&    grid,
+                                      const std::string&     wname,
+                                      const WDFAC&           wdfac,
+                                      const KeywordLocation& location)
+    {
+        loadCOMPDATX(record,grid,wname,wdfac,location);
+    }
+
+
+    void WellConnections::loadCOMPDATL(const DeckRecord&      record,
+                                       const ScheduleGrid&    grid,
+                                       const std::string&     wname,
+                                       const WDFAC&           wdfac,
+                                       const KeywordLocation& location)
+    {
+        const std::string& lgr_tag = record.getItem("LGR").get<std::string>(0);
+        loadCOMPDATX(record,grid,wname,wdfac,location, lgr_tag);
+    }
+
     void WellConnections::loadCOMPTRAJ(const DeckRecord&      record,
                                        const ScheduleGrid&    grid,
                                        const std::string&     wname,
@@ -582,20 +609,34 @@ The cell ({},{},{}) in well {} is not active and the connection will be ignored)
         // Get the grid
         const auto& ecl_grid = grid.get_grid();
 
-        // Calulate the x,y,z coordinates of the begin and end of a perforation
-        external::cvf::Vec3d p_top;
-        external::cvf::Vec3d p_bot;
-        for (size_t i = 0; i < 3 ; ++i) {
-            p_top[i] = linearInterpolation(this->md, this->coord[i], perf_top.getSIDouble(0));
-            p_bot[i] = linearInterpolation(this->md, this->coord[i], perf_bot.getSIDouble(0));
-        }
+        std::vector<external::cvf::Vec3d> points;
+        std::vector<double> measured_depths;
 
-        std::vector<external::cvf::Vec3d> points{p_top, p_bot};
-        std::vector<double> md_interval{perf_top.getSIDouble(0), perf_bot.getSIDouble(0)};
+        // Calulate the x,y,z coordinates of the begin and end of a perforation
+        external::cvf::Vec3d p_top, p_bot;
+        double m_top = perf_top.getSIDouble(0), m_bot = perf_bot.getSIDouble(0);
+        for (size_t i = 0; i < 3 ; ++i) {
+            p_top[i] = linearInterpolation(this->md, this->coord[i], m_top);
+            p_bot[i] = linearInterpolation(this->md, this->coord[i], m_bot);
+        }
+        points.push_back(p_top);
+        measured_depths.push_back(m_top);
 
         external::cvf::ref<external::RigWellPath> wellPathGeometry { new external::RigWellPath };
+        points.reserve(this->coord[0].size());
+        measured_depths.reserve(this->coord[0].size());
+        for (size_t i = 0; i < coord[0].size(); ++i) {
+            if (this->md[i] > m_top and this->md[i] < m_bot) {
+                points.push_back(external::cvf::Vec3d(coord[0][i], coord[1][i], coord[2][i]));
+                measured_depths.push_back(this->md[i]);
+            }
+        }
+
+        points.push_back(p_bot);
+        measured_depths.push_back(m_bot);
+
         wellPathGeometry->setWellPathPoints(points);
-        wellPathGeometry->setMeasuredDepths(md_interval);
+        wellPathGeometry->setMeasuredDepths(measured_depths);
 
         external::cvf::ref<external::RigEclipseWellLogExtractor> e {
             new external::RigEclipseWellLogExtractor {
@@ -648,6 +689,10 @@ The cell ({},{},{}) in well {} is not active and the connection will be ignored)
             ctf_props.skin_factor = skin_factor;
             ctf_props.d_factor = d_factor;
 
+            if (defaultSatTable) {
+                satTableId = props->satnum;
+            }
+
             ctf_props.r0 = -1.0;
             ctf_props.Kh = -1.0;
             if (KhItem.hasValue(0) && (KhItem.getSIDouble(0) > 0.0)) {
@@ -721,7 +766,8 @@ CF and Kh items for well {} must both be specified or both defaulted/negative)",
                                     cell.global_index, state,
                                     cell.depth, ctf_props, satTableId,
                                     direction, ctf_kind,
-                                    noConn, defaultSatTable);
+                                    noConn, 0,
+                                    defaultSatTable);
             }
             else {
                 const auto compl_num = prev->complnum();
@@ -747,8 +793,14 @@ CF and Kh items for well {} must both be specified or both defaulted/negative)",
                                       [[maybe_unused]] const std::string&     wname,
                                       [[maybe_unused]] const KeywordLocation& location)
     {
-        this->coord[0].push_back(record.getItem("X").getSIDouble(0));
-        this->coord[1].push_back(record.getItem("Y").getSIDouble(0));
+        double x = record.getItem("X").getSIDouble(0);
+        double y = record.getItem("Y").getSIDouble(0);
+        const auto& mapaxes = grid.get_grid()->getMapAxes();
+        if (mapaxes.has_value())
+            mapaxes.value().inv_transform(x, y);
+
+        this->coord[0].push_back(x);
+        this->coord[1].push_back(y);
         this->coord[2].push_back(record.getItem("TVD").getSIDouble(0));
 
         this->md.push_back(record.getItem("MD").getSIDouble(0));
@@ -858,6 +910,20 @@ CF and Kh items for well {} must both be specified or both defaulted/negative)",
         throw std::runtime_error(" the connection is not found! \n ");
     }
 
+    Connection* WellConnections::maybeGetFromGlobalIndex(const std::size_t global_index)
+    {
+        auto conn_iter =
+            std::find_if(this->begin(), this->end(),
+                         [global_index] (const Connection& conn)
+                         { return conn.global_index() == global_index; });
+
+        if (conn_iter == this->end()) {
+            return nullptr;
+        }
+
+        return &*conn_iter;
+    }
+
     bool WellConnections::allConnectionsShut() const
     {
         if (this->empty()) {
