@@ -115,6 +115,7 @@ WellProducerCMode WellProducerCModeFromString(const std::string& stringValue);
 WellProducerCMode WellProducerCModeFromInt(const int intValue);
 std::ostream& operator<<(std::ostream& os, const WellProducerCMode& cm);
 
+std::string WellWELTARGCMode2String(WellWELTARGCMode cmode);
 WellWELTARGCMode WellWELTARGCModeFromString(const std::string& stringValue);
 
 std::string WellGuideRateTarget2String(WellGuideRateTarget enumValue);
@@ -123,6 +124,6 @@ WellGuideRateTarget WellGuideRateTargetFromString(const std::string& stringValue
 std::string WellGasInflowEquation2String(WellGasInflowEquation enumValue);
 WellGasInflowEquation WellGasInflowEquationFromString(const std::string& stringValue);
 
-}
+} // Namespace Opm
 
 #endif
