@@ -18,7 +18,6 @@
  */
 
 #include "WellKeywordHandlers.hpp"
-
 #include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/common/utility/OpmInputError.hpp>
 #include <opm/common/utility/String.hpp>
@@ -29,6 +28,7 @@
 #include <opm/input/eclipse/Parser/ParserKeywords/F.hpp>
 #include <opm/input/eclipse/Parser/ParserKeywords/W.hpp>
 
+#include <opm/input/eclipse/Schedule/Network/ExtNetwork.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleState.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleStatic.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQActive.hpp>
@@ -48,6 +48,7 @@
 #include "../HandlerContext.hpp"
 
 #include <fmt/format.h>
+#include <fmt/ranges.h>
 
 #include <algorithm>
 #include <memory>
@@ -163,6 +164,7 @@ void handleWCONHIST(HandlerContext& handlerContext)
                 handlerContext.state().events().addEvent( ScheduleEvents::PRODUCTION_UPDATE );
                 handlerContext.state().wellgroup_events().addEvent( well2.name(), ScheduleEvents::PRODUCTION_UPDATE);
                 handlerContext.state().wells.update( well2 );
+                handlerContext.affected_well(well_name);
             }
 
             // Always check if well can be opened (it could have been closed for numerical reasons and possible to operate with new params)
@@ -241,6 +243,7 @@ void handleWCONINJE(HandlerContext& handlerContext)
                     handlerContext.state().wellgroup_events().addEvent( well_name, ScheduleEvents::INJECTION_TYPE_CHANGED);
                 }
                 handlerContext.state().wells.update( std::move(well2) );
+                handlerContext.affected_well(well_name);
             }
 
             if (handlerContext.state().wells.get( well_name ).getStatus() == Well::Status::OPEN) {
@@ -251,8 +254,6 @@ void handleWCONINJE(HandlerContext& handlerContext)
             if (injection->updateUDQActive(handlerContext.state().udq.get(), udq_active)) {
                 handlerContext.state().udq_active.update( std::move(udq_active) );
             }
-
-            handlerContext.affected_well(well_name);
         }
     }
 }
@@ -320,17 +321,38 @@ void handleWCONINJH(HandlerContext& handlerContext)
                     handlerContext.state().wellgroup_events().addEvent( well_name, ScheduleEvents::INJECTION_TYPE_CHANGED);
                 }
                 handlerContext.state().wells.update( std::move(well2) );
+                handlerContext.affected_well(well_name);
             }
 
             // Always check if well can be opened (it could have been closed for numerical reasons and possible to operate with new params)
             if (handlerContext.getWellStatus(well_name) == WellStatus::OPEN) {
                 handlerContext.state().wellgroup_events().addEvent( well_name, ScheduleEvents::REQUEST_OPEN_WELL);
             }
-
         }
     }
 }
 
+bool belongsToAutoChokeGroup(const Well& well, const ScheduleState& state) {
+    const auto& network = state.network.get();
+    if (!network.active())
+        return false;
+    auto group_name = well.groupName();
+    while (group_name != "FIELD") {
+        if (network.has_node(group_name)) {
+            auto node_name = group_name;
+            if (network.node(node_name).as_choke())
+                return true;
+            while (network.uptree_branch(node_name)) {
+                node_name = network.uptree_branch(node_name)->uptree_node();
+                if (network.node(node_name).as_choke())
+                    return true;
+            }
+        }
+        group_name = state.groups.get(group_name).parent();
+    }
+    return false;
+}
+
 void handleWCONPROD(HandlerContext& handlerContext)
 {
     for (const auto& record : handlerContext.keyword) {
@@ -347,7 +369,7 @@ void handleWCONPROD(HandlerContext& handlerContext)
             const bool switching_from_injector = !well2.isProducer();
             auto properties = std::make_shared<Well::WellProductionProperties>(well2.getProductionProperties());
             properties->clearControls();
-            if (well2.isAvailableForGroupControl()) {
+            if (well2.isAvailableForGroupControl() || belongsToAutoChokeGroup(well2, handlerContext.state())) {
                 properties->addProductionControl(Well::ProducerCMode::GRUP);
             }
 
@@ -402,18 +424,49 @@ void handleWCONPROD(HandlerContext& handlerContext)
                 handlerContext.state().events().addEvent( ScheduleEvents::PRODUCTION_UPDATE );
                 handlerContext.state().wellgroup_events().addEvent( well2.name(), ScheduleEvents::PRODUCTION_UPDATE);
                 handlerContext.state().wells.update( std::move(well2) );
+                handlerContext.affected_well(well_name);
             }
 
             auto udq_active = handlerContext.state().udq_active.get();
             if (properties->updateUDQActive(handlerContext.state().udq.get(), udq_active)) {
                 handlerContext.state().udq_active.update( std::move(udq_active));
             }
-
-            handlerContext.affected_well(well_name);
         }
     }
 }
 
+void handleWCYCLE(HandlerContext& handlerContext)
+{
+    auto new_config = handlerContext.state().wcycle();
+    for (const auto& record : handlerContext.keyword) {
+        new_config.addRecord(record);
+    }
+    handlerContext.state().wcycle.update(std::move(new_config));
+}
+
+void handleWELLSTRE(HandlerContext& handlerContext)
+{
+    auto& inj_streams = handlerContext.state().inj_streams;
+    for (const auto& record : handlerContext.keyword) {
+        const auto stream_name = record.getItem<ParserKeywords::WELLSTRE::STREAM>().getTrimmedString(0);
+        const auto& composition = record.getItem<ParserKeywords::WELLSTRE::COMPOSITIONS>().getSIDoubleData();
+        const std::size_t num_comps = handlerContext.static_schedule().m_runspec.numComps();
+        if (composition.size() != num_comps) {
+            const std::string msg = fmt::format("The number of the composition values for stream '{}' is not the same as the number of components.", stream_name);
+            throw OpmInputError(msg, handlerContext.keyword.location());
+        }
+
+        const double sum = std::accumulate(composition.begin(), composition.end(), 0.0);
+        if (std::abs(sum - 1.0) > std::numeric_limits<double>::epsilon()) {
+            const std::string msg = fmt::format("The sum of the composition values for stream '{}' is not 1.0, but {}.", stream_name, sum);
+            throw OpmInputError(msg, handlerContext.keyword.location());
+        }
+        auto composition_ptr = std::make_shared<std::vector<double>>(composition);
+        inj_streams.update(stream_name, std::move(composition_ptr));
+    }
+
+}
+
 void handleWELOPEN(HandlerContext& handlerContext)
 {
     const auto& keyword = handlerContext.keyword;
@@ -496,6 +549,44 @@ void handleWELOPEN(HandlerContext& handlerContext)
     }
 }
 
+void handleWINJGAS(HandlerContext& handlerContext)
+{
+    // \Note: we do not support the item 4 MAKEUPGAS and item 5 STAGE in WINJGAS keyword yet
+    for (const auto& record : handlerContext.keyword) {
+        const std::string fluid_nature = record.getItem<ParserKeywords::WINJGAS::FLUID>().getTrimmedString(0);
+
+        // \Note: technically, only the first two characters are significant
+        // with some testing, we can determine whether we want to enforce this.
+        // at the moment, we only support full string STREAM for fluid nature
+        if (fluid_nature != "STREAM") {
+            const std::string msg = fmt::format("The fluid nature '{}' is not supported in WINJGAS keyword.", fluid_nature);
+            throw OpmInputError(msg, handlerContext.keyword.location());
+        }
+
+        const std::string stream_name = record.getItem<ParserKeywords::WINJGAS::STREAM>().getTrimmedString(0);
+        // we make sure the stream is defined in WELLSTRE keyword
+        const auto& inj_streams = handlerContext.state().inj_streams;
+        if (!inj_streams.has(stream_name)) {
+            const std::string msg = fmt::format("The stream '{}' is not defined in WELLSTRE keyword.", stream_name);
+            throw OpmInputError(msg, handlerContext.keyword.location());
+        }
+
+        const std::string wellNamePattern = record.getItem<ParserKeywords::WINJGAS::WELL>().getTrimmedString(0);
+        const auto well_names = handlerContext.wellNames(wellNamePattern, false);
+        for (const auto& well_name : well_names) {
+            auto well2 = handlerContext.state().wells.get(well_name);
+            auto injection = std::make_shared<Well::WellInjectionProperties>(well2.getInjectionProperties());
+
+            const auto& inj_stream = inj_streams.get(stream_name);
+            injection->setGasInjComposition(inj_stream);
+
+            if (well2.updateInjection(injection)) {
+                handlerContext.state().wells.update(std::move(well2));
+            }
+        }
+    }
+}
+
 void handleWELSPECS(HandlerContext& handlerContext)
 {
     using Kw = ParserKeywords::WELSPECS;
@@ -603,6 +694,26 @@ Well{0} entered with 'FIELD' parent group:
     }
 }
 
+
+void handleWELSPECL(HandlerContext& handlerContext)
+{
+    using Kw = ParserKeywords::WELSPECL;
+    auto getTrimmedName = [&handlerContext](const auto& item)
+    {
+        return trim_wgname(handlerContext.keyword,
+                           item.template get<std::string>(0),
+                           handlerContext.parseContext,
+                           handlerContext.errors);
+    };
+    handleWELSPECS(handlerContext);
+    for (const auto& record : handlerContext.keyword) {
+        const auto wellName = getTrimmedName(record.getItem<Kw::WELL>());
+        const auto lgrTag = getTrimmedName(record.getItem<Kw::LGR>());
+        handlerContext.state().wells.get(wellName).flag_lgr_well();
+        handlerContext.state().wells.get(wellName).set_lgr_well_tag(lgrTag);
+    }
+}
+
 /*
   The documentation for the WELTARG keyword says that the well
   must have been fully specified and initialized using one of the
@@ -619,6 +730,7 @@ Well{0} entered with 'FIELD' parent group:
   WCONPROD / WCONHIST before WELTARG is applied, if not the units for the
   rates will be wrong.
 */
+
 void handleWELTARG(HandlerContext& handlerContext)
 {
     const double SiFactorP = handlerContext.static_schedule().m_unit_system.parse("Pressure").getSIScaling();
@@ -671,9 +783,8 @@ void handleWELTARG(HandlerContext& handlerContext)
                     handlerContext.state().events().addEvent( ScheduleEvents::INJECTION_UPDATE );
                 }
                 handlerContext.state().wells.update( std::move(well2) );
+                handlerContext.affected_well(well_name);
             }
-
-            handlerContext.affected_well(well_name);
         }
     }
 }
@@ -891,11 +1002,15 @@ getWellHandlers()
         { "WCONINJE", &handleWCONINJE },
         { "WCONINJH", &handleWCONINJH },
         { "WCONPROD", &handleWCONPROD },
+        { "WCYCLE",   &handleWCYCLE   },
         { "WELOPEN" , &handleWELOPEN  },
+        { "WELLSTRE", &handleWELLSTRE },
         { "WELSPECS", &handleWELSPECS },
+        { "WELSPECL", &handleWELSPECL },
         { "WELTARG" , &handleWELTARG  },
         { "WELTRAJ" , &handleWELTRAJ  },
         { "WHISTCTL", &handleWHISTCTL },
+        { "WINJGAS",  &handleWINJGAS  },
         { "WLIST"   , &handleWLIST    },
         { "WPAVE"   , &handleWPAVE    },
         { "WPAVEDEP", &handleWPAVEDEP },
