@@ -32,6 +32,7 @@
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
 #include <opm/input/eclipse/Schedule/VFPProdTable.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
+#include <opm/input/eclipse/Units/Dimension.hpp>
 
 #include "../eval_uda.hpp"
 
@@ -95,8 +96,9 @@ namespace Opm {
 
     void Well::WellProductionProperties::init_vfp(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type, const int vfp_table_nr, const UnitSystem& unit_system_arg, const DeckRecord& record) {
         this->VFPTableNumber = vfp_table_nr;
-        if (alq_type) {
-            const auto alq_dim = VFPProdTable::ALQDimension(*alq_type, unit_system_arg);
+        // May have ALQ values used in UDQ calculations event without any table - use identity dimension in this case
+        if (alq_type || vfp_table_nr==0) {
+            Dimension alq_dim = alq_type ? VFPProdTable::ALQDimension(*alq_type, unit_system_arg) : Dimension(1.0);
             const auto& alq_input = record.getItem("ALQ").get<UDAValue>(0);
             if (alq_input.is<double>())
                 this->ALQValue = UDAValue(alq_input.get<double>(), alq_dim);
@@ -118,12 +120,12 @@ namespace Opm {
         this->predictionMode = false;
         // update LiquidRate. The funny construction with explicitly making a new
         // UDAValue is to ensure that the UDAValue has the correct dimension.
-        this->LiquidRate = UDAValue(this->WaterRate.get<double>() + this->OilRate.get<double>(), this->OilRate.get_dim());
+        this->LiquidRate = UDAValue(this->WaterRate.raw_value_or(0.0) + this->OilRate.raw_value_or(0.0), this->OilRate.get_dim());
 
-        if ( record.getItem( "BHP" ).hasValue(0) )
+        if ( const auto& item = record.getItem( "BHP" ); item.hasValue(0) && !item.defaultApplied(0) )
             this->BHPH = record.getItem("BHP").get<UDAValue>(0).getSI();
 
-        if ( record.getItem( "THP" ).hasValue(0) )
+        if ( const auto& item = record.getItem( "THP" ); item.hasValue(0) && !item.defaultApplied(0) )
             this->THPH = record.getItem("THP").get<UDAValue>(0).getSI();
 
         const auto& cmodeItem = record.getItem("CMODE");
@@ -300,30 +302,38 @@ void Well::WellProductionProperties::handleWCONHIST(const std::optional<VFPProdT
 
 
     void Well::WellProductionProperties::handleWTMULT(Well::WELTARGCMode cmode, double factor) {
+        auto update_target = [cmode, factor](UDAValue& target) {
+            if (target.is_defined()) {
+                target *= factor;
+                return;
+            }
+            throw std::invalid_argument(fmt::format("Cannot apply WTMULT to undefined {} target", WellWELTARGCMode2String(cmode)));
+        };
+
         switch (cmode) {
         case Well::WELTARGCMode::ORAT:
-            this->OilRate *= factor;
+            update_target(this->OilRate);
             break;
         case Well::WELTARGCMode::GRAT:
-            this->GasRate *= factor;
+            update_target(this->GasRate);
             break;
         case Well::WELTARGCMode::WRAT:
-            this->WaterRate *= factor;
+            update_target(this->WaterRate);
             break;
         case Well::WELTARGCMode::LRAT:
-            this->LiquidRate *= factor;
+            update_target(this->LiquidRate);
             break;
         case Well::WELTARGCMode::RESV:
-            this->ResVRate *= factor;
+            update_target(this->ResVRate);
             break;
         case Well::WELTARGCMode::BHP:
-            this->BHPTarget *= factor;
+            update_target(this->BHPTarget);
             break;
         case Well::WELTARGCMode::THP:
-            this->THPTarget *= factor;
+            update_target(this->THPTarget);
             break;
         case Well::WELTARGCMode::LIFT:
-            this->ALQValue *= factor;
+            update_target(this->ALQValue);
             break;
         default:
             throw std::logic_error("Unhandled WTMULT control");
