@@ -140,18 +140,34 @@ void handleWECON(HandlerContext& handlerContext)
 
 void handleWEFAC(HandlerContext& handlerContext)
 {
+    using Kw = ParserKeywords::WEFAC;
+
     for (const auto& record : handlerContext.keyword) {
-        const std::string& wellNamePattern = record.getItem("WELLNAME").getTrimmedString(0);
+        const auto wellNamePattern = record.getItem<Kw::WELLNAME>().getTrimmedString(0);
         const auto well_names = handlerContext.wellNames(wellNamePattern);
 
-        const double& efficiencyFactor = record.getItem("EFFICIENCY_FACTOR").get<double>(0);
+        if (well_names.empty()) {
+            handlerContext.invalidNamePattern(wellNamePattern);
+        }
+
+        const auto efficiencyFactor = record.getItem<Kw::EFFICIENCY_FACTOR>().get<double>(0);
+        const bool useEfficiencyInNetwork =
+            DeckItem::to_bool(record.getItem<Kw::USE_WEFAC_IN_NETWORK>().getTrimmedString(0));
 
         for (const auto& well_name : well_names) {
-            auto well2 = handlerContext.state().wells.get( well_name );
-            if (well2.updateEfficiencyFactor(efficiencyFactor)){
-                handlerContext.state().wellgroup_events().addEvent( well_name, ScheduleEvents::WELLGROUP_EFFICIENCY_UPDATE);
-                handlerContext.state().events().addEvent(ScheduleEvents::WELLGROUP_EFFICIENCY_UPDATE);
-                handlerContext.state().wells.update( std::move(well2) );
+            auto well2 = handlerContext.state().wells.get(well_name);
+
+            if (well2.updateEfficiencyFactor(efficiencyFactor, useEfficiencyInNetwork)) {
+                handlerContext.state().wells.update(std::move(well2));
+
+                handlerContext.affected_well(well_name);
+                handlerContext.record_well_structure_change();
+
+                handlerContext.state().events()
+                    .addEvent(ScheduleEvents::WELLGROUP_EFFICIENCY_UPDATE);
+
+                handlerContext.state().wellgroup_events()
+                    .addEvent(well_name, ScheduleEvents::WELLGROUP_EFFICIENCY_UPDATE);
             }
         }
     }
@@ -185,14 +201,14 @@ void handleWELPIRuntime(HandlerContext& handlerContext)
             handlerContext.state().target_wellpi
                 .insert_or_assign(well_name, targetPI);
 
-            handlerContext.affected_well(well_name);
+            handlerContext.welpi_well(well_name);
         }
     }
 }
 
 void handleWELPI(HandlerContext& handlerContext)
 {
-    if (handlerContext.actionx_mode) {
+    if (handlerContext.action_mode) {
         handleWELPIRuntime(handlerContext);
         return;
     }
@@ -332,10 +348,10 @@ void handleWINJTEMP(HandlerContext& handlerContext)
 
         for (const auto& well_name : well_names) {
             const auto& well = handlerContext.state().wells.get(well_name);
-            const double current_temp = !well.isProducer()? well.temperature(): 0.0;
+            const double current_temp = well.hasInjTemperature()? well.inj_temperature(): 0.0;
             if (current_temp != temp) {
                 auto well2 = handlerContext.state().wells( well_name );
-                well2.setWellTemperature(temp);
+                well2.setWellInjTemperature(temp);
                 handlerContext.state().wells.update( std::move(well2) );
             }
         }
@@ -500,10 +516,10 @@ void handleWTEMP(HandlerContext& handlerContext)
 
         for (const auto& well_name : well_names) {
             const auto& well = handlerContext.state().wells.get(well_name);
-            const double current_temp = !well.isProducer()? well.temperature(): 0.0;
+            const double current_temp = well.hasInjTemperature()? well.inj_temperature(): 0.0;
             if (current_temp != temp) {
                 auto well2 = handlerContext.state().wells( well_name );
-                well2.setWellTemperature(temp);
+                well2.setWellInjTemperature(temp);
                 handlerContext.state().wells.update( std::move(well2) );
             }
         }
@@ -526,57 +542,85 @@ void handleWTEMP(HandlerContext& handlerContext)
 void handleWTMULT(HandlerContext& handlerContext)
 {
     for (const auto& record : handlerContext.keyword) {
-        const auto& wellNamePattern = record.getItem<ParserKeywords::WTMULT::WELL>().getTrimmedString(0);
-        const auto& control = record.getItem<ParserKeywords::WTMULT::CONTROL>().get<std::string>(0);
         const auto& factor = record.getItem<ParserKeywords::WTMULT::FACTOR>().get<UDAValue>(0);
-        const auto& num = record.getItem<ParserKeywords::WTMULT::NUM>().get<int>(0);
-
         if (factor.is<std::string>()) {
-            std::string reason = fmt::format("Use of UDA value: {} is not supported as multiplier", factor.get<std::string>());
-            throw OpmInputError(reason, handlerContext.keyword.location());
+            const auto reason =
+                fmt::format("UDA value {} is not supported "
+                            "as multiplier", factor.get<std::string>());
+
+            throw OpmInputError {
+                reason, handlerContext.keyword.location()
+            };
         }
 
+        const auto& control = record.getItem<ParserKeywords::WTMULT::CONTROL>().get<std::string>(0);
         if (handlerContext.state().udq().has_keyword(control)) {
-            std::string reason = fmt::format("Use of UDA value: {} is not supported for control target", control);
-            throw OpmInputError(reason, handlerContext.keyword.location());
+            const auto reason =
+                fmt::format("UDA value {} is not supported "
+                            "for control target", control);
+
+            throw OpmInputError {
+                reason, handlerContext.keyword.location()
+            };
         }
 
+        const auto num = record.getItem<ParserKeywords::WTMULT::NUM>().get<int>(0);
         if (num != 1) {
-            std::string reason = fmt::format("Only NUM=1 is supported in WTMULT keyword");
-            throw OpmInputError(reason, handlerContext.keyword.location());
+            throw OpmInputError {
+                "Only NUM=1 is supported in WTMULT keyword",
+                handlerContext.keyword.location()
+            };
         }
 
         const auto cmode = WellWELTARGCModeFromString(control);
-        if (cmode == Well::WELTARGCMode::GUID)
-            throw std::logic_error("Multiplying guide rate is not implemented");
+        if (cmode == Well::WELTARGCMode::GUID) {
+            throw OpmInputError {
+                "Multiplying the guide rate is not supported",
+                handlerContext.keyword.location()
+            };
+        }
 
+        const auto& wellNamePattern = record.getItem<ParserKeywords::WTMULT::WELL>().getTrimmedString(0);
         const auto well_names = handlerContext.wellNames(wellNamePattern);
         for (const auto& well_name : well_names) {
             auto well = handlerContext.state().wells.get(well_name);
+
             if (well.isInjector()) {
-                bool update_well = true;
+                const bool update_well = true;
+
                 auto properties = std::make_shared<Well::WellInjectionProperties>(well.getInjectionProperties());
-                properties->handleWTMULT( cmode, factor.get<double>());
+                properties->handleWTMULT(cmode, factor.get<double>());
 
                 well.updateInjection(properties);
                 if (update_well) {
-                    handlerContext.state().events().addEvent(ScheduleEvents::INJECTION_UPDATE);
-                    handlerContext.state().wellgroup_events().addEvent(well_name, ScheduleEvents::INJECTION_UPDATE);
-                    handlerContext.state().wells.update(std::move(well));
-                }
-            } else {
-                bool update_well = true;
-                auto properties = std::make_shared<Well::WellProductionProperties>(well.getProductionProperties());
-                properties->handleWTMULT( cmode, factor.get<double>());
+                    handlerContext.state().events()
+                        .addEvent(ScheduleEvents::INJECTION_UPDATE);
+
+                    handlerContext.state().wellgroup_events()
+                        .addEvent(well_name, ScheduleEvents::INJECTION_UPDATE);
 
-                well.updateProduction(properties);
-                if (update_well) {
-                    handlerContext.state().events().addEvent(ScheduleEvents::PRODUCTION_UPDATE);
-                    handlerContext.state().wellgroup_events().addEvent(well_name,
-                                                                     ScheduleEvents::PRODUCTION_UPDATE);
                     handlerContext.state().wells.update(std::move(well));
                 }
             }
+            else {
+                const bool update_well = true;
+
+                auto properties = std::make_shared<Well::WellProductionProperties>(well.getProductionProperties());
+                properties->handleWTMULT(cmode, factor.get<double>());
+
+                well.updateProduction(properties);
+                if (update_well) {
+                    handlerContext.state().events()
+                        .addEvent(ScheduleEvents::PRODUCTION_UPDATE);
+
+                    handlerContext.state().wellgroup_events()
+                        .addEvent(well_name, ScheduleEvents::PRODUCTION_UPDATE);
+
+                    handlerContext.state().wells.update(std::move(well));
+                }
+            }
+
+            handlerContext.affected_well(well_name);
         }
     }
 }
