@@ -260,28 +260,9 @@ public:
     void serializeOp(Serializer& serializer)
     {
         serializer(this->wells);
-        if (serializer.isSerializing()) {
-            std::size_t size = this->completions.size();
-            serializer(size);
-            for (auto& [well, comp_map] : this->completions) {
-                serializer(well);
-                serializer(comp_map);
-            }
-        } else {
-            std::size_t size = 0;
-            serializer(size);
-            for (std::size_t i=0; i < size; i++) {
-                std::string well;
-                std::unordered_map<int, ClosedCompletion> comp_map;
-
-                serializer(well);
-                serializer(comp_map);
-                this->completions.emplace(well, std::move(comp_map));
-            }
-        }
+        serializer(this->completions);
     }
 
-
     static WellTestState serializationTestObject();
     bool operator==(const WellTestState& other) const;
 
