@@ -32,6 +32,9 @@ namespace UDA {
     if (value.is<double>())
         return value.getSI();
 
+    if (!value.is<std::string>())
+        return udq_default;
+
     const std::string& string_var = value.get<std::string>();
     double output_value = udq_default;
 
@@ -58,6 +61,9 @@ double eval_group_uda(const UDAValue& value, const std::string& group, const Sum
     if (value.is<double>())
         return value.getSI();
 
+    if (!value.is<std::string>())
+        return udq_undefined;
+
     const std::string& string_var = value.get<std::string>();
     double output_value = udq_undefined;
 
