@@ -32,10 +32,11 @@
 #include <string>
 #include <stdexcept>
 
+#include <fmt/format.h>
 
-namespace Opm { namespace EclIO {
+namespace Opm::EclIO {
 
-using NNCentry = std::tuple<int, int, int, int, int,int, float>;
+using NNCentry = std::tuple<int, int, int, int, int, int, float>;
 
 EGrid::EGrid(const std::string& filename, const std::string& grid_name)
     : EclFile(filename), inputFileName { filename }, m_grid_name {grid_name}
@@ -55,6 +56,8 @@ EGrid::EGrid(const std::string& filename, const std::string& grid_name)
     nnc2_array_index = -1;
     coordsys_array_index = -1;
     m_radial = false;
+    m_mapaxes_loaded = false;
+    double length_factor = 1.0;
 
     int hostnum_index = -1;
 
@@ -81,10 +84,25 @@ EGrid::EGrid(const std::string& filename, const std::string& grid_name)
         if (array_name[n] == "MAPUNITS"){
             auto mapunits = this->get<std::string>(n);
             m_mapunits = mapunits[0];
+            if (m_mapunits == "METRES")
+                length_factor = 1.0;
+            else if (m_mapunits == "FEET")
+                length_factor = 0.3048;
+            else if (m_mapunits == "CM")
+                length_factor = 0.01;
+            else{
+                std::string message = "Unit system " + m_mapunits + " not supported for MAPUNITS";
+                OPM_THROW(std::invalid_argument, message);
+            }
         }
 
-        if (array_name[n] == "MAPAXES")
-            m_mapaxes = this->get<float>(n);
+        if (array_name[n] == "MAPAXES"){
+            const auto& mapAx = this->get<float>(n);
+            std::transform(mapAx.begin(), mapAx.end(), this->m_mapaxes.begin(),
+                           [length_factor](const float elm) { return elm * length_factor; });
+            mapaxes_init();
+            m_mapaxes_loaded = true;
+        }
 
         if (lgrname == grid_name) {
             if (array_name[n] == "GRIDHEAD") {
@@ -228,32 +246,32 @@ void EGrid::load_nnc_data()
         nnc2_array = getImpl(nnc2_array_index, Opm::EclIO::INTE, inte_array, "inte");
 
         if ((std::filesystem::exists(initFileName)) && (nnc1_array.size() > 0)){
-            Opm::EclIO::EInit init(initFileName);
+            Opm::EclIO::EInit init(initFileName.generic_string());
 
             auto init_dims = init.grid_dimension(m_grid_name);
             int init_nactive = init.activeCells(m_grid_name);
 
             if (init_dims != nijk){
-                std::string message = "Dimensions of Egrid differ from dimensions found in init file. ";
-                std::string grid_str = std::to_string(nijk[0]) + "x" + std::to_string(nijk[1]) + "x" + std::to_string(nijk[2]);
-                std::string init_str = std::to_string(init_dims[0]) + "x" + std::to_string(init_dims[1]) + "x" + std::to_string(init_dims[2]);
-                message = message + "Egrid: " + grid_str + ". INIT file: " + init_str;
-                OPM_THROW(std::invalid_argument, message);
+                OPM_THROW(std::invalid_argument,
+                          fmt::format("Dimensions of Egrid differ from dimensions found in init file. "
+                                      "Egrid: {}x{}x{}. INIT file: {}x{}x{}",
+                                      nijk[0], nijk[1], nijk[2],
+                                      init_dims[0], init_dims[1], init_dims[2]));
             }
 
             if (init_nactive != nactive){
-                std::string message = "Number of active cells are different in Egrid and Init file.";
-                message = message + " Egrid: " + std::to_string(nactive) + ". INIT file: " + std::to_string(init_nactive);
-                OPM_THROW(std::invalid_argument, message);
+                OPM_THROW(std::invalid_argument,
+                          fmt::format("Number of active cells are different in Egrid and Init file."
+                                      " Egrid: {}. INIT file: {}", nactive, init_nactive));
             }
 
             auto trans_data = init.getInitData<float>("TRANNNC", m_grid_name);
 
             if (trans_data.size() != nnc1_array.size()){
-                std::string message = "inconsistent size of array TRANNNC in init file. ";
-                message = message + " Size of NNC1 and NNC2: " + std::to_string(nnc1_array.size());
-                message = message + " Size of TRANNNC: " + std::to_string(trans_data.size());
-                OPM_THROW(std::invalid_argument, message);
+                OPM_THROW(std::invalid_argument,
+                          fmt::format("inconsistent size of array TRANNNC in init file. "
+                                      "Size of NNC1 and NNC2: {} Size of TRANNNC: {}",
+                                      nnc1_array.size(), trans_data.size()));
             }
 
             transnnc_array = trans_data;
@@ -322,11 +340,31 @@ std::array<int, 3> EGrid::ijk_from_global_index(int globInd) const
     return result;
 }
 
+void EGrid::mapaxes_transform(double& x, double& y) const {
+    double tmpx = x;
+    x = origin[0] + tmpx * unit_x[0] + y * unit_y[0];
+    y = origin[1] + tmpx * unit_x[1] + y * unit_y[1];
+}
+
+void EGrid::mapaxes_init()
+{
+    origin = {m_mapaxes[2], m_mapaxes[3]};
+    unit_x = {m_mapaxes[4] - m_mapaxes[2], m_mapaxes[5] - m_mapaxes[3]};
+    unit_y = {m_mapaxes[0] - m_mapaxes[2], m_mapaxes[1] - m_mapaxes[3]};
+
+    auto norm_x = 1.0 / std::hypot(unit_x[0], unit_x[1]);
+    auto norm_y = 1.0 / std::hypot(unit_y[0], unit_y[1]);
+
+    unit_x[0] *= norm_x;
+    unit_x[1] *= norm_x;
+    unit_y[0] *= norm_y;
+    unit_y[1] *= norm_y;
+}
 
 void EGrid::getCellCorners(const std::array<int, 3>& ijk,
-                           std::array<double,8>& X,
-                           std::array<double,8>& Y,
-                           std::array<double,8>& Z)
+                           std::array<double, 8>& X,
+                           std::array<double, 8>& Y,
+                           std::array<double, 8>& Z)
 {
     if (coord_array.empty())
         load_grid_data();
@@ -351,7 +389,7 @@ void EGrid::getCellCorners(const std::array<int, 3>& ijk,
     for (int n = 0; n < 4; n++)
         zind.push_back(zind[n] + nijk[0]*nijk[1]*4);
 
-    for (int n = 0; n< 8; n++)
+    for (int n = 0; n < 8; n++)
         Z[n] = zcorn_array[zind[n]];
 
     for (int  n = 0; n < 4; n++) {
@@ -366,8 +404,8 @@ void EGrid::getCellCorners(const std::array<int, 3>& ijk,
         if (m_radial) {
             xt = coord_array[pind[n]] * cos(coord_array[pind[n] + 1] / 180.0 * M_PI);
             yt = coord_array[pind[n]] * sin(coord_array[pind[n] + 1] / 180.0 * M_PI);
-            xb = coord_array[pind[n]+3] * cos(coord_array[pind[n] + 4] / 180.0 * M_PI);
-            yb = coord_array[pind[n]+3] * sin(coord_array[pind[n] + 4] / 180.0 * M_PI);
+            xb = coord_array[pind[n] + 3] * cos(coord_array[pind[n] + 4] / 180.0 * M_PI);
+            yb = coord_array[pind[n] + 3] * sin(coord_array[pind[n] + 4] / 180.0 * M_PI);
         } else {
             xt = coord_array[pind[n]];
             yt = coord_array[pind[n] + 1];
@@ -390,11 +428,10 @@ void EGrid::getCellCorners(const std::array<int, 3>& ijk,
 }
 
 
-
-void EGrid::getCellCorners(int globindex, std::array<double,8>& X,
-                           std::array<double,8>& Y, std::array<double,8>& Z)
+void EGrid::getCellCorners(int globindex, std::array<double, 8>& X,
+                           std::array<double, 8>& Y, std::array<double, 8>& Z)
 {
-    return getCellCorners(ijk_from_global_index(globindex),X,Y,Z);
+    return getCellCorners(ijk_from_global_index(globindex), X, Y, Z);
 }
 
 
@@ -403,16 +440,15 @@ std::vector<std::array<float, 3>> EGrid::getXYZ_layer(int layer, const std::arra
    // layer is layer index, zero based. The box array is i and j range (i1,i2,j1,j2), also zero based
 
     if ((layer < 0) || (layer > (nijk[2] -1))){
-        std::string message = "invalid layer index " + std::to_string(layer) + ". Valid range [0, ";
-        message = message + std::to_string(nijk[2] -1) + "]";
-        throw std::invalid_argument(message);
+        throw std::invalid_argument(fmt::format("invalid layer index {}. Valid range [0,{}]",
+                                                layer, nijk[2] - 1));
     }
 
     if ((box[0] < 0) || (box[0]+1 > nijk[0]) || (box[1] < 0) || (box[1]+1 > nijk[0]) ||
          (box[2] < 0) || (box[2]+1 > nijk[1]) || (box[3] < 0) || (box[3]+1 > nijk[1]) ||
            (box[0] > box[1]) || (box[2] > box[3])){
 
-        throw std::invalid_argument("invalid box input, i1,i2,j1 or j2 out of valid range ");
+        throw std::invalid_argument("invalid box input, i1, i2, j1 or j2 out of valid range ");
     }
 
     int nodes_pr_surf = nijk[0]*nijk[1]*4;
@@ -474,7 +510,7 @@ std::vector<std::array<float, 3>> EGrid::getXYZ_layer(int layer, bool bottom)
 }
 
 
-std::vector<float> EGrid::get_zcorn_from_disk(int layer, bool bottom)
+std::vector<float> EGrid::  get_zcorn_from_disk(int layer, bool bottom)
 {
     if (formatted)
         throw std::invalid_argument("partial loading of zcorn arrays not possible when using formatted input");
@@ -608,6 +644,4 @@ void EGrid::getCellCorners(const std::array<int, 3>& ijk, const std::vector<floa
     }
 }
 
-
-
-}} // namespace Opm::ecl
+} // namespace Opm::EclIO
