@@ -116,7 +116,7 @@ bool is_well_completion(const std::string& keyword)
 }
 
 
-namespace Opm { namespace EclIO {
+namespace Opm::EclIO {
 
 ESmry::ESmry(const std::string &filename, bool loadBaseRunData) :
     inputFileName { filename },
@@ -578,13 +578,13 @@ ESmry::ESmry(const std::string &filename, bool loadBaseRunData) :
         while  (i < arraySourceList.size()) {
 
             if (std::get<0>(arraySourceList[i]) != "MINISTEP") {
-                std::string message="Reading summary file, expecting keyword MINISTEP, found '" + std::get<0>(arraySourceList[i]) + "'";
-                throw std::invalid_argument(message);
+                throw std::invalid_argument("Reading summary file, expecting keyword MINISTEP, "
+                                            "found '" + std::get<0>(arraySourceList[i]) + "'");
             }
 
             if (std::get<0>(arraySourceList[i+1]) != "PARAMS") {
-                std::string message="Reading summary file, expecting keyword PARAMS, found '" + std::get<0>(arraySourceList[i]) + "'";
-                throw std::invalid_argument(message);
+                throw std::invalid_argument("Reading summary file, expecting keyword PARAMS, "
+                                            "found '" + std::get<0>(arraySourceList[i]) + "'");
             }
 
             i++;
@@ -634,6 +634,9 @@ ESmry::ESmry(const std::string &filename, bool loadBaseRunData) :
 
 void ESmry::read_ministeps_from_disk()
 {
+    if (miniStepList.empty())
+        return;
+
     auto specInd = std::get<0>(miniStepList[0]);
     auto dataFileIndex = std::get<1>(miniStepList[0]);
 
@@ -740,8 +743,7 @@ void ESmry::loadData(const std::vector<std::string>& vectList) const
     std::uint64_t blockSize_f;
 
     {
-        const int rest = MaxBlockSizeReal % numColumnsReal;
-        const int nLinesBlock = MaxBlockSizeReal / numColumnsReal + (rest > 0);
+        const int nLinesBlock = MaxBlockSizeReal / numColumnsReal;
 
         blockSize_f= static_cast<std::uint64_t>(MaxNumBlockReal * numColumnsReal * columnWidthReal + nLinesBlock);
     }
@@ -842,6 +844,9 @@ std::vector<int> ESmry::makeKeywPosVector(int specInd) const
 
 void ESmry::loadData() const
 {
+    if (timeStepList.empty())
+        return;
+
     std::fstream fileH;
 
     auto specInd = std::get<0>(timeStepList[0]);
@@ -917,7 +922,8 @@ void ESmry::loadData() const
 
                 const int num = dhead / sizeOfInte;
                 if ((num > maxNumberOfElements) || (num < 0))
-                    OPM_THROW(std::runtime_error, "??Error reading binary data, inconsistent header data or incorrect number of elements");
+                    OPM_THROW(std::runtime_error, "??Error reading binary data, inconsistent header "
+                                                  "data or incorrect number of elements");
 
                 for (int i = 0; i < num; ++i, ++p) {
                     float value;
@@ -932,8 +938,7 @@ void ESmry::loadData() const
                 if (( num < maxNumberOfElements && rest != 0) ||
                         (num == maxNumberOfElements && rest < 0))
                 {
-                    std::string message = "Error reading binary data, incorrect number of elements";
-                    OPM_THROW(std::runtime_error, message);
+                    OPM_THROW(std::runtime_error, "Error reading binary data, incorrect number of elements");
                 }
 
                 int dtail;
@@ -1071,7 +1076,7 @@ bool ESmry::make_esmry_file()
     std::filesystem::path smryDataFile = path / rootName;
     smryDataFile.replace_extension(".ESMRY");
 
-    if (Opm::EclIO::fileExists(smryDataFile))
+    if (Opm::EclIO::fileExists(smryDataFile.generic_string()))
     {
         return false;
 
@@ -1106,7 +1111,7 @@ bool ESmry::make_esmry_file()
             std::transform(keyword.begin(), keyword.end(), std::back_inserter(units),
                            [this](const auto& key) { return kwunits.at(key); });
 
-            Opm::EclIO::EclOutput outFile(smryDataFile, false, std::ios::out);
+            Opm::EclIO::EclOutput outFile(smryDataFile.generic_string(), false, std::ios::out);
 
             outFile.write<int>("START", start_date_vect);
 
@@ -1357,8 +1362,7 @@ const std::vector<float>& ESmry::get(const std::string& name) const
     auto it = std::find(keyword.begin(), keyword.end(), name);
 
     if (it == keyword.end()) {
-        const std::string message="keyword " + name + " not found ";
-        OPM_THROW(std::invalid_argument, message);
+        OPM_THROW(std::invalid_argument, "keyword " + name + " not found ");
     }
 
     int ind = std::distance(keyword.begin(), it);
@@ -1427,7 +1431,7 @@ std::vector<Opm::time_point> ESmry::dates() const
                    {
                        using Seconds = std::chrono::duration<double, std::chrono::seconds::period>;
                        return this->tp_startdat +
-                              std::chrono::duration_cast<std::chrono::seconds>(Seconds{t * time_unit});
+                           std::chrono::duration_cast<time_point::duration>(Seconds{t * time_unit});
                    });
 
     return d;
@@ -1445,4 +1449,4 @@ std::tuple<double, double> ESmry::get_io_elapsed() const
 }
 
 
-}} // namespace Opm::EclIO
+} // namespace Opm::EclIO
