@@ -48,11 +48,9 @@ Opm::time_point make_date(const std::vector<int>& datetime) {
     if (datetime.size() == 7) {
         hour = datetime[3];
         minute = datetime[4];
-        auto total_usec = datetime[5];
-        second = total_usec / 1000000;
+        second = datetime[5] ;
     }
 
-
     const auto ts = Opm::TimeStampUTC{ Opm::TimeStampUTC::YMD{ year, month, day}}.hour(hour).minutes(minute).seconds(second);
     return Opm::TimeService::from_time_t( Opm::asTimeT(ts) );
 }
@@ -553,8 +551,8 @@ void ExtESmry::loadData(const std::vector<std::string>& stringVect)
         }
 
         if (n_attempts == 10){
-            std::string emsry_file_name = m_esmry_files[ind].string();
-            OPM_THROW( std::runtime_error, "when loading data from ESMRY file" + emsry_file_name );
+            OPM_THROW(std::runtime_error,
+                      "when loading data from ESMRY file" + m_esmry_files[ind].string());
         }
 
         ind--;
@@ -596,7 +594,7 @@ std::vector<Opm::time_point> ExtESmry::dates()
                   [this, time_unit](const auto& t)
                   {
                       using Seconds = std::chrono::duration<double, std::chrono::seconds::period>;
-                      return this->m_startdat + std::chrono::duration_cast<std::chrono::seconds>(Seconds{t * time_unit});
+                      return this->m_startdat + std::chrono::duration_cast<time_point::duration>(Seconds{t * time_unit});
                   });
 
     return d;
