@@ -34,24 +34,43 @@
 #include <opm/common/utility/String.hpp>
 #include <opm/common/utility/TimeService.hpp>
 
-#include <opm/input/eclipse/Schedule/Action/Actdims.hpp>
-#include <opm/input/eclipse/Schedule/Action/Condition.hpp>
-
-#include <opm/input/eclipse/Deck/Deck.hpp>
-#include <opm/input/eclipse/Parser/Parser.hpp>
-
 #include <opm/output/eclipse/UDQDims.hpp>
+
 #include <opm/output/eclipse/VectorItems/connection.hpp>
 #include <opm/output/eclipse/VectorItems/doubhead.hpp>
 #include <opm/output/eclipse/VectorItems/intehead.hpp>
 #include <opm/output/eclipse/VectorItems/well.hpp>
+
 #include <opm/output/eclipse/WriteRestartHelpers.hpp>
 
+#include <opm/input/eclipse/Schedule/Action/Actdims.hpp>
+#include <opm/input/eclipse/Schedule/Action/Condition.hpp>
+
+#include <opm/input/eclipse/Deck/Deck.hpp>
+
+#include <opm/input/eclipse/Parser/InputErrorAction.hpp>
+#include <opm/input/eclipse/Parser/ParseContext.hpp>
+#include <opm/input/eclipse/Parser/Parser.hpp>
+
 #include <algorithm>
+#include <array>
+#include <cstddef>
+#include <functional>
 #include <iterator>
 #include <memory>
 #include <numeric>
 #include <optional>
+#include <stdexcept>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <fmt/format.h>
+#include <fmt/ranges.h>
+
+#include <boost/range.hpp>
+
+namespace VI = ::Opm::RestartIO::Helpers::VectorItems;
 
 namespace {
     std::string
@@ -60,12 +79,35 @@ namespace {
     {
         auto zudl_begin = zudl.begin();
         auto zudl_end = zudl_begin;
-        std::advance( zudl_begin, (udq_index + 0) * Opm::UDQDims::entriesPerZUDL() );
-        std::advance( zudl_end  , (udq_index + 1) * Opm::UDQDims::entriesPerZUDL() );
+        std::advance(zudl_begin, (udq_index + 0) * Opm::UDQDims::entriesPerZUDL());
+        std::advance(zudl_end  , (udq_index + 1) * Opm::UDQDims::entriesPerZUDL());
 
-        auto define = std::accumulate(zudl_begin, zudl_end, std::string{}, std::plus<>());
-        if (!define.empty() && (define[0] == '~')) {
-            define[0] = '-';
+        // Note: We inject each ZUDL substring into a field of exactly 8
+        // characters ({:<8}) in order to preserve any requisite trailing
+        // whitespace.  In particular, we don't want to risk consecutive
+        // slots of the form
+        //
+        //   ")/(GEFF "
+        //   "TEST)*(1"
+        //
+        // being fused into a string fragment of the form
+        //
+        //   )/(GEFFTEST)*(1
+        //
+        // This would change the meaning of the summary vector (scalar value
+        // 'GEFF TEST' -> group level UDQ set 'GEFFTEST').
+        //
+        // We do nevertheless use rtrim_copy() on the final string in order
+        // to ensure that a UDQ without a defining expression--i.e., an
+        // assignment--comes back as an empty string.  The 8 character
+        // explicit field width in fmt::format() generates a result string
+        // which is always of size 128 regardless of the actual character
+        // data (e.g., 128 space characters).  If we were to return that
+        // string value unchanged, we would break some of the higher-level
+        // logic.
+        auto define = Opm::rtrim_copy(fmt::format("{:<8}", fmt::join(zudl_begin, zudl_end, "")));
+        if (!define.empty() && (define.front() == '~')) {
+            define.front() = '-';
         }
 
         return define;
@@ -76,11 +118,225 @@ namespace {
     {
         return Opm::UDQ::updateType(iudq[udq_index * Opm::UDQDims::entriesPerIUDQ()]);
     }
+
+    template <typename T>
+    boost::iterator_range<typename std::vector<T>::const_iterator>
+    getDataWindow(const std::vector<T>& arr,
+                  const std::size_t     windowSize,
+                  const std::size_t     entity,
+                  const std::size_t     subEntity               = 0,
+                  const std::size_t     maxSubEntitiesPerEntity = 1)
+    {
+        const auto off =
+            windowSize * (subEntity + maxSubEntitiesPerEntity*entity);
+
+        auto begin = arr.begin() + off;
+        auto end   = begin       + windowSize;
+
+        return { begin, end };
+    }
+
+    class UDQVectors
+    {
+    public:
+        explicit UDQVectors(std::shared_ptr<Opm::EclIO::RestartFileView> rst_view)
+            : rstView_{ std::move(rst_view) }
+        {
+            const auto& intehead = this->rstView_->getKeyword<int>("INTEHEAD");
+
+            this->maxNumMsWells_  = intehead[VI::intehead::NSWLMX];
+            this->maxNumSegments_ = intehead[VI::intehead::NSEGMX];
+            this->numGroups_      = intehead[VI::intehead::NGMAXZ]; // Including FIELD
+            this->numWells_       = intehead[VI::intehead::NWMAXZ];
+        }
+
+        enum Type : std::size_t {
+            Field, Group, Segment, Well,
+            // ---- Implementation Helper ----
+            NumTypes
+        };
+
+        void prepareNext(const Type t) { ++this->varIx_[t]; }
+
+        const std::vector<std::string>& zudn() const
+        {
+            return this->rstView_->getKeyword<std::string>("ZUDN");
+        }
+
+        bool hasGroup()   const { return this->rstView_->hasKeyword<double>("DUDG"); }
+        bool hasSegment() const { return this->rstView_->hasKeyword<double>("DUDS"); }
+        bool hasWell()    const { return this->rstView_->hasKeyword<double>("DUDW"); }
+
+        double currentFieldUDQValue() const
+        {
+            return this->rstView_->getKeyword<double>("DUDF")[ this->varIx_[Type::Field] ];
+        }
+
+        auto currentGroupUDQValue() const
+        {
+            return getDataWindow(this->rstView_->getKeyword<double>("DUDG"),
+                                 this->numGroups_, this->varIx_[Type::Group]);
+        }
+
+        auto currentSegmentUDQValue(const std::size_t msWellIx) const
+        {
+            return getDataWindow(this->rstView_->getKeyword<double>("DUDS"),
+                                 this->maxNumSegments_, this->varIx_[Type::Segment],
+                                 msWellIx, this->maxNumMsWells_);
+        }
+
+        auto currentWellUDQValue() const
+        {
+            return getDataWindow(this->rstView_->getKeyword<double>("DUDW"),
+                                 this->numWells_, this->varIx_[Type::Well]);
+        }
+
+    private:
+        std::shared_ptr<Opm::EclIO::RestartFileView> rstView_;
+
+        std::size_t maxNumMsWells_{0};
+        std::size_t maxNumSegments_{0};
+        std::size_t numGroups_{0};
+        std::size_t numWells_{0};
+
+        std::array<std::size_t, Type::NumTypes> varIx_{};
+    };
+
+    bool isDefaultedUDQ(const double x)
+    {
+        return x == Opm::UDQ::restart_default;
+    }
+
+    void restoreFieldUDQValue(const UDQVectors&       udqs,
+                              Opm::RestartIO::RstUDQ& udq)
+    {
+        const auto dudf = udqs.currentFieldUDQValue();
+
+        if (! isDefaultedUDQ(dudf)) {
+            udq.assignScalarValue(dudf);
+        }
+    }
+
+    void restoreGroupUDQValue(const UDQVectors&                            udqs,
+                              const std::vector<Opm::RestartIO::RstGroup>& groups,
+                              Opm::RestartIO::RstUDQ&                      udq)
+    {
+        const auto nGrp = groups.size();
+        const auto dudg = udqs.currentGroupUDQValue();
+
+        const auto subEntity = 0;
+        auto entity = 0;
+        for (auto iGrp = 0*nGrp; iGrp < nGrp; ++iGrp) {
+            if (isDefaultedUDQ(dudg[iGrp])) {
+                continue;
+            }
+
+            udq.addValue(entity++, subEntity, dudg[iGrp]);
+            udq.addEntityName(groups[iGrp].name);
+        }
+    }
+
+    void restoreSegmentUDQValue(const UDQVectors&                           udqs,
+                                const std::vector<Opm::RestartIO::RstWell>& wells,
+                                Opm::RestartIO::RstUDQ&                     udq)
+    {
+        // Counter for MS wells with a non-defaulted UDQ value for at least
+        // one segment.
+        auto activeMsWellID = 0;
+
+        for (const auto& well : wells) {
+            if (well.msw_index == 0) {
+                // Not a multi-segmented well.
+                continue;
+            }
+
+            // Subtract one for zero-based indexing.
+            const auto duds = udqs.currentSegmentUDQValue(well.msw_index - 1);
+            const auto nSeg = duds.size();
+
+            auto isActiveMsWell = false;
+            for (auto iSeg = 0*nSeg; iSeg < nSeg; ++iSeg) {
+                if (isDefaultedUDQ(duds[iSeg])) {
+                    continue;
+                }
+
+                // If we get here, there is at least one non-defaulted SU*
+                // value for this well.  Record the well as such, in order
+                // to properly associate the entity to a well name.
+                isActiveMsWell = true;
+
+                // Note: Use 'iSeg' directly as the sub-entity, as this
+                // simplifies ordering.  Trust clients to make this into a
+                // one-based segment number when needed.
+                udq.addValue(activeMsWellID, iSeg, duds[iSeg]);
+            }
+
+            if (isActiveMsWell) {
+                // The current well has a non-default UDQ value for at least
+                // one segment.  Associate the current entity to the well's
+                // name and prepare for handling the next MS well.
+                udq.addEntityName(well.name);
+                ++activeMsWellID;
+            }
+        }
+    }
+
+    void restoreWellUDQValue(const UDQVectors&                           udqs,
+                             const std::vector<Opm::RestartIO::RstWell>& wells,
+                             Opm::RestartIO::RstUDQ&                     udq)
+    {
+        const auto nWell = wells.size();
+        const auto dudw  = udqs.currentWellUDQValue();
+
+        const auto subEntity = 0;
+        auto entity = 0;
+        for (auto iWell = 0*nWell; iWell < nWell; ++iWell) {
+            if (isDefaultedUDQ(dudw[iWell])) {
+                continue;
+            }
+
+            udq.addValue(entity++, subEntity, dudw[iWell]);
+            udq.addEntityName(wells[iWell].name);
+        }
+    }
+
+    void restoreSingleUDQ(const Opm::RestartIO::RstState& rst,
+                          UDQVectors&                     udqValues,
+                          Opm::RestartIO::RstUDQ&         udq)
+    {
+        udq.prepareValues();
+
+        // Note: Categories ordered by enumerator values in UDQEnums.hpp.
+        switch (udq.category) {
+        case Opm::UDQVarType::FIELD_VAR:
+            restoreFieldUDQValue(udqValues, udq);
+            udqValues.prepareNext(UDQVectors::Type::Field);
+            break;
+
+        case Opm::UDQVarType::SEGMENT_VAR:
+            restoreSegmentUDQValue(udqValues, rst.wells, udq);
+            udqValues.prepareNext(UDQVectors::Type::Segment);
+            break;
+
+        case Opm::UDQVarType::WELL_VAR:
+            restoreWellUDQValue(udqValues, rst.wells, udq);
+            udqValues.prepareNext(UDQVectors::Type::Well);
+            break;
+
+        case Opm::UDQVarType::GROUP_VAR:
+            restoreGroupUDQValue(udqValues, rst.groups, udq);
+            udqValues.prepareNext(UDQVectors::Type::Group);
+            break;
+
+        default:
+            break;
+        }
+
+        udq.commitValues();
+    }
 }
 
-namespace VI = ::Opm::RestartIO::Helpers::VectorItems;
-
-namespace Opm { namespace RestartIO {
+namespace Opm::RestartIO {
 
 RstState::RstState(std::shared_ptr<EclIO::RestartFileView> rstView,
                    const Runspec&                          runspec,
@@ -118,12 +374,21 @@ void RstState::load_tuning(const std::vector<int>& intehead,
     this->tuning.MXWSIT  = intehead[ VI::intehead::MXWSIT ];
     this->tuning.MXWPIT  = intehead[ VI::intehead::MXWPIT ];
 
-    double tsinit = this->unit_system.to_si(M::time, doubhead[VI::doubhead::TsInit]);
-    //tuning.TSINIT = tsinit > 0 ? std::optional<double>{ tsinit } : std::nullopt;
-    tuning.TSINIT = VI::DoubHeadValue::TSINITHasNoValue(tsinit) ? std::nullopt : std::optional<double>{ tsinit };
+    this->tuning.WSEG_MAX_RESTART = intehead[ VI::intehead::WSEGITR_IT2 ];
+
+    {
+        const auto tsinit = this->unit_system
+            .to_si(M::time, doubhead[VI::doubhead::TsInit]);
+
+        tuning.TSINIT = VI::DoubHeadValue::TSINITHasNoValue(tsinit)
+            ? std::nullopt
+            : std::optional<double>{ tsinit };
+    }
+
     tuning.TSMAXZ = this->unit_system.to_si(M::time, doubhead[VI::doubhead::TsMaxz]);
     tuning.TSMINZ = this->unit_system.to_si(M::time, doubhead[VI::doubhead::TsMinz]);
     tuning.TSMCHP = this->unit_system.to_si(M::time, doubhead[VI::doubhead::TsMchp]);
+
     tuning.TSFMAX = doubhead[VI::doubhead::TsFMax];
     tuning.TSFMIN = doubhead[VI::doubhead::TsFMin];
     tuning.TSFCNV = doubhead[VI::doubhead::TsFcnv];
@@ -144,6 +409,9 @@ void RstState::load_tuning(const std::vector<int>& intehead,
     tuning.XXXDPR = doubhead[VI::doubhead::XxxDPR];
     tuning.DDPLIM = doubhead[VI::doubhead::DdpLim];
     tuning.DDSLIM = doubhead[VI::doubhead::DdsLim];
+
+    this->tuning.WSEG_REDUCTION_FACTOR = doubhead[VI::doubhead::WsegRedFac];
+    this->tuning.WSEG_INCREASE_FACTOR = doubhead[VI::doubhead::WsegIncFac];
 }
 
 void RstState::add_groups(const std::vector<std::string>& zgrp,
@@ -186,7 +454,8 @@ void RstState::add_wells(const std::vector<std::string>& zwel,
                          const std::vector<double>& xwel,
                          const std::vector<int>& icon,
                          const std::vector<float>& scon,
-                         const std::vector<double>& xcon) {
+                         const std::vector<double>& xcon)
+{
 
     for (int iw = 0; iw < this->header.num_wells; iw++) {
         std::size_t zwel_offset = iw * this->header.nzwelz;
@@ -223,7 +492,8 @@ void RstState::add_msw(const std::vector<std::string>& zwel,
                        const std::vector<float>& scon,
                        const std::vector<double>& xcon,
                        const std::vector<int>& iseg,
-                       const std::vector<double>& rseg) {
+                       const std::vector<double>& rseg)
+{
 
     for (int iw = 0; iw < this->header.num_wells; iw++) {
         std::size_t zwel_offset = iw * this->header.nzwelz;
@@ -251,59 +521,35 @@ void RstState::add_msw(const std::vector<std::string>& zwel,
     }
 }
 
-void RstState::add_udqs(const std::vector<int>& iudq,
-                        const std::vector<std::string>& zudn,
-                        const std::vector<std::string>& zudl,
-                        const std::vector<double>& dudw,
-                        const std::vector<double>& dudg,
-                        const std::vector<double>& dudf)
+void RstState::add_udqs(std::shared_ptr<EclIO::RestartFileView> rstView)
 {
-    auto well_var  = 0*this->header.num_wells;
-    auto group_var = 0*this->header.ngroup;
-    auto field_var = 0*this->header.num_udq();
+    const auto& iudq = rstView->getKeyword<int>("IUDQ");
+    const auto& zudn = rstView->getKeyword<std::string>("ZUDN");
+    const auto& zudl = rstView->getKeyword<std::string>("ZUDL");
 
-    for (auto udq_index = 0*this->header.num_udq(); udq_index < this->header.num_udq(); ++udq_index) {
+    if (rstView->hasKeyword<int>("IUAD")) {
+        const auto& iuad = rstView->getKeyword<int>("IUAD");
+        const auto& iuap = rstView->getKeyword<int>("IUAP");
+        const auto& igph = rstView->getKeyword<int>("IGPH");
+
+        this->udq_active = RstUDQActive(iuad, iuap, igph);
+    }
+
+    auto udqValues = UDQVectors { std::move(rstView) };
+
+    for (auto udq_index = 0*this->header.num_udq();
+         udq_index < this->header.num_udq(); ++udq_index)
+    {
         const auto& name = zudn[udq_index*UDQDims::entriesPerZUDN() + 0];
         const auto& unit = zudn[udq_index*UDQDims::entriesPerZUDN() + 1];
 
         const auto define = udq_define(zudl, udq_index);
+
         auto& udq = define.empty()
             ? this->udqs.emplace_back(name, unit)
             : this->udqs.emplace_back(name, unit, define, udq_update(iudq, udq_index));
 
-        if (udq.var_type == UDQVarType::WELL_VAR) {
-            for (std::size_t well_index = 0; well_index < this->wells.size(); well_index++) {
-                auto well_value = dudw[ well_var * this->header.max_wells_in_field + well_index ];
-                if (well_value == UDQ::restart_default)
-                    continue;
-
-                const auto& well_name = this->wells[well_index].name;
-                udq.add_value(well_name, well_value);
-            }
-
-            ++well_var;
-        }
-
-        if (udq.var_type == UDQVarType::GROUP_VAR) {
-            for (std::size_t group_index = 0; group_index < this->groups.size(); group_index++) {
-                auto group_value = dudg[ group_var * this->header.max_groups_in_field + group_index ];
-                if (group_value == UDQ::restart_default)
-                    continue;
-
-                const auto& group_name = this->groups[group_index].name;
-                udq.add_value(group_name, group_value);
-            }
-
-            ++group_var;
-        }
-
-        if (udq.var_type == UDQVarType::FIELD_VAR) {
-            auto field_value = dudf[ field_var ];
-            if (field_value != UDQ::restart_default)
-                udq.add_value(field_value);
-
-            ++field_var;
-        }
+        restoreSingleUDQ(*this, udqValues, udq);
     }
 }
 
@@ -372,7 +618,10 @@ void RstState::add_actions(const Parser& parser,
             action_deck += line + "\n";
             zlact_offset += actdims.line_size();
         }
-        const auto& deck = parser.parseString( action_deck );
+        // Ignore invalid keyword combinaions in actions, since these decks are typically incomplete
+        ParseContext parseContext;
+        parseContext.update(ParseContext::PARSE_INVALID_KEYWORD_COMBINATION, InputErrorAction::IGNORE);
+        const auto& deck = parser.parseString( action_deck, parseContext );
         for (auto keyword : deck)
             this->actions.back().keywords.push_back(std::move(keyword));
     }
@@ -402,7 +651,8 @@ void RstState::add_wlist(const std::vector<std::string>& zwls,
     }
 }
 
-const RstWell& RstState::get_well(const std::string& wname) const {
+const RstWell& RstState::get_well(const std::string& wname) const
+{
     const auto well_iter = std::find_if(this->wells.begin(),
                                         this->wells.end(),
                                         [&wname] (const auto& well) {
@@ -428,6 +678,7 @@ RstState RstState::load(std::shared_ptr<EclIO::RestartFileView> rstView,
         const auto& igrp = rstView->getKeyword<int>("IGRP");
         const auto& sgrp = rstView->getKeyword<float>("SGRP");
         const auto& xgrp = rstView->getKeyword<double>("XGRP");
+
         state.add_groups(zgrp, igrp, sgrp, xgrp);
     }
 
@@ -442,15 +693,19 @@ RstState RstState::load(std::shared_ptr<EclIO::RestartFileView> rstView,
         const auto& xcon = rstView->getKeyword<double>("XCON");
 
         if (rstView->hasKeyword<int>("ISEG")) {
+            // Multi-segmented wells in restart file.
             const auto& iseg = rstView->getKeyword<int>("ISEG");
             const auto& rseg = rstView->getKeyword<double>("RSEG");
 
             state.add_msw(zwel, iwel, swel, xwel,
                           icon, scon, xcon,
                           iseg, rseg);
-        } else
+        }
+        else {
+            // Standard wells only.
             state.add_wells(zwel, iwel, swel, xwel,
                             icon, scon, xcon);
+        }
 
         if (rstView->hasKeyword<int>("IWLS")) {
             const auto& iwls = rstView->getKeyword<int>("IWLS");
@@ -461,22 +716,7 @@ RstState RstState::load(std::shared_ptr<EclIO::RestartFileView> rstView,
     }
 
     if (state.header.num_udq() > 0) {
-        const auto& iudq = rstView->getKeyword<int>("IUDQ");
-        const auto& zudn = rstView->getKeyword<std::string>("ZUDN");
-        const auto& zudl = rstView->getKeyword<std::string>("ZUDL");
-
-        const auto& dudw = state.header.nwell_udq  > 0 ? rstView->getKeyword<double>("DUDW") : std::vector<double>{};
-        const auto& dudg = state.header.ngroup_udq > 0 ? rstView->getKeyword<double>("DUDG") : std::vector<double>{};
-        const auto& dudf = state.header.nfield_udq > 0 ? rstView->getKeyword<double>("DUDF") : std::vector<double>{};
-
-        state.add_udqs(iudq, zudn, zudl, dudw, dudg, dudf);
-
-        if (rstView->hasKeyword<int>("IUAD")) {
-            const auto& iuad = rstView->getKeyword<int>("IUAD");
-            const auto& iuap = rstView->getKeyword<int>("IUAP");
-            const auto& igph = rstView->getKeyword<int>("IGPH");
-            state.udq_active = RstUDQActive(iuad, iuap, igph);
-        }
+        state.add_udqs(rstView);
     }
 
     if (state.header.num_action > 0) {
@@ -490,8 +730,7 @@ RstState RstState::load(std::shared_ptr<EclIO::RestartFileView> rstView,
         state.add_actions(parser, runspec, state.header.sim_time(), zact, iact, sact, zacn, iacn, sacn, zlact);
     }
 
-
     return state;
 }
 
-}} // namespace Opm::RestartIO
+} // namespace Opm::RestartIO
