@@ -122,10 +122,10 @@ struct RstWell
     float wtest_interval;
     float wtest_startup;
     float grupcon_gr_scaling;
-    float glift_max_rate;
-    float glift_min_rate;
-    float glift_weight_factor;
-    float glift_inc_weight_factor;
+    double glift_max_rate;
+    double glift_min_rate;
+    double glift_weight_factor;
+    double glift_inc_weight_factor;
     float dfac_corr_coeff_a{};
     float dfac_corr_exponent_b{};
     float dfac_corr_exponent_c{};
@@ -157,6 +157,8 @@ struct RstWell
     double water_void_rate;
     double gas_void_rate;
 
+    static constexpr auto UNDEFINED_VALUE = 1.0e20f;
+
     const RstSegment& segment(int segment_number) const;
     std::vector<RstConnection> connections;
     std::vector<RstSegment> segments;
