@@ -31,6 +31,7 @@
 #include <opm/material/IdealGas.hpp>
 #include <opm/material/common/Valgrind.hpp>
 #include <opm/common/TimingMacros.hpp>
+#include <opm/common/utility/gpuDecorators.hpp>
 
 #include <array>
 
@@ -55,14 +56,14 @@ public:
      * \param temperature the temperature [K]
      * \param pressure the phase pressure [Pa]
      */
-    template <class Evaluation>
-    static Evaluation gasDiffCoeff(const Evaluation& temperature, const Evaluation& pressure, bool extrapolate = false)
+    template <class Evaluation, class CO2Params>
+    OPM_HOST_DEVICE static Evaluation gasDiffCoeff(const CO2Params& params, const Evaluation& temperature, const Evaluation& pressure, bool extrapolate = false)
     {
         //Diffusion coefficient of water in the CO2 phase
         Scalar k = 1.3806504e-23; // Boltzmann constant
         Scalar c = 4; // slip parameter, can vary between 4 (slip condition) and 6 (stick condition)
         Scalar R_h = 1.72e-10; // hydrodynamic radius of the solute
-        const Evaluation& mu = CO2::gasViscosity(temperature, pressure, extrapolate); // CO2 viscosity
+        const Evaluation& mu = CO2::gasViscosity(params, temperature, pressure, extrapolate); // CO2 viscosity
         return k / (c * M_PI * R_h) * (temperature / mu);
     }
 
@@ -73,7 +74,7 @@ public:
      * \param pressure the phase pressure [Pa]
      */
     template <class Evaluation>
-    static Evaluation liquidDiffCoeff(const Evaluation& /*temperature*/, const Evaluation& /*pressure*/)
+    OPM_HOST_DEVICE static Evaluation liquidDiffCoeff(const Evaluation& /*temperature*/, const Evaluation& /*pressure*/)
     {
         //Diffusion coefficient of CO2 in the brine phase
         return 2e-9;
@@ -96,8 +97,9 @@ public:
      * \param xlCO2 mole fraction of CO2 in brine [mol/mol]
      * \param ygH2O mole fraction of water in the gas phase [mol/mol]
      */
-    template <class Evaluation>
-    static void calculateMoleFractions(const Evaluation& temperature,
+    template <class Evaluation, class CO2Params>
+    OPM_HOST_DEVICE static void calculateMoleFractions(const CO2Params& params,
+                                       const Evaluation& temperature,
                                        const Evaluation& pg,
                                        const Evaluation& salinity,
                                        const int knownPhaseIdx,
@@ -124,7 +126,7 @@ public:
             // Technically only valid below T = 100 C, but we use low-temp. parameters and formulas even above 100 C as
             // an approximation.
             if (activityModel == 3) {
-                auto [xCO2, yH2O] = mutualSolubilitySpycherPruess2005_(temperature, pg, molalityNaCl, extrapolate);
+                auto [xCO2, yH2O] = mutualSolubilitySpycherPruess2005_(params, temperature, pg, molalityNaCl, extrapolate);
                 xlCO2 = xCO2;
                 ygH2O = yH2O;
 
@@ -132,14 +134,14 @@ public:
             else {
                 // Fixed-point iterations to calculate solubility
                 if (iterate) {
-                    auto [xCO2, yH2O] = fixPointIterSolubility_(temperature, pg, molalityNaCl, activityModel, extrapolate);
+                    auto [xCO2, yH2O] = fixPointIterSolubility_(params, temperature, pg, molalityNaCl, activityModel, extrapolate);
                     xlCO2 = xCO2;
                     ygH2O = yH2O;
                 }
 
                 // Solve mutual solubility equation with back substitution (no need for iterations)
                 else {
-                    auto [xCO2, yH2O] = nonIterSolubility_(temperature, pg, molalityNaCl, activityModel, extrapolate);
+                    auto [xCO2, yH2O] = nonIterSolubility_(params, temperature, pg, molalityNaCl, activityModel, extrapolate);
                     xlCO2 = xCO2;
                     ygH2O = yH2O;
                 }
@@ -151,7 +153,7 @@ public:
         // with the mutual solubility function
         else if (knownPhaseIdx == liquidPhaseIdx && activityModel == 3) {
             Evaluation x_NaCl = salinityToMolFrac_(salinity);
-            const Evaluation& A = computeA_(temperature, pg, Evaluation(0.0), Evaluation(0.0), false, extrapolate, true);
+            const Evaluation& A = computeA_(params, temperature, pg, Evaluation(0.0), Evaluation(0.0), false, extrapolate, true);
             ygH2O = A * (1 - xlCO2 - x_NaCl);
         }
 
@@ -161,7 +163,7 @@ public:
         else if (knownPhaseIdx == gasPhaseIdx && activityModel == 3) {
             //y_H2o = fluidstate.
             Evaluation x_NaCl = salinityToMolFrac_(salinity);
-            const Evaluation& A = computeA_(temperature, pg, Evaluation(0.0), Evaluation(0.0), false, extrapolate, true);
+            const Evaluation& A = computeA_(params, temperature, pg, Evaluation(0.0), Evaluation(0.0), false, extrapolate, true);
             xlCO2 = 1 - x_NaCl - ygH2O / A;
         }
     }
@@ -181,8 +183,9 @@ public:
      * \param T the temperature [K]
      * \param pg the gas phase pressure [Pa]
      */
-    template <class Evaluation>
-    static Evaluation fugacityCoefficientCO2(const Evaluation& temperature, 
+    template <class Evaluation, class CO2Params>
+    static Evaluation fugacityCoefficientCO2(const CO2Params& params,
+                                             const Evaluation& temperature, 
                                              const Evaluation& pg,
                                              const Evaluation& yH2O, 
                                              const bool highTemp, 
@@ -193,29 +196,30 @@ public:
         Valgrind::CheckDefined(temperature);
         Valgrind::CheckDefined(pg);
 
-        Evaluation V = 1 / (CO2::gasDensity(temperature, pg, extrapolate) / CO2::molarMass()) * 1.e6; // molar volume in cm^3/mol
-        Evaluation pg_bar = pg / 1.e5; // gas phase pressure in bar
-        Scalar R = IdealGas::R * 10.; // ideal gas constant with unit bar cm^3 /(K mol)
+        const Evaluation V = 1 / (CO2::gasDensity(params, temperature, pg, extrapolate) / CO2::molarMass()) * 1.e6; // molar volume in cm^3/mol
+        const Evaluation pg_bar = pg / 1.e5; // gas phase pressure in bar
+        const Scalar R = IdealGas::R * 10.; // ideal gas constant with unit bar cm^3 /(K mol)
 
         // Parameters in Redlich-Kwong equation
-        Evaluation a_CO2 = aCO2_(temperature, highTemp);
-        Evaluation a_CO2_H2O = aCO2_H2O_(temperature, yH2O, highTemp);
-        Evaluation a_mix = aMix_(temperature, yH2O, highTemp);
-        Scalar b_CO2 = bCO2_(highTemp); 
-        Evaluation b_mix = bMix_(yH2O, highTemp);
+        const Evaluation a_CO2 = aCO2_(temperature, highTemp);
+        const Evaluation a_CO2_H2O = aCO2_H2O_(temperature, yH2O, highTemp);
+        const Evaluation a_mix = aMix_(temperature, yH2O, highTemp);
+        const Scalar b_CO2 = bCO2_(highTemp);
+        const Evaluation b_mix = bMix_(yH2O, highTemp);
+        const Evaluation Rt15 = R * pow(temperature, 1.5);
 
         Evaluation lnPhiCO2;
         if (spycherPruess2005) {
+            const Evaluation logVpb_V = log((V + b_CO2) / V);
             lnPhiCO2 = log(V / (V - b_CO2));
             lnPhiCO2 += b_CO2 / (V - b_CO2);
-            lnPhiCO2 -= 2 * a_CO2 / (R * pow(temperature, 1.5) * b_CO2) * log((V + b_CO2) / V);
+            lnPhiCO2 -= 2 * a_CO2 / (Rt15 * b_CO2) * logVpb_V;
             lnPhiCO2 +=
                 a_CO2 * b_CO2
-                / (R
-                   * pow(temperature, 1.5)
+                / (Rt15
                    * b_CO2
                    * b_CO2)
-                * (log((V + b_CO2) / V)
+                * (logVpb_V
                    - b_CO2 / (V + b_CO2));
             lnPhiCO2 -= log(pg_bar * V / (R * temperature));
         }
@@ -223,7 +227,7 @@ public:
             lnPhiCO2 = (b_CO2 / b_mix) * (pg_bar * V / (R * temperature) - 1);
             lnPhiCO2 -= log(pg_bar * (V - b_mix) / (R * temperature));
             lnPhiCO2 += (2 * (yH2O * a_CO2_H2O + (1 - yH2O) * a_CO2) / a_mix - (b_CO2 / b_mix)) *
-                        a_mix / (b_mix * R * pow(temperature, 1.5)) * log(V / (V + b_mix));
+                        a_mix / (b_mix * Rt15) * log(V / (V + b_mix));
         }
         return exp(lnPhiCO2); // fugacity coefficient of CO2
     }
@@ -236,8 +240,9 @@ public:
      * \param temperature the temperature [K]
      * \param pg the gas phase pressure [Pa]
      */
-    template <class Evaluation>
-    static Evaluation fugacityCoefficientH2O(const Evaluation& temperature, 
+    template <class Evaluation, class CO2Params>
+    static Evaluation fugacityCoefficientH2O(const CO2Params& params,
+                                             const Evaluation& temperature, 
                                              const Evaluation& pg,
                                              const Evaluation& yH2O, 
                                              const bool highTemp, 
@@ -248,32 +253,34 @@ public:
         Valgrind::CheckDefined(temperature);
         Valgrind::CheckDefined(pg);
 
-        const Evaluation& V = 1 / (CO2::gasDensity(temperature, pg, extrapolate) / CO2::molarMass()) * 1.e6; // molar volume in cm^3/mol
+        const Evaluation& V = 1 / (CO2::gasDensity(params, temperature, pg, extrapolate) / CO2::molarMass()) * 1.e6; // molar volume in cm^3/mol
         const Evaluation& pg_bar = pg / 1.e5; // gas phase pressure in bar
-        Scalar R = IdealGas::R * 10.; // ideal gas constant with unit bar cm^3 /(K mol)
+        const Scalar R = IdealGas::R * 10.; // ideal gas constant with unit bar cm^3 /(K mol)
 
         // Mixture parameter of  Redlich-Kwong equation
-        Evaluation a_H2O = aH2O_(temperature, highTemp);
-        Evaluation a_CO2_H2O = aCO2_H2O_(temperature, yH2O, highTemp);
-        Evaluation a_mix = aMix_(temperature, yH2O, highTemp);
-        Scalar b_H2O = bH2O_(highTemp); 
-        Evaluation b_mix = bMix_(yH2O, highTemp);
+        const Evaluation a_H2O = aH2O_(temperature, highTemp);
+        const Evaluation a_CO2_H2O = aCO2_H2O_(temperature, yH2O, highTemp);
+        const Evaluation a_mix = aMix_(temperature, yH2O, highTemp);
+        const Scalar b_H2O = bH2O_(highTemp);
+        const Evaluation b_mix = bMix_(yH2O, highTemp);
+        const Evaluation Rt15 = R * pow(temperature, 1.5);
 
         Evaluation lnPhiH2O;
         if (spycherPruess2005) {
+            const Evaluation logVpb_V = log((V + b_mix) / V);
             lnPhiH2O =
                 log(V/(V - b_mix))
                 + b_H2O/(V - b_mix) - 2*a_CO2_H2O
-                / (R*pow(temperature, 1.5)*b_mix)*log((V + b_mix)/V)
-                + a_mix*b_H2O/(R*pow(temperature, 1.5)*b_mix*b_mix)
-                *(log((V + b_mix)/V) - b_mix/(V + b_mix))
+                / (Rt15*b_mix)*logVpb_V
+                + a_mix*b_H2O/(Rt15*b_mix*b_mix)
+                *(logVpb_V - b_mix/(V + b_mix))
                 - log(pg_bar*V/(R*temperature));
         }
         else {
             lnPhiH2O = (b_H2O / b_mix) * (pg_bar * V / (R * temperature) - 1);
             lnPhiH2O -= log(pg_bar * (V - b_mix) / (R * temperature));
             lnPhiH2O += (2 * (yH2O * a_H2O + (1 - yH2O) * a_CO2_H2O) / a_mix - (b_H2O / b_mix)) *
-                        a_mix / (b_mix * R * pow(temperature, 1.5)) * log(V / (V + b_mix));
+                        a_mix / (b_mix * Rt15) * log(V / (V + b_mix));
         }
         return exp(lnPhiH2O); // fugacity coefficient of H2O
     }
@@ -283,7 +290,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation aCO2_(const Evaluation& temperature, const bool& highTemp)
+    OPM_HOST_DEVICE static Evaluation aCO2_(const Evaluation& temperature, const bool& highTemp)
     {
         if (highTemp) {
             return 8.008e7 - 4.984e4 * temperature;
@@ -297,7 +304,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation aH2O_(const Evaluation& temperature, const bool& highTemp)
+    OPM_HOST_DEVICE static Evaluation aH2O_(const Evaluation& temperature, const bool& highTemp)
     {
         if (highTemp) {
             return 1.337e8 - 1.4e4 * temperature;
@@ -311,7 +318,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation aCO2_H2O_(const Evaluation& temperature, const Evaluation& yH2O, const bool& highTemp)
+    OPM_HOST_DEVICE static Evaluation aCO2_H2O_(const Evaluation& temperature, const Evaluation& yH2O, const bool& highTemp)
     {
         if (highTemp) {
             // Pure parameters
@@ -335,7 +342,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation aMix_(const Evaluation& temperature, const Evaluation& yH2O, const bool& highTemp)
+    OPM_HOST_DEVICE static Evaluation aMix_(const Evaluation& temperature, const Evaluation& yH2O, const bool& highTemp)
     {
         if (highTemp) {
             // Parameters
@@ -353,7 +360,7 @@ private:
     /*!
     * \brief
     */
-    static Scalar bCO2_(const bool& highTemp)
+    OPM_HOST_DEVICE static Scalar bCO2_(const bool& highTemp)
     {
         if (highTemp) {
             return 28.25;
@@ -366,7 +373,7 @@ private:
     /*!
     * \brief
     */
-    static Scalar bH2O_(const bool& highTemp)
+    OPM_HOST_DEVICE static Scalar bH2O_(const bool& highTemp)
     {
         if (highTemp) {
             return 15.7;
@@ -380,7 +387,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation bMix_(const Evaluation& yH2O, const bool& highTemp)
+    OPM_HOST_DEVICE static Evaluation bMix_(const Evaluation& yH2O, const bool& highTemp)
     {
         if (highTemp) {
             // Parameters
@@ -398,7 +405,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation V_avg_CO2_(const Evaluation& temperature, const bool& highTemp)
+    OPM_HOST_DEVICE static Evaluation V_avg_CO2_(const Evaluation& temperature, const bool& highTemp)
     {
         if (highTemp && (temperature > 373.15)) {
             return 32.6 + 3.413e-2 * (temperature - 373.15);
@@ -412,7 +419,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation V_avg_H2O_(const Evaluation& temperature, const bool& highTemp)
+    OPM_HOST_DEVICE static Evaluation V_avg_H2O_(const Evaluation& temperature, const bool& highTemp)
     {
         if (highTemp && (temperature > 373.15)) {
             return 18.1 + 3.137e-2 * (temperature - 373.15);
@@ -426,7 +433,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation AM_(const Evaluation& temperature, const bool& highTemp)
+    OPM_HOST_DEVICE static Evaluation AM_(const Evaluation& temperature, const bool& highTemp)
     {
         if (highTemp && temperature > 373.15) {
             Evaluation deltaTk = temperature - 373.15;
@@ -441,7 +448,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation Pref_(const Evaluation& temperature, const bool& highTemp)
+    OPM_HOST_DEVICE static Evaluation Pref_(const Evaluation& temperature, const bool& highTemp)
     {
         if (highTemp && temperature > 373.15) {
             const Evaluation& temperatureCelcius = temperature - 273.15;
@@ -458,7 +465,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation activityCoefficientCO2_(const Evaluation& temperature, 
+    OPM_HOST_DEVICE static Evaluation activityCoefficientCO2_(const Evaluation& temperature, 
                                               const Evaluation& xCO2, 
                                               const bool& highTemp)
     {
@@ -477,7 +484,7 @@ private:
     * \brief
     */
     template <class Evaluation>
-    static Evaluation activityCoefficientH2O_(const Evaluation& temperature, 
+    OPM_HOST_DEVICE static Evaluation activityCoefficientH2O_(const Evaluation& temperature, 
                                               const Evaluation& xCO2, 
                                               const bool& highTemp)
     {
@@ -498,7 +505,7 @@ private:
      * \param salinity the salinity [kg NaCl / kg solution]
      */
     template <class Evaluation>
-    static Evaluation salinityToMolFrac_(const Evaluation& salinity) {
+    OPM_HOST_DEVICE static Evaluation salinityToMolFrac_(const Evaluation& salinity) {
         OPM_TIMEFUNCTION_LOCAL();
         const Scalar Mw = H2O::molarMass(); /* molecular weight of water [kg/mol] */
         const Scalar Ms = 58.44e-3; /* molecular weight of NaCl  [kg/mol] */
@@ -514,15 +521,17 @@ private:
     *
     * \param x_NaCl mole fraction of NaCL in brine [mol/mol]
     */
+#if 0
     template <class Evaluation>
-    static Evaluation moleFracToMolality_(const Evaluation& x_NaCl)
+    OPM_HOST_DEVICE static Evaluation moleFracToMolality_(const Evaluation& x_NaCl)
     {
         // conversion from mol fraction to molality (dissolved CO2 neglected)
         return 55.508 * x_NaCl / (1 - x_NaCl);
     }
+#endif
 
     template <class Evaluation>
-    static Evaluation massFracToMolality_(const Evaluation& X_NaCl)
+    OPM_HOST_DEVICE static Evaluation massFracToMolality_(const Evaluation& X_NaCl)
     {
         const Scalar MmNaCl = 58.44e-3;
         return X_NaCl / (MmNaCl * (1 - X_NaCl));
@@ -534,7 +543,7 @@ private:
     * \param x_NaCl mole fraction of NaCL in brine [mol/mol]
     */
     template <class Evaluation>
-    static Evaluation molalityToMoleFrac_(const Evaluation& m_NaCl)
+    OPM_HOST_DEVICE static Evaluation molalityToMoleFrac_(const Evaluation& m_NaCl)
     {
         // conversion from molality to mole fractio (dissolved CO2 neglected)
         return m_NaCl / (55.508 + m_NaCl);
@@ -543,8 +552,9 @@ private:
     /*!
     * \brief Fixed-point iterations for high-temperature cases
     */
-    template <class Evaluation>
-    static std::pair<Evaluation, Evaluation> fixPointIterSolubility_(const Evaluation& temperature, 
+    template <class Evaluation, class CO2Parameters>
+    OPM_HOST_DEVICE static std::pair<Evaluation, Evaluation> fixPointIterSolubility_(const CO2Parameters& params,
+                                                                     const Evaluation& temperature, 
                                                                      const Evaluation& pg,
                                                                      const Evaluation& m_NaCl,
                                                                      const int& activityModel,
@@ -578,7 +588,7 @@ private:
             }
 
             // F(x_i) is the mutual solubilities
-            auto [xCO2_new, yH2O_new] = mutualSolubility_(temperature, pg, xCO2, yH2O, m_NaCl, gammaNaCl, highTemp, 
+            auto [xCO2_new, yH2O_new] = mutualSolubility_(params, temperature, pg, xCO2, yH2O, m_NaCl, gammaNaCl, highTemp, 
                                                           iterate, extrapolate);
             
             // Check for convergence
@@ -601,8 +611,9 @@ private:
     /*!
     * \brief Fixed-point iterations for high-temperature cases
     */
-    template <class Evaluation>
-    static std::pair<Evaluation, Evaluation> nonIterSolubility_(const Evaluation& temperature, 
+    template <class Evaluation, class CO2Parameters>
+    OPM_HOST_DEVICE static std::pair<Evaluation, Evaluation> nonIterSolubility_(const CO2Parameters& params,
+                                                                const Evaluation& temperature, 
                                                                 const Evaluation& pg,
                                                                 const Evaluation& m_NaCl,
                                                                 const int& activityModel,
@@ -618,7 +629,7 @@ private:
         // Note that we don't use xCO2 and yH2O input in low-temperature case, so we set them to 0.0
         const bool highTemp = false;
         const bool iterate = false;
-        auto [xCO2, yH2O] = mutualSolubility_(temperature, pg, Evaluation(0.0), Evaluation(0.0), m_NaCl, gammaNaCl, 
+        auto [xCO2, yH2O] = mutualSolubility_(params, temperature, pg, Evaluation(0.0), Evaluation(0.0), m_NaCl, gammaNaCl, 
                                               highTemp, iterate, extrapolate);
 
         return {xCO2, yH2O};
@@ -627,8 +638,9 @@ private:
     /*!
     * \brief Mutual solubility according to Spycher & Pruess (2009)
     */
-    template <class Evaluation>
-    static std::pair<Evaluation, Evaluation> mutualSolubility_(const Evaluation& temperature, 
+    template <class Evaluation, class CO2Parameters>
+    OPM_HOST_DEVICE static std::pair<Evaluation, Evaluation> mutualSolubility_(const CO2Parameters& params,
+                                                               const Evaluation& temperature,
                                                                const Evaluation& pg,
                                                                const Evaluation& xCO2,
                                                                const Evaluation& yH2O,
@@ -639,8 +651,8 @@ private:
                                                                bool extrapolate = false)
     {
         // Calculate A and B (without salt effect); Eqs. (8) and (9)
-        const Evaluation& A = computeA_(temperature, pg, yH2O, xCO2, highTemp, extrapolate);
-        Evaluation B = computeB_(temperature, pg, yH2O, xCO2, highTemp, extrapolate);
+        const Evaluation& A = computeA_(params, temperature, pg, yH2O, xCO2, highTemp, extrapolate);
+        Evaluation B = computeB_(params, temperature, pg, yH2O, xCO2, highTemp, extrapolate);
 
         // Add salt effect to B, Eq. (17)
         B /= gammaNaCl;
@@ -661,15 +673,16 @@ private:
     /*!
     * \brief Mutual solubility according to Spycher & Pruess (2009)
     */
-    template <class Evaluation>
-    static std::pair<Evaluation, Evaluation> mutualSolubilitySpycherPruess2005_(const Evaluation& temperature, 
+    template <class Evaluation, class CO2Parameters>
+    OPM_HOST_DEVICE static std::pair<Evaluation, Evaluation> mutualSolubilitySpycherPruess2005_(const CO2Parameters& params,
+                                                                                const Evaluation& temperature,
                                                                                 const Evaluation& pg,
                                                                                 const Evaluation& m_NaCl,
                                                                                 bool extrapolate = false)
     {
         // Calculate A and B (without salt effect); Eqs. (8) and (9)
-        const Evaluation& A = computeA_(temperature, pg, Evaluation(0.0), Evaluation(0.0), false, extrapolate, true);
-        const Evaluation& B = computeB_(temperature, pg, Evaluation(0.0), Evaluation(0.0), false, extrapolate, true);
+        const Evaluation& A = computeA_(params, temperature, pg, Evaluation(0.0), Evaluation(0.0), false, extrapolate, true);
+        const Evaluation& B = computeB_(params, temperature, pg, Evaluation(0.0), Evaluation(0.0), false, extrapolate, true);
 
         // Mole fractions and molality in pure water
         Evaluation yH2O = (1 - B) / (1. / A - B);
@@ -700,8 +713,9 @@ private:
      * \param T the temperature [K]
      * \param pg the gas phase pressure [Pa]
      */
-    template <class Evaluation>
-    static Evaluation computeA_(const Evaluation& temperature, 
+    template <class Evaluation, class CO2Params>
+    OPM_HOST_DEVICE static Evaluation computeA_(const CO2Params& params,
+                                const Evaluation& temperature, 
                                 const Evaluation& pg, 
                                 const Evaluation& yH2O,
                                 const Evaluation& xCO2,
@@ -714,7 +728,7 @@ private:
         const Evaluation& deltaP = pg / 1e5 - Pref_(temperature, highTemp); // pressure range [bar] from pref to pg[bar]
         Evaluation v_av_H2O = V_avg_H2O_(temperature, highTemp); // average partial molar volume of H2O [cm^3/mol]
         Evaluation k0_H2O = equilibriumConstantH2O_(temperature, highTemp); // equilibrium constant for H2O at 1 bar
-        Evaluation phi_H2O = fugacityCoefficientH2O(temperature, pg, yH2O, highTemp, extrapolate, spycherPruess2005); // fugacity coefficient of H2O for the water-CO2 system
+        Evaluation phi_H2O = fugacityCoefficientH2O(params, temperature, pg, yH2O, highTemp, extrapolate, spycherPruess2005); // fugacity coefficient of H2O for the water-CO2 system
         Evaluation gammaH2O = activityCoefficientH2O_(temperature, xCO2, highTemp);
 
         // In the intermediate temperature range 99-109 C, equilibrium constants and fugacity coeff. are linearly
@@ -722,7 +736,7 @@ private:
         if ( temperature > 372.15 && temperature < 382.15 && !spycherPruess2005) {
             const Evaluation weight = (382.15 - temperature) / 10.;
             const Evaluation& k0_H2O_low = equilibriumConstantH2O_(temperature, false);
-            const Evaluation& phi_H2O_low = fugacityCoefficientH2O(temperature, pg, Evaluation(0.0), false, extrapolate);
+            const Evaluation& phi_H2O_low = fugacityCoefficientH2O(params, temperature, pg, Evaluation(0.0), false, extrapolate);
             k0_H2O = k0_H2O * (1 - weight) + k0_H2O_low * weight;
             phi_H2O = phi_H2O * (1 - weight) + phi_H2O_low * weight;
         }
@@ -741,8 +755,9 @@ private:
      * \param temperature the temperature [K]
      * \param pg the gas phase pressure [Pa]
      */
-    template <class Evaluation>
-    static Evaluation computeB_(const Evaluation& temperature, 
+    template <class Evaluation, class CO2Parameters>
+    static Evaluation computeB_(const CO2Parameters& params,
+                                const Evaluation& temperature, 
                                 const Evaluation& pg, 
                                 const Evaluation& yH2O,
                                 const Evaluation& xCO2,
@@ -755,7 +770,7 @@ private:
         const Evaluation& deltaP = pg / 1e5 - Pref_(temperature, highTemp); // pressure range [bar] from pref to pg[bar]
         Evaluation v_av_CO2 = V_avg_CO2_(temperature, highTemp); // average partial molar volume of CO2 [cm^3/mol]
         Evaluation k0_CO2 = equilibriumConstantCO2_(temperature, pg, highTemp, spycherPruess2005); // equilibrium constant for CO2 at 1 bar
-        Evaluation phi_CO2 = fugacityCoefficientCO2(temperature, pg, yH2O, highTemp, extrapolate, spycherPruess2005); // fugacity coefficient of CO2 for the water-CO2 system
+        Evaluation phi_CO2 = fugacityCoefficientCO2(params, temperature, pg, yH2O, highTemp, extrapolate, spycherPruess2005); // fugacity coefficient of CO2 for the water-CO2 system
         Evaluation gammaCO2 = activityCoefficientCO2_(temperature, xCO2, highTemp);
 
         // In the intermediate temperature range 99-109 C, equilibrium constants and fugacity coeff. are linearly
@@ -763,7 +778,7 @@ private:
         if ( temperature > 372.15 && temperature < 382.15 && !spycherPruess2005) {
             const Evaluation weight = (382.15 - temperature) / 10.;
             const Evaluation& k0_CO2_low = equilibriumConstantCO2_(temperature, pg, false, spycherPruess2005);
-            const Evaluation& phi_CO2_low = fugacityCoefficientCO2(temperature, pg, Evaluation(0.0), false, extrapolate);
+            const Evaluation& phi_CO2_low = fugacityCoefficientCO2(params, temperature, pg, Evaluation(0.0), false, extrapolate);
             k0_CO2 = k0_CO2 * (1 - weight) + k0_CO2_low * weight;
             phi_CO2 = phi_CO2 * (1 - weight) + phi_CO2_low * weight;
         }
@@ -778,7 +793,7 @@ private:
     * \brief Activity model of salt in Spycher & Pruess (2009)
     */
     template <class Evaluation>
-    static Evaluation activityCoefficientSalt_(const Evaluation& temperature,
+    OPM_HOST_DEVICE static Evaluation activityCoefficientSalt_(const Evaluation& temperature,
                                                const Evaluation& pg, 
                                                const Evaluation& m_NaCl,
                                                const Evaluation& xCO2,
@@ -822,7 +837,7 @@ private:
     * \brief Lambda parameter in Duan & Sun model, as modified and detailed in Spycher & Pruess (2009)
     */
     template <class Evaluation>
-    static Evaluation computeLambdaSpycherPruess2009_(const Evaluation& temperature)
+    OPM_HOST_DEVICE static Evaluation computeLambdaSpycherPruess2009_(const Evaluation& temperature)
     {
         // Table 1
         static const Scalar c[3] = { 2.217e-4, 1.074, 2648. };
@@ -835,7 +850,7 @@ private:
     * \brief Xi parameter in Duan & Sun model, as modified and detailed in Spycher & Pruess (2009)
     */
     template <class Evaluation>
-    static Evaluation computeXiSpycherPruess2009_(const Evaluation& temperature)
+    OPM_HOST_DEVICE static Evaluation computeXiSpycherPruess2009_(const Evaluation& temperature)
     {
         // Table 1
         static const Scalar c[3] = { 1.3e-5, -20.12, 5259. };
@@ -848,7 +863,7 @@ private:
     * \brief Lambda parameter in Rumpf et al. (1994), as detailed in Spycher & Pruess (2005)
     */
     template <class Evaluation>
-    static Evaluation computeLambdaRumpfetal_(const Evaluation& temperature)
+    OPM_HOST_DEVICE static Evaluation computeLambdaRumpfetal_(const Evaluation& temperature)
     {
         // B^(0) below Eq. (A-6)
         static const Scalar c[4] = { 0.254, -76.82, -10656, 6312e3 };
@@ -865,7 +880,7 @@ private:
      * \param pg the gas phase pressure [Pa]
      */
     template <class Evaluation>
-    static Evaluation computeLambdaDuanSun_(const Evaluation& temperature, const Evaluation& pg)
+    OPM_HOST_DEVICE static Evaluation computeLambdaDuanSun_(const Evaluation& temperature, const Evaluation& pg)
     {
         static const Scalar c[6] =
             { -0.411370585, 6.07632013E-4, 97.5347708, -0.0237622469, 0.0170656236, 1.41335834E-5 };
@@ -883,7 +898,7 @@ private:
      * \param pg the gas phase pressure [Pa]
      */
     template <class Evaluation>
-    static Evaluation computeXiDuanSun_(const Evaluation& temperature, const Evaluation& pg)
+    OPM_HOST_DEVICE static Evaluation computeXiDuanSun_(const Evaluation& temperature, const Evaluation& pg)
     {
         static const Scalar c[4] =
             { 3.36389723E-4, -1.98298980E-5, 2.12220830E-3, -5.24873303E-3 };
@@ -911,9 +926,14 @@ private:
             c = { 1.668, 3.992e-3, -1.156e-5, 1.593e-9 };
         }
         else {
-            // For temperature below 31 C and pressures above saturation pressure, separate parameters are needed
-            Evaluation psat = CO2::vaporPressure(temperature);
-            if (temperatureCelcius < 31 && pg > psat && !spycherPruess2005) {
+            // For temperature below critical temperature and pressures above saturation pressure, separate parameters are needed
+            bool model1 = temperature < CO2::criticalTemperature() && !spycherPruess2005;
+            if (model1) {
+                // Computing the vapor pressure is not trivial and is also not defined for T > criticalTemperature
+                Evaluation psat = CO2::vaporPressure(temperature);
+                model1 = pg > psat;
+            }
+            if (model1) {
                 c = { 1.169, 1.368e-2, -5.38e-5, 0.0 };
             }
             else {
@@ -933,7 +953,7 @@ private:
      * \param temperature the temperature [K]
      */
     template <class Evaluation>
-    static Evaluation equilibriumConstantH2O_(const Evaluation& temperature, const bool& highTemp)
+    OPM_HOST_DEVICE static Evaluation equilibriumConstantH2O_(const Evaluation& temperature, const bool& highTemp)
     {
         Evaluation temperatureCelcius = temperature - 273.15;
         std::array<Scalar, 5> c;
