@@ -93,8 +93,8 @@ public:
      *               have a size() method)
      */
     template <class PointContainer>
-    Tabulated1DFunction(const PointContainer& points,
-                        bool sortInputs = true)
+    explicit Tabulated1DFunction(const PointContainer& points,
+                                 bool sortInputs = true)
     { this->setContainerOfTuples(points, sortInputs); }
 
     /*!
@@ -388,9 +388,9 @@ public:
 
             Scalar m = evalDerivative_(xMax(), /*segmentIdx=*/numSamples() - 2);
             if (m < 0)
-                return (r < 0 || r==3)?-1:0;
+                return (r < 0 || r==3) ? -1 : 0;
             else if (m > 0)
-                return (r > 0 || r==3)?1:0;
+                return r > 0 ? 1 : 0;
 
             return r;
         }
@@ -551,7 +551,7 @@ private:
      */
     struct ComparatorX_
     {
-        ComparatorX_(const std::vector<Scalar>& x)
+        explicit ComparatorX_(const std::vector<Scalar>& x)
             : x_(x)
         {}
 
