@@ -32,12 +32,24 @@
 #include <opm/common/Exceptions.hpp>
 
 #include <opm/material/common/MathToolbox.hpp>
+#include <opm/common/utility/gpuDecorators.hpp>
 
 #include <algorithm>
 #include <cassert>
 #include <string>
 #include <vector>
 
+// forward declaration of the class so the function in the next namespace can be declared
+template <class Scalar, class ContainerT = std::vector<Scalar>>
+class UniformTabulated2DFunction;
+
+// declaration of make_view in correct namespace so friend function can be declared in the class
+namespace Opm::gpuistl
+{
+    template <class ViewType, class ScalarT, class ContainerType>
+    UniformTabulated2DFunction<ScalarT, ViewType> make_view(UniformTabulated2DFunction<ScalarT, ContainerType>& params);
+}
+
 namespace Opm {
 
 /*!
@@ -47,13 +59,21 @@ namespace Opm {
  * This class can be used when the sampling points are calculated at
  * run time.
  */
-template <class Scalar>
+template <class Scalar, class ContainerT = std::vector<Scalar>>
 class UniformTabulated2DFunction
 {
 public:
-    UniformTabulated2DFunction()
+    OPM_HOST_DEVICE UniformTabulated2DFunction()
     { }
 
+
+    // Intended for construction of UniformTabulated2DFunction<double, GPUBuffer>
+    UniformTabulated2DFunction(Scalar minX, Scalar maxX, unsigned m,
+                               Scalar minY, Scalar maxY, unsigned n,
+                               const ContainerT& samples)
+        : samples_(samples), m_(m), n_(n), xMin_(minX), yMin_(minY), xMax_(maxX), yMax_(maxY){
+    }
+
      /*!
       * \brief Constructor where the tabulation parameters are already
       *        provided.
@@ -75,6 +95,19 @@ public:
                 this->setSamplePoint(i, j, vals[i][j]);
     }
 
+    // Both CO2Tables and H2Tables have values of dimes [200][500]
+    // suboptimal hardcoding for now but easier than templating this function etc
+    UniformTabulated2DFunction(Scalar minX, Scalar maxX, unsigned m,
+                               Scalar minY, Scalar maxY, unsigned n,
+                               const double vals[200][500])
+    {
+        resize(minX, maxX, m, minY, maxY, n);
+
+        for (unsigned i = 0; i < m; ++i)
+            for (unsigned j = 0; j < n; ++j)
+                this->setSamplePoint(i, j, vals[i][j]);
+    }
+
     /*!
      * \brief Resize the tabulation to a new range.
      */
@@ -96,43 +129,50 @@ public:
     /*!
      * \brief Returns the minimum of the X coordinate of the sampling points.
      */
-    Scalar xMin() const
+    OPM_HOST_DEVICE Scalar xMin() const
     { return xMin_; }
 
     /*!
      * \brief Returns the maximum of the X coordinate of the sampling points.
      */
-    Scalar xMax() const
+    OPM_HOST_DEVICE Scalar xMax() const
     { return xMax_; }
 
     /*!
      * \brief Returns the minimum of the Y coordinate of the sampling points.
      */
-    Scalar yMin() const
+    OPM_HOST_DEVICE Scalar yMin() const
     { return yMin_; }
 
     /*!
      * \brief Returns the maximum of the Y coordinate of the sampling points.
      */
-    Scalar yMax() const
+    OPM_HOST_DEVICE Scalar yMax() const
     { return yMax_; }
 
     /*!
      * \brief Returns the number of sampling points in X direction.
      */
-    unsigned numX() const
+    OPM_HOST_DEVICE unsigned numX() const
     { return m_; }
 
     /*!
      * \brief Returns the number of sampling points in Y direction.
      */
-    unsigned numY() const
+    OPM_HOST_DEVICE unsigned numY() const
     { return n_; }
 
+    /*!
+     * \brief Returns the sampling points.
+     */
+    OPM_HOST_DEVICE const ContainerT& samples() const
+    { return samples_; }
+
+
     /*!
      * \brief Return the position on the x-axis of the i-th interval.
      */
-    Scalar iToX(unsigned i) const
+    OPM_HOST_DEVICE Scalar iToX(unsigned i) const
     {
         assert(i < numX());
 
@@ -142,7 +182,7 @@ public:
     /*!
      * \brief Return the position on the y-axis of the j-th interval.
       */
-    Scalar jToY(unsigned j) const
+    OPM_HOST_DEVICE Scalar jToY(unsigned j) const
     {
         assert(j < numY());
 
@@ -158,7 +198,7 @@ public:
      * sample point.
       */
     template <class Evaluation>
-    Evaluation xToI(const Evaluation& x) const
+    OPM_HOST_DEVICE Evaluation xToI(const Evaluation& x) const
     { return (x - xMin())/(xMax() - xMin())*(numX() - 1); }
 
     /*!
@@ -170,14 +210,14 @@ public:
      * sample point.
      */
     template <class Evaluation>
-    Evaluation yToJ(const Evaluation& y) const
+    OPM_HOST_DEVICE Evaluation yToJ(const Evaluation& y) const
     { return (y - yMin())/(yMax() - yMin())*(numY() - 1); }
 
     /*!
      * \brief Returns true iff a coordinate lies in the tabulated range
      */
     template <class Evaluation>
-    bool applies(const Evaluation& x, const Evaluation& y) const
+    OPM_HOST_DEVICE bool applies(const Evaluation& x, const Evaluation& y) const
     {
         return
             xMin() <= x && x <= xMax() &&
@@ -193,11 +233,12 @@ public:
      * an exception might be thrown.
      */
     template <class Evaluation>
-    Evaluation eval(const Evaluation& x,
+    OPM_HOST_DEVICE Evaluation eval(const Evaluation& x,
                     const Evaluation& y,
                     [[maybe_unused]] bool extrapolate) const
     {
 #ifndef NDEBUG
+#if !OPM_IS_INSIDE_DEVICE_FUNCTION
         if (!extrapolate && !applies(x,y)) {
             std::string msg = "Attempt to get tabulated value for ("
                 +std::to_string(double(scalarValue(x)))+", "+std::to_string(double(scalarValue(y)))
@@ -205,16 +246,9 @@ public:
                 +std::to_string(xMin())+" to "+std::to_string(xMax())+" times "
                 +std::to_string(yMin())+" to "+std::to_string(yMax());
 
-            if (!extrapolate)
-            {
-                throw NumericalProblem(msg);
-            }
-            else
-            {
-                OpmLog::warning("PVT Table evaluation:" + msg + ". Will use extrapolation");
-            }
-
+            throw NumericalProblem(msg);
         }
+#endif
 #endif
 
         Evaluation alpha = xToI(x);
@@ -243,7 +277,7 @@ public:
      *         intersection of the \f$i\f$-th interval of the x-Axis
      *         and the \f$j\f$-th of the y-Axis.
      */
-    Scalar getSamplePoint(unsigned i, unsigned j) const
+    OPM_HOST_DEVICE Scalar getSamplePoint(unsigned i, unsigned j) const
     {
         assert(i < m_);
         assert(j < n_);
@@ -264,7 +298,7 @@ public:
         samples_[j*m_ + i] = value;
     }
 
-    bool operator==(const UniformTabulated2DFunction<Scalar>& data) const
+    OPM_HOST_DEVICE bool operator==(const UniformTabulated2DFunction<Scalar>& data) const
     {
         return samples_ == data.samples_ &&
                m_ == data.m_ &&
@@ -275,12 +309,14 @@ public:
                yMax_ == data.yMax_;
     }
 
-
 private:
+    template <class ViewType, class ScalarT, class Container>
+    friend UniformTabulated2DFunction<ScalarT, ViewType> gpuistl::make_view(UniformTabulated2DFunction<ScalarT, Container>&);
+
     // the vector which contains the values of the sample points
     // f(x_i, y_j). don't use this directly, use getSamplePoint(i,j)
     // instead!
-    std::vector<Scalar> samples_;
+    ContainerT samples_;
 
     // the number of sample points in x direction
     unsigned m_;
@@ -299,4 +335,21 @@ private:
 
 } // namespace Opm
 
+namespace Opm::gpuistl{
+    template<class GPUContainer, class ScalarT>
+    UniformTabulated2DFunction<ScalarT, GPUContainer>
+    copy_to_gpu(const UniformTabulated2DFunction<ScalarT>& tab){
+        return UniformTabulated2DFunction<ScalarT, GPUContainer>(tab.xMin(), tab.xMax(), tab.numX(), tab.yMin(), tab.yMax(), tab.numY(), GPUContainer(tab.samples()));
+    }
+
+    template <class ViewType, class ScalarT, class ContainerType>
+    UniformTabulated2DFunction<ScalarT, ViewType>
+    make_view(UniformTabulated2DFunction<ScalarT, ContainerType>& tab) {
+
+        ViewType newTab = make_view<typename ViewType::value_type>(tab.samples_);
+
+        return UniformTabulated2DFunction<ScalarT, ViewType>(tab.xMin(), tab.xMax(), tab.numX(), tab.yMin(), tab.yMax(), tab.numY(), newTab);
+    }
+}
+
 #endif
