@@ -40,6 +40,38 @@
 
 namespace Opm {
 
+// this class collects all the H2tabulated quantities in one convenient place
+struct H2Tables {
+	static const Opm::UniformTabulated2DFunction<double>   tabulatedEnthalpy;
+	static const Opm::UniformTabulated2DFunction<double>   tabulatedDensity;
+	static constexpr double brineSalinity = 1.000000000000000e-01;
+};
+
+struct H2TabulatedDensityTraits {
+    typedef double Scalar;
+    static const char  *name;
+    static const int    numX = 200;
+    static const Scalar xMin;
+    static const Scalar xMax;
+    static const int    numY = 500;
+    static const Scalar yMin;
+    static const Scalar yMax;
+
+    static const Scalar vals[200][500];
+};
+
+struct H2TabulatedEnthalpyTraits {
+    typedef double Scalar;
+    static const char  *name;
+    static const int    numX = 200;
+    static const Scalar xMin;
+    static const Scalar xMax;
+    static const int    numY = 500;
+    static const Scalar yMin;
+    static const Scalar yMax;
+    static const Scalar vals[200][500];
+};
+
 /*!
  * \ingroup Components
  * \brief Properties of pure molecular hydrogen \f$H_2\f$. Most of the properties are calculated based on Leachman JW,
@@ -693,6 +725,7 @@ private:
     static constexpr Scalar gamma_[5] = {0.7164, 1.3444, 1.4517, 0.7204, 1.5445};
     static constexpr Scalar D_[5] = {1.506, 0.156, 1.736, 0.670, 1.662};
 
+#if 0
     /*!
     * \brief Objective function in root-finding done in convertPgToReducedRho.
     * 
@@ -714,6 +747,7 @@ private:
         Evaluation obj = rho_red * rho_cRT * (1 + rho_red * dResHelm_dRedRho) - p_MPa;
         return obj;
     }
+#endif
 };
 
 } // end namespace Opm
