@@ -37,7 +37,9 @@
 #include <opm/material/common/MathToolbox.hpp>
 #include <opm/material/common/Valgrind.hpp>
 #include <opm/material/Constants.hpp>
-#include <opm/material/eos/PengRobinsonMixture.hpp>
+#include <opm/material/eos/CubicEOS.hpp>
+
+#include <opm/input/eclipse/EclipseState/Compositional/CompositionalConfig.hpp>
 
 #include <dune/common/fvector.hh>
 #include <dune/common/fmatrix.hh>
@@ -61,17 +63,16 @@ namespace Opm {
 template <class Scalar, class FluidSystem>
 class PTFlash
 {
-    enum { numPhases = FluidSystem::numPhases };
-    enum { numComponents = FluidSystem::numComponents };
+    static constexpr int numPhases = FluidSystem::numPhases;
+    static constexpr int numComponents = FluidSystem::numComponents;
     enum { oilPhaseIdx = FluidSystem::oilPhaseIdx};
     enum { gasPhaseIdx = FluidSystem::gasPhaseIdx};
-    enum { numMiscibleComponents = FluidSystem::numMiscibleComponents};
-    enum { numMisciblePhases = FluidSystem::numMisciblePhases}; //oil, gas
-    enum {
-        numEq =
-        numMisciblePhases+
-        numMisciblePhases*numMiscibleComponents
-    };
+    enum { waterPhaseIdx = FluidSystem::waterPhaseIdx};
+    static constexpr int numMiscibleComponents = FluidSystem::numMiscibleComponents;
+    static constexpr int numMisciblePhases = FluidSystem::numMisciblePhases; //oil, gas
+    static constexpr int numEq = numMisciblePhases + numMisciblePhases * numMiscibleComponents;
+
+    using EOSType = CompositionalConfig::EOSType;
 
 public:
     /*!
@@ -79,53 +80,21 @@ public:
      *
      */
     template <class FluidState>
-    static void solve(FluidState& fluid_state,
-                      const Dune::FieldVector<typename FluidState::Scalar, numComponents>& z,
-                      std::string twoPhaseMethod,
-                      Scalar tolerance = -1.,
+    static bool solve(FluidState& fluid_state,
+                      const std::string& twoPhaseMethod,
+                      Scalar flash_tolerance,
+                      const EOSType& eos_type,
                       int verbosity = 0)
     {
-
-        using InputEval = typename FluidState::Scalar;
-        using ComponentVector = Dune::FieldVector<typename FluidState::Scalar, numComponents>;
-
-        if (tolerance <= 0) {
-            tolerance = std::min<Scalar>(1e-3, 1e8 * std::numeric_limits<Scalar>::epsilon());
-        }
-
-        // K and L from previous timestep (wilson and -1 initially)
-        ComponentVector K;
-        for(int compIdx = 0; compIdx < numComponents; ++compIdx) {
-            K[compIdx] = fluid_state.K(compIdx);
-        }
-        InputEval L;
-        // TODO: L has all the derivatives to be all ZEROs here.
-        L = fluid_state.L();
-
-        // Print header
-        if (verbosity >= 1) {
-            std::cout << "********" << std::endl;
-            std::cout << "Inputs are K = [" << K << "], L = [" << L << "], z = [" << z << "], P = " << fluid_state.pressure(0) << ", and T = " << fluid_state.temperature(0) << std::endl;
-        }
-
-        using ScalarVector = Dune::FieldVector<Scalar, numComponents>;
-        Scalar L_scalar = Opm::getValue(L);
-        ScalarVector z_scalar;
-        ScalarVector K_scalar;
-        for (unsigned i = 0; i < numComponents; ++i) {
-            z_scalar[i] = Opm::getValue(z[i]);
-            K_scalar[i] = Opm::getValue(K[i]);
-        }
         using ScalarFluidState = CompositionalFluidState<Scalar, FluidSystem>;
         ScalarFluidState fluid_state_scalar;
 
         for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
-            fluid_state_scalar.setMoleFraction(oilPhaseIdx, compIdx, Opm::getValue(fluid_state.moleFraction(oilPhaseIdx, compIdx)));
-            fluid_state_scalar.setMoleFraction(gasPhaseIdx, compIdx, Opm::getValue(fluid_state.moleFraction(gasPhaseIdx, compIdx)));
-            fluid_state_scalar.setKvalue(compIdx, Opm::getValue(fluid_state.K(compIdx)));
+            fluid_state_scalar.setKvalue(compIdx, Opm::getValue(fluid_state.K(compIdx) ) );
+            fluid_state_scalar.setMoleFraction(compIdx, Opm::getValue(fluid_state.moleFraction(compIdx) ) );
         }
 
-        fluid_state_scalar.setLvalue(L_scalar);
+        fluid_state_scalar.setLvalue(Opm::getValue(fluid_state.L()));
         // other values need to be Scalar, but I guess the fluidstate does not support it yet.
         fluid_state_scalar.setPressure(FluidSystem::oilPhaseIdx,
                                        Opm::getValue(fluid_state.pressure(FluidSystem::oilPhaseIdx)));
@@ -134,36 +103,7 @@ public:
 
         fluid_state_scalar.setTemperature(Opm::getValue(fluid_state.temperature(0)));
 
-        // Do a stability test to check if cell is is_single_phase-phase (do for all cells the first time).
-        bool is_stable = false;
-        if ( L <= 0 || L == 1 ) {
-             if (verbosity >= 1) {
-                 std::cout << "Perform stability test (L <= 0 or L == 1)!" << std::endl;
-             }
-            phaseStabilityTest_(is_stable, K_scalar, fluid_state_scalar, z_scalar, verbosity);
-        }
-        if (verbosity >= 1) {
-            std::cout << "Inputs after stability test are K = [" << K_scalar << "], L = [" << L_scalar << "], z = [" << z_scalar << "], P = " << fluid_state.pressure(0) << ", and T = " << fluid_state.temperature(0) << std::endl;
-        }
-        // TODO: we do not need two variables is_stable and is_single_hase, while lacking a good name
-        // TODO: from the later code, is good if we knows whether single_phase_gas or single_phase_oil here
-        const bool is_single_phase = is_stable;
-
-        // Update the composition if cell is two-phase
-        if ( !is_single_phase ) {
-            // Rachford Rice equation to get initial L for composition solver
-            L_scalar = solveRachfordRice_g_(K_scalar, z_scalar, verbosity);
-            flash_2ph(z_scalar, twoPhaseMethod, K_scalar, L_scalar, fluid_state_scalar, verbosity);
-        } else {
-            // Cell is one-phase. Use Li's phase labeling method to see if it's liquid or vapor
-            L_scalar = li_single_phase_label_(fluid_state_scalar, z_scalar, verbosity);
-        }
-        fluid_state_scalar.setLvalue(L_scalar);
-
-        // Print footer
-        if (verbosity >= 1) {
-            std::cout << "********" << std::endl;
-        }
+        const auto is_single_phase = flash_solve_scalar_(fluid_state_scalar, twoPhaseMethod, flash_tolerance, eos_type, verbosity);
 
         // the flash solution process were performed in scalar form, after the flash calculation finishes,
         // ensure that things in fluid_state_scalar is transformed to fluid_state
@@ -174,14 +114,11 @@ public:
                 fluid_state.setMoleFraction(gasPhaseIdx, compIdx, y_i);
         }
 
-        for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
-            fluid_state.setKvalue(compIdx, K_scalar[compIdx]);
-            fluid_state_scalar.setKvalue(compIdx, K_scalar[compIdx]);
-        }
-        fluid_state.setLvalue(L_scalar);
         // we update the derivatives in fluid_state
-        updateDerivatives_(fluid_state_scalar, z, fluid_state, is_single_phase);
-    }//end solve
+        updateDerivatives_(fluid_state_scalar, fluid_state, eos_type, is_single_phase);
+
+        return is_single_phase;
+    } //end solve
 
     /*!
      * \brief Calculates the chemical equilibrium from the component
@@ -253,14 +190,12 @@ public:
                     }
 
                     // Run bisection
-                    auto Lmin = 1 - Vmax;
-                    auto Lmax = 1 - Vmin;
                     // TODO: This is required for some cases. Not clear why
                     // since the objective function should be monotone with a
                     // single zero between the Lmin/Lmax interval defined by
                     // K-values.
-                    Lmax = 1.0;
-                    Lmin = 0.0;
+                    decltype(Vmax) Lmin = 1.0;
+                    decltype(Vmin) Lmax = 0.0;
                     auto L = bisection_g_(K, Lmin, Lmax, z, verbosity);
 
                     // Print final result
@@ -291,6 +226,50 @@ public:
         throw std::runtime_error(" Rachford-Rice did not converge within maximum number of iterations" );
     }
 
+    // performing the flash calculation, which is done with Scalar without touching derivatives
+    template <typename FluidState>
+    static bool flash_solve_scalar_(FluidState& fluid_state,
+                                    const std::string& twoPhaseMethod,
+                                    const Scalar flash_tolerance,
+                                    const EOSType& eos_type,
+                                    const int verbosity = 0)
+    {
+        // Do a stability test to check if cell is is_single_phase-phase (do for all cells the first time).
+        bool is_stable = false;
+        auto L_scalar = fluid_state.L();
+        using ScalarVector = Dune::FieldVector<Scalar, numComponents>;
+        ScalarVector K_scalar, z_scalar;
+        for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
+            K_scalar[compIdx] = fluid_state.K(compIdx);
+            z_scalar[compIdx] = fluid_state.moleFraction(compIdx);
+        }
+
+        if ( L_scalar <= 0 || L_scalar == 1 ) {
+            if (verbosity >= 1) {
+                std::cout << "Perform stability test (L <= 0 or L == 1)!" << std::endl;
+            }
+            phaseStabilityTest_(is_stable, K_scalar, fluid_state, z_scalar, eos_type, verbosity);
+        }
+        if (verbosity >= 1) {
+            std::cout << "Inputs after stability test are K = [" << K_scalar << "], L = [" << L_scalar << "], z = [" << z_scalar << "], P = " << fluid_state.pressure(0) << ", and T = " << fluid_state.temperature(0) << std::endl;
+        }
+        // TODO: we do not need two variables is_stable and is_single_hase, while lacking a good name
+        // TODO: from the later code, is good if we knows whether single_phase_gas or single_phase_oil here
+        const bool is_single_phase = is_stable;
+
+        // Update the composition if cell is two-phase
+        if ( !is_single_phase ) {
+            // Rachford Rice equation to get initial L for composition solver
+            L_scalar = solveRachfordRice_g_(K_scalar, z_scalar, verbosity);
+            flash_2ph(z_scalar, twoPhaseMethod, K_scalar, L_scalar, fluid_state, flash_tolerance, eos_type, verbosity);
+        } else {
+            // Cell is one-phase. Use Li's phase labeling method to see if it's liquid or vapor
+            L_scalar = li_single_phase_label_(fluid_state, z_scalar, verbosity);
+        }
+        fluid_state.setLvalue(L_scalar);
+        return is_single_phase;
+    }
+
     template <class Vector>
     static typename Vector::field_type bisection_g_(const Vector& K, typename Vector::field_type Lmin,
                                                     typename Vector::field_type Lmax, const Vector& z, int verbosity)
@@ -394,7 +373,7 @@ public:
     }
 
     template <class FlashFluidState, class ComponentVector>
-    static void phaseStabilityTest_(bool& isStable, ComponentVector& K, FlashFluidState& fluid_state, const ComponentVector& z, int verbosity)
+    static void phaseStabilityTest_(bool& isStable, ComponentVector& K, FlashFluidState& fluid_state, const ComponentVector& z, const EOSType& eos_type, int verbosity)
     {
         // Declarations
         bool isTrivialL, isTrivialV;
@@ -407,14 +386,14 @@ public:
         if (verbosity == 3 || verbosity == 4) {
             std::cout << "Stability test for vapor phase:" << std::endl;
         }
-        checkStability_(fluid_state, isTrivialV, K0, y, S_v, z, /*isGas=*/true, verbosity);
+        checkStability_(fluid_state, isTrivialV, K0, y, S_v, z, /*isGas=*/true, eos_type, verbosity);
         bool V_unstable = (S_v < (1.0 + 1e-5)) || isTrivialV;
 
         // Check for liquids stable phase
         if (verbosity == 3 || verbosity == 4) {
             std::cout << "Stability test for liquid phase:" << std::endl;
         }
-        checkStability_(fluid_state, isTrivialL, K1, x, S_l, z, /*isGas=*/false, verbosity);
+        checkStability_(fluid_state, isTrivialL, K1, x, S_l, z, /*isGas=*/false, eos_type, verbosity);
         bool L_stable = (S_l < (1.0 + 1e-5)) || isTrivialL;
 
         // L-stable means success in making liquid, V-unstable means no success in making vapour
@@ -472,17 +451,18 @@ protected:
 
     template <class FlashFluidState, class ComponentVector>
     static void checkStability_(const FlashFluidState& fluid_state, bool& isTrivial, ComponentVector& K, ComponentVector& xy_loc,
-                                typename FlashFluidState::Scalar& S_loc, const ComponentVector& z, bool isGas, int verbosity)
+                                typename FlashFluidState::Scalar& S_loc, const ComponentVector& z, bool isGas, const EOSType& eos_type, 
+                                int verbosity)
     {
         using FlashEval = typename FlashFluidState::Scalar;
-        using PengRobinsonMixture = typename Opm::PengRobinsonMixture<Scalar, FluidSystem>;
+        using CubicEOS = typename Opm::CubicEOS<Scalar, FluidSystem>;
 
         // Declarations
         FlashFluidState fluid_state_fake = fluid_state;
         FlashFluidState fluid_state_global = fluid_state;
 
         // Setup output
-        if (verbosity >= 3 || verbosity >= 4) {
+        if (verbosity >= 3) {
             std::cout << std::setw(10) << "Iteration" << std::setw(16) << "K-Norm" << std::setw(16) << "R-Norm" << std::endl;
         }
 
@@ -518,16 +498,16 @@ protected:
                 fluid_state_global.setMoleFraction(phaseIdx2, compIdx, z[compIdx]);
             }
 
-            typename FluidSystem::template ParameterCache<FlashEval> paramCache_fake;
+            typename FluidSystem::template ParameterCache<FlashEval> paramCache_fake(eos_type);
             paramCache_fake.updatePhase(fluid_state_fake, phaseIdx);
 
-            typename FluidSystem::template ParameterCache<FlashEval> paramCache_global;
+            typename FluidSystem::template ParameterCache<FlashEval> paramCache_global(eos_type);
             paramCache_global.updatePhase(fluid_state_global, phaseIdx2);
 
             //fugacity for fake phases each component
             for (int compIdx=0; compIdx<numComponents; ++compIdx){
-                auto phiFake = PengRobinsonMixture::computeFugacityCoefficient(fluid_state_fake, paramCache_fake, phaseIdx, compIdx);
-                auto phiGlobal = PengRobinsonMixture::computeFugacityCoefficient(fluid_state_global, paramCache_global, phaseIdx2, compIdx);
+                auto phiFake = CubicEOS::computeFugacityCoefficient(fluid_state_fake, paramCache_fake, phaseIdx, compIdx);
+                auto phiGlobal = CubicEOS::computeFugacityCoefficient(fluid_state_global, paramCache_global, phaseIdx2, compIdx);
 
                 fluid_state_fake.setFugacityCoefficient(phaseIdx, compIdx, phiFake);
                 fluid_state_global.setFugacityCoefficient(phaseIdx2, compIdx, phiGlobal);
@@ -607,6 +587,8 @@ protected:
                           ComponentVector& K_scalar,
                           typename FluidState::Scalar& L_scalar,
                           FluidState& fluid_state_scalar,
+                          const Scalar flash_tolerance,
+                          const EOSType& eos_type,
                           int verbosity = 0) {
         if (verbosity >= 1) {
             std::cout << "Cell is two-phase! Solve Rachford-Rice with initial K = [" << K_scalar << "]" << std::endl;
@@ -614,28 +596,39 @@ protected:
 
         // Calculate composition using nonlinear solver
         // Newton
+        bool converged = false;
         if (flash_2p_method == "newton") {
             if (verbosity >= 1) {
                 std::cout << "Calculate composition using Newton." << std::endl;
             }
-            newtonComposition_(K_scalar, L_scalar, fluid_state_scalar, z_scalar, verbosity);
+            converged = newtonComposition_(K_scalar, L_scalar, fluid_state_scalar, z_scalar, flash_tolerance, eos_type, verbosity);
         } else if (flash_2p_method == "ssi") {
             // Successive substitution
             if (verbosity >= 1) {
                 std::cout << "Calculate composition using Succcessive Substitution." << std::endl;
             }
-            successiveSubstitutionComposition_(K_scalar, L_scalar, fluid_state_scalar, z_scalar, false, verbosity);
+            converged = successiveSubstitutionComposition_(K_scalar, L_scalar, fluid_state_scalar, z_scalar, false, flash_tolerance, eos_type, verbosity);
         } else if (flash_2p_method == "ssi+newton") {
-            successiveSubstitutionComposition_(K_scalar, L_scalar, fluid_state_scalar, z_scalar, true, verbosity);
-            newtonComposition_(K_scalar, L_scalar, fluid_state_scalar, z_scalar, verbosity);
+            converged = successiveSubstitutionComposition_(K_scalar, L_scalar, fluid_state_scalar, z_scalar, true, flash_tolerance, eos_type, verbosity);
+            if (!converged) {
+                converged = newtonComposition_(K_scalar, L_scalar, fluid_state_scalar, z_scalar, flash_tolerance, eos_type, verbosity);
+            }
         } else {
-            throw std::runtime_error("unknown two phase flash method " + flash_2p_method + " is specified");
+            throw std::logic_error("unknown two phase flash method " + flash_2p_method + " is specified");
+        }
+
+        if (!converged) {
+            throw std::runtime_error("flash calculation did not get converged with " + flash_2p_method);
         }
     }
 
     template <class FlashFluidState, class ComponentVector>
-    static void newtonComposition_(ComponentVector& K, typename FlashFluidState::Scalar& L,
-                                   FlashFluidState& fluid_state, const ComponentVector& z,
+    static bool newtonComposition_(ComponentVector& K,
+                                   typename FlashFluidState::Scalar& L,
+                                   FlashFluidState& fluid_state,
+                                   const ComponentVector& z,
+                                   const Scalar tolerance,
+                                   const EOSType& eos_type,
                                    int verbosity)
     {
         // Note: due to the need for inverse flash update for derivatives, the following two can be different
@@ -644,7 +637,6 @@ protected:
         constexpr size_t num_primary_variables = numMisciblePhases * numMiscibleComponents + 1;
         using NewtonVector = Dune::FieldVector<Scalar, num_equations>;
         using NewtonMatrix = Dune::FieldMatrix<Scalar, num_equations, num_primary_variables>;
-        constexpr Scalar tolerance = 1.e-8;
 
         NewtonVector soln(0.);
         NewtonVector res(0.);
@@ -652,12 +644,10 @@ protected:
 
         // Compute x and y from K, L and Z
         computeLiquidVapor_(fluid_state, L, K, z);
-        if (verbosity >= 1) {
-            std::cout << " the current L is " << Opm::getValue(L) << std::endl;
-        }
 
         // Print initial condition
         if (verbosity >= 1) {
+            std::cout << " the current L is " << Opm::getValue(L) << std::endl;
             std::cout << "Initial guess: x = [";
             for (int compIdx=0; compIdx<numComponents; ++compIdx){
                 if (compIdx < numComponents - 1)
@@ -717,9 +707,9 @@ protected:
         flash_fluid_state.setTemperature(fluid_state.temperature(0));
 
         using ParamCache = typename FluidSystem::template ParameterCache<typename CompositionalFluidState<Eval, FluidSystem>::Scalar>;
-        ParamCache paramCache;
+        ParamCache paramCache(eos_type);
 
-        for (unsigned phaseIdx = 0; phaseIdx < numPhases; ++phaseIdx) {
+        for (unsigned phaseIdx = 0; phaseIdx < numMisciblePhases; ++phaseIdx) {
             paramCache.updatePhase(flash_fluid_state, phaseIdx);
             for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
                 // TODO: will phi here carry the correct derivatives?
@@ -758,13 +748,14 @@ protected:
             }
             flash_fluid_state.setLvalue(l);
 
-            for (unsigned phaseIdx = 0; phaseIdx < numPhases; ++phaseIdx) {
+            for (unsigned phaseIdx = 0; phaseIdx < numMisciblePhases; ++phaseIdx) {
                 paramCache.updatePhase(flash_fluid_state, phaseIdx);
                 for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
                     Eval phi = FluidSystem::fugacityCoefficient(flash_fluid_state, paramCache, phaseIdx, compIdx);
                     flash_fluid_state.setFugacityCoefficient(phaseIdx, compIdx, phi);
                 }
             }
+            ++iter;
         }
         if (verbosity >= 1) {
             for (unsigned i = 0; i < num_equations; ++i) {
@@ -792,6 +783,7 @@ protected:
         }
         L = Opm::getValue(l);
         fluid_state.setLvalue(L);
+        return converged;
     }
 
     // TODO: the interface will need to refactor for later usage
@@ -845,77 +837,31 @@ protected:
         }
     }
 
-     // TODO: the interface will need to refactor for later usage
-    template<typename FlashFluidState, typename ComponentVector, size_t num_primary, size_t num_equation >
-    static void assembleNewtonSingle_(const FlashFluidState& fluid_state,
-                                const ComponentVector& global_composition,
-                                Dune::FieldMatrix<double, num_equation, num_primary>& jac,
-                                Dune::FieldVector<double, num_equation>& res)
-    {
-        using Eval = DenseAd::Evaluation<double, num_primary>;
-        std::vector<Eval> x(numComponents), y(numComponents);
-        for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
-            x[compIdx] = fluid_state.moleFraction(oilPhaseIdx, compIdx);
-            y[compIdx] = fluid_state.moleFraction(gasPhaseIdx, compIdx);
-        }
-        const Eval& l = fluid_state.L();
-
-        // TODO: clearing zero whether necessary?
-        jac = 0.;
-        res = 0.;
-        for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
-            {
-                // z - L*x - (1-L) * y  ---> z - x;
-                auto local_res = -global_composition[compIdx] + x[compIdx];
-                res[compIdx] = Opm::getValue(local_res);
-                for (unsigned i = 0; i < num_primary; ++i) {
-                    jac[compIdx][i] = local_res.derivative(i);
-                }
-            }
-
-            {
-                // f_liquid - f_vapor = 0  -->z - y;
-                auto local_res = -global_composition[compIdx] + y[compIdx];
-                res[compIdx + numComponents] = Opm::getValue(local_res);
-                for (unsigned i = 0; i < num_primary; ++i) {
-                    jac[compIdx + numComponents][i] = local_res.derivative(i);
-                }
-            }
-        }
-
-        // TODO: better we have isGas or isLiquid here
-        const bool isGas = Opm::abs(l - 1.0) > std::numeric_limits<double>::epsilon();
-
-        // sum(x) - sum(y) = 0
-        auto local_res = l;
-        if(isGas) {
-            local_res = l-1;
-        }
-
-        res[num_equation - 1] = Opm::getValue(local_res);
-        for (unsigned i = 0; i < num_primary; ++i) {
-            jac[num_equation - 1][i] = local_res.derivative(i);
-        }
-    }
-
-        template <typename FlashFluidStateScalar, typename FluidState, typename ComponentVector>
+    template <typename FlashFluidStateScalar, typename FluidState>
     static void updateDerivatives_(const FlashFluidStateScalar& fluid_state_scalar,
-                                   const ComponentVector& z,
                                    FluidState& fluid_state,
+                                   const EOSType& eos_type,
                                    bool is_single_phase)
     {
         if(!is_single_phase)
-            updateDerivativesTwoPhase_(fluid_state_scalar, z, fluid_state);
+            updateDerivativesTwoPhase_(fluid_state_scalar, fluid_state, eos_type);
         else
-            updateDerivativesSinglePhase_(fluid_state_scalar, z, fluid_state);
+            updateDerivativesSinglePhase_(fluid_state_scalar, fluid_state);
 
     }
 
-    template <typename FlashFluidStateScalar, typename FluidState, typename ComponentVector>
+    template <typename FlashFluidStateScalar, typename FluidState>
     static void updateDerivativesTwoPhase_(const FlashFluidStateScalar& fluid_state_scalar,
-                                   const ComponentVector& z,
-                                   FluidState& fluid_state)
+                                           FluidState& fluid_state,
+                                           const EOSType& eos_type)
     {
+        using InputEval = typename FluidState::Scalar;
+        using ComponentVector = Dune::FieldVector<InputEval, numComponents>;
+        ComponentVector z;
+        for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
+            z[compIdx] = fluid_state.moleFraction(compIdx);
+        }
+
         // getting the secondary Jocobian matrix
         constexpr size_t num_equations = numMisciblePhases * numMiscibleComponents + 1;
         constexpr size_t secondary_num_pv = numComponents + 1; // pressure, z for all the components
@@ -954,8 +900,8 @@ protected:
         // compositions
         // TODO: we probably can simplify SecondaryFlashFluidState::Scalar
         using SecondaryParamCache = typename FluidSystem::template ParameterCache<typename SecondaryFlashFluidState::Scalar>;
-        SecondaryParamCache secondary_param_cache;
-        for (unsigned phase_idx = 0; phase_idx < numPhases; ++phase_idx) {
+        SecondaryParamCache secondary_param_cache(eos_type);
+        for (unsigned phase_idx = 0; phase_idx < numMisciblePhases; ++phase_idx) {
             secondary_param_cache.updatePhase(secondary_fluid_state, phase_idx);
             for (unsigned comp_idx = 0; comp_idx < numComponents; ++comp_idx) {
                 SecondaryEval phi = FluidSystem::fugacityCoefficient(secondary_fluid_state, secondary_param_cache, phase_idx, comp_idx);
@@ -1004,8 +950,8 @@ protected:
 
         // TODO: is PrimaryFlashFluidState::Scalar> PrimaryEval here?
         using PrimaryParamCache = typename FluidSystem::template ParameterCache<typename PrimaryFlashFluidState::Scalar>;
-        PrimaryParamCache primary_param_cache;
-        for (unsigned phase_idx = 0; phase_idx < numPhases; ++phase_idx) {
+        PrimaryParamCache primary_param_cache(eos_type);
+        for (unsigned phase_idx = 0; phase_idx < numMisciblePhases; ++phase_idx) {
             primary_param_cache.updatePhase(primary_fluid_state, phase_idx);
             for (unsigned comp_idx = 0; comp_idx < numComponents; ++comp_idx) {
                 PrimaryEval phi = FluidSystem::fugacityCoefficient(primary_fluid_state, primary_param_cache, phase_idx, comp_idx);
@@ -1026,12 +972,9 @@ protected:
         // SecondaryNewtonMatrix xx;
         // pri_jac.solve(xx, sec_jac);
         pri_jac.invert();
-        sec_jac.template leftmultiply(pri_jac);
+        sec_jac.template leftmultiply<PrimaryNewtonMatrix>(pri_jac);
 
-        using InputEval = typename FluidState::Scalar;
-        using ComponentVectorMoleFraction = Dune::FieldVector<InputEval, numComponents>;
-
-        ComponentVectorMoleFraction x(numComponents), y(numComponents);
+        ComponentVector x(numComponents), y(numComponents);
         InputEval L_eval = L;
 
         // use the chainrule (and using partial instead of total
@@ -1041,10 +984,8 @@ protected:
 
         const auto p_l = fluid_state.pressure(FluidSystem::oilPhaseIdx);
         const auto p_v = fluid_state.pressure(FluidSystem::gasPhaseIdx);
-        std::vector<double> K(numComponents);
 
         for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
-            K[compIdx] = fluid_state_scalar.K(compIdx);
             x[compIdx] = fluid_state_scalar.moleFraction(FluidSystem::oilPhaseIdx,compIdx);//;z[compIdx] * 1. / (L + (1 - L) * K[compIdx]);
             y[compIdx] = fluid_state_scalar.moleFraction(FluidSystem::gasPhaseIdx,compIdx);//;x[compIdx] * K[compIdx];
         }
@@ -1053,7 +994,7 @@ protected:
         // p_l and p_v are the same here, in the future, there might be slightly more complicated scenarios when capillary
         // pressure joins
 
-        constexpr size_t num_deri = numComponents;
+        constexpr size_t num_deri = InputEval::numVars;
         for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
             std::vector<double> deri(num_deri, 0.);
             // derivatives from P
@@ -1112,9 +1053,8 @@ protected:
         fluid_state.setLvalue(L_eval);
     } //end updateDerivativesTwoPhase
 
-    template <typename FlashFluidStateScalar, typename FluidState, typename ComponentVector>
+    template <typename FlashFluidStateScalar, typename FluidState>
     static void updateDerivativesSinglePhase_(const FlashFluidStateScalar& fluid_state_scalar,
-                                              const ComponentVector& z,
                                               FluidState& fluid_state)
     {
         using InputEval = typename FluidState::Scalar;
@@ -1124,8 +1064,8 @@ protected:
         // for single phase situation, x = y = z;
         // and L_eval have all zero derivatives
         for (unsigned compIdx = 0; compIdx < numComponents; ++compIdx) {
-            fluid_state.setMoleFraction(FluidSystem::oilPhaseIdx, compIdx, z[compIdx]);
-            fluid_state.setMoleFraction(FluidSystem::gasPhaseIdx, compIdx, z[compIdx]);
+            fluid_state.setMoleFraction(FluidSystem::oilPhaseIdx, compIdx, fluid_state.moleFraction(compIdx) );
+            fluid_state.setMoleFraction(FluidSystem::gasPhaseIdx, compIdx, fluid_state.moleFraction(compIdx) );
         }
         fluid_state.setLvalue(L_eval);
     } //end updateDerivativesSinglePhase
@@ -1133,11 +1073,17 @@ protected:
 
     // TODO: or use typename FlashFluidState::Scalar
     template <class FlashFluidState, class ComponentVector>
-    static void successiveSubstitutionComposition_(ComponentVector& K, typename ComponentVector::field_type& L, FlashFluidState& fluid_state, const ComponentVector& z,
-                                                   const bool newton_afterwards, const int verbosity)
+    static bool successiveSubstitutionComposition_(ComponentVector& K,
+                                                   typename ComponentVector::field_type& L,
+                                                   FlashFluidState& fluid_state,
+                                                   const ComponentVector& z,
+                                                   const bool newton_afterwards,
+                                                   const Scalar flash_tolerance,
+                                                   const EOSType& eos_type,
+                                                   const int verbosity)
     {
         // Determine max. iterations based on if it will be used as a standalone flash or as a pre-process to Newton (or other) method.
-        const int maxIterations = newton_afterwards ? 3 : 100;
+        const int maxIterations = newton_afterwards ? 5 : 100;
 
         // Store cout format before manipulation
         std::ios_base::fmtflags f(std::cout.flags());
@@ -1161,8 +1107,8 @@ protected:
 
             // Calculate fugacity coefficient
             using ParamCache = typename FluidSystem::template ParameterCache<typename FlashFluidState::Scalar>;
-            ParamCache paramCache;
-            for (int phaseIdx=0; phaseIdx<numPhases; ++phaseIdx){
+            ParamCache paramCache(eos_type);
+            for (int phaseIdx=0; phaseIdx<numMisciblePhases; ++phaseIdx){
                 paramCache.updatePhase(fluid_state, phaseIdx);
                 for (int compIdx=0; compIdx<numComponents; ++compIdx){
                     auto phi = FluidSystem::fugacityCoefficient(fluid_state, paramCache, phaseIdx, compIdx);
@@ -1194,7 +1140,7 @@ protected:
             }
 
             // Check convergence
-            if (convFugRatio.two_norm() < 1e-6) {
+            if (convFugRatio.two_norm() < flash_tolerance) {
                 // Restore cout format
                 std::cout.flags(f);
 
@@ -1202,7 +1148,7 @@ protected:
                 if (verbosity >= 1) {
                     std::cout << "Solution converged to the following result :" << std::endl;
                     std::cout << "x = [";
-                    for (int compIdx=0; compIdx<numComponents; ++compIdx){
+                    for (int compIdx = 0; compIdx < numComponents; ++compIdx) {
                         if (compIdx < numComponents - 1)
                             std::cout << fluid_state.moleFraction(oilPhaseIdx, compIdx) << " ";
                         else
@@ -1210,7 +1156,7 @@ protected:
                     }
                     std::cout << "]" << std::endl;
                     std::cout << "y = [";
-                    for (int compIdx=0; compIdx<numComponents; ++compIdx){
+                    for (int compIdx = 0; compIdx < numComponents; ++compIdx) {
                         if (compIdx < numComponents - 1)
                             std::cout << fluid_state.moleFraction(gasPhaseIdx, compIdx) << " ";
                         else
@@ -1221,11 +1167,10 @@ protected:
                     std::cout << "L = " << L << std::endl;
                 }
                 // Restore cout format format
-                return;
+                return true;
             }
-
             //  If convergence is not met, K is updated in a successive substitution manner
-            else{
+            else {
                 // Update K
                 for (int compIdx=0; compIdx<numComponents; ++compIdx){
                     K[compIdx] *= newFugRatio[compIdx];
@@ -1234,12 +1179,18 @@ protected:
                 // Solve Rachford-Rice to get L from updated K
                 L = solveRachfordRice_g_(K, z, 0);
             }
+        }
+        // did not get converged. check whether we will do more newton later afterward
+        {
+           const std::string msg = fmt::format("Successive substitution composition update did not converge within maxIterations {}.", maxIterations);
+           if (!newton_afterwards) {
+               throw std::runtime_error(msg);
+           } else if (verbosity > 0) {
+               std::cout << msg << std::endl;
+           }
+        }
 
-        }
-        if (!newton_afterwards) {
-            throw std::runtime_error(
-                    "Successive substitution composition update did not converge within maxIterations " + std::to_string(maxIterations));
-        }
+        return false;
     }
 
 };//end PTFlash
