@@ -27,10 +27,11 @@
 #ifndef OPM_ECL_DEFAULT_MATERIAL_HPP
 #define OPM_ECL_DEFAULT_MATERIAL_HPP
 
-#include "EclDefaultMaterialParams.hpp"
+#include <opm/common/TimingMacros.hpp>
 
 #include <opm/material/common/MathToolbox.hpp>
 #include <opm/material/common/Valgrind.hpp>
+#include <opm/material/fluidmatrixinteractions/EclDefaultMaterialParams.hpp>
 
 #include <algorithm>
 #include <stdexcept>
@@ -150,76 +151,97 @@ public:
 
     /*
      * Hysteresis parameters for oil-water
-     * @see EclHysteresisTwoPhaseLawParams::pcSwMdc(...)
-     * @see EclHysteresisTwoPhaseLawParams::krnSwMdc(...)
+     * @see EclHysteresisTwoPhaseLawParams::soMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::swMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::swMin(...)
      * \param params Parameters
      */
-    static void oilWaterHysteresisParams(Scalar& pcSwMdc,
-                                         Scalar& krnSwMdc,
+    static void oilWaterHysteresisParams(Scalar& soMax,
+                                         Scalar& swMax,
+                                         Scalar& swMin,
                                          const Params& params)
     {
-        pcSwMdc = params.oilWaterParams().pcSwMdc();
-        krnSwMdc = params.oilWaterParams().krnSwMdc();
-
-        Valgrind::CheckDefined(pcSwMdc);
-        Valgrind::CheckDefined(krnSwMdc);
+        soMax = 1.0 - params.oilWaterParams().krnSwMdc();
+        swMax = params.oilWaterParams().krwSwMdc();
+        swMin = params.oilWaterParams().pcSwMdc();
+        Valgrind::CheckDefined(soMax);
+        Valgrind::CheckDefined(swMax);
+        Valgrind::CheckDefined(swMin);
     }
 
     /*
      * Hysteresis parameters for oil-water
-     * @see EclHysteresisTwoPhaseLawParams::pcSwMdc(...)
-     * @see EclHysteresisTwoPhaseLawParams::krnSwMdc(...)
+     * @see EclHysteresisTwoPhaseLawParams::soMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::swMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::swMin(...)
      * \param params Parameters
      */
-    static void setOilWaterHysteresisParams(const Scalar& pcSwMdc,
-                                            const Scalar& krnSwMdc,
+    static void setOilWaterHysteresisParams(const Scalar& soMax,
+                                            const Scalar& swMax,
+                                            const Scalar& swMin,
                                             Params& params)
     {
-        constexpr const double krwSw = 2.0; //Should not be used
-        params.oilWaterParams().update(pcSwMdc, krwSw, krnSwMdc);
+        params.oilWaterParams().update(swMin, swMax, 1.0 - soMax);
     }
 
     /*
      * Hysteresis parameters for gas-oil
-     * @see EclHysteresisTwoPhaseLawParams::pcSwMdc(...)
-     * @see EclHysteresisTwoPhaseLawParams::krnSwMdc(...)
+     * @see EclHysteresisTwoPhaseLawParams::sgMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::shMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::soMin(...)
      * \param params Parameters
      */
-    static void gasOilHysteresisParams(Scalar& pcSwMdc,
-                                       Scalar& krnSwMdc,
+    static void gasOilHysteresisParams(Scalar& sgMax,
+                                       Scalar& shMax,
+                                       Scalar& soMin,
                                        const Params& params)
     {
         const auto Swco = params.Swl();
+        sgMax = 1.0 - params.gasOilParams().krnSwMdc() - Swco;
+        shMax = params.gasOilParams().krwSwMdc();
+        soMin = params.gasOilParams().pcSwMdc();
 
-        // Pretend oil saturation is 'Swco' larger than it really is in
-        // order to infer correct maximum Sg values in output layer.
-        pcSwMdc = std::min(params.gasOilParams().pcSwMdc() + Swco, Scalar{2.0});
-        krnSwMdc = std::min(params.gasOilParams().krnSwMdc() + Swco, Scalar{2.0});
-
-        Valgrind::CheckDefined(pcSwMdc);
-        Valgrind::CheckDefined(krnSwMdc);
+        Valgrind::CheckDefined(sgMax);
+        Valgrind::CheckDefined(shMax);
+        Valgrind::CheckDefined(soMin);
     }
 
-
-    static Scalar trappedGasSaturation(const Params& params)
-    {
-
-        return params.gasOilParams().SnTrapped();
-    }
     /*
      * Hysteresis parameters for gas-oil
-     * @see EclHysteresisTwoPhaseLawParams::pcSwMdc(...)
-     * @see EclHysteresisTwoPhaseLawParams::krnSwMdc(...)
+     * @see EclHysteresisTwoPhaseLawParams::sgMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::shMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::soMin(...)
      * \param params Parameters
      */
-    static void setGasOilHysteresisParams(const Scalar& pcSwMdc,
-                                          const Scalar& krnSwMdc,
+    static void setGasOilHysteresisParams(const Scalar& sgMax,
+                                          const Scalar& shMax,
+                                          const Scalar& soMin,
                                           Params& params)
     {
-        // Maximum attainable oil saturation is 1-SWL
         const auto Swco = params.Swl();
-        constexpr const double krwSw = 2.0; //Should not be used
-        params.gasOilParams().update(pcSwMdc - Swco, krwSw, krnSwMdc - Swco);
+        params.gasOilParams().update(soMin, shMax, 1.0 - sgMax - Swco);
+    }
+
+    static Scalar trappedGasSaturation(const Params& params, bool maximumTrapping)
+    {
+        const auto Swco = params.Swl();
+        return params.gasOilParams().SnTrapped(maximumTrapping) - Swco;
+    }
+
+    static Scalar trappedOilSaturation(const Params& params, bool maximumTrapping)
+    {
+        return params.oilWaterParams().SnTrapped(maximumTrapping) + params.gasOilParams().SwTrapped();
+    }
+
+    static Scalar trappedWaterSaturation(const Params& params)
+    {
+        return params.oilWaterParams().SwTrapped();
+    }
+
+    static Scalar strandedGasSaturation(const Params& params, Scalar Sg, Scalar Kg)
+    {
+        const auto Swco = params.Swl();
+        return params.gasOilParams().SnStranded(Sg, Kg) - Swco;
     }
 
     /*!
@@ -337,8 +359,8 @@ public:
     {
         OPM_TIMEFUNCTION_LOCAL();
         // Maximum attainable oil saturation is 1-SWL.
-        const Evaluation Sw = 1.0 - params.Swl() - decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
-        return GasOilMaterialLaw::twoPhaseSatKrn(params.gasOilParams(), Sw);
+        const Evaluation sw = 1.0 - params.Swl() - decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
+        return GasOilMaterialLaw::twoPhaseSatKrn(params.gasOilParams(), sw);
     }
 
     /*!
@@ -349,8 +371,8 @@ public:
                           const FluidState& fluidState)
     {
         OPM_TIMEFUNCTION_LOCAL();
-        const Evaluation Sw = decay<Evaluation>(fluidState.saturation(waterPhaseIdx));
-        return OilWaterMaterialLaw::twoPhaseSatKrw(params.oilWaterParams(), Sw);
+        const Evaluation sw = decay<Evaluation>(fluidState.saturation(waterPhaseIdx));
+        return OilWaterMaterialLaw::twoPhaseSatKrw(params.oilWaterParams(), sw);
     }
 
     /*!
@@ -363,13 +385,13 @@ public:
         OPM_TIMEFUNCTION_LOCAL();
         const Scalar Swco = params.Swl();
 
-        const Evaluation Sw =
+        const Evaluation sw =
             max(Evaluation(Swco),
                      decay<Evaluation>(fluidState.saturation(waterPhaseIdx)));
 
-        const Evaluation Sg = decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
+        const Evaluation sg = decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
 
-        const Evaluation Sw_ow = Sg + Sw;
+        const Evaluation Sw_ow = sg + sw;
         const Evaluation kro_ow = relpermOilInOilWaterSystem<Evaluation>(params, fluidState);
         const Evaluation kro_go = relpermOilInOilGasSystem<Evaluation>(params, fluidState);
 
@@ -380,16 +402,16 @@ public:
         if (scalarValue(Sw_ow) - Swco < epsilon) {
             const Evaluation kro2 = (kro_ow + kro_go)/2;
             if (scalarValue(Sw_ow) - Swco > epsilon/2) {
-                const Evaluation kro1 = (Sg*kro_go + (Sw - Swco)*kro_ow)/(Sw_ow - Swco);
-                const Evaluation alpha = (epsilon - (Sw_ow - Swco))/(epsilon/2);
+                const Evaluation kro1 = (sg * kro_go + (sw - Swco) * kro_ow) / (Sw_ow - Swco);
+                const Evaluation alpha = (epsilon - (Sw_ow - Swco)) / (epsilon / 2);
 
-                return kro2*alpha + kro1*(1 - alpha);
+                return kro2 * alpha + kro1 * (1 - alpha);
             }
 
             return kro2;
         }
 
-        return (Sg*kro_go + (Sw - Swco)*kro_ow) / (Sw_ow - Swco);
+        return (sg * kro_go + (sw - Swco) * kro_ow) / (Sw_ow - Swco);
     }
 
     /*!
@@ -400,12 +422,12 @@ public:
                                                const FluidState& fluidState)
     {
         OPM_TIMEFUNCTION_LOCAL();
-        const Evaluation Sw =
+        const Evaluation sw =
             max(Evaluation{ params.Swl() },
                      decay<Evaluation>(fluidState.saturation(waterPhaseIdx)));
 
-        const Evaluation Sg = decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
-        const Evaluation So_go = 1.0 - (Sg + Sw);
+        const Evaluation sg = decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
+        const Evaluation So_go = 1.0 - (sg + sw);
 
         return GasOilMaterialLaw::twoPhaseSatKrw(params.gasOilParams(), So_go);
     }
@@ -418,12 +440,12 @@ public:
                                                  const FluidState& fluidState)
     {
         OPM_TIMEFUNCTION_LOCAL();
-        const Evaluation Sw =
+        const Evaluation sw =
             max(Evaluation{ params.Swl() },
                      decay<Evaluation>(fluidState.saturation(waterPhaseIdx)));
 
-        const Evaluation Sg = decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
-        const Evaluation Sw_ow = Sg + Sw;
+        const Evaluation sg = decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
+        const Evaluation Sw_ow = sg + sw;
 
         return OilWaterMaterialLaw::twoPhaseSatKrn(params.oilWaterParams(), Sw_ow);
     }
@@ -438,49 +460,15 @@ public:
     template <class FluidState>
     static bool updateHysteresis(Params& params, const FluidState& fluidState)
     {
-        OPM_TIMEFUNCTION_LOCAL();
-        bool changed = false;
         const Scalar Swco = params.Swl();
-
-        const Scalar Sw = clampSaturation(fluidState, waterPhaseIdx);
+        const Scalar sw = clampSaturation(fluidState, waterPhaseIdx);
         const Scalar So = clampSaturation(fluidState, oilPhaseIdx);
-        const Scalar Sg = clampSaturation(fluidState, gasPhaseIdx);
-
-        if (params.inconsistentHysteresisUpdate()) {
-            // NOTE: the saturations which are passed to update the hysteresis curves are
-            // inconsistent with the ones used to calculate the relative permabilities. We do
-            // it like this anyway because (a) the saturation functions of opm-core do it
-            // this way (b) the simulations seem to converge better (which is not too much
-            // surprising actually, because the time step does not start on a kink in the
-            // solution) and (c) the Eclipse 100 simulator may do the same.
-            //
-            // Though be aware that from a physical perspective this is definitively
-            // incorrect!
-            bool oilchanged = params.oilWaterParams().update(/*pcSw=*/  Sw, //1.0 - So, (Effect is significant vs benchmark.)
-                                           /*krwSw=*/ 1.0 - So,
-                                           /*krnSw=*/ 1.0 - So);
-
-            changed = changed || oilchanged;
-
-            bool gaschanged = params.gasOilParams().update(/*pcSw=*/  1.0 - Swco - Sg,
-                                         /*krwSw=*/ 1.0 - std::max(Swco, Sw) - Sg, //(Three phase behavior ...)
-                                         /*krnSw=*/ 1.0 - Swco - Sg);
-            changed = changed || gaschanged;
-        }
-        else {
-            const Scalar Sw_ow = Sg + std::max(Swco, Sw);
-            const Scalar So_go = 1.0 - Sw_ow;
-            bool oilchanged = params.oilWaterParams().update(/*pcSw=*/  Sw,
-                                                             /*krwSw=*/ 1 - Sg,
-                                                             /*krnSw=*/ Sw_ow);
-            changed = changed || oilchanged;
-            bool gaschanged = params.gasOilParams().update(/*pcSw=*/  1.0 - Swco - Sg,
-                                                              /*krwSw=*/ So_go,
-                                                              /*krnSw=*/ 1.0 - Swco - Sg);
-
-            changed = changed || gaschanged;
-        }
-        return changed;
+        const Scalar sg = clampSaturation(fluidState, gasPhaseIdx);
+        bool owChanged = params.oilWaterParams().update(/*pcSw=*/sw, /*krwSw=*/sw, /*krnSw=*/1 - So);
+        bool gochanged = params.gasOilParams().update(/*pcSw=*/  So,
+                                                          /*krwSw=*/ So,
+                                                          /*krnSw=*/ 1.0 - Swco - sg);
+        return owChanged || gochanged;
     }
 
     template <class FluidState>
@@ -491,6 +479,7 @@ public:
         return std::clamp(sat, Scalar{0.0}, Scalar{1.0});
     }
 };
+
 } // namespace Opm
 
 #endif
