@@ -54,36 +54,6 @@ class EclHysteresisTwoPhaseLawParams : public EnsureFinalized
 public:
     using Traits = typename EffLawParams::Traits;
 
-    EclHysteresisTwoPhaseLawParams()
-    {
-        // These are initialized to two (even though they represent saturations)
-        // to signify that the values are larger than physically possible, and force
-        // using the drainage curve before the first saturation update.
-        pcSwMdc_ = 2.0;
-        krnSwMdc_ = 2.0;
-        krwSwMdc_ = -2.0;
-        krnSwDrainRevert_ = 2.0;
-        krnSwDrainStart_ = -2.0;
-        krnSwWAG_ = 2.0;
-
-        pcSwMic_ = -1.0;
-        initialImb_ = false;
-        oilWaterSystem_ = false;
-        gasOilSystem_ = false;
-        pcmaxd_ = 0.0;
-        pcmaxi_ = 0.0;
-
-        deltaSwImbKrn_ = 0.0;
-        //deltaSwImbKrw_ = 0.0;
-
-        Swco_ = 0.0;
-        swatImbStart_ = 0.0;
-        isDrain_ = true;
-        cTransf_ = 0.0;
-        tolWAG_ = 0.001;
-        nState_ = 0;
-    }
-
     static EclHysteresisTwoPhaseLawParams serializationTestObject()
     {
         EclHysteresisTwoPhaseLawParams<EffLawT> result;
@@ -114,10 +84,10 @@ public:
             }
             if (config().krHysteresisModel() == 4) {
                 Cw_ = 1.0/(Swcri_ - Swcrd_ + 1.0e-12) - 1.0/(Swmaxd_ - Swcrd_);
+                Krwd_sncri_ = EffLawT::twoPhaseSatKrw(drainageParams(), 1 - Sncri());
             }
             updateDynamicParams_();
         }
-
         EnsureFinalized :: finalize();
     }
 
@@ -165,9 +135,9 @@ public:
         if (config().krHysteresisModel() == 2 || config().krHysteresisModel() == 3 || config().krHysteresisModel() == 4 || config().pcHysteresisModel() == 0 || gasOilHysteresisWAG()) {
             Swco_ = info.Swl;
             if (twoPhaseSystem == EclTwoPhaseSystemType::GasOil) {
-                Sncrd_ = info.Sgcr+info.Swl;
-                Swcrd_ = info.Sogcr + info.Swl;
-                Snmaxd_ = info.Sgu+info.Swl;
+                Sncrd_ = info.Sgcr + info.Swl;
+                Swcrd_ = info.Sogcr;
+                Snmaxd_ = info.Sgu + info.Swl;
                 KrndMax_ = EffLawT::twoPhaseSatKrn(drainageParams(), 1.0-Snmaxd_);
             }
             else if (twoPhaseSystem == EclTwoPhaseSystemType::GasWater) {
@@ -257,7 +227,7 @@ public:
 
         // Store critical nonwetting saturation
         if (twoPhaseSystem == EclTwoPhaseSystemType::GasOil) {
-            Sncri_ = info.Sgcr+info.Swl;
+            Sncri_ = info.Sgcr + info.Swl;
             Swcri_ = info.Sogcr;
         }
         else if (twoPhaseSystem == EclTwoPhaseSystemType::GasWater) {
@@ -270,7 +240,6 @@ public:
             Swcri_ = info.Swcr;
         }
 
-
         // Killough hysteresis model for pc
         if (config().pcHysteresisModel() == 0) {
             if (twoPhaseSystem == EclTwoPhaseSystemType::GasOil) {
@@ -286,6 +255,11 @@ public:
                 pcmaxi_ = info.maxPcow;
             }
         }
+
+        if (config().krHysteresisModel() == 4) {
+            Krwi_snmax_ = EffLawT::twoPhaseSatKrw(imbibitionParams(), 1 - Snmaxd());
+            Krwi_snrmax_ = EffLawT::twoPhaseSatKrw(imbibitionParams(), 1 - Sncri());
+        }
     }
 
     /*!
@@ -407,10 +381,9 @@ public:
     Scalar Swcrt() const
     { return Swcrt_; }
 
-    Scalar SnTrapped() const
+    Scalar SnTrapped(bool maximumTrapping) const
     {
-
-        if(isDrain_)
+        if(!maximumTrapping && isDrain_)
             return 0.0;
 
         // For Killough the trapped saturation is already computed
@@ -420,6 +393,11 @@ public:
             return Sncri_ + deltaSwImbKrn_;
     }
 
+    Scalar SnStranded(Scalar sg, Scalar krg) const {
+        const Scalar sn = EffLawT::twoPhaseSatKrnInv(drainageParams_, krg);
+        return sg - (1.0 - sn) + Sncrd_;
+    }
+
     Scalar SwTrapped() const
     {
         if( config().krHysteresisModel() == 0 || config().krHysteresisModel() == 2)
@@ -459,13 +437,43 @@ public:
     { return KrndHy_/KrndMax_; }
 
     Scalar krwWght() const
-    { 
-        return KrwdHy_/KrwdMax_; }
+    {
+        // a = 1 (deltaKrw)^a Formulation according to KILLOUGH 1976
+        Scalar deltaKrw = Krwi_snrmax() - Krwd_sncri();
+        Scalar Krwi_snr = Krwd_sncrt() + deltaKrw * (Sncrt() / max(1e-12, Sncri()));
+        return (Krwi_snr - KrwdHy()) / ( Krwi_snrmax() - Krwi_snmax());
+    }
 
     Scalar krwdMax() const
     { 
         return KrwdMax_; }
 
+    Scalar KrwdHy() const
+    {
+        return KrwdHy_;
+    }
+
+
+    Scalar Krwd_sncri() const
+    {
+        return Krwd_sncri_;
+    }
+
+    Scalar Krwi_snmax() const
+    {
+        return Krwi_snmax_;
+    }
+
+    Scalar Krwi_snrmax() const
+    {
+        return Krwi_snrmax_;
+    }
+
+    Scalar Krwd_sncrt() const
+    {
+        return Krwd_sncrt_;
+    }
+
     Scalar pcWght() const // Aligning pci and pcd at Swir
     {
         if (pcmaxd_ < 0.0)
@@ -594,23 +602,19 @@ public:
         if (krnSw < krnSwMdc_) {
             krnSwMdc_ = krnSw;
             KrndHy_ = EffLawT::twoPhaseSatKrn(drainageParams(), krnSwMdc_);
+            if (config().krHysteresisModel() == 4) {
+                KrwdHy_ = EffLawT::twoPhaseSatKrw(drainageParams(), krnSwMdc_);
+            }
             updateParams = true;
         }
-
         if (krwSw > krwSwMdc_) {
-            krwSwMdc_ = krwSw;
-            KrwdHy_ = EffLawT::twoPhaseSatKrw(drainageParams(), krwSwMdc_);
-            updateParams = true;
+            krwSwMdc_ = krwSw; // Only used for output at the moment
         }
 
         // for non WAG hysteresis we still keep track of the process
         // for output purpose.
         if (!gasOilHysteresisWAG()) {
-            if (krnSw < krnSwMdc_) {
-                isDrain_ = true;
-            } else {
-                isDrain_ = false;
-            }
+            this->isDrain_ = (krnSw <= this->krnSwMdc_);
         } else {
             wasDrain_ = isDrain_;
 
@@ -713,43 +717,50 @@ private:
         // Scalar SwPcMdcImbibition = EffLawT::twoPhaseSatPcnwInv(imbibitionParams(), pcMdcDrainage);
         // deltaSwImbPc_ = SwPcMdcImbibition - pcSwMdc_;
 
-        if (config().krHysteresisModel() == 2 || config().krHysteresisModel() == 3 || config().krHysteresisModel() == 4 || config().pcHysteresisModel() == 0) {
-            Scalar Snhy = 1.0 - krnSwMdc_;
-            if (Snhy > Sncrd_) {
-                Sncrt_ = Sncrd_ + (Snhy - Sncrd_)/((1.0+config().modParamTrapped()*(Snmaxd_-Snhy)) + C_*(Snhy - Sncrd_));
+        if (config().krHysteresisModel() == 2 ||
+            config().krHysteresisModel() == 3 ||
+            config().krHysteresisModel() == 4 ||
+            config().pcHysteresisModel() == 0)
+        {
+            const Scalar snhy = 1.0 - krnSwMdc_;
+            if (snhy > Sncrd_) {
+                Sncrt_ = Sncrd_ + (snhy - Sncrd_) /
+                        ((1.0 + config().modParamTrapped()*(Snmaxd_ - snhy)) + C_ * (snhy - Sncrd_));
             }
-            else
-            {
+            else {
                 Sncrt_ = Sncrd_;
             }
         }
 
         if (config().krHysteresisModel() == 4) {
-            Scalar Swhy = krwSwMdc_;
-            if (Swhy >= Swcrd_) {
-                Swcrt_ = Swcrd_ + (Swhy - Swcrd_)/((1.0+config().modParamTrapped()*(Swmaxd_-Swhy)) + Cw_*(Swhy - Swcrd_));
-            } else
-            {
+            Scalar swhy = krnSwMdc_;
+            if (swhy >= Swcrd_) {
+                Swcrt_ = Swcrd_ + (swhy - Swcrd_) /
+                        ((1.0 + config().modParamTrapped() * (Swmaxd_ - swhy)) + Cw_ * (swhy - Swcrd_));
+            } else {
                 Swcrt_ = Swcrd_;
             }
+            Krwd_sncrt_ = EffLawT::twoPhaseSatKrw(drainageParams(), 1 - Sncrt());
         }
 
 
         if (gasOilHysteresisWAG()) {
             if (isDrain_ && krnSwMdc_ == krnSwWAG_) {
-                Scalar Snhy = 1.0 - krnSwMdc_;
+                Scalar snhy = 1.0 - krnSwMdc_;
                 SncrtWAG_ = Sncrd_;
-                if (Snhy > Sncrd_) {
-                    SncrtWAG_ += (Snhy - Sncrd_)/(1.0+config().modParamTrapped()*(Snmaxd_-Snhy) + wagConfig().wagLandsParam()*(Snhy - Sncrd_));
+                if (snhy > Sncrd_) {
+                    SncrtWAG_ += (snhy - Sncrd_) /
+                        (1.0 + config().modParamTrapped() * (Snmaxd_ - snhy) +
+                         wagConfig().wagLandsParam() * (snhy - Sncrd_));
                 }
             }
 
-            if (isDrain_ && (1.0-krnSwDrainRevert_) > SncrtWAG_) { //Reversal from drain to imb
-                cTransf_ = 1.0/(SncrtWAG_-Sncrd_ + 1.0e-12) - 1.0/(1.0-krnSwDrainRevert_-Sncrd_);
+            if (isDrain_ && (1.0 - krnSwDrainRevert_) > SncrtWAG_) { // Reversal from drain to imb
+                cTransf_ = 1.0 / (SncrtWAG_ - Sncrd_ + 1.0e-12) - 1.0 / (1.0 - krnSwDrainRevert_ - Sncrd_);
             }
 
             if (!wasDrain_ && isDrain_) { // Start of new drainage cycle
-                if (threePhaseState() || nState_>1) { // Never return to primary (two-phase) state after leaving
+                if (threePhaseState() || nState_ > 1) { // Never return to primary (two-phase) state after leaving
                     nState_ += 1;
                     krnDrainStart_ = EffLawT::twoPhaseSatKrn(drainageParams(), krnSwDrainStart_);
                     krnImbStart_ = krnImbStartNxt_;
@@ -781,24 +792,24 @@ private:
     // largest wettinging phase saturation which is on the main-drainage curve. These are
     // three different values because the sourounding code can choose to use different
     // definitions for the saturations for different quantities
-    Scalar krwSwMdc_;
-    Scalar krnSwMdc_;
-    Scalar pcSwMdc_;
+    Scalar krwSwMdc_{-2.0};
+    Scalar krnSwMdc_{2.0};
+    Scalar pcSwMdc_{2.0};
 
     // largest wettinging phase saturation along main imbibition curve
-    Scalar pcSwMic_;
+    Scalar pcSwMic_{1.0};
     // Initial process is imbibition (for initial saturations at or below critical drainage saturation)
-    bool initialImb_;
+    bool initialImb_{false};
 
-    bool oilWaterSystem_;
-    bool gasOilSystem_;
+    bool oilWaterSystem_{false};
+    bool gasOilSystem_{false};
 
 
     // offsets added to wetting phase saturation uf using the imbibition curves need to
     // be used to calculate the wetting phase relperm, the non-wetting phase relperm and
     // the capillary pressure
-    //Scalar deltaSwImbKrw_;
-    Scalar deltaSwImbKrn_;
+    //Scalar deltaSwImbKrw_{};
+    Scalar deltaSwImbKrn_{};
     //Scalar deltaSwImbPc_;
 
     // the following uses the conventions of the Eclipse technical description:
@@ -821,45 +832,52 @@ private:
     Scalar Swcrd_{};
     Scalar Swmaxi_{};
     Scalar Snmaxd_{};
-    Scalar Swmaxd_;
+    Scalar Swmaxd_{};
     Scalar C_{};
-    Scalar Cw_{};
 
     Scalar KrndMax_{}; // Krn_drain(Snmaxd_)
     Scalar KrwdMax_{}; // Krw_drain(Swmaxd_)
     Scalar KrndHy_{};  // Krn_drain(1-krnSwMdc_)
-    Scalar KrwdHy_{};  // Krw_drain(1-krwSwMdc_)
 
-    Scalar pcmaxd_;  // max pc for drain
-    Scalar pcmaxi_;  // max pc for imb
+
+    // For wetting hysterese Killough
+    Scalar Cw_{};
+    Scalar KrwdHy_{};
+    Scalar Krwd_sncri_{};
+    Scalar Krwi_snmax_{};
+    Scalar Krwi_snrmax_{};
+    Scalar Krwd_sncrt_{};
+    Scalar Swcrt_{}; // trapped wetting phase saturation
+
+    Scalar pcmaxd_{};  // max pc for drain
+    Scalar pcmaxi_{};  // max pc for imb
 
     Scalar curvatureCapPrs_{}; // curvature parameter used for capillary pressure hysteresis
 
     Scalar Sncrt_{}; // trapped non-wetting phase saturation
-    Scalar Swcrt_{}; // trapped wetting phase saturation
 
     // Used for WAG hysteresis
-    Scalar Swco_;               // Connate water.
-    Scalar swatImbStart_;     // Water saturation at start of current drainage curve (end of previous imb curve).
-    Scalar swatImbStartNxt_{};    // Water saturation at start of next drainage curve (end of current imb curve).
-    Scalar krnSwWAG_;           // Saturation value after latest completed timestep.
-    Scalar krnSwDrainRevert_;   // Saturation value at end of current drainage curve.
-    Scalar cTransf_;            // Modified Lands constant used for free gas calculations to obtain consistent scanning curve
-                                //  when reversion to imb occurs above historical maximum gas saturation (i.e. Sw > krwSwMdc_).
-    Scalar krnSwDrainStart_;    // Saturation value at start of current drainage curve (end of previous imb curve).
-    Scalar krnSwDrainStartNxt_{}; // Saturation value at start of current drainage curve (end of previous imb curve).
-    Scalar krnImbStart_{};      // Relperm at start of current drainage curve (end of previous imb curve).
-    Scalar krnImbStartNxt_{};   // Relperm at start of next drainage curve (end of current imb curve).
-    Scalar krnDrainStart_{};    // Primary (input) relperm evaluated at start of current drainage curve.
-    Scalar krnDrainStartNxt_{}; // Primary (input) relperm evaluated at start of next drainage curve.
-    bool isDrain_;              // Status is either drainage or imbibition
-    bool wasDrain_{};           // Previous status.
-    Scalar krnSwImbStart_{};    // Saturation value where primary drainage relperm equals krnImbStart_
+    Scalar Swco_{};                // Connate water.
+    Scalar swatImbStart_{};        // Water saturation at start of current drainage curve (end of previous imb curve).
+    Scalar swatImbStartNxt_{};     // Water saturation at start of next drainage curve (end of current imb curve).
+    Scalar krnSwWAG_{2.0};         // Saturation value after latest completed timestep.
+    Scalar krnSwDrainRevert_{2.0}; // Saturation value at end of current drainage curve.
+    Scalar cTransf_{};             // Modified Lands constant used for free gas calculations to obtain consistent scanning curve
+                                   //  when reversion to imb occurs above historical maximum gas saturation (i.e. Sw > krwSwMdc_).
+    Scalar krnSwDrainStart_{-2.0}; // Saturation value at start of current drainage curve (end of previous imb curve).
+    Scalar krnSwDrainStartNxt_{};  // Saturation value at start of current drainage curve (end of previous imb curve).
+    Scalar krnImbStart_{};         // Relperm at start of current drainage curve (end of previous imb curve).
+    Scalar krnImbStartNxt_{};      // Relperm at start of next drainage curve (end of current imb curve).
+    Scalar krnDrainStart_{};       // Primary (input) relperm evaluated at start of current drainage curve.
+    Scalar krnDrainStartNxt_{};    // Primary (input) relperm evaluated at start of next drainage curve.
+    bool isDrain_{true};           // Status is either drainage or imbibition
+    bool wasDrain_{};              // Previous status.
+    Scalar krnSwImbStart_{};       // Saturation value where primary drainage relperm equals krnImbStart_
 
-    int nState_;                // Number of cycles. Primary cycle is nState_=1.
+    int nState_{};                 // Number of cycles. Primary cycle is nState_=1.
 
     Scalar SncrtWAG_{};
-    Scalar tolWAG_;
+    Scalar tolWAG_{0.001};
 };
 
 } // namespace Opm
