@@ -8,6 +8,7 @@
   OPM is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
   OPM is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -338,5 +339,7 @@ satOrImbRegion_(std::vector<int>& array, std::vector<int>& default_vec, unsigned
 // Make some actual code, by realizing the previously defined templated class
 template class EclMaterialLawManager<ThreePhaseMaterialTraits<double,0,1,2>>::InitParams;
 template class EclMaterialLawManager<ThreePhaseMaterialTraits<float,0,1,2>>::InitParams;
+template class EclMaterialLawManager<ThreePhaseMaterialTraits<double,2,0,1>>::InitParams;
+template class EclMaterialLawManager<ThreePhaseMaterialTraits<float,2,0,1>>::InitParams;
 
 } // namespace Opm
