@@ -37,6 +37,7 @@
 
 #include <opm/material/components/Brine.hpp>
 #include <opm/material/components/CO2.hpp>
+#include <opm/material/components/CO2Tables.hpp>
 #include <opm/material/components/SimpleCO2.hpp>
 #include <opm/material/components/SimpleH2O.hpp>
 #include <opm/material/components/TabulatedComponent.hpp>
@@ -49,13 +50,6 @@
 
 namespace Opm {
 
-// Silence compiler warnings about use of variables
-// that are instantiated in a different compilation unit.
-template<>
-const float CO2<float>::brineSalinity;
-template<>
-const double CO2<double>::brineSalinity;
-
 /*!
  * \brief A two-phase fluid system with water and CO2.
  *
@@ -305,7 +299,7 @@ public:
         }
 
         assert(phaseIdx == gasPhaseIdx);
-        LhsEval result = CO2::gasViscosity(temperature, pressure);
+        LhsEval result = CO2::gasViscosity(getTableInstance(), temperature, pressure);
         Valgrind::CheckDefined(result);
         return result;
     }
@@ -342,7 +336,8 @@ public:
         // could use some cleanup.
         LhsEval xlH2O, xgH2O;
         LhsEval xlCO2, xgCO2;
-        BinaryCoeffBrineCO2::calculateMoleFractions(temperature,
+        BinaryCoeffBrineCO2::calculateMoleFractions(getTableInstance(),
+                                                    temperature,
                                                     pressure,
                                                     LhsEval(Brine_IAPWS::salinity),
                                                     /*knownPhaseIdx=*/-1,
@@ -384,7 +379,7 @@ public:
             return BinaryCoeffBrineCO2::liquidDiffCoeff(temperature, pressure);
 
         assert(phaseIdx == gasPhaseIdx);
-        return BinaryCoeffBrineCO2::gasDiffCoeff(temperature, pressure);
+        return BinaryCoeffBrineCO2::gasDiffCoeff(getTableInstance(), temperature, pressure);
     }
 
     /*!
@@ -415,7 +410,7 @@ public:
 
             LhsEval result = 0;
             result += XBrine * Brine::gasEnthalpy(temperature, pressure);
-            result += XCO2 * CO2::gasEnthalpy(temperature, pressure);
+            result += XCO2 * CO2::gasEnthalpy(getTableInstance(), temperature, pressure);
             Valgrind::CheckDefined(result);
             return result;
         }
@@ -459,13 +454,20 @@ public:
         const LhsEval& temperature = decay<LhsEval>(fluidState.temperature(phaseIdx));
         const LhsEval& pressure = decay<LhsEval>(fluidState.pressure(phaseIdx));
 
-        if(phaseIdx == liquidPhaseIdx)
+        if(phaseIdx == liquidPhaseIdx){
             return H2O::liquidHeatCapacity(temperature, pressure);
-        else
+        }
+        else{
             return CO2::gasHeatCapacity(temperature, pressure);
+        }
     }
 
 private:
+    static const CO2Tables<double, std::vector<double>>& getTableInstance() {
+        static CO2Tables<double, std::vector<double>> instance;
+        return instance;
+    }
+
     template <class LhsEval>
     static LhsEval gasDensity_(const LhsEval& T,
                                const LhsEval& pg,
@@ -477,7 +479,7 @@ private:
         Valgrind::CheckDefined(xgH2O);
         Valgrind::CheckDefined(xgCO2);
 
-        return CO2::gasDensity(T, pg);
+        return CO2::gasDensity(getTableInstance(), T, pg);
     }
 
     /***********************************************************************/
@@ -574,12 +576,12 @@ private:
         /* same function as enthalpy_brine, only extended by CO2 content */
 
         /*Numerical coefficents from PALLISER*/
-        static Scalar f[] = {
+        static const Scalar f[] = {
             2.63500E-1, 7.48368E-6, 1.44611E-6, -3.80860E-10
         };
 
         /*Numerical coefficents from MICHAELIDES for the enthalpy of brine*/
-        static Scalar a[4][3] = {
+        static const Scalar a[4][3] = {
             { 9633.6, -4080.0, +286.49 },
             { +166.58, +68.577, -4.6856 },
             { -0.90963, -0.36524, +0.249667E-1 },
@@ -625,7 +627,7 @@ private:
         delta_hCO2 = (-57.4375 + T * 0.1325) * 1000/44;
 
         /* enthalpy contribution of CO2 (kJ/kg) */
-        hg = CO2::gasEnthalpy(T, p)/1E3 + delta_hCO2;
+        hg = CO2::gasEnthalpy(getTableInstance(), T, p)/1E3 + delta_hCO2;
 
         /* Enthalpy of brine with dissolved CO2 */
         return (h_ls1 - X_CO2_w*hw + hg*X_CO2_w)*1E3; /*J/kg*/
