@@ -40,6 +40,8 @@
 #include "BaseFluidSystem.hpp"
 #include "NullParameterCache.hpp"
 
+#include <string_view>
+
 namespace Opm {
 
 /*!
@@ -126,7 +128,7 @@ public:
     }
 
     //! \copydoc BaseFluidSystem::phaseName
-    static const char* phaseName(unsigned phaseIdx)
+    static std::string_view phaseName(unsigned phaseIdx)
     {
         switch (phaseIdx) {
         case waterPhaseIdx: return "water";
@@ -137,7 +139,7 @@ public:
     }
 
     //! \copydoc BaseFluidSystem::componentName
-    static const char* componentName(unsigned compIdx)
+    static std::string_view componentName(unsigned compIdx)
     {
         switch (compIdx) {
         case H2OIdx: return H2O::name();
