@@ -30,9 +30,12 @@
 #ifndef OPM_PTFlash_PARAMETER_CACHE_HPP
 #define OPM_PTFlash_PARAMETER_CACHE_HPP
 
+#include <opm/material/common/Valgrind.hpp>
 #include <opm/material/fluidsystems/ParameterCacheBase.hpp>
-#include <opm/material/eos/PengRobinson.hpp>
-#include <opm/material/eos/PengRobinsonParamsMixture.hpp>
+#include <opm/material/eos/CubicEOS.hpp>
+#include <opm/material/eos/CubicEOSParams.hpp>
+
+#include <opm/input/eclipse/EclipseState/Compositional/CompositionalConfig.hpp>
 
 #include <cassert>
 
@@ -48,11 +51,15 @@ class PTFlashParameterCache
 {
     using ThisType = PTFlashParameterCache<Scalar, FluidSystem>;
     using ParentType = Opm::ParameterCacheBase<ThisType>;
-    using PengRobinson = Opm::PengRobinson<Scalar, false>; // false refer to discard some old code
+    using CubicEOS = Opm::CubicEOS<Scalar, FluidSystem>;
+    using EOSType = CompositionalConfig::EOSType;
 
     enum { numPhases = FluidSystem::numPhases };
     enum { oilPhaseIdx = FluidSystem::oilPhaseIdx };
     enum { gasPhaseIdx = FluidSystem::gasPhaseIdx };
+    enum { waterPhaseIdx = FluidSystem::waterPhaseIdx };
+
+    static constexpr bool waterEnabled = FluidSystem::waterEnabled;
 
     static_assert(static_cast<int>(oilPhaseIdx) >= 0, "Oil phase index must be non-negative");
     static_assert(static_cast<int>(oilPhaseIdx) < static_cast<int>(numPhases),
@@ -64,16 +71,20 @@ class PTFlashParameterCache
 
 public:
     //! The cached parameters for the oil phase
-    using OilPhaseParams = Opm::PengRobinsonParamsMixture<Scalar, FluidSystem, oilPhaseIdx, /*useChi=*/false>;
-    //! The cached parameters for the gas phase
-    using GasPhaseParams = Opm::PengRobinsonParamsMixture<Scalar, FluidSystem, gasPhaseIdx, /*useChi=*/false>;
+    using OilPhaseParams = Opm::CubicEOSParams<Scalar, FluidSystem, oilPhaseIdx>;
 
-    PTFlashParameterCache()
+    //! The cached parameters for the gas phase
+    using GasPhaseParams = Opm::CubicEOSParams<Scalar, FluidSystem, gasPhaseIdx>;
+
+    explicit PTFlashParameterCache(EOSType eos_type)
     {
-            VmUpToDate_[oilPhaseIdx] = false;
-            Valgrind::SetUndefined(Vm_[oilPhaseIdx]);
-            VmUpToDate_[gasPhaseIdx] = false;
-            Valgrind::SetUndefined(Vm_[gasPhaseIdx]);
+        VmUpToDate_[oilPhaseIdx] = false;
+        Valgrind::SetUndefined(Vm_[oilPhaseIdx]);
+        VmUpToDate_[gasPhaseIdx] = false;
+        Valgrind::SetUndefined(Vm_[gasPhaseIdx]);
+    
+        oilPhaseParams_.setEOSType(eos_type);
+        gasPhaseParams_.setEOSType(eos_type);
     }
 
     //! \copydoc ParameterCacheBase::updatePhase
@@ -82,6 +93,10 @@ public:
                      unsigned phaseIdx,
                      int exceptQuantities = ParentType::None)
     {
+        if (waterEnabled && phaseIdx == static_cast<unsigned int>(waterPhaseIdx)) {
+            return;
+        }
+
         assert ((phaseIdx == static_cast<unsigned int>(oilPhaseIdx)) ||
                 (phaseIdx == static_cast<unsigned int>(gasPhaseIdx)));
 
@@ -108,6 +123,66 @@ public:
         updateMolarVolume_(fluidState, phaseIdx);
     }
 
+    Scalar A(unsigned phaseIdx) const
+    {
+        switch (phaseIdx)
+        {
+        case oilPhaseIdx: return oilPhaseParams_.A();
+        case gasPhaseIdx: return gasPhaseParams_.A();
+        default:
+            throw std::logic_error("The A parameter is only defined for "
+                                   "oil and gas phases");
+        };
+    }
+    
+    Scalar B(unsigned phaseIdx) const
+    {
+        switch (phaseIdx)
+        {
+        case oilPhaseIdx: return oilPhaseParams_.B();
+        case gasPhaseIdx: return gasPhaseParams_.B();
+        default:
+            throw std::logic_error("The B parameter is only defined for "
+                                   "oil and gas phases");
+        };
+    }
+    
+    Scalar Bi(unsigned phaseIdx, unsigned compIdx) const
+    {
+        switch (phaseIdx)
+        {
+        case oilPhaseIdx: return oilPhaseParams_.Bi(compIdx);
+        case gasPhaseIdx: return gasPhaseParams_.Bi(compIdx);
+        default:
+            throw std::logic_error("The Bi parameter is only defined for "
+                                   "oil and gas phases");
+        };
+    }
+
+    Scalar m1(unsigned phaseIdx) const
+    {
+        switch (phaseIdx)
+        {
+        case oilPhaseIdx: return oilPhaseParams_.m1();
+        case gasPhaseIdx: return gasPhaseParams_.m1();
+        default:
+            throw std::logic_error("The m1 parameter is only defined for "
+                                   "oil and gas phases");
+        };
+    }
+
+    Scalar m2(unsigned phaseIdx) const
+    {
+        switch (phaseIdx)
+        {
+        case oilPhaseIdx: return oilPhaseParams_.m2();
+        case gasPhaseIdx: return gasPhaseParams_.m2();
+        default:
+            throw std::logic_error("The m2 parameter is only defined for "
+                                   "oil and gas phases");
+        };
+    }
+
     /*!
      * \brief The Peng-Robinson attractive parameter for a phase.
      *
@@ -181,8 +256,6 @@ public:
         };
     }
 
-
-
     /*!
      * \brief TODO
      *
@@ -194,10 +267,10 @@ public:
     {
         switch (phaseIdx)
         {
-        case oilPhaseIdx: return oilPhaseParams_.getaCache(compIdx,compJIdx);
-        case gasPhaseIdx: return gasPhaseParams_.getaCache(compIdx,compJIdx);
+        case oilPhaseIdx: return oilPhaseParams_.aCache(compIdx, compJIdx);
+        case gasPhaseIdx: return gasPhaseParams_.aCache(compIdx, compJIdx);
         default:
-            throw std::logic_error("The aCache() parameter is only defined for "
+            throw std::logic_error("The aCache parameter is only defined for "
                                    "oil and gas phase");
         };
     }
@@ -208,7 +281,10 @@ public:
      * \param phaseIdx The fluid phase of interest
      */
     Scalar molarVolume(unsigned phaseIdx) const
-    { assert(VmUpToDate_[phaseIdx]); return Vm_[phaseIdx]; }
+    { 
+        assert(VmUpToDate_[phaseIdx]); 
+        return Vm_[phaseIdx]; 
+    }
 
 
     /*!
@@ -316,7 +392,7 @@ protected:
             // system can get queried, so it is okay to calculate it
             // here...
             Vm_[gasPhaseIdx] = decay<Scalar> (
-                PengRobinson::computeMolarVolume(fluidState,
+                CubicEOS::computeMolarVolume(fluidState,
                                                  *this,
                                                  phaseIdx,
                                                  /*isGasPhase=*/true) );
@@ -329,7 +405,7 @@ protected:
             // system can get queried, so it is okay to calculate it
             // here...
             Vm_[oilPhaseIdx] = decay<Scalar> (
-                PengRobinson::computeMolarVolume(fluidState,
+                CubicEOS::computeMolarVolume(fluidState,
                                                  *this,
                                                  phaseIdx,
                                                  /*isGasPhase=*/false) );
