@@ -29,12 +29,15 @@
 
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
+#include <opm/input/eclipse/Units/UnitSystem.hpp>
+
+#include <fmt/format.h>
 
 namespace Opm {
 
-template<class Scalar>
-BrineCo2Pvt<Scalar>::
-BrineCo2Pvt(const std::vector<Scalar>& salinity,
+template<class Scalar, class Params, class ContainerT>
+BrineCo2Pvt<Scalar, Params, ContainerT>::
+BrineCo2Pvt(const ContainerT& salinity,
             int activityModel,
             int thermalMixingModelSalt,
             int thermalMixingModelLiquid,
@@ -52,17 +55,20 @@ BrineCo2Pvt(const std::vector<Scalar>& salinity,
     int num_regions =  salinity_.size();
     co2ReferenceDensity_.resize(num_regions);
     brineReferenceDensity_.resize(num_regions);
+
     for (int i = 0; i < num_regions; ++i) {
-        co2ReferenceDensity_[i] = CO2::gasDensity(T_ref, P_ref, true);
+        co2ReferenceDensity_[i] = CO2::gasDensity(co2Tables_, T_ref, P_ref, true);
         brineReferenceDensity_[i] = Brine::liquidDensity(T_ref, P_ref, salinity_[i], true);
     }
 }
 
 #if HAVE_ECL_INPUT
-template<class Scalar>
-void BrineCo2Pvt<Scalar>::
+template<class Scalar, class Params, class ContainerT>
+void BrineCo2Pvt<Scalar, Params, ContainerT>::
 initFromState(const EclipseState& eclState, const Schedule&)
 {
+    using Meas = UnitSystem::measure;
+    auto usys = eclState.getDeckUnitSystem();
     bool co2sol = eclState.runspec().co2Sol();
     if (!co2sol && !eclState.getTableManager().getDensityTable().empty()) {
         OpmLog::warning("CO2STORE is enabled but DENSITY is in the deck. \n"
@@ -81,6 +87,7 @@ initFromState(const EclipseState& eclState, const Schedule&)
                         "BRINE PVT properties are computed based on the Hu et al. "
                         "pvt model and PVTW input is ignored.");
     }
+    OpmLog::info("CO2STORE/CO2SOL is enabled.");
     // enable co2 dissolution into brine for co2sol case with DISGASW
     // or co2store case with DISGASW or DISGAS
     bool co2sol_dis = co2sol && eclState.getSimulationConfig().hasDISGASW();
@@ -94,7 +101,9 @@ initFromState(const EclipseState& eclState, const Schedule&)
     // set the surface conditions using the STCOND keyword
     Scalar T_ref = eclState.getTableManager().stCond().temperature;
     Scalar P_ref = eclState.getTableManager().stCond().pressure;
-    
+
+    co2Tables_ = Params();
+
     // Throw an error if STCOND is not (T, p) = (15.56 C, 1 atm) = (288.71 K, 1.01325e5 Pa)
     if (T_ref != Scalar(288.71) || P_ref != Scalar(1.01325e5)) {
         OPM_THROW(std::runtime_error, "CO2STORE can only be used with default values for STCOND!");
@@ -103,22 +112,6 @@ initFromState(const EclipseState& eclState, const Schedule&)
     // Check for Ezrokhi tables DENAQA and VISCAQA
     setEzrokhiDenCoeff(eclState.getCo2StoreConfig().getDenaqaTables());
     setEzrokhiViscCoeff(eclState.getCo2StoreConfig().getViscaqaTables());
-    std::string ezrokhi_msg;
-    if (enableEzrokhiDensity_) {
-        ezrokhi_msg = "\nEzrokhi density coefficients : \n\tNaCl = " 
-                      + std::to_string(ezrokhiDenNaClCoeff_[0]) + " " + std::to_string(ezrokhiDenNaClCoeff_[1]) 
-                      + " " + std::to_string(ezrokhiDenNaClCoeff_[2])
-                      + "\n\tCO2 = " + std::to_string(ezrokhiDenCo2Coeff_[0]) + " " + std::to_string(ezrokhiDenCo2Coeff_[1]) 
-                      + " " + std::to_string(ezrokhiDenCo2Coeff_[2]);
-    }
-    else {
-        ezrokhi_msg = "";
-    }
-    if (enableEzrokhiViscosity_) {
-        ezrokhi_msg += "\nEzrokhi viscosity coefficients : \n\tNaCl = " 
-                       + std::to_string(ezrokhiViscNaClCoeff_[0]) + " " + std::to_string(ezrokhiViscNaClCoeff_[1]) 
-                       + " " + std::to_string(ezrokhiViscNaClCoeff_[1]);
-    }
     
     std::size_t regions = eclState.runspec().tabdims().getNumPVTTables();
     setNumRegions(regions);
@@ -133,22 +126,30 @@ initFromState(const EclipseState& eclState, const Schedule&)
         else {
             brineReferenceDensity_[regionIdx] = Brine::liquidDensity(T_ref, P_ref, salinity_[regionIdx], extrapolate);
         }
-        co2ReferenceDensity_[regionIdx] = CO2::gasDensity(T_ref, P_ref, extrapolate);
+        co2ReferenceDensity_[regionIdx] = CO2::gasDensity(co2Tables_, T_ref, P_ref, extrapolate);
     }
 
-    // The reference densities are the same across regions. Only output info for region 0
-    OpmLog::info("CO2STORE/CO2SOL is enabled. \n The surface density of CO2 is  " + std::to_string(co2ReferenceDensity_[0])
-            + "kg/m3 \n The surface density of Brine is  " + std::to_string(brineReferenceDensity_[0])
-            + "kg/m3"
-            + "\n The surface densities are computed using the reference pressure ( " + std::to_string(P_ref) 
-            + "Pa) and the reference temperature (" +  std::to_string(T_ref) + "K)."
-            + ezrokhi_msg
-            );
+    OpmLog::info(fmt::format("The surface density of CO2 is {:.6f} {}.",
+                             usys.from_si(Meas::density, co2ReferenceDensity_[0]), usys.name(Meas::density)));
+    OpmLog::info(fmt::format("The surface density of brine is {:.6f} {}.",
+                             usys.from_si(Meas::density, brineReferenceDensity_[0]), usys.name(Meas::density)));
+    OpmLog::info(fmt::format("The surface densities are computed using the reference pressure ({:.3f} {}) and reference temperature ({:.2f} {}).",
+                             usys.from_si(Meas::pressure , P_ref), usys.name(Meas::pressure),
+                             usys.from_si(Meas::temperature , T_ref), usys.name(Meas::temperature)));
+    if (enableEzrokhiDensity_) {
+        OpmLog::info(fmt::format("Ezrokhi density coefficients : \n\tNaCl = {:.3E} {:.3E} {:.3E} \n\tCO2 = {:.3E} {:.3E} {:.3E}",
+                                 ezrokhiDenNaClCoeff_[0], ezrokhiDenNaClCoeff_[1], ezrokhiDenNaClCoeff_[2],
+                                 ezrokhiDenCo2Coeff_[0], ezrokhiDenCo2Coeff_[1], ezrokhiDenCo2Coeff_[2]));
+    }
+    if (enableEzrokhiViscosity_) {
+       OpmLog::info(fmt::format("Ezrokhi viscosity coefficients : \n\tNaCl = {:.3E} {:.3E} {:.3E}",
+                    ezrokhiViscNaClCoeff_[0], ezrokhiViscNaClCoeff_[1], ezrokhiViscNaClCoeff_[2])); 
+    }
 }
 #endif
 
-template<class Scalar>
-void BrineCo2Pvt<Scalar>::
+template<class Scalar, class Params, class ContainerT>
+void BrineCo2Pvt<Scalar, Params, ContainerT>::
 setNumRegions(std::size_t numRegions)
 {
     brineReferenceDensity_.resize(numRegions);
@@ -156,8 +157,8 @@ setNumRegions(std::size_t numRegions)
     salinity_.resize(numRegions);
 }
 
-template<class Scalar>
-void BrineCo2Pvt<Scalar>::
+template<class Scalar, class Params, class ContainerT>
+void BrineCo2Pvt<Scalar, Params, ContainerT>::
 setReferenceDensities(unsigned regionIdx,
                       Scalar rhoRefBrine,
                       Scalar rhoRefCO2,
@@ -167,8 +168,8 @@ setReferenceDensities(unsigned regionIdx,
     co2ReferenceDensity_[regionIdx] = rhoRefCO2;
 }
 
-template<class Scalar>
-void BrineCo2Pvt<Scalar>::
+template<class Scalar, class Params, class ContainerT>
+void BrineCo2Pvt<Scalar, Params, ContainerT>::
 setActivityModelSalt(int activityModel)
 {
     switch (activityModel) {
@@ -179,8 +180,8 @@ setActivityModelSalt(int activityModel)
     }
 }
 
-template<class Scalar>
-void BrineCo2Pvt<Scalar>::
+template<class Scalar, class Params, class ContainerT>
+void BrineCo2Pvt<Scalar, Params, ContainerT>::
 setThermalMixingModel(int thermalMixingModelSalt, int thermalMixingModelLiquid)
 {
     switch (thermalMixingModelSalt) {
@@ -197,8 +198,8 @@ setThermalMixingModel(int thermalMixingModelSalt, int thermalMixingModelLiquid)
     }
 }
 
-template<class Scalar>
-void BrineCo2Pvt<Scalar>::
+template<class Scalar, class Params, class ContainerT>
+void BrineCo2Pvt<Scalar, Params, ContainerT>::
 setEzrokhiDenCoeff(const std::vector<EzrokhiTable>& denaqa)
 {
     if (denaqa.empty())
@@ -213,8 +214,8 @@ setEzrokhiDenCoeff(const std::vector<EzrokhiTable>& denaqa)
                            static_cast<Scalar>(denaqa[0].getC2("CO2"))};
 }
 
-template<class Scalar>
-void BrineCo2Pvt<Scalar>::
+template<class Scalar, class Params, class ContainerT>
+void BrineCo2Pvt<Scalar, Params, ContainerT>::
 setEzrokhiViscCoeff(const std::vector<EzrokhiTable>& viscaqa)
 {
     if (viscaqa.empty())
