@@ -30,6 +30,7 @@
 #include <opm/common/Exceptions.hpp>
 #include <opm/common/TimingMacros.hpp>
 #include <opm/common/ErrorMacros.hpp>
+#include <opm/common/utility/gpuDecorators.hpp>
 
 #include <opm/material/Constants.hpp>
 
@@ -38,6 +39,7 @@
 #include <opm/material/components/CO2.hpp>
 #include <opm/material/common/UniformTabulated2DFunction.hpp>
 #include <opm/material/components/TabulatedComponent.hpp>
+#include <opm/material/components/CO2Tables.hpp>
 #include <opm/material/binarycoefficients/H2O_CO2.hpp>
 #include <opm/material/binarycoefficients/Brine_CO2.hpp>
 
@@ -53,11 +55,22 @@ class Schedule;
 class Co2StoreConfig;
 class EzrokhiTable;
 
+// forward declaration of the class so the function in the next namespace can be declared
+template <class Scalar, class Params, class ContainerT>
+class BrineCo2Pvt;
+
+// declaration of make_view in correct namespace so friend function can be declared in the class
+namespace gpuistl {
+    template <class ViewType, class OutputParams, class InputParams, class ContainerType, class ScalarT>
+    BrineCo2Pvt<ScalarT, OutputParams, ViewType>
+    make_view(BrineCo2Pvt<ScalarT, InputParams, ContainerType>&);
+}
+
 /*!
  * \brief This class represents the Pressure-Volume-Temperature relations of the liquid phase
  * for a CO2-Brine system
  */
-template <class Scalar>
+template <class Scalar, class Params = Opm::CO2Tables<double, std::vector<double>>, class ContainerT = std::vector<Scalar>>
 class BrineCo2Pvt
 {
     static constexpr bool extrapolate = true;
@@ -73,20 +86,37 @@ class BrineCo2Pvt
 public:
     using H2O = SimpleHuDuanH2O<Scalar>;
     using Brine = ::Opm::BrineDynamic<Scalar, H2O>;
-    using CO2 = ::Opm::CO2<Scalar>;
+    using CO2 = ::Opm::CO2<Scalar, Params>;
 
     //! The binary coefficients for brine and CO2 used by this fluid system
     using BinaryCoeffBrineCO2 = BinaryCoeff::Brine_CO2<Scalar, H2O, CO2>;
 
-    explicit BrineCo2Pvt() = default;
+    BrineCo2Pvt() = default;
 
-    explicit BrineCo2Pvt(const std::vector<Scalar>& salinity,
+    explicit BrineCo2Pvt(const ContainerT& salinity,
                          int activityModel = 3,
                          int thermalMixingModelSalt = 1,
                          int thermalMixingModelLiquid = 2,
                          Scalar T_ref = 288.71, //(273.15 + 15.56)
                          Scalar P_ref = 101325);
 
+    BrineCo2Pvt(const ContainerT& brineReferenceDensity,
+                const ContainerT& co2ReferenceDensity,
+                const ContainerT& salinity,
+                int activityModel,
+                Co2StoreConfig::SaltMixingType thermalMixingModelSalt,
+                Co2StoreConfig::LiquidMixingType thermalMixingModelLiquid,
+                Params params)
+                : brineReferenceDensity_(brineReferenceDensity)
+                , co2ReferenceDensity_(co2ReferenceDensity)
+                , salinity_(salinity)
+                , activityModel_(activityModel)
+                , liquidMixType_(thermalMixingModelLiquid)
+                , saltMixType_(thermalMixingModelSalt)
+                , co2Tables_(params)
+{
+}
+
 #if HAVE_ECL_INPUT
     /*!
      * \brief Initialize the parameters for Brine-CO2 system using an ECL deck.
@@ -154,7 +184,7 @@ public:
     unsigned numRegions() const
     { return brineReferenceDensity_.size(); }
 
-    Scalar hVap(unsigned ) const{
+    OPM_HOST_DEVICE Scalar hVap(unsigned ) const{
         return 0;
     }
 
@@ -162,7 +192,7 @@ public:
      * \brief Returns the specific enthalpy [J/kg] of gas given a set of parameters.
      */
     template <class Evaluation>
-    Evaluation internalEnergy(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation internalEnergy(unsigned regionIdx,
                               const Evaluation& temperature,
                               const Evaluation& pressure,
                               const Evaluation& Rs,
@@ -181,7 +211,7 @@ public:
      * \brief Returns the specific enthalpy [J/kg] of gas given a set of parameters.
      */
     template <class Evaluation>
-    Evaluation internalEnergy(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation internalEnergy(unsigned regionIdx,
                         const Evaluation& temperature,
                         const Evaluation& pressure,
                         const Evaluation& Rs) const
@@ -199,7 +229,7 @@ public:
      * \brief Returns the dynamic viscosity [Pa s] of the fluid phase given a set of parameters.
      */
     template <class Evaluation>
-    Evaluation viscosity(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation viscosity(unsigned regionIdx,
                          const Evaluation& temperature,
                          const Evaluation& pressure,
                          const Evaluation& /*Rs*/) const
@@ -212,7 +242,7 @@ public:
      * \brief Returns the dynamic viscosity [Pa s] of the fluid phase given a set of parameters.
      */
     template <class Evaluation>
-    Evaluation saturatedViscosity(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedViscosity(unsigned regionIdx,
                                  const Evaluation& temperature,
                                  const Evaluation& pressure,
                                  const Evaluation& saltConcentration) const
@@ -233,7 +263,7 @@ public:
      * \brief Returns the dynamic viscosity [Pa s] of the fluid phase given a set of parameters.
      */
     template <class Evaluation>
-    Evaluation viscosity(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation viscosity(unsigned regionIdx,
                          const Evaluation& temperature,
                          const Evaluation& pressure,
                          const Evaluation& /*Rsw*/,
@@ -248,7 +278,7 @@ public:
      * \brief Returns the dynamic viscosity [Pa s] of oil saturated gas at given pressure.
      */
     template <class Evaluation>
-    Evaluation saturatedViscosity(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedViscosity(unsigned regionIdx,
                                   const Evaluation& temperature,
                                   const Evaluation& pressure) const
     {
@@ -269,7 +299,7 @@ public:
      * \brief Returns the formation volume factor [-] of the fluid phase.
      */
     template <class Evaluation>
-    Evaluation saturatedInverseFormationVolumeFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedInverseFormationVolumeFactor(unsigned regionIdx,
                                                      const Evaluation& temperature,
                                                      const Evaluation& pressure,
                                                      const Evaluation& saltconcentration) const
@@ -286,7 +316,7 @@ public:
      * \brief Returns the formation volume factor [-] of the fluid phase.
      */
     template <class Evaluation>
-    Evaluation inverseFormationVolumeFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation inverseFormationVolumeFactor(unsigned regionIdx,
                                             const Evaluation& temperature,
                                             const Evaluation& pressure,
                                             const Evaluation& Rs,
@@ -303,7 +333,7 @@ public:
      * \brief Returns the formation volume factor [-] of the fluid phase.
      */
     template <class Evaluation>
-    Evaluation inverseFormationVolumeFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation inverseFormationVolumeFactor(unsigned regionIdx,
                                             const Evaluation& temperature,
                                             const Evaluation& pressure,
                                             const Evaluation& Rs) const
@@ -317,7 +347,7 @@ public:
      * \brief Returns the formation volume factor [-] of brine saturated with CO2 at a given pressure.
      */
     template <class Evaluation>
-    Evaluation saturatedInverseFormationVolumeFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedInverseFormationVolumeFactor(unsigned regionIdx,
                                                      const Evaluation& temperature,
                                                      const Evaluation& pressure) const
     {
@@ -335,12 +365,16 @@ public:
      * \param Rs
      */
     template <class Evaluation>
-    Evaluation saturationPressure(unsigned /*regionIdx*/,
+    OPM_HOST_DEVICE Evaluation saturationPressure(unsigned /*regionIdx*/,
                                   const Evaluation& /*temperature*/,
                                   const Evaluation& /*Rs*/) const
     {
+#if OPM_IS_INSIDE_DEVICE_FUNCTION
+        assert(false && "Requested the saturation pressure for the brine-co2 pvt module. Not yet implemented.");
+#else
         throw std::runtime_error("Requested the saturation pressure for the brine-co2 pvt module. "
                                  "Not yet implemented.");
+#endif
     }
 
     /*!
@@ -350,20 +384,24 @@ public:
      * \param Rs
      */
     template <class Evaluation>
-    Evaluation saturationPressure(unsigned /*regionIdx*/,
+    OPM_HOST_DEVICE Evaluation saturationPressure(unsigned /*regionIdx*/,
                                   const Evaluation& /*temperature*/,
                                   const Evaluation& /*Rs*/,
                                   const Evaluation& /*saltConcentration*/) const
     {
+#if OPM_IS_INSIDE_DEVICE_FUNCTION
+        assert(false && "Requested the saturation pressure for the brine-co2 pvt module. Not yet implemented.");
+#else
         throw std::runtime_error("Requested the saturation pressure for the brine-co2 pvt module. "
                                  "Not yet implemented.");
+#endif
     }
 
     /*!
      * \brief Returns the gas dissoluiton factor \f$R_s\f$ [m^3/m^3] of the liquid phase.
      */
     template <class Evaluation>
-    Evaluation saturatedGasDissolutionFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedGasDissolutionFactor(unsigned regionIdx,
                                              const Evaluation& temperature,
                                              const Evaluation& pressure,
                                              const Evaluation& /*oilSaturation*/,
@@ -377,7 +415,7 @@ public:
      * \brief Returns the gas dissoluiton factor \f$R_s\f$ [m^3/m^3] of the liquid phase.
      */
     template <class Evaluation>
-    Evaluation saturatedGasDissolutionFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedGasDissolutionFactor(unsigned regionIdx,
                                              const Evaluation& temperature,
                                              const Evaluation& pressure,
                                              const Evaluation& saltConcentration) const
@@ -391,27 +429,48 @@ public:
      * \brief Returns thegas dissoluiton factor  \f$R_s\f$ [m^3/m^3] of the liquid phase.
      */
     template <class Evaluation>
-    Evaluation saturatedGasDissolutionFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedGasDissolutionFactor(unsigned regionIdx,
                                              const Evaluation& temperature,
                                              const Evaluation& pressure) const
     {
         return rsSat(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx]));
     }
 
-    Scalar oilReferenceDensity(unsigned regionIdx) const
+    OPM_HOST_DEVICE Scalar oilReferenceDensity(unsigned regionIdx) const
     { return brineReferenceDensity_[regionIdx]; }
 
-    Scalar waterReferenceDensity(unsigned regionIdx) const
+    OPM_HOST_DEVICE Scalar waterReferenceDensity(unsigned regionIdx) const
     { return brineReferenceDensity_[regionIdx]; }
 
-    Scalar gasReferenceDensity(unsigned regionIdx) const
+    OPM_HOST_DEVICE Scalar gasReferenceDensity(unsigned regionIdx) const
     { return co2ReferenceDensity_[regionIdx]; }
 
-    Scalar salinity(unsigned regionIdx) const
+    OPM_HOST_DEVICE Scalar salinity(unsigned regionIdx) const
     { return salinity_[regionIdx]; }
 
+    OPM_HOST_DEVICE const ContainerT& getBrineReferenceDensity() const
+    { return brineReferenceDensity_; }
+
+    OPM_HOST_DEVICE const ContainerT& getCo2ReferenceDensity() const
+    { return co2ReferenceDensity_; }
+
+    OPM_HOST_DEVICE const ContainerT& getSalinity() const
+    { return salinity_; }
+
+    OPM_HOST_DEVICE const Params& getParams() const
+    { return co2Tables_; }
+
+    OPM_HOST_DEVICE Co2StoreConfig::SaltMixingType getThermalMixingModelSalt() const
+    { return saltMixType_; }
+
+    OPM_HOST_DEVICE Co2StoreConfig::LiquidMixingType getThermalMixingModelLiquid() const
+    { return liquidMixType_; }
+
+    OPM_HOST_DEVICE int getActivityModel() const
+    { return activityModel_; }
+
     template <class Evaluation>
-    Evaluation diffusionCoefficient(const Evaluation& temperature,
+    OPM_HOST_DEVICE Evaluation diffusionCoefficient(const Evaluation& temperature,
                                     const Evaluation& pressure,
                                     unsigned /*compIdx*/) const
     {
@@ -442,7 +501,7 @@ public:
     }
 
     template <class Evaluation>
-    Evaluation density(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation density(unsigned regionIdx,
                        const Evaluation& temperature,
                        const Evaluation& pressure,
                        const Evaluation& Rs,
@@ -460,7 +519,7 @@ public:
     }
 
     template <class Evaluation>
-    Evaluation rsSat(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation rsSat(unsigned regionIdx,
                      const Evaluation& temperature,
                      const Evaluation& pressure,
                      const Evaluation& salinity) const
@@ -474,7 +533,8 @@ public:
         // temperature and pressure.
         Evaluation xgH2O;
         Evaluation xlCO2;
-        BinaryCoeffBrineCO2::calculateMoleFractions(temperature,
+        BinaryCoeffBrineCO2::calculateMoleFractions(co2Tables_,
+                                                    temperature,
                                                     pressure,
                                                     salinity,
                                                     /*knownPhaseIdx=*/-1,
@@ -491,15 +551,15 @@ public:
 
 private:
     template <class LhsEval>
-    LhsEval ezrokhiExponent_(const LhsEval& temperature,
-                             const std::vector<Scalar>& ezrokhiCoeff) const
+    OPM_HOST_DEVICE LhsEval ezrokhiExponent_(const LhsEval& temperature,
+                             const ContainerT& ezrokhiCoeff) const
     {
         const LhsEval& tempC = temperature - 273.15;
         return ezrokhiCoeff[0] + tempC * (ezrokhiCoeff[1] + ezrokhiCoeff[2] * tempC);
     }
     
     template <class LhsEval>
-    LhsEval liquidDensity_(const LhsEval& T,
+    OPM_HOST_DEVICE LhsEval liquidDensity_(const LhsEval& T,
                            const LhsEval& pl,
                            const LhsEval& xlCO2,
                            const LhsEval& salinity) const
@@ -510,18 +570,26 @@ private:
         Valgrind::CheckDefined(xlCO2);
 
         if (!extrapolate && T < 273.15) {
+#if OPM_IS_INSIDE_DEVICE_FUNCTION
+            assert(false && "Liquid density for Brine and CO2 is only defined above 273.15K");
+#else
             const std::string msg =
                 "Liquid density for Brine and CO2 is only "
                 "defined above 273.15K (is " +
                 std::to_string(getValue(T)) + "K)";
             throw NumericalProblem(msg);
+#endif
         }
         if (!extrapolate && pl >= 2.5e8) {
+#if OPM_IS_INSIDE_DEVICE_FUNCTION
+            assert(false && "Liquid density for Brine and CO2 is only defined below 250MPa");
+#else
             const std::string msg  =
                 "Liquid density for Brine and CO2 is only "
                 "defined below 250MPa (is " +
                 std::to_string(getValue(pl)) + "Pa)";
             throw NumericalProblem(msg);
+#endif
         }
 
         const LhsEval& rho_pure = H2O::liquidDensity(T, pl, extrapolate);
@@ -540,7 +608,7 @@ private:
     }
 
     template <class LhsEval>
-    LhsEval liquidDensityWaterCO2_(const LhsEval& temperature,
+    OPM_HOST_DEVICE LhsEval liquidDensityWaterCO2_(const LhsEval& temperature,
                                           const LhsEval& pl,
                                           const LhsEval& xlCO2) const
     {
@@ -568,7 +636,7 @@ private:
      *        of the gas component in the oil phase.
      */
     template <class LhsEval>
-    LhsEval convertRsToXoG_(const LhsEval& Rs, unsigned regionIdx) const
+    OPM_HOST_DEVICE LhsEval convertRsToXoG_(const LhsEval& Rs, unsigned regionIdx) const
     {
         OPM_TIMEFUNCTION_LOCAL();
         Scalar rho_oRef = brineReferenceDensity_[regionIdx];
@@ -582,7 +650,7 @@ private:
      * \brief Convert a gas mass fraction in the oil phase the corresponding mole fraction.
      */
     template <class LhsEval>
-    LhsEval convertXoGToxoG_(const LhsEval& XoG, const LhsEval& salinity) const
+    OPM_HOST_DEVICE LhsEval convertXoGToxoG_(const LhsEval& XoG, const LhsEval& salinity) const
     {
         OPM_TIMEFUNCTION_LOCAL();
         Scalar M_CO2 = CO2::molarMass();
@@ -594,7 +662,7 @@ private:
      * \brief Convert a gas mole fraction in the oil phase the corresponding mass fraction.
      */
     template <class LhsEval>
-    LhsEval convertxoGToXoG(const LhsEval& xoG, const LhsEval& salinity) const
+    OPM_HOST_DEVICE LhsEval convertxoGToXoG(const LhsEval& xoG, const LhsEval& salinity) const
     {
         OPM_TIMEBLOCK_LOCAL(convertxoGToXoG);
         Scalar M_CO2 = CO2::molarMass();
@@ -608,7 +676,7 @@ private:
      *        corresponding gas dissolution factor.
      */
     template <class LhsEval>
-    LhsEval convertXoGToRs(const LhsEval& XoG, unsigned regionIdx) const
+    OPM_HOST_DEVICE LhsEval convertXoGToRs(const LhsEval& XoG, unsigned regionIdx) const
     {
         Scalar rho_oRef = brineReferenceDensity_[regionIdx];
         Scalar rho_gRef = co2ReferenceDensity_[regionIdx];
@@ -617,7 +685,7 @@ private:
     }
 
     template <class LhsEval>
-    LhsEval liquidEnthalpyBrineCO2_(const LhsEval& T,
+    OPM_HOST_DEVICE LhsEval liquidEnthalpyBrineCO2_(const LhsEval& T,
                                     const LhsEval& p,
                                     const LhsEval& salinity, 
                                     const LhsEval& X_CO2_w) const
@@ -699,14 +767,14 @@ private:
         }
             
         /* enthalpy contribution of CO2 (kJ/kg) */
-        hg = CO2::gasEnthalpy(T, p, extrapolate)/1E3 + delta_hCO2;
+        hg = CO2::gasEnthalpy(co2Tables_, T, p, extrapolate)/1E3 + delta_hCO2;
 
         /* Enthalpy of brine with dissolved CO2 */
         return (h_ls1 - X_CO2_w*hw + hg*X_CO2_w)*1E3; /*J/kg*/
     }
 
     template <class LhsEval>
-    const LhsEval salinityFromConcentration(unsigned regionIdx,
+    OPM_HOST_DEVICE const LhsEval salinityFromConcentration(unsigned regionIdx,
                                             const LhsEval&T,
                                             const LhsEval& P,
                                             const LhsEval& saltConcentration) const
@@ -718,12 +786,16 @@ private:
         return salinity(regionIdx);
     }
 
-    std::vector<Scalar> brineReferenceDensity_{};
-    std::vector<Scalar> co2ReferenceDensity_{};
-    std::vector<Scalar> salinity_{};
-    std::vector<Scalar> ezrokhiDenNaClCoeff_{};
-    std::vector<Scalar> ezrokhiDenCo2Coeff_{};
-    std::vector<Scalar> ezrokhiViscNaClCoeff_{};
+    template <class ViewType, class OutputParams, class InputParams, class ContainerType, class ScalarT>
+    friend BrineCo2Pvt<ScalarT, OutputParams, ViewType>
+    gpuistl::make_view(BrineCo2Pvt<ScalarT, InputParams, ContainerType>&);
+
+    ContainerT brineReferenceDensity_{};
+    ContainerT co2ReferenceDensity_{};
+    ContainerT salinity_{};
+    ContainerT ezrokhiDenNaClCoeff_{};
+    ContainerT ezrokhiDenCo2Coeff_{};
+    ContainerT ezrokhiViscNaClCoeff_{};
     bool enableEzrokhiDensity_ = false;
     bool enableEzrokhiViscosity_ = false;
     bool enableDissolution_ = true;
@@ -731,9 +803,50 @@ private:
     int activityModel_{};
     Co2StoreConfig::LiquidMixingType liquidMixType_{};
     Co2StoreConfig::SaltMixingType saltMixType_{};
-
+    Params co2Tables_;
 };
 
 } // namespace Opm
 
+namespace Opm::gpuistl
+{
+
+    template<class Params, class GPUContainer, class ScalarT>
+    BrineCo2Pvt<ScalarT, Params, GPUContainer>
+    copy_to_gpu(const BrineCo2Pvt<ScalarT>& cpuBrineCo2)
+    {
+        return BrineCo2Pvt<ScalarT, Params, GPUContainer>(
+            GPUContainer(cpuBrineCo2.getBrineReferenceDensity()),
+            GPUContainer(cpuBrineCo2.getCo2ReferenceDensity()),
+            GPUContainer(cpuBrineCo2.getSalinity()),
+            cpuBrineCo2.getActivityModel(),
+            cpuBrineCo2.getThermalMixingModelSalt(),
+            cpuBrineCo2.getThermalMixingModelLiquid(),
+            copy_to_gpu<GPUContainer>(cpuBrineCo2.getParams())
+        );
+    }
+
+    template <class ViewType, class OutputParams, class InputParams, class ContainerType, class ScalarT>
+    BrineCo2Pvt<ScalarT, OutputParams, ViewType>
+    make_view(BrineCo2Pvt<ScalarT, InputParams, ContainerType>& brineCo2Pvt)
+    {
+
+        using ContainedType = typename ViewType::value_type;
+
+        ViewType newBrineReferenceDensity = make_view<ContainedType>(brineCo2Pvt.brineReferenceDensity_);
+        ViewType newGasReferenceDensity = make_view<ContainedType>(brineCo2Pvt.co2ReferenceDensity_);
+        ViewType newSalinity = make_view<ContainedType>(brineCo2Pvt.salinity_);
+
+        return BrineCo2Pvt<ScalarT, OutputParams, ViewType>(
+            newBrineReferenceDensity,
+            newGasReferenceDensity,
+            newSalinity,
+            brineCo2Pvt.getActivityModel(),
+            brineCo2Pvt.getThermalMixingModelSalt(),
+            brineCo2Pvt.getThermalMixingModelLiquid(),
+            make_view<ViewType>(brineCo2Pvt.co2Tables_)
+        );
+    }
+}
+
 #endif
