@@ -29,6 +29,8 @@
 
 #include <opm/material/Constants.hpp>
 #include <opm/common/TimingMacros.hpp>
+#include <opm/common/ErrorMacros.hpp>
+#include <opm/common/utility/gpuDecorators.hpp>
 
 #include <opm/material/components/CO2.hpp>
 #include <opm/material/components/BrineDynamic.hpp>
@@ -36,6 +38,9 @@
 #include <opm/material/common/UniformTabulated2DFunction.hpp>
 #include <opm/material/binarycoefficients/Brine_CO2.hpp>
 #include <opm/input/eclipse/EclipseState/Co2StoreConfig.hpp>
+#include <opm/material/components/CO2Tables.hpp>
+#include <opm/input/eclipse/EclipseState/EclipseState.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
 
 #include <cstddef>
 #include <vector>
@@ -46,47 +51,76 @@ class EclipseState;
 class Schedule;
 class Co2StoreConfig;
 
+// forward declaration of the class so the function in the next namespace can be declared
+template <class Scalar, class ParamsT, class ContainerT>
+class Co2GasPvt;
+
+// declaration of make_view in correct namespace so friend function can be declared in the class
+namespace gpuistl {
+    template <class ViewType, class OutputParams, class InputParams, class ContainerType, class Scalar>
+    Co2GasPvt<Scalar, OutputParams, ViewType>
+    make_view(Co2GasPvt<Scalar, InputParams, ContainerType>&);
+}
+
 /*!
  * \brief This class represents the Pressure-Volume-Temperature relations of the gas phase
  *        for CO2.
  */
-template <class Scalar>
+template <class Scalar, class ParamsT = Opm::CO2Tables<double, std::vector<double>>, class ContainerT = std::vector<Scalar>>
 class Co2GasPvt
 {
-    using CO2 = ::Opm::CO2<Scalar>;
+    using CO2 = ::Opm::CO2<Scalar, ParamsT>;
     using H2O = SimpleHuDuanH2O<Scalar>;
     using Brine = ::Opm::BrineDynamic<Scalar, H2O>;
+    using Params = ParamsT;
     static constexpr bool extrapolate = true;
 
 public:
     //! The binary coefficients for brine and CO2 used by this fluid system
     using BinaryCoeffBrineCO2 = BinaryCoeff::Brine_CO2<Scalar, H2O, CO2>;
 
-    explicit Co2GasPvt() = default;
+    Co2GasPvt() = default;
 
-    explicit Co2GasPvt(const std::vector<Scalar>& salinity,
+    explicit Co2GasPvt(const ContainerT& salinity,
                        int activityModel = 3,
                        int thermalMixingModel = 1,
                        Scalar T_ref = 288.71, //(273.15 + 15.56)
                        Scalar P_ref = 101325);
 
+    Co2GasPvt(const Params& params,
+              const ContainerT& brineReferenceDensity,
+              const ContainerT& gasReferenceDensity,
+              const ContainerT& salinity,
+              bool enableEzrokhiDensity,
+              bool enableVaporization,
+              int activityModel,
+              Co2StoreConfig::GasMixingType gastype)
+        : brineReferenceDensity_(brineReferenceDensity)
+        , gasReferenceDensity_(gasReferenceDensity)
+        , salinity_(salinity)
+        , enableEzrokhiDensity_(enableEzrokhiDensity)
+        , enableVaporization_(enableVaporization)
+        , activityModel_(activityModel)
+        , gastype_(gastype)
+        , co2Tables(params)
+{
+    assert(enableEzrokhiDensity == false && "Ezrokhi density not supported by GPUs");
+}
+
 #if HAVE_ECL_INPUT
-    /*!
-     * \brief Initialize the parameters for CO2 gas using an ECL deck.
-     */
     void initFromState(const EclipseState& eclState, const Schedule&);
 #endif
 
     void setNumRegions(std::size_t numRegions);
 
-    void setVapPars(const Scalar, const Scalar)
+    OPM_HOST_DEVICE void setVapPars(const Scalar, const Scalar)
     {
     }
 
     /*!
      * \brief Initialize the reference densities of all fluids for a given PVT region
      */
-    void setReferenceDensities(unsigned regionIdx,
+    OPM_HOST_DEVICE void setReferenceDensities(unsigned regionIdx,
                                Scalar rhoRefBrine,
                                Scalar rhoRefGas,
                                Scalar /*rhoRefWater*/);
@@ -97,40 +131,40 @@ public:
      *
      * By default, vaporized water is considered.
      */
-    void setEnableVaporizationWater(bool yesno)
+    OPM_HOST_DEVICE void setEnableVaporizationWater(bool yesno)
     { enableVaporization_ = yesno; }
 
     /*!
     * \brief Set activity coefficient model for salt in solubility model
     */
-    void setActivityModelSalt(int activityModel);
+    OPM_HOST_DEVICE void setActivityModelSalt(int activityModel);
 
    /*!
     * \brief Set thermal mixing model for co2 in brine
     */
-    void setThermalMixingModel(int thermalMixingModel);
+    OPM_HOST_DEVICE void setThermalMixingModel(int thermalMixingModel);
 
     /*!
      * \brief Finish initializing the co2 phase PVT properties.
      */
-    void initEnd()
+    OPM_HOST_DEVICE void initEnd()
     {
     }
 
     /*!
      * \brief Return the number of PVT regions which are considered by this PVT-object.
      */
-    unsigned numRegions() const
+    OPM_HOST_DEVICE unsigned numRegions() const
     { return gasReferenceDensity_.size(); }
 
-    Scalar hVap(unsigned ) const
+    OPM_HOST_DEVICE Scalar hVap(unsigned ) const
     { return 0.0;  }
 
     /*!
      * \brief Returns the specific enthalpy [J/kg] of gas given a set of parameters.
      */
     template <class Evaluation>
-    Evaluation internalEnergy(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation internalEnergy(unsigned regionIdx,
                         const Evaluation& temperature,
                         const Evaluation& pressure,
                         const Evaluation& rv,
@@ -139,7 +173,7 @@ public:
         OPM_TIMEBLOCK_LOCAL(internalEnergy);
         if (gastype_ == Co2StoreConfig::GasMixingType::NONE) {
             // use the gasInternalEnergy of CO2
-            return CO2::gasInternalEnergy(temperature, pressure, extrapolate);
+            return CO2::gasInternalEnergy(co2Tables, temperature, pressure, extrapolate);
         }
 
         assert(gastype_ == Co2StoreConfig::GasMixingType::IDEAL);
@@ -150,7 +184,7 @@ public:
         assert(rv == 0.0 || rvw == 0.0);
         const Evaluation xBrine = convertRvwToXgW_(max(rvw,rv),regionIdx);
         result += xBrine * H2O::gasInternalEnergy(temperature, pressure);
-        result += (1 - xBrine) * CO2::gasInternalEnergy(temperature, pressure, extrapolate);
+        result += (1 - xBrine) * CO2::gasInternalEnergy(co2Tables, temperature, pressure, extrapolate);
         return result;
     }
 
@@ -159,7 +193,7 @@ public:
      *        given a set of parameters.
      */
     template <class Evaluation>
-    Evaluation viscosity(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation viscosity(unsigned regionIdx,
                          const Evaluation& temperature,
                          const Evaluation& pressure,
                          const Evaluation& /*Rv*/,
@@ -170,20 +204,20 @@ public:
      * \brief Returns the dynamic viscosity [Pa s] of fluid phase at saturated conditions.
      */
     template <class Evaluation>
-    Evaluation saturatedViscosity(unsigned /*regionIdx*/,
+    OPM_HOST_DEVICE Evaluation saturatedViscosity(unsigned /*regionIdx*/,
                                   const Evaluation& temperature,
                                   const Evaluation& pressure) const
     {
         OPM_TIMEBLOCK_LOCAL(saturatedViscosity);
         // Neglects impact of vaporized water on the visosity
-        return CO2::gasViscosity(temperature, pressure, extrapolate);
+        return CO2::gasViscosity(co2Tables, temperature, pressure, extrapolate);
     }
 
     /*!
      * \brief Returns the formation volume factor [-] of the fluid phase.
      */
     template <class Evaluation>
-    Evaluation inverseFormationVolumeFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation inverseFormationVolumeFactor(unsigned regionIdx,
                                             const Evaluation& temperature,
                                             const Evaluation& pressure,
                                             const Evaluation& rv,
@@ -191,12 +225,12 @@ public:
     {
         OPM_TIMEFUNCTION_LOCAL();
         if (!enableVaporization_) {
-            return CO2::gasDensity(temperature, pressure, extrapolate) /
+            return CO2::gasDensity(co2Tables, temperature, pressure, extrapolate) /
                    gasReferenceDensity_[regionIdx];
         }
 
         // Use CO2 density for the gas phase.
-        const auto& rhoCo2 = CO2::gasDensity(temperature, pressure, extrapolate);
+        const auto& rhoCo2 = CO2::gasDensity(co2Tables, temperature, pressure, extrapolate);
         //const auto& rhoH2O = H2O::gasDensity(temperature, pressure);
         //The CO2STORE option both works for GAS/WATER and GAS/OIL systems
         //Either rv og rvw should be zero
@@ -211,7 +245,7 @@ public:
      * \brief Returns the formation volume factor [-] of water saturated gas at given pressure.
      */
     template <class Evaluation>
-    Evaluation saturatedInverseFormationVolumeFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedInverseFormationVolumeFactor(unsigned regionIdx,
                                                      const Evaluation& temperature,
                                                      const Evaluation& pressure) const
     {
@@ -230,7 +264,7 @@ public:
      *            in what will yield one cubic meter of water at the surface [-]
      */
     template <class Evaluation>
-    Evaluation saturationPressure(unsigned /*regionIdx*/,
+    OPM_HOST_DEVICE Evaluation saturationPressure(unsigned /*regionIdx*/,
                                   const Evaluation& /*temperature*/,
                                   const Evaluation& /*Rvw*/) const
     { return 0.0; /* not implemented */ }
@@ -239,7 +273,7 @@ public:
      * \brief Returns the water vaporization factor \f$R_vw\f$ [m^3/m^3] of the water phase.
      */
     template <class Evaluation>
-    Evaluation saturatedWaterVaporizationFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedWaterVaporizationFactor(unsigned regionIdx,
                                               const Evaluation& temperature,
                                               const Evaluation& pressure) const
     { return rvwSat_(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx])); }
@@ -248,7 +282,7 @@ public:
     * \brief Returns the water vaporization factor \f$R_vw\f$ [m^3/m^3] of water phase.
     */
     template <class Evaluation = Scalar>
-    Evaluation saturatedWaterVaporizationFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedWaterVaporizationFactor(unsigned regionIdx,
                                               const Evaluation& temperature,
                                               const Evaluation& pressure,
                                               const Evaluation& saltConcentration) const
@@ -263,7 +297,7 @@ public:
      * \brief Returns the oil vaporization factor \f$R_v\f$ [m^3/m^3] of the oil phase.
      */
     template <class Evaluation>
-    Evaluation saturatedOilVaporizationFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedOilVaporizationFactor(unsigned regionIdx,
                                               const Evaluation& temperature,
                                               const Evaluation& pressure,
                                               const Evaluation& /*oilSaturation*/,
@@ -274,44 +308,69 @@ public:
      * \brief Returns the oil vaporization factor \f$R_v\f$ [m^3/m^3] of the oil phase.
      */
     template <class Evaluation>
-    Evaluation saturatedOilVaporizationFactor(unsigned regionIdx,
+    OPM_HOST_DEVICE Evaluation saturatedOilVaporizationFactor(unsigned regionIdx,
                                               const Evaluation& temperature,
                                               const Evaluation& pressure) const
     { return rvwSat_(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx])); }
 
     template <class Evaluation>
-    Evaluation diffusionCoefficient(const Evaluation& temperature,
+    OPM_HOST_DEVICE Evaluation diffusionCoefficient(const Evaluation& temperature,
                                     const Evaluation& pressure,
                                     unsigned /*compIdx*/) const
     {
-        return BinaryCoeffBrineCO2::gasDiffCoeff(temperature, pressure, extrapolate);
+        return BinaryCoeffBrineCO2::gasDiffCoeff(co2Tables, temperature, pressure, extrapolate);
     }
 
-    Scalar gasReferenceDensity(unsigned regionIdx) const
+    OPM_HOST_DEVICE Scalar gasReferenceDensity(unsigned regionIdx) const
     { return gasReferenceDensity_[regionIdx]; }
 
-    Scalar oilReferenceDensity(unsigned regionIdx) const
+    OPM_HOST_DEVICE Scalar oilReferenceDensity(unsigned regionIdx) const
     { return brineReferenceDensity_[regionIdx]; }
 
-    Scalar waterReferenceDensity(unsigned regionIdx) const
+    OPM_HOST_DEVICE Scalar waterReferenceDensity(unsigned regionIdx) const
     { return brineReferenceDensity_[regionIdx]; }
 
-    Scalar salinity(unsigned regionIdx) const
+    OPM_HOST_DEVICE Scalar salinity(unsigned regionIdx) const
     { return salinity_[regionIdx]; }
 
     void setEzrokhiDenCoeff(const std::vector<EzrokhiTable>& denaqa);
 
+    // new get functions that will be needed to move a cpu based Co2GasPvt object to the GPU
+    OPM_HOST_DEVICE const ContainerT& getBrineReferenceDensity() const
+    { return brineReferenceDensity_; }
+
+    OPM_HOST_DEVICE const ContainerT& getGasReferenceDensity() const
+    { return gasReferenceDensity_; }
+
+    OPM_HOST_DEVICE const ContainerT& getSalinity() const
+    { return salinity_; }
+
+    OPM_HOST_DEVICE bool getEnableEzrokhiDensity() const
+    { return enableEzrokhiDensity_; }
+
+    OPM_HOST_DEVICE bool getEnableVaporization() const
+    { return enableVaporization_; }
+
+    OPM_HOST_DEVICE int getActivityModel() const
+    { return activityModel_; }
+
+    OPM_HOST_DEVICE Co2StoreConfig::GasMixingType getGasType() const
+    { return gastype_; }
+
+    OPM_HOST_DEVICE const Params& getParams() const
+    { return co2Tables; }
+
 private:
     template <class LhsEval>
     LhsEval ezrokhiExponent_(const LhsEval& temperature,
-                             const std::vector<Scalar>& ezrokhiCoeff) const
+                             const ContainerT& ezrokhiCoeff) const
     {
         const LhsEval& tempC = temperature - 273.15;
         return ezrokhiCoeff[0] + tempC * (ezrokhiCoeff[1] + ezrokhiCoeff[2] * tempC);
     }
 
     template <class LhsEval>
-    LhsEval rvwSat_(unsigned regionIdx,
+    OPM_HOST_DEVICE LhsEval rvwSat_(unsigned regionIdx,
                     const LhsEval& temperature,
                     const LhsEval& pressure,
                     const LhsEval& salinity) const
@@ -325,7 +384,8 @@ private:
         // temperature and pressure.
         LhsEval xgH2O;
         LhsEval xlCO2;
-        BinaryCoeffBrineCO2::calculateMoleFractions(temperature,
+        BinaryCoeffBrineCO2::calculateMoleFractions(co2Tables,
+                                                    temperature,
                                                     pressure,
                                                     salinity,
                                                     /*knownPhaseIdx=*/-1,
@@ -345,7 +405,7 @@ private:
      *        corresponding water vaporization factor.
      */
     template <class LhsEval>
-    LhsEval convertXgWToRvw(const LhsEval& XgW, unsigned regionIdx) const
+    OPM_HOST_DEVICE LhsEval convertXgWToRvw(const LhsEval& XgW, unsigned regionIdx) const
     {
         OPM_TIMEFUNCTION_LOCAL();
         Scalar rho_wRef = brineReferenceDensity_[regionIdx];
@@ -359,7 +419,7 @@ private:
      *        of the water component in the gas phase.
      */
     template <class LhsEval>
-    LhsEval convertRvwToXgW_(const LhsEval& Rvw, unsigned regionIdx) const
+    OPM_HOST_DEVICE LhsEval convertRvwToXgW_(const LhsEval& Rvw, unsigned regionIdx) const
     {
         OPM_TIMEFUNCTION_LOCAL();
         Scalar rho_wRef = brineReferenceDensity_[regionIdx];
@@ -372,7 +432,7 @@ private:
      * \brief Convert a water mole fraction in the gas phase the corresponding mass fraction.
      */
     template <class LhsEval>
-    LhsEval convertxgWToXgW(const LhsEval& xgW, const LhsEval& salinity) const
+    OPM_HOST_DEVICE LhsEval convertxgWToXgW(const LhsEval& xgW, const LhsEval& salinity) const
     {
         OPM_TIMEFUNCTION_LOCAL();
         Scalar M_CO2 = CO2::molarMass();
@@ -381,21 +441,65 @@ private:
         return xgW * M_Brine / (xgW * (M_Brine - M_CO2) + M_CO2);
     }
 
+    template <class ViewType, class OutputParams, class InputParams, class ContainerType, class ScalarT>
+    friend Co2GasPvt<ScalarT, OutputParams, ViewType>
+    gpuistl::make_view(Co2GasPvt<ScalarT, InputParams, ContainerType>&);
+
     template <class LhsEval>
-    const LhsEval salinityFromConcentration(const LhsEval&T, const LhsEval& P,
+    OPM_HOST_DEVICE const LhsEval salinityFromConcentration(const LhsEval&T, const LhsEval& P,
                                             const LhsEval& saltConcentration) const
     { return saltConcentration/H2O::liquidDensity(T, P, true); }
 
-    std::vector<Scalar> brineReferenceDensity_{};
-    std::vector<Scalar> gasReferenceDensity_{};
-    std::vector<Scalar> salinity_{};
-    std::vector<Scalar> ezrokhiDenNaClCoeff_{};
+    ContainerT brineReferenceDensity_{};
+    ContainerT gasReferenceDensity_{};
+    ContainerT salinity_{};
+    ContainerT ezrokhiDenNaClCoeff_{};
     bool enableEzrokhiDensity_ = false;
     bool enableVaporization_ = true;
     int activityModel_{};
     Co2StoreConfig::GasMixingType gastype_{};
+    Params co2Tables;
 };
 
 } // namespace Opm
 
+namespace Opm::gpuistl{
+    template<class GPUContainer, class Params, class ScalarT>
+    Co2GasPvt<ScalarT, Params, GPUContainer>
+    copy_to_gpu(const Co2GasPvt<ScalarT>& cpuCo2)
+    {
+        return Co2GasPvt<ScalarT, Params, GPUContainer>(
+            copy_to_gpu<GPUContainer>(cpuCo2.getParams()),
+            GPUContainer(cpuCo2.getBrineReferenceDensity()),
+            GPUContainer(cpuCo2.getGasReferenceDensity()),
+            GPUContainer(cpuCo2.getSalinity()),
+            cpuCo2.getEnableEzrokhiDensity(),
+            cpuCo2.getEnableVaporization(),
+            cpuCo2.getActivityModel(),
+            cpuCo2.getGasType());
+    }
+
+    template <class ViewType, class OutputParams, class InputParams, class ContainerType, class ScalarT>
+    Co2GasPvt<ScalarT, OutputParams, ViewType>
+    make_view(Co2GasPvt<ScalarT, InputParams, ContainerType>& co2GasPvt)
+    {
+        using ContainedType = typename ContainerType::value_type;
+
+        ViewType newBrineReferenceDensity = make_view<ContainedType>(co2GasPvt.brineReferenceDensity_);
+        ViewType newGasReferenceDensity = make_view<ContainedType>(co2GasPvt.gasReferenceDensity_);
+        ViewType newSalinity = make_view<ContainedType>(co2GasPvt.salinity_);
+
+        return Co2GasPvt<ScalarT, OutputParams, ViewType>(
+            make_view<ViewType>(co2GasPvt.co2Tables),
+            newBrineReferenceDensity,
+            newGasReferenceDensity,
+            newSalinity,
+            co2GasPvt.getEnableEzrokhiDensity(),
+            co2GasPvt.getEnableVaporization(),
+            co2GasPvt.getActivityModel(),
+            co2GasPvt.getGasType());
+    }
+}
+
+
 #endif
