@@ -27,41 +27,42 @@
 #ifndef OPM_WATER_PVT_MULTIPLEXER_HPP
 #define OPM_WATER_PVT_MULTIPLEXER_HPP
 
-#include "ConstantCompressibilityWaterPvt.hpp"
-#include "ConstantCompressibilityBrinePvt.hpp"
-#include "WaterPvtThermal.hpp"
-#include "BrineCo2Pvt.hpp"
-#include "BrineH2Pvt.hpp"
+#include <opm/material/fluidsystems/blackoilpvt/BrineCo2Pvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/BrineH2Pvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/ConstantCompressibilityWaterPvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/ConstantCompressibilityBrinePvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/WaterPvtThermal.hpp>
 
-#define OPM_WATER_PVT_MULTIPLEXER_CALL(codeToCall)                      \
-    switch (approach_) {                                                \
-    case WaterPvtApproach::ConstantCompressibilityWater: {           \
+#define OPM_WATER_PVT_MULTIPLEXER_CALL(codeToCall, ...)                                \
+    switch (approach_) {                                                               \
+    case WaterPvtApproach::ConstantCompressibilityWater: {                             \
         auto& pvtImpl = getRealPvt<WaterPvtApproach::ConstantCompressibilityWater>();  \
-        codeToCall;                                                     \
-        break;                                                          \
-    }                                                                   \
-    case WaterPvtApproach::ConstantCompressibilityBrine: {           \
+        codeToCall;                                                                    \
+        __VA_ARGS__;                                                                   \
+    }                                                                                  \
+    case WaterPvtApproach::ConstantCompressibilityBrine: {                             \
         auto& pvtImpl = getRealPvt<WaterPvtApproach::ConstantCompressibilityBrine>();  \
-        codeToCall;                                                     \
-        break;                                                          \
-    }                                                                   \
-    case WaterPvtApproach::ThermalWater: {                           \
-        auto& pvtImpl = getRealPvt<WaterPvtApproach::ThermalWater>();   \
-        codeToCall;                                                     \
-        break;                                                          \
-    }                                                                    \
-    case WaterPvtApproach::BrineCo2: {                                              \
-        auto& pvtImpl = getRealPvt<WaterPvtApproach::BrineCo2>();                   \
-        codeToCall;                                                                  \
-        break;                                                                       \
-    }                                                                    \
-    case WaterPvtApproach::BrineH2: {                                              \
-        auto& pvtImpl = getRealPvt<WaterPvtApproach::BrineH2>();                   \
-        codeToCall;                                                                  \
-        break;                                                                       \
-    }                                                                    \
-    case WaterPvtApproach::NoWater:                                  \
-        throw std::logic_error("Not implemented: Water PVT of this deck!"); \
+        codeToCall;                                                                    \
+        __VA_ARGS__;                                                                   \
+    }                                                                                  \
+    case WaterPvtApproach::ThermalWater: {                                             \
+        auto& pvtImpl = getRealPvt<WaterPvtApproach::ThermalWater>();                  \
+        codeToCall;                                                                    \
+        __VA_ARGS__;                                                                   \
+    }                                                                                  \
+    case WaterPvtApproach::BrineCo2: {                                                 \
+        auto& pvtImpl = getRealPvt<WaterPvtApproach::BrineCo2>();                      \
+        codeToCall;                                                                    \
+        __VA_ARGS__;                                                                   \
+    }                                                                                  \
+    case WaterPvtApproach::BrineH2: {                                                  \
+        auto& pvtImpl = getRealPvt<WaterPvtApproach::BrineH2>();                       \
+        codeToCall;                                                                    \
+        __VA_ARGS__;                                                                   \
+    }                                                                                  \
+    default:                                                                           \
+    case WaterPvtApproach::NoWater:                                                    \
+        throw std::logic_error("Not implemented: Water PVT of this deck!");            \
     }
 
 namespace Opm {
@@ -89,9 +90,9 @@ class WaterPvtMultiplexer
 {
 public:
     WaterPvtMultiplexer()
+        : approach_(WaterPvtApproach::NoWater)
+        , realWaterPvt_(nullptr)
     {
-        approach_ = WaterPvtApproach::NoWater;
-        realWaterPvt_ = nullptr;
     }
 
     WaterPvtMultiplexer(WaterPvtApproach approach, void* realWaterPvt)
@@ -104,32 +105,11 @@ public:
         *this = data;
     }
 
-    ~WaterPvtMultiplexer()
+    ~WaterPvtMultiplexer();
+
+    bool mixingEnergy() const
     {
-        switch (approach_) {
-        case WaterPvtApproach::ConstantCompressibilityWater: {
-            delete &getRealPvt<WaterPvtApproach::ConstantCompressibilityWater>();
-            break;
-        }
-        case WaterPvtApproach::ConstantCompressibilityBrine: {
-            delete &getRealPvt<WaterPvtApproach::ConstantCompressibilityBrine>();
-            break;
-        }
-        case WaterPvtApproach::ThermalWater: {
-            delete &getRealPvt<WaterPvtApproach::ThermalWater>();
-            break;
-        }
-        case WaterPvtApproach::BrineCo2: {
-            delete &getRealPvt<WaterPvtApproach::BrineCo2>();
-            break;
-        }
-        case WaterPvtApproach::BrineH2: {
-            delete &getRealPvt<WaterPvtApproach::BrineH2>();
-            break;
-        }
-        case WaterPvtApproach::NoWater:
-            break;
-        }
+        return approach_ == WaterPvtApproach::ThermalWater;
     }
 
 #if HAVE_ECL_INPUT
@@ -141,25 +121,19 @@ public:
     void initFromState(const EclipseState& eclState, const Schedule& schedule);
 #endif // HAVE_ECL_INPUT
 
-    void initEnd()
-    { OPM_WATER_PVT_MULTIPLEXER_CALL(pvtImpl.initEnd()); }
+    void initEnd();
 
     /*!
      * \brief Return the number of PVT regions which are considered by this PVT-object.
      */
-    unsigned numRegions() const
-    { OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.numRegions()); return 1; }
+    unsigned numRegions() const;
 
-    void setVapPars(const Scalar par1, const Scalar par2)
-    {
-        OPM_WATER_PVT_MULTIPLEXER_CALL(pvtImpl.setVapPars(par1, par2));
-    }
+    void setVapPars(const Scalar par1, const Scalar par2);
 
     /*!
      * \brief Return the reference density which are considered by this PVT-object.
      */
-    const Scalar waterReferenceDensity(unsigned regionIdx) const
-    { OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.waterReferenceDensity(regionIdx)); return 1000.; }
+    Scalar waterReferenceDensity(unsigned regionIdx) const;
 
     /*!
      * \brief Returns the specific enthalpy [J/kg] of gas given a set of parameters.
@@ -170,7 +144,9 @@ public:
                         const Evaluation& pressure,
                         const Evaluation& Rsw,
                         const Evaluation& saltconcentration) const
-    { OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.internalEnergy(regionIdx, temperature, pressure, Rsw, saltconcentration)); return 0; }
+    { OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.internalEnergy(regionIdx, temperature, pressure, Rsw, saltconcentration)); }
+
+    Scalar hVap(unsigned regionIdx) const;
 
     /*!
      * \brief Returns the dynamic viscosity [Pa s] of the fluid phase given a set of parameters.
@@ -183,7 +159,6 @@ public:
                          const Evaluation& saltconcentration) const
     {
         OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.viscosity(regionIdx, temperature, pressure, Rsw, saltconcentration));
-        return 0;
     }
 
     /*!
@@ -196,7 +171,6 @@ public:
                                   const Evaluation& saltconcentration) const
     {
         OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedViscosity(regionIdx, temperature, pressure, saltconcentration));
-        return 0;
     }
 
     /*!
@@ -208,9 +182,8 @@ public:
                                             const Evaluation& pressure,
                                             const Evaluation& Rsw,
                                             const Evaluation& saltconcentration) const
-    {   
+    {
         OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.inverseFormationVolumeFactor(regionIdx, temperature, pressure, Rsw, saltconcentration));
-        return 0;
     }
 
         /*!
@@ -221,9 +194,8 @@ public:
                                                      const Evaluation& temperature,
                                                      const Evaluation& pressure,
                                                      const Evaluation& saltconcentration) const
-    {   
+    {
         OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedInverseFormationVolumeFactor(regionIdx, temperature, pressure, saltconcentration));
-        return 0;
     }
 
     /*!
@@ -234,9 +206,8 @@ public:
                                              const Evaluation& temperature,
                                              const Evaluation& pressure,
                                              const Evaluation& saltconcentration) const
-    { 
-        OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedGasDissolutionFactor(regionIdx, temperature, pressure, saltconcentration)); 
-        return 0; 
+    {
+        OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedGasDissolutionFactor(regionIdx, temperature, pressure, saltconcentration));
     }
 
     /*!
@@ -251,8 +222,7 @@ public:
                                   const Evaluation& temperature,
                                   const Evaluation& Rs,
                                   const Evaluation& saltconcentration) const
-    { OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.saturationPressure(regionIdx, temperature, Rs, saltconcentration)); return 0; }
-
+    { OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.saturationPressure(regionIdx, temperature, Rs, saltconcentration)); }
 
     /*!
      * \copydoc BaseFluidSystem::diffusionCoefficient
@@ -262,39 +232,10 @@ public:
                                     const Evaluation& pressure,
                                     unsigned compIdx) const
     {
-      OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.diffusionCoefficient(temperature, pressure, compIdx)); 
-      return 0;
+      OPM_WATER_PVT_MULTIPLEXER_CALL(return pvtImpl.diffusionCoefficient(temperature, pressure, compIdx));
     }
 
-    void setApproach(WaterPvtApproach appr)
-    {
-        switch (appr) {
-        case WaterPvtApproach::ConstantCompressibilityWater:
-            realWaterPvt_ = new ConstantCompressibilityWaterPvt<Scalar>;
-            break;
-
-        case WaterPvtApproach::ConstantCompressibilityBrine:
-            realWaterPvt_ = new ConstantCompressibilityBrinePvt<Scalar>;
-            break;
-
-        case WaterPvtApproach::ThermalWater:
-            realWaterPvt_ = new WaterPvtThermal<Scalar, enableBrine>;
-            break;
-
-        case WaterPvtApproach::BrineCo2:
-            realWaterPvt_ = new BrineCo2Pvt<Scalar>;
-            break;
-
-        case WaterPvtApproach::BrineH2:
-            realWaterPvt_ = new BrineH2Pvt<Scalar>;
-            break;
-
-        case WaterPvtApproach::NoWater:
-            throw std::logic_error("Not implemented: Water PVT of this deck!");
-        }
-
-        approach_ = appr;
-    }
+    void setApproach(WaterPvtApproach appr);
 
     /*!
      * \brief Returns the concrete approach for calculating the PVT relations.
@@ -377,35 +318,12 @@ public:
 
     const void* realWaterPvt() const { return realWaterPvt_; }
 
-    WaterPvtMultiplexer<Scalar,enableThermal,enableBrine>& operator=(const WaterPvtMultiplexer<Scalar,enableThermal,enableBrine>& data)
-    {
-        approach_ = data.approach_;
-        switch (approach_) {
-        case WaterPvtApproach::ConstantCompressibilityWater:
-            realWaterPvt_ = new ConstantCompressibilityWaterPvt<Scalar>(*static_cast<const ConstantCompressibilityWaterPvt<Scalar>*>(data.realWaterPvt_));
-            break;
-        case WaterPvtApproach::ConstantCompressibilityBrine:
-            realWaterPvt_ = new ConstantCompressibilityBrinePvt<Scalar>(*static_cast<const ConstantCompressibilityBrinePvt<Scalar>*>(data.realWaterPvt_));
-            break;
-        case WaterPvtApproach::ThermalWater:
-            realWaterPvt_ = new WaterPvtThermal<Scalar, enableBrine>(*static_cast<const WaterPvtThermal<Scalar, enableBrine>*>(data.realWaterPvt_));
-            break;
-        case WaterPvtApproach::BrineCo2:
-            realWaterPvt_ = new BrineCo2Pvt<Scalar>(*static_cast<const BrineCo2Pvt<Scalar>*>(data.realWaterPvt_));
-            break;
-        case WaterPvtApproach::BrineH2:
-            realWaterPvt_ = new BrineH2Pvt<Scalar>(*static_cast<const BrineH2Pvt<Scalar>*>(data.realWaterPvt_));
-            break;
-        default:
-            break;
-        }
-
-        return *this;
-    }
+    WaterPvtMultiplexer<Scalar,enableThermal,enableBrine>&
+    operator=(const WaterPvtMultiplexer<Scalar,enableThermal,enableBrine>& data);
 
 private:
-    WaterPvtApproach approach_;
-    void* realWaterPvt_;
+    WaterPvtApproach approach_{WaterPvtApproach::NoWater};
+    void* realWaterPvt_{nullptr};
 };
 
 } // namespace Opm
