@@ -34,6 +34,8 @@
 #include <opm/material/common/UniformXTabulated2DFunction.hpp>
 #include <opm/material/common/Tabulated1DFunction.hpp>
 
+#include <cstddef>
+
 namespace Opm {
 
 #if HAVE_ECL_INPUT
@@ -77,7 +79,7 @@ private:
 public:
 #endif // HAVE_ECL_INPUT
 
-    void setNumRegions(size_t numRegions);
+    void setNumRegions(std::size_t numRegions);
 
     void setVapPars(const Scalar par1, const Scalar)
     {
@@ -97,7 +99,8 @@ public:
      *
      * \param samplePoints A container of (x,y) values.
      */
-    void setSaturatedGasWaterVaporizationFactor(unsigned regionIdx, const SamplingPoints& samplePoints)
+    void setSaturatedGasWaterVaporizationFactor(unsigned regionIdx,
+                                                const SamplingPoints& samplePoints)
     { saturatedWaterVaporizationFactorTable_[regionIdx].setContainerOfTuples(samplePoints); }
 
     /*!
@@ -105,10 +108,10 @@ public:
      *
      * \param samplePoints A container of (x,y) values.
      */
-    void setSaturatedGasOilVaporizationFactor(unsigned regionIdx, const SamplingPoints& samplePoints)
+    void setSaturatedGasOilVaporizationFactor(unsigned regionIdx,
+                                              const SamplingPoints& samplePoints)
     { saturatedOilVaporizationFactorTable_[regionIdx].setContainerOfTuples(samplePoints); }
 
-
     /*!
      * \brief Finish initializing the gas phase PVT properties.
      */
@@ -125,12 +128,19 @@ public:
      */
     template <class Evaluation>
     Evaluation internalEnergy(unsigned,
-                        const Evaluation&,
-                        const Evaluation&,
-                        const Evaluation&,
-                        const Evaluation&) const
+                              const Evaluation&,
+                              const Evaluation&,
+                              const Evaluation&,
+                              const Evaluation&) const
     {
-        throw std::runtime_error("Requested the enthalpy of gas but the thermal option is not enabled");
+        throw std::runtime_error("Requested the enthalpy of gas but the thermal "
+                                 "option is not enabled");
+    }
+
+    Scalar hVap(unsigned) const
+    {
+        throw std::runtime_error("Requested the hvap of oil but the thermal "
+                                 "option is not enabled");
     }
 
     /*!
@@ -145,16 +155,16 @@ public:
     {
         const Evaluation& temperature = 1E30;
 
-        if (Rv >= (1.0 - 1e-10)*saturatedOilVaporizationFactor(regionIdx, temperature, pressure)) {
+        if (Rv >= (1.0 - 1e-10) * saturatedOilVaporizationFactor(regionIdx, temperature, pressure)) {
             const Evaluation& invBg = inverseGasBRvSat_[regionIdx].eval(pressure, Rvw, /*extrapolate=*/true);
             const Evaluation& invMugBg = inverseGasBMuRvSat_[regionIdx].eval(pressure, Rvw, /*extrapolate=*/true);
-            return invBg/invMugBg;
+            return invBg / invMugBg;
         }
         else {
             // for Rv undersaturated viscosity is evaluated at saturated Rvw values
             const Evaluation& invBg = inverseGasBRvwSat_[regionIdx].eval(pressure, Rv, /*extrapolate=*/true);
             const Evaluation& invMugBg = inverseGasBMuRvwSat_[regionIdx].eval(pressure, Rv, /*extrapolate=*/true);
-            return invBg/invMugBg;
+            return invBg / invMugBg;
         }
     }
 
@@ -169,7 +179,7 @@ public:
         const Evaluation& invBg = inverseSaturatedGasB_[regionIdx].eval(pressure, /*extrapolate=*/true);
         const Evaluation& invMugBg = inverseSaturatedGasBMu_[regionIdx].eval(pressure, /*extrapolate=*/true);
 
-        return invBg/invMugBg;
+        return invBg / invMugBg;
     }
 
     /*!
@@ -190,18 +200,16 @@ public:
                                             const Evaluation& Rvw) const
     {
         const Evaluation& temperature = 1E30;
-        
-        if (Rv >= (1.0 - 1e-10)*saturatedOilVaporizationFactor(regionIdx, temperature, pressure)) {
+
+        if (Rv >= (1.0 - 1e-10) * saturatedOilVaporizationFactor(regionIdx, temperature, pressure)) {
             return inverseGasBRvSat_[regionIdx].eval(pressure, Rvw, /*extrapolate=*/true);
         }
         else {
             // for Rv undersaturated Bg^-1 is evaluated at saturated Rvw values
             return inverseGasBRvwSat_[regionIdx].eval(pressure, Rv, /*extrapolate=*/true);
         }
-
     }
 
-
     /*!
      * \brief Returns the formation volume factor [-] of water saturated gas at a given pressure.
      */
@@ -216,8 +224,8 @@ public:
      */
     template <class Evaluation>
     Evaluation saturatedWaterVaporizationFactor(unsigned regionIdx,
-                                              const Evaluation& /*temperature*/,
-                                              const Evaluation& pressure) const
+                                                const Evaluation& /*temperature*/,
+                                                const Evaluation& pressure) const
     {
         return saturatedWaterVaporizationFactorTable_[regionIdx].eval(pressure, /*extrapolate=*/true);
     }
@@ -227,19 +235,19 @@ public:
     */
     template <class Evaluation>
     Evaluation saturatedWaterVaporizationFactor(unsigned regionIdx,
-                                              const Evaluation& /*temperature*/,
-                                              const Evaluation& pressure,
-                                              const Evaluation& saltConcentration) const
+                                                const Evaluation& /*temperature*/,
+                                                const Evaluation& pressure,
+                                                const Evaluation& saltConcentration) const
     {
-        if (enableRwgSalt_)
+        if (enableRwgSalt_) {
             return saturatedWaterVaporizationSaltFactorTable_[regionIdx].eval(pressure, saltConcentration, /*extrapolate=*/true);
+        }
         else {
             return saturatedWaterVaporizationFactorTable_[regionIdx].eval(pressure, /*extrapolate=*/true);
         }
-
     }
 
-   template <class Evaluation>
+    template <class Evaluation>
     Evaluation saturatedOilVaporizationFactor(unsigned regionIdx,
                                               const Evaluation& /*temperature*/,
                                               const Evaluation& pressure) const
@@ -270,7 +278,7 @@ public:
         if (vapPar1_ > 0.0 && maxOilSaturation > 0.01 && oilSaturation < maxOilSaturation) {
             constexpr const Scalar eps = 0.001;
             const Evaluation& So = max(oilSaturation, eps);
-            tmp *= max(1e-3, pow(So/maxOilSaturation, vapPar1_));
+            tmp *= max(1e-3, pow(So / maxOilSaturation, vapPar1_));
         }
 
         return tmp;
@@ -292,7 +300,7 @@ public:
         using Toolbox = MathToolbox<Evaluation>;
 
         const auto& RwTable = saturatedWaterVaporizationFactorTable_[regionIdx];
-        constexpr const Scalar eps = std::numeric_limits<typename Toolbox::Scalar>::epsilon()*1e6;
+        constexpr const Scalar eps = std::numeric_limits<typename Toolbox::Scalar>::epsilon() * 1e6;
 
         // use the tabulated saturation pressure function to get a pretty good initial value
         Evaluation pSat = saturationPressure_[regionIdx].eval(Rw, /*extrapolate=*/true);
@@ -311,22 +319,24 @@ public:
                 return pSat;
             }
 
-            const Evaluation& delta = f/fPrime;
+            const Evaluation& delta = f / fPrime;
 
             pSat -= delta;
 
             if (pSat < 0.0) {
                 // if the pressure is lower than 0 Pascals, we set it back to 0. if this
                 // happens twice, we give up and just return 0 Pa...
-                if (onProbation)
+                if (onProbation) {
                     return 0.0;
+                }
 
                 onProbation = true;
                 pSat = 0.0;
             }
 
-            if (std::abs(scalarValue(delta)) < std::abs(scalarValue(pSat))*eps)
+            if (std::abs(scalarValue(delta)) < std::abs(scalarValue(pSat))*eps) {
                 return pSat;
+            }
         }
 
         const std::string msg =
@@ -342,7 +352,8 @@ public:
                                     const Evaluation& /*pressure*/,
                                     unsigned /*compIdx*/) const
     {
-        throw std::runtime_error("Not implemented: The PVT model does not provide a diffusionCoefficient()");
+        throw std::runtime_error("Not implemented: The PVT model does not provide "
+                                 "a diffusionCoefficient()");
     }
 
     Scalar gasReferenceDensity(unsigned regionIdx) const
@@ -354,64 +365,54 @@ public:
     Scalar waterReferenceDensity(unsigned regionIdx) const
     { return waterReferenceDensity_[regionIdx]; }
 
-    const std::vector<TabulatedTwoDFunction>& inverseGasB() const {
-        return inverseGasBRvSat_;
-    }
+    const std::vector<TabulatedTwoDFunction>& inverseGasB() const
+    { return inverseGasBRvSat_; }
 
-    const std::vector<TabulatedOneDFunction>& inverseSaturatedGasB() const {
-        return inverseSaturatedGasB_;
-    }
+    const std::vector<TabulatedOneDFunction>& inverseSaturatedGasB() const
+    { return inverseSaturatedGasB_; }
 
-    const std::vector<TabulatedTwoDFunction>& gasMu() const {
-        return gasMuRvSat_;
-    }
+    const std::vector<TabulatedTwoDFunction>& gasMu() const
+    { return gasMuRvSat_; }
 
-    const std::vector<TabulatedTwoDFunction>& inverseGasBMu() const {
-        return inverseGasBMuRvSat_;
-    }
+    const std::vector<TabulatedTwoDFunction>& inverseGasBMu() const
+    { return inverseGasBMuRvSat_; }
 
-    const std::vector<TabulatedOneDFunction>& inverseSaturatedGasBMu() const {
-        return inverseSaturatedGasBMu_;
-    }
+    const std::vector<TabulatedOneDFunction>& inverseSaturatedGasBMu() const
+    { return inverseSaturatedGasBMu_; }
 
-    const std::vector<TabulatedOneDFunction>& saturatedWaterVaporizationFactorTable() const {
-        return saturatedWaterVaporizationFactorTable_;
-    }
+    const std::vector<TabulatedOneDFunction>& saturatedWaterVaporizationFactorTable() const
+    { return saturatedWaterVaporizationFactorTable_; }
 
-    const std::vector<TabulatedTwoDFunction>& saturatedWaterVaporizationSaltFactorTable() const {
-        return saturatedWaterVaporizationSaltFactorTable_;
-    }
+    const std::vector<TabulatedTwoDFunction>& saturatedWaterVaporizationSaltFactorTable() const
+    { return saturatedWaterVaporizationSaltFactorTable_; }
 
-    const std::vector<TabulatedOneDFunction>& saturatedOilVaporizationFactorTable() const {
-        return saturatedOilVaporizationFactorTable_;
-    }
+    const std::vector<TabulatedOneDFunction>& saturatedOilVaporizationFactorTable() const
+    { return saturatedOilVaporizationFactorTable_; }
 
-    const std::vector<TabulatedOneDFunction>& saturationPressure() const {
-        return saturationPressure_;
-    }
+    const std::vector<TabulatedOneDFunction>& saturationPressure() const
+    { return saturationPressure_; }
 
-    Scalar vapPar1() const {
-        return vapPar1_;
-    }
+    Scalar vapPar1() const
+    { return vapPar1_; }
 
 private:
     void updateSaturationPressure_(unsigned regionIdx);
 
-    std::vector<Scalar> gasReferenceDensity_;
-    std::vector<Scalar> oilReferenceDensity_;
-    std::vector<Scalar> waterReferenceDensity_;
-    std::vector<TabulatedTwoDFunction> inverseGasBRvwSat_;
-    std::vector<TabulatedTwoDFunction> inverseGasBRvSat_;
-    std::vector<TabulatedOneDFunction> inverseSaturatedGasB_;
-    std::vector<TabulatedTwoDFunction> gasMuRvwSat_;
-    std::vector<TabulatedTwoDFunction> gasMuRvSat_;
-    std::vector<TabulatedTwoDFunction> inverseGasBMuRvwSat_;
-    std::vector<TabulatedTwoDFunction> inverseGasBMuRvSat_;
-    std::vector<TabulatedOneDFunction> inverseSaturatedGasBMu_;
-    std::vector<TabulatedOneDFunction> saturatedWaterVaporizationFactorTable_;
-    std::vector<TabulatedTwoDFunction> saturatedWaterVaporizationSaltFactorTable_;
-    std::vector<TabulatedOneDFunction> saturatedOilVaporizationFactorTable_;
-    std::vector<TabulatedOneDFunction> saturationPressure_;
+    std::vector<Scalar> gasReferenceDensity_{};
+    std::vector<Scalar> oilReferenceDensity_{};
+    std::vector<Scalar> waterReferenceDensity_{};
+    std::vector<TabulatedTwoDFunction> inverseGasBRvwSat_{};
+    std::vector<TabulatedTwoDFunction> inverseGasBRvSat_{};
+    std::vector<TabulatedOneDFunction> inverseSaturatedGasB_{};
+    std::vector<TabulatedTwoDFunction> gasMuRvwSat_{};
+    std::vector<TabulatedTwoDFunction> gasMuRvSat_{};
+    std::vector<TabulatedTwoDFunction> inverseGasBMuRvwSat_{};
+    std::vector<TabulatedTwoDFunction> inverseGasBMuRvSat_{};
+    std::vector<TabulatedOneDFunction> inverseSaturatedGasBMu_{};
+    std::vector<TabulatedOneDFunction> saturatedWaterVaporizationFactorTable_{};
+    std::vector<TabulatedTwoDFunction> saturatedWaterVaporizationSaltFactorTable_{};
+    std::vector<TabulatedOneDFunction> saturatedOilVaporizationFactorTable_{};
+    std::vector<TabulatedOneDFunction> saturationPressure_{};
 
     bool enableRwgSalt_ = false;
     Scalar vapPar1_ = 0.0;
