@@ -27,26 +27,31 @@
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
-#include <opm/input/eclipse/Schedule/ArrayDimChecker.hpp>
-#include <opm/input/eclipse/Schedule/Group/GuideRateConfig.hpp>
-#include <opm/input/eclipse/Schedule/Group/GuideRateModel.hpp>
-#include <opm/input/eclipse/Schedule/GasLiftOpt.hpp>
-#include <opm/input/eclipse/Schedule/Network/Balance.hpp>
-#include <opm/input/eclipse/Schedule/Network/ExtNetwork.hpp>
-#include <opm/input/eclipse/Schedule/Schedule.hpp>
-#include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
-#include <opm/input/eclipse/Schedule/UDQ/UDQActive.hpp>
-#include <opm/input/eclipse/Schedule/Well/Well.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
-#include <opm/input/eclipse/Schedule/Action/Actions.hpp>
-#include <opm/input/eclipse/Schedule/Action/ActionX.hpp>
-#include <opm/input/eclipse/Schedule/Tuning.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/Regdims.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
 
+#include <opm/input/eclipse/Schedule/Action/ActionX.hpp>
+#include <opm/input/eclipse/Schedule/Action/Actions.hpp>
+#include <opm/input/eclipse/Schedule/ArrayDimChecker.hpp>
+#include <opm/input/eclipse/Schedule/GasLiftOpt.hpp>
+#include <opm/input/eclipse/Schedule/Group/GuideRateConfig.hpp>
+#include <opm/input/eclipse/Schedule/Group/GuideRateModel.hpp>
+#include <opm/input/eclipse/Schedule/Network/Balance.hpp>
+#include <opm/input/eclipse/Schedule/Network/ExtNetwork.hpp>
+#include <opm/input/eclipse/Schedule/Schedule.hpp>
+#include <opm/input/eclipse/Schedule/Tuning.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQActive.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQEnums.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQInput.hpp>
+#include <opm/input/eclipse/Schedule/Well/Well.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
+
 #include <algorithm>
+#include <array>
 #include <cstddef>
 #include <iterator>
+#include <numeric>
 #include <stdexcept>
 #include <string>
 #include <vector>
@@ -136,93 +141,43 @@ namespace {
         return gctrl;
     }
 
-    int noWellUdqs(const Opm::Schedule& sched,
-                   const std::size_t    rptStep,
-                   const std::size_t    simStep)
-    {
-        if (rptStep == std::size_t{0}) {
-            return 0;
-        }
-
-        const auto& udqCfg = sched.getUDQConfig(simStep);
-        const auto input = udqCfg.input();
-        return std::count_if(input.begin(), input.end(),
-                             [](const auto& udq_input)
-                             {
-                                 return udq_input.var_type() == Opm::UDQVarType::WELL_VAR;
-                             });
-    }
-
-    int noGroupUdqs(const Opm::Schedule& sched,
-                    const std::size_t    rptStep,
-                    const std::size_t    simStep)
-    {
-        if (rptStep == std::size_t{0}) {
-            return 0;
-        }
-
-        const auto& udqCfg = sched.getUDQConfig(simStep);
-        const auto& input = udqCfg.input();
-
-        return std::count_if(input.begin(), input.end(), [](const Opm::UDQInput inp) { return (inp.var_type() == Opm::UDQVarType::GROUP_VAR); });
-    }
-
     int noIuads(const Opm::Schedule& sched,
-                    const std::size_t    rptStep,
-                    const std::size_t    simStep)
+                const std::size_t    rptStep,
+                const std::size_t    simStep)
     {
         if (rptStep == std::size_t{0}) {
             return 0;
         }
 
-        const auto& udqAct = sched[simStep].udq_active.get();
-        const auto& iuad = udqAct.iuad();
-
-        return std::count_if(iuad.begin(), iuad.end(),
-                             [](const Opm::UDQActive::OutputRecord& rec)
-                             {
-                                  return (!(((Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONPROD)
-                                            || (Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONINJE))
-                                            && (rec.wg_name() == "FIELD")));
-                            });
+        return static_cast<int>
+            (sched[simStep].udq_active().iuad().size());
     }
 
     int noIuaps(const Opm::Schedule& sched,
-                    const std::size_t    rptStep,
-                    const std::size_t    simStep)
+                const std::size_t    rptStep,
+                const std::size_t    simStep)
     {
         if (rptStep == std::size_t{0}) {
             return 0;
         }
 
-        const auto& udqAct = sched[simStep].udq_active.get();
-        const auto& iuap = udqAct.iuap();
+        // UDQActive::iuap() returns a vector<> by value.
+        const auto iuap = sched[simStep].udq_active().iuap();
 
-        return std::count_if(iuap.begin(), iuap.end(),
-                             [](const Opm::UDQActive::InputRecord& rec)
-                             {
-                                return (!(((Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONPROD)
-                                         || (Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONINJE))
-                                         && (rec.wgname == "FIELD")));
-                             });
-    }
+        return std::accumulate(iuap.begin(), iuap.end(), 0,
+            [](const int n, const auto& rec)
+        {
+            const auto kw = Opm::UDQ::keyword(rec.control);
 
+            const auto is_field_uda =
+                ((kw == Opm::UDAKeyword::GCONPROD) ||
+                 (kw == Opm::UDAKeyword::GCONINJE))
+                && (rec.wgname == "FIELD");
 
-    int noFieldUdqs(const Opm::Schedule& sched,
-                    const std::size_t    rptStep,
-                    const std::size_t    simStep)
-    {
-        if (rptStep == std::size_t{0}) {
-            return 0;
-        }
-
-        const auto& udqCfg = sched.getUDQConfig(simStep);
-        const auto input = udqCfg.input();
-        return std::count_if(input.begin(), input.end(),
-                             [](const auto& udq_input)
-                             {
-                                 return udq_input.var_type() == Opm::UDQVarType::FIELD_VAR;
-                             });
+            // One IUAP entry for each "regular" UDA in WCON* or GCON*.  Two
+            // IUAP entries for each field level UDA in GCON*.
+            return n + 1 + static_cast<int>(is_field_uda);
+        });
     }
 
     int numMultiSegWells(const ::Opm::Schedule& sched,
@@ -373,13 +328,11 @@ namespace {
 
         param.udqParam_1 = rspec.udqParams().rand_seed();
 
-        param.num_field_udqs = noFieldUdqs(sched, rptStep, simStep);
-        param.num_group_udqs = noGroupUdqs(sched, rptStep, simStep);
-        param.num_well_udqs  = noWellUdqs (sched, rptStep, simStep);
-
         param.num_iuads = noIuads(sched, rptStep, simStep);
         param.num_iuaps = noIuaps(sched, rptStep, simStep);
 
+        sched[simStep].udq().exportTypeCount(param.numUDQs);
+
         return param;
     }
 
@@ -547,8 +500,8 @@ namespace {
     {
         const int noactnod = sched[lookup_step].network().node_names().size();
         const int noactbr  = sched[lookup_step].network().NoOfBranches();
-        const int nodmax = rspec.networkDimensions().maxNONodes();
-        const int nbrmax = rspec.networkDimensions().maxNoBranches();
+        const int nodmax = std::max(rspec.networkDimensions().maxNONodes(), sched[lookup_step].network().NoOfNodes());
+        const int nbrmax = std::max(rspec.networkDimensions().maxNoBranches(), sched[lookup_step].network().NoOfBranches());
 
         //the following dimensions are fixed
         const int nibran = 14;
