@@ -71,12 +71,24 @@ public:
         CO2MassInGasPhaseMob = 27,
         CO2MassInGasPhaseInMobKrg = 28,
         CO2MassInGasPhaseMobKrg = 29,
+        CO2MassInGasPhaseEffectiveTrapped = 30,
+        CO2MassInGasPhaseEffectiveUnTrapped = 31,
+        CO2MassInGasPhaseMaximumTrapped = 32,
+        CO2MassInGasPhaseMaximumUnTrapped = 33,
+        MicrobialMass = 34,
+        OxygenMass = 35,
+        UreaMass = 36,
+        BiofilmMass = 37,
+        CalciteMass = 38,
     };
 
     /// Create non-defaulted object suitable for testing the serialisation
     /// operation.
     static Inplace serializationTestObject();
 
+    /// Converts phase enum to ECL textual representation.
+    static std::string EclString(const Phase phase);
+
     /// Assign value of particular quantity in specific region of named
     /// region set.
     ///
