@@ -283,6 +283,10 @@ namespace {
             rstFile.write("DUDG", dudg->data());
         }
 
+        if (const auto& duds = udqData.getDUDS(); duds.has_value()) {
+            rstFile.write("DUDS", duds->data());
+        }
+
         if (const auto& dudw = udqData.getDUDW(); dudw.has_value()) {
             rstFile.write("DUDW", dudw->data());
         }
@@ -433,8 +437,8 @@ namespace {
         writeGroup(sim_step, schedule.getUnits(), schedule, sumState, inteHD, rstFile);
 
         // Write network data if the network option is used and network defined
-        if ((es.runspec().networkDimensions().maxNONodes() >= 1) &&
-            schedule[sim_step].network().active())
+        const auto& network = schedule[sim_step].network();
+        if (network.active())
         {
             writeNetwork(es, sim_step, schedule.getUnits(), schedule, sumState, inteHD, rstFile);
         }
