@@ -40,6 +40,7 @@
 
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
+#include <algorithm>
 #include <cstddef>
 #include <initializer_list>
 #include <stdexcept>
@@ -464,6 +465,7 @@ namespace {
             {"THERMEXR" , ::Opm::UnitSystem::measure::identity}, // 1/(temperature difference)
             {"THELCOEF" , ::Opm::UnitSystem::measure::identity}, // Pressure/Temperature
             {"YMODULE"  , ::Opm::UnitSystem::measure::ymodule},
+            {"THCONR"   , ::Opm::UnitSystem::measure::thermalconductivity},
         };
 
         // The INIT file should always contain the NTG property.  Guarantee
@@ -500,7 +502,7 @@ namespace {
         initFile.write("TAB"    , tables.tab());
     }
 
-    void writeIntegerMaps(std::map<std::string, std::vector<int>> mapData,
+    void writeIntegerMaps(const std::map<std::string, std::vector<int>>& mapData,
                           ::Opm::EclIO::OutputStream::Init&       initFile)
     {
         for (const auto& pair : mapData) {
@@ -571,9 +573,8 @@ namespace {
         auto tran = std::vector<double>{};
         tran.reserve(nnc.size());
 
-        for (const auto& nd : nnc) {
-            tran.push_back(nd.trans);
-        }
+        std::transform(nnc.begin(), nnc.end(), std::back_inserter(tran),
+                       [](const auto& nd) { return nd.trans; });
 
         units.from_si(::Opm::UnitSystem::measure::transmissibility, tran);
 
@@ -667,10 +668,15 @@ void Opm::InitIO::write(const ::Opm::EclipseState&              es,
     {
         const auto writeAll = es.cfg().io().writeAllTransMultipliers();
 
-        const auto tranMult = es.getTransMult()
+        auto multipliers = es.getTransMult()
             .convertToSimProps(grid.getNumActive(), writeAll);
+        if (es.globalFieldProps().has_double("MULTPV")) {
+            multipliers.insert("MULTPV", UnitSystem::measure::identity,
+                        es.globalFieldProps().get_double("MULTPV"),
+                        data::TargetType::INIT);
+        }
 
-        writeSimulatorProperties(grid, tranMult, initFile);
+        writeSimulatorProperties(grid, multipliers, initFile);
     }
 
     writeTableData(es, units, initFile);
