@@ -20,6 +20,8 @@
 #include "export.hpp"
 #include "converters.hpp"
 
+#include <python/cxx/OpmCommonPythonDoc.hpp>
+
 namespace py = pybind11;
 
 
@@ -84,7 +86,7 @@ public:
             return convert::numpy_array( m_ext_esmry->get_at_rstep(key) );
     }
 
-    time_point smry_start_date()
+    std::tuple<int, int, int,int, int, int, bool> smry_start_date()
     {
         time_point utc_chrono;
 
@@ -94,11 +96,26 @@ public:
             utc_chrono = m_ext_esmry->startdate();
 
         auto utc_time_t   = std::chrono::system_clock::to_time_t( utc_chrono );
+
         auto utc_ts       = Opm::TimeStampUTC( utc_time_t );
+
         auto local_time_t = Opm::asLocalTimeT( utc_ts );
-        return TimeService::from_time_t( local_time_t );
+
+        std::tm* local_tm = std::localtime(&local_time_t);
+
+        int y = local_tm->tm_year + 1900;
+        int m = local_tm->tm_mon + 1;
+        int d = local_tm->tm_mday;
+        int h = local_tm->tm_hour;
+        int mi = local_tm->tm_min;
+        int s = local_tm->tm_sec;
+
+        bool dst = local_tm->tm_isdst == 1 ? true : false;
+
+        return std::make_tuple(y, m, d, h, mi, s, dst);
     }
 
+
     std::vector<time_point> dates() const
     {
         std::vector<time_point> result;
@@ -156,7 +173,7 @@ class EclOutputBind {
 
 public:
 
-    EclOutputBind(const std::string& filename,const bool formatted, const bool append)
+    EclOutputBind(const std::string& filename, const bool formatted, const bool append)
     {
         if (append == true)
             m_output = std::make_unique<Opm::EclIO::EclOutput>(filename, formatted, std::ios::app);
@@ -292,26 +309,51 @@ npArray get_erst_vector(Opm::EclIO::ERst * file_ptr, const std::string& key, siz
     return get_erst_by_index(file_ptr, array_index, rstep);
 }
 
-
 std::tuple<std::array<double,8>, std::array<double,8>, std::array<double,8>>
-get_xyz_from_ijk(Opm::EclIO::EGrid * file_ptr,int i, int j, int k)
+get_xyz_from_ijk(Opm::EclIO::EGrid * file_ptr, int i, int j, int k)
 {
     std::array<double,8> X = {0.0};
     std::array<double,8> Y = {0.0};
     std::array<double,8> Z = {0.0};
 
-    std::array<int, 3> ijk = {i, j, k };
+    std::array<int, 3> ijk = {i, j, k};
 
     file_ptr->getCellCorners(ijk, X, Y, Z);
 
-    return std::make_tuple( X, Y, Z);
+    return std::make_tuple(X, Y, Z);
+}
+
+std::tuple<std::array<double,8>, std::array<double,8>, std::array<double,8>>
+get_xyz_from_ijk_mapaxes(Opm::EclIO::EGrid * file_ptr, int i, int j, int k, bool mapaxes)
+{
+    auto xyz = get_xyz_from_ijk(file_ptr, i, j, k);
+
+    if (file_ptr->with_mapaxes() && mapaxes){
+        for (int n = 0; n < 8; n++)
+            file_ptr->mapaxes_transform(std::get<0>(xyz)[n], std::get<1>(xyz)[n]);
+    }
+
+    return xyz;
 }
 
 std::tuple<std::array<double,8>, std::array<double,8>, std::array<double,8>>
 get_xyz_from_active_index(Opm::EclIO::EGrid * file_ptr, int actIndex)
 {
     std::array<int, 3> ijk = file_ptr->ijk_from_active_index(actIndex);
-    return get_xyz_from_ijk(file_ptr,ijk[0], ijk[1], ijk[2]);
+    return get_xyz_from_ijk(file_ptr, ijk[0], ijk[1], ijk[2]);
+}
+
+std::tuple<std::array<double,8>, std::array<double,8>, std::array<double,8>>
+get_xyz_from_active_index_mapaxes(Opm::EclIO::EGrid * file_ptr, int actIndex, bool mapaxes)
+{
+    auto xyz = get_xyz_from_active_index(file_ptr, actIndex);
+
+    if (file_ptr->with_mapaxes() && mapaxes){
+        for (int n = 0; n < 8; n++)
+            file_ptr->mapaxes_transform(std::get<0>(xyz)[n], std::get<1>(xyz)[n]);
+    }
+
+    return xyz;
 }
 
 py::array get_cellvolumes_mask(Opm::EclIO::EGrid * file_ptr, std::vector<int> mask)
@@ -412,68 +454,75 @@ npArray get_rft_vector_Index(Opm::EclIO::ERft * file_ptr,const std::string& name
 
 void python::common::export_IO(py::module& m) {
 
-    py::enum_<Opm::EclIO::eclArrType>(m, "eclArrType", py::arithmetic())
-        .value("INTE", Opm::EclIO::INTE)
-        .value("REAL", Opm::EclIO::REAL)
-        .value("DOUB", Opm::EclIO::DOUB)
-        .value("CHAR", Opm::EclIO::CHAR)
-        .value("C0nn", Opm::EclIO::C0NN)
-        .value("LOGI", Opm::EclIO::LOGI)
-        .value("MESS", Opm::EclIO::MESS)
+    using namespace Opm::Common::DocStrings;
+
+    py::enum_<Opm::EclIO::eclArrType>(m, "eclArrType", py::arithmetic(), eclArrType_docstring)
+        .value("INTE", Opm::EclIO::INTE, eclArrType_INTE_docstring)
+        .value("REAL", Opm::EclIO::REAL, eclArrType_REAL_docstring)
+        .value("DOUB", Opm::EclIO::DOUB, eclArrType_DOUB_docstring)
+        .value("CHAR", Opm::EclIO::CHAR, eclArrType_CHAR_docstring)
+        .value("C0nn", Opm::EclIO::C0NN, eclArrType_C0nn_docstring)
+        .value("LOGI", Opm::EclIO::LOGI, eclArrType_LOGI_docstring)
+        .value("MESS", Opm::EclIO::MESS, eclArrType_MESS_docstring)
         .export_values();
 
-    py::class_<Opm::EclIO::EclFile>(m, "EclFile")
-        .def(py::init<const std::string &, bool>(), py::arg("filename"), py::arg("preload") = false)
-        .def_property_readonly("arrays", &Opm::EclIO::EclFile::getList)
-        .def("__contains__", &Opm::EclIO::EclFile::hasKey)
-        .def("__len__", &Opm::EclIO::EclFile::size)
-        .def("count", &Opm::EclIO::EclFile::count)
-        .def("__get_data", &get_vector_index)
-        .def("__get_data", &get_vector_name)
-        .def("__get_data", &get_vector_occurrence);
+    py::class_<Opm::EclIO::EclFile>(m, "EclFile", EclFile_docstring)
+        .def(py::init<const std::string &, bool>(), py::arg("filename"), py::arg("preload") = false, EclFile_init_docstring)
+        .def_property_readonly("arrays", &Opm::EclIO::EclFile::getList, EclFile_arrays_docstring)
+        .def("__contains__", &Opm::EclIO::EclFile::hasKey, py::arg("name"), EclFile_contains_docstring)
+        .def("__len__", &Opm::EclIO::EclFile::size, EclFile_len_docstring)
+        .def("count", &Opm::EclIO::EclFile::count, py::arg("name"), EclFile_count_docstring)
+        .def("__get_data", &get_vector_index, py::arg("index"), EclFile_get_data_index_docstring)
+        .def("__get_data", &get_vector_name, py::arg("name"), EclFile_get_data_name_docstring)
+        .def("__get_data", &get_vector_occurrence, py::arg("name"), py::arg("occurrence"), EclFile_get_data_occurrence_docstring);
 
-    py::class_<Opm::EclIO::ERst>(m, "ERst")
-        .def(py::init<const std::string &>())
-        .def("__has_report_step", &Opm::EclIO::ERst::hasReportStepNumber)
-        .def("load_report_step", &Opm::EclIO::ERst::loadReportStepNumber)
-        .def_property_readonly("report_steps", &Opm::EclIO::ERst::listOfReportStepNumbers)
-        .def("__len__", &Opm::EclIO::ERst::numberOfReportSteps)
-        .def("count", &Opm::EclIO::ERst::occurrence_count)
-        .def("__contains", &erst_contains)
+    py::class_<Opm::EclIO::ERst>(m, "ERst", ERst_docstring)
+        .def(py::init<const std::string &>(), py::arg("filename"), ERst_init_docstring)
+        .def("__has_report_step", &Opm::EclIO::ERst::hasReportStepNumber, py::arg("report_step"), ERst_has_report_step_docstring)
+        .def("load_report_step", &Opm::EclIO::ERst::loadReportStepNumber, py::arg("report_step"), ERst_load_report_step_docstring)
+        .def_property_readonly("report_steps", &Opm::EclIO::ERst::listOfReportStepNumbers, ERst_report_steps_docstring)
+        .def("__len__", &Opm::EclIO::ERst::numberOfReportSteps, ERst_len_docstring)
+        .def("count", &Opm::EclIO::ERst::occurrence_count, py::arg("name"), py::arg("report_step"), ERst_count_docstring)
+        .def("__contains", &erst_contains, py::arg("tuple"), ERst_contains_docstring)
         .def("arrays", (std::vector< std::tuple<std::string, Opm::EclIO::eclArrType, int64_t> >
-                        (Opm::EclIO::ERst::*)(int) ) &Opm::EclIO::ERst::listOfRstArrays)
+                        (Opm::EclIO::ERst::*)(int) ) &Opm::EclIO::ERst::listOfRstArrays, py::arg("report_step"), ERst_arrays_docstring)
         .def("arrays", (std::vector< std::tuple<std::string, Opm::EclIO::eclArrType, int64_t> >
-                        (Opm::EclIO::ERst::*)(int, const std::string&) ) &Opm::EclIO::ERst::listOfRstArrays)
-        .def("__get_data", &get_erst_by_index)
-        .def("__get_data", &get_erst_vector);
+                        (Opm::EclIO::ERst::*)(int, const std::string&) ) &Opm::EclIO::ERst::listOfRstArrays, py::arg("report_step"), py::arg("lgr_name"), ERst_arrays_with_string_docstring)
+        .def("__get_data", &get_erst_by_index, py::arg("index"), py::arg("report_step"), ERst_get_data_by_index_docstring)
+        .def("__get_data", &get_erst_vector, py::arg("name"), py::arg("report_step"), py::arg("occurrence"), ERst_get_data_vector_docstring);
 
-   py::class_<ESmryBind>(m, "ESmry")
-        .def(py::init<const std::string &, const bool>(), py::arg("filename"), py::arg("load_base_run") = false)
-        .def("__contains__", &ESmryBind::hasKey)
-        .def("make_esmry_file", &ESmryBind::make_esmry_file)
-        .def("__len__", &ESmryBind::numberOfTimeSteps)
-        .def("__get_all", &ESmryBind::get_smry_vector)
-        .def("__get_at_rstep", &ESmryBind::get_smry_vector_at_rsteps)
-        .def_property_readonly("start_date", &ESmryBind::smry_start_date)
+
+   py::class_<ESmryBind>(m, "ESmry", ESmry_docstring)
+        .def(py::init<const std::string &, const bool>(), py::arg("filename"), py::arg("load_base_run") = false, ESmry_init_docstring)
+        .def("__contains__", &ESmryBind::hasKey, py::arg("key"), ESmry_contains_docstring)
+        .def("make_esmry_file", &ESmryBind::make_esmry_file, ESmry_make_esmry_file_docstring)
+        .def("__len__", &ESmryBind::numberOfTimeSteps, ESmry_len_docstring)
+        .def("__get_all", &ESmryBind::get_smry_vector, py::arg("key"), ESmry_get_all_docstring)
+        .def("__get_at_rstep", &ESmryBind::get_smry_vector_at_rsteps, py::arg("key"), ESmry_get_at_rstep_docstring)
+        .def("__start_date", &ESmryBind::smry_start_date, ESmry_start_date_docstring)
         .def("keys", (const std::vector<std::string>& (ESmryBind::*) (void) const)
-            &ESmryBind::keywordList)
+            &ESmryBind::keywordList, ESmry_keys1_docstring)
         .def("keys", (std::vector<std::string> (ESmryBind::*) (const std::string&) const)
-            &ESmryBind::keywordList)
-        .def("dates", &ESmryBind::dates)
-        .def("units", &ESmryBind::units);
+            &ESmryBind::keywordList, py::arg("pattern"), ESmry_keys2_docstring)
+        .def("dates", &ESmryBind::dates, ESmry_dates_docstring)
+        .def("units", &ESmryBind::units, py::arg("field"), ESmry_units_docstring);
+
+   py::class_<Opm::EclIO::EGrid>(m, "EGrid", EGrid_docstring)
+        .def(py::init<const std::string &, const std::string &>(), py::arg("filename"), py::arg("grid_name") = "global", EGrid_init_docstring)
+        .def_property_readonly("active_cells", &Opm::EclIO::EGrid::activeCells, EGrid_active_cells_docstring)
+        .def_property_readonly("dimension", &Opm::EclIO::EGrid::dimension, EGrid_dimension_docstring)
+        .def("ijk_from_global_index", &Opm::EclIO::EGrid::ijk_from_global_index, py::arg("global_index"), EGrid_ijk_from_global_index_docstring)
+        .def("ijk_from_active_index", &Opm::EclIO::EGrid::ijk_from_active_index, py::arg("active_index"), EGrid_ijk_from_active_index_docstring)
+        .def("active_index", &Opm::EclIO::EGrid::active_index, py::arg("i"), py::arg("j"), py::arg("k"), EGrid_active_index_docstring)
+        .def("global_index", &Opm::EclIO::EGrid::global_index, py::arg("i"), py::arg("j"), py::arg("k"), EGrid_global_index_docstring)
+        .def("export_mapaxes", &Opm::EclIO::EGrid::get_mapaxes, EGrid_export_mapaxes_docstring)
+        .def("xyz_from_ijk", &get_xyz_from_ijk, py::arg("i"), py::arg("j"), py::arg("k"), EGrid_xyz_from_ijk_docstring)
+        .def("xyz_from_ijk", &get_xyz_from_ijk_mapaxes, py::arg("i"), py::arg("j"), py::arg("k"), py::arg("apply_mapaxes"), EGrid_xyz_from_ijk_mapaxes_docstring)
+        .def("xyz_from_active_index", &get_xyz_from_active_index, py::arg("active_index"), EGrid_xyz_from_active_index_docstring)
+        .def("xyz_from_active_index", &get_xyz_from_active_index_mapaxes, py::arg("active_index"), py::arg("apply_mapaxes"), EGrid_xyz_from_active_index_mapaxes_docstring)
+        .def("cellvolumes", &get_cellvolumes, EGrid_cellvolumes_docstring)
+        .def("cellvolumes", &get_cellvolumes_mask, py::arg("mask"), EGrid_cellvolumes_mask_docstring);
 
-   py::class_<Opm::EclIO::EGrid>(m, "EGrid")
-        .def(py::init<const std::string &>())
-        .def_property_readonly("active_cells", &Opm::EclIO::EGrid::activeCells)
-        .def_property_readonly("dimension", &Opm::EclIO::EGrid::dimension)
-        .def("ijk_from_global_index", &Opm::EclIO::EGrid::ijk_from_global_index)
-        .def("ijk_from_active_index", &Opm::EclIO::EGrid::ijk_from_active_index)
-        .def("active_index", &Opm::EclIO::EGrid::active_index)
-        .def("global_index", &Opm::EclIO::EGrid::global_index)
-        .def("xyz_from_ijk", &get_xyz_from_ijk)
-        .def("xyz_from_active_index", &get_xyz_from_active_index)
-        .def("cellvolumes", &get_cellvolumes)
-        .def("cellvolumes", &get_cellvolumes_mask);
 
    py::class_<Opm::EclIO::ERft>(m, "ERft")
         .def(py::init<const std::string &>())
