@@ -1,6 +1,7 @@
 #include <string>
 
 #include <opm/common/OpmLog/OpmLog.hpp>
+#include <opm/common/OpmLog/Logger.hpp>
 
 #include "export.hpp"
 
@@ -38,10 +39,12 @@ void note(const std::string& msg) {
 
 void python::common::export_Log(py::module& module)
 {
-    py::class_<OpmLog, std::shared_ptr<OpmLog> >( module, "OpmLog", R"(
+    py::class_<Logger, std::shared_ptr<Logger>>(module, "Logger");
+    py::class_<OpmLog>( module, "OpmLog", R"(
             The Opm::OpmLog class - this is a fully static class which manages a proper
             Logger instance.
         )")
+        .def_static("_setLogger", &OpmLog::setLogger, "Internal function to set the logger.")
         .def_static("info", info, "Add an info message to the opm log.")
         .def_static("warning", warning, "Add a warning message to the opm log.")
         .def_static("error", error, "Add an error message to the opm log.")
