@@ -34,15 +34,14 @@
 #include <opm/common/utility/numeric/cmp.hpp>
 
 #include <algorithm>
-#include <chrono>
 #include <cmath>
 #include <filesystem>
 #include <fstream>
 #include <iomanip>
 #include <iostream>
 #include <set>
-#include <type_traits>
 #include <typeinfo>
+#include <unordered_set>
 #include <vector>
 
 // helper macro to handle error throws or not
@@ -327,10 +326,9 @@ void ECLRegressionTest::checkSpecificKeyword(std::vector<std::string>& keywords1
         OPM_THROW(std::runtime_error, "\n" + msg);
     }
 
-    eclArrType arrType;
     if (search1 != keywords1.end()) {
         int ind = std::distance(keywords1.begin(), search1);
-        arrType = arrayType1[ind];
+        const eclArrType arrType = arrayType1[ind];
 
         if (search2 == keywords2.end()) {
             const std::string msg =
@@ -612,8 +610,6 @@ void ECLRegressionTest::results_init()
         init1.loadData();
         init2.loadData();
 
-        std::string reference = "Init file";
-
         auto arrayList1 = init1.getList();
         auto arrayList2 = init2.getList();
 
@@ -637,6 +633,7 @@ void ECLRegressionTest::results_init()
             printComparisonForKeywordLists(keywords1,keywords2, arrayType1, arrayType2);
         } else {
             std::cout << "\nComparing init files \n" << std::endl;
+            std::string reference = "Init file";
 
             if (specificKeyword.empty()) {
                 if (keywords1.size() == keywords2.size() && keywords1 != keywords2) {
@@ -793,15 +790,15 @@ void ECLRegressionTest::results_rst()
             std::vector<std::string> seqnStrList1;
             std::vector<std::string> seqnStrList2;
 
-            for (auto& val : seqnums1) {
-                seqnStrList1.push_back(std::to_string(val));
-            }
+            std::transform(seqnums1.begin(), seqnums1.end(),
+                           std::back_inserter(seqnStrList1),
+                           [](const auto& val) { return std::to_string(val); });
+
+            std::transform(seqnums2.begin(), seqnums2.end(),
+                           std::back_inserter(seqnStrList2),
+                           [](const auto& val) { return std::to_string(val); });
 
-            for (auto& val : seqnums2) {
-                seqnStrList2.push_back(std::to_string(val));
-            }
             std::cout << "\nrestart sequences " << std::endl;
-
             printComparisonForKeywordLists(seqnStrList1, seqnStrList2);
             OPM_THROW(std::runtime_error, "\nRestart files not having the same report steps: ");
         }
@@ -864,9 +861,6 @@ void ECLRegressionTest::results_rst()
                     checkSpecificKeyword(keywords1, keywords2, arrayType1, arrayType2, reference);
                 }
 
-                std::unordered_set<std::string> keywords = {"IGRP"};
-
-
                 for (size_t i = 0; i < keywords1.size(); i++) {
                     //if (keywords.count(keywords1[i]) == 0)
                     //    continue;
@@ -1029,16 +1023,21 @@ void ECLRegressionTest::results_smry()
 
             // Remove blacklisted keywords
             std::vector<std::string> blackListed;
-            keywords1.erase(std::remove_if(keywords1.begin(), keywords1.end(),
-                                           [&blacklist = keywordsBlackList, &blackListed](const auto& kw)
-                                           {
-                                               const auto it =
-                                                  std::find(blacklist.begin(), blacklist.end(), kw);
-                                               if (it != blacklist.end()) {
-                                                   blackListed.push_back(kw);
-                                               }
-                                               return it != blacklist.end();
-                                           }), keywords1.end());
+            auto make_remover = [&blackListed](const std::vector<std::string>& _blacklist) {
+                                    return [&blacklist = _blacklist, &blackListed](const auto& kw)
+                                            {
+                                                const auto it = std::find(blacklist.begin(), blacklist.end(), kw);
+                                                if (it != blacklist.end()) {
+                                                    blackListed.push_back(kw);
+                                                }
+                                                return it != blacklist.end();
+                                            };
+                                };
+
+            keywords1.erase(std::remove_if(keywords1.begin(), keywords1.end(), make_remover(keywordsBlackList)), keywords1.end());
+            if (loadBaseRunData) {
+                keywords1.erase(std::remove_if(keywords1.begin(), keywords1.end(), make_remover(keywordsBlackListExtraRestart)), keywords1.end());
+            }
 
             std::cout << "\nChecking " << keywords1.size() << "  vectors  ... ";
 
@@ -1275,7 +1274,7 @@ void ECLRegressionTest::printComparisonForKeywordLists(const std::vector<std::st
         commonList.insert(key);
     }
 
-    for (auto& key : commonList) {
+    for (const auto& key : commonList) {
         if (key.size() > maxLen) {
             maxLen = key.size();
         }
@@ -1364,7 +1363,7 @@ void ECLRegressionTest::printComparisonForKeywordLists(const std::vector<std::st
         commonList.insert(key);
     }
 
-    for (auto& key : commonList) {
+    for (const auto& key : commonList) {
         if (key.size() > maxLen) {
             maxLen = key.size();
         }
