@@ -20,17 +20,18 @@
 
 #include <opm/common/ErrorMacros.hpp>
 
+#include <algorithm>
+#include <filesystem>
+#include <getopt.h>
 #include <iostream>
 #include <sstream>
 #include <string>
-#include <getopt.h>
-#include <fstream>
 
 static void printHelp() {
     std::cout << "\ncompareECL compares ECLIPSE files (restart (.RST), unified restart (.UNRST), initial (.INIT), summary (.SMRY), unified summary (.UNSMRY) or .RFT) and gridsizes (from .EGRID or .GRID file) from two simulations.\n"
               << "The program takes four arguments:\n\n"
-              << "1. Case number 1, reference case (full path without extension)\n"
-              << "2. Case number 2, test case (full path without extension)\n"
+              << "1. Case number 1, reference case (path without extension)\n"
+              << "2. Case number 2, test case (path without extension)\n"
               << "3. Absolute tolerance\n"
               << "4. Relative tolerance (between 0 and 1)\n\n"
               << "In addition, the program takes these options (which must be given before the arguments):\n\n"
@@ -70,14 +71,11 @@ static bool has_result_files(const std::string& rootName)
 {
     std::vector<std::string> extList = { "EGRID", "INIT", "UNRST", "SMSPEC", "RFT" };
 
-    for (const auto& ext : extList) {
-        std::ifstream is(rootName + '.' + ext);
-        if (is) {
-            return true;
-        }
-    }
-
-    return false;
+    return std::any_of(extList.begin(), extList.end(),
+                       [&rootName](const auto& ext)
+                       {
+                           return std::filesystem::exists(rootName + '.' + ext);
+                       });
 }
 
 //------------------------------------------------//
