@@ -168,7 +168,7 @@ BOOST_AUTO_TEST_CASE(RUN) {
             }
             auto rst_view = std::make_shared<EclIO::RestartFileView>(std::move(rst), report_step);
             const auto rst_state = Opm::RestartIO::RstState::load(std::move(rst_view), state.runspec(), parser);
-            Schedule sched_rst(deck, state, python, false, true, {}, &rst_state);
+            Schedule sched_rst(deck, state, python, false,  /*slave_mode=*/false, true, {}, &rst_state);
             const auto& rfti_well = sched_rst.getWell("RFTI", report_step);
             const auto& rftp_well = sched_rst.getWell("RFTP", report_step);
             BOOST_CHECK(rftp_well.getStatus() == Well::Status::SHUT);
