@@ -15,22 +15,23 @@
 
   You should have received a copy of the GNU General Public License
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
- */
-
-#include <stdexcept>
+*/
 
 #define BOOST_TEST_MODULE GroupTests
 #include <boost/test/unit_test.hpp>
-#include <opm/input/eclipse/Python/Python.hpp>
 
-#include <opm/input/eclipse/Deck/Deck.hpp>
-#include <opm/input/eclipse/Parser/Parser.hpp>
 #include <opm/input/eclipse/Parser/ParseContext.hpp>
+
+#include <opm/input/eclipse/EclipseState/Aquifer/NumericalAquifer/NumericalAquifers.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
+
+#include <opm/input/eclipse/Python/Python.hpp>
+
 #include <opm/input/eclipse/Schedule/Group/GConSump.hpp>
+#include <opm/input/eclipse/Schedule/Group/GSatProd.hpp>
 #include <opm/input/eclipse/Schedule/Group/GConSale.hpp>
 #include <opm/input/eclipse/Schedule/Group/GroupEconProductionLimits.hpp>
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
@@ -42,6 +43,15 @@
 
 #include <opm/common/utility/TimeService.hpp>
 
+#include <opm/input/eclipse/Deck/Deck.hpp>
+
+#include <opm/input/eclipse/Parser/Parser.hpp>
+
+#include <memory>
+#include <optional>
+#include <string>
+#include <stdexcept>
+
 using namespace Opm;
 
 namespace {
@@ -55,7 +65,7 @@ Schedule create_schedule(const std::string& deck_string)
     const FieldPropsManager fp(deck, Phases{true, true, true}, grid, table);
     const Runspec runspec(deck);
 
-    return { deck, grid, fp, runspec, std::make_shared<Python>() };
+    return { deck, grid, fp, NumericalAquifers{}, runspec, std::make_shared<Python>() };
 }
 
 } // Anonymous namespace
@@ -131,7 +141,7 @@ GEFAC
 
     const auto& group1 = schedule.getGroup("PRODUC", 0);
     BOOST_CHECK_EQUAL(group1.getGroupEfficiencyFactor(), 0.85);
-    BOOST_CHECK(group1.getTransferGroupEfficiencyFactor());
+    BOOST_CHECK(group1.useEfficiencyInNetwork());
 }
 
 
@@ -380,6 +390,40 @@ TSTEP
     GuideRate gr(schedule);
 }
 
+BOOST_AUTO_TEST_CASE(TESTGSatProd) {
+    const std::string input = R"(
+START             -- 0
+31 AUG 1993 /
+SCHEDULE
+
+GRUPTREE
+  'G1'  'FIELD' /
+  'G2'  'FIELD' /
+/
+
+GCONPROD
+  'G1' 'ORAT' 10000 /
+/
+
+GSATPROD
+   'G2' 1000 /
+/
+
+TSTEP
+  1 /)";
+
+    auto schedule = create_schedule(input);
+    double metric_to_si = 1.0 / (24.0 * 3600.0);  //cubic meters / day
+    const auto& gsatprod = schedule[0].gsatprod.get();
+    BOOST_CHECK_EQUAL(gsatprod.size(), 1U);
+    BOOST_CHECK(!gsatprod.has("G1"));
+    BOOST_CHECK(gsatprod.has("G2"));
+    const GSatProd::GSatProdGroup& group = gsatprod.get("G2");
+    using Rate = GSatProd::GSatProdGroup::Rate;
+    BOOST_CHECK_EQUAL(group.rate[Rate::Oil], 1000*metric_to_si);
+    BOOST_CHECK_EQUAL(group.rate[Rate::Water], 0.0);
+}
+
 BOOST_AUTO_TEST_CASE(TESTGCONSALE) {
     const std::string input = R"(
 START             -- 0
@@ -604,10 +648,10 @@ GCONINJE
         GuideRate gr = GuideRate(schedule);
         const auto& g1 = schedule.getGroup("G1", 1);
         const auto& g2 = schedule.getGroup("G2", 1);
-        gr.compute(g1.name(), Phase::WATER, 1, 0.0);
-        gr.compute(g1.name(), Phase::GAS, 1, 0.0);
-        gr.compute(g2.name(), Phase::WATER, 1, 0.0);
-        gr.compute(g2.name(), Phase::GAS, 1, 0.0);
+        gr.compute(g1.name(), Phase::WATER, 1, std::nullopt);
+        gr.compute(g1.name(), Phase::GAS, 1, std::nullopt);
+        gr.compute(g2.name(), Phase::WATER, 1, std::nullopt);
+        gr.compute(g2.name(), Phase::GAS, 1, std::nullopt);
 
         BOOST_CHECK( gr.has(g1.name(), Phase::WATER));
         BOOST_CHECK( gr.has(g1.name(), Phase::GAS));
@@ -785,13 +829,17 @@ GCONPROD
         BOOST_CHECK( !prod_group.has_control(Phase::GAS, Group::InjectionCMode::RESV) );
         BOOST_CHECK( f1_group.has_control(Group::ProductionCMode::RESV) );
 
+        {
         auto [name, number] = *gpm_prod->region();
         BOOST_CHECK_EQUAL(number, 2);
         BOOST_CHECK_EQUAL(name, "FIPNUM");
-
+        }
+        {
         const auto& gpm_c1 = c1_group.gpmaint();
-        BOOST_CHECK(!gpm_c1->region());
-
+        auto [name, number] = *gpm_c1->region();
+        BOOST_CHECK_EQUAL(number, 0);
+        BOOST_CHECK_EQUAL(name, "FIPNUM");
+        }
         const auto& plat_prod = plat_group.gpmaint();
         BOOST_CHECK( !plat_prod );
     }
