@@ -26,7 +26,9 @@
 #include <boost/test/unit_test.hpp>
 
 #include <opm/input/eclipse/Deck/Deck.hpp>
+#include <opm/input/eclipse/Deck/DeckSection.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/FaultCollection.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
 #include <opm/input/eclipse/EclipseState/InitConfig/InitConfig.hpp>
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
@@ -35,7 +37,7 @@
 #include <opm/input/eclipse/Parser/Parser.hpp>
 #include <opm/input/eclipse/Parser/ParseContext.hpp>
 #include <opm/input/eclipse/Parser/ErrorGuard.hpp>
-
+#include <opm/input/eclipse/Parser/InputErrorAction.hpp>
 
 using namespace Opm;
 
@@ -54,6 +56,39 @@ THPRES
 /
 )";
 
+const std::string& inputStrft = R"(
+RUNSPEC
+EQLOPTS
+THPRES /
+DIMENS
+10 10 10 /
+
+REGIONS
+EQLNUM
+   120*3 /
+SOLUTION
+THPRES
+1 2 12.0/
+1 3 5.0/
+2 3 33.0 /
+2 3 7.0/
+/
+
+GRID
+DX
+1000*0.25 /
+DY
+1000*0.25 /
+DZ
+1000*0.25 /
+FAULTS
+  'F1'  1  1  1  4   1  4  'X' /
+/
+THPRESFT
+  'F1' 1.0 /
+/
+)";
+
 const std::string& inputStr_RESTART = R"(
 RUNSPEC
 EQLOPTS
@@ -235,7 +270,7 @@ struct Setup
     InitConfig initConfig;
     ThresholdPressure threshPres;
 
-    explicit Setup(const std::string& input) :
+    explicit Setup(const std::string& input, bool ft = false) :
             deck(createDeck(ParseContext(), input)),
             tablemanager(deck),
             grid(10, 3, 4),
@@ -243,6 +278,12 @@ struct Setup
             initConfig(deck),
             threshPres(initConfig.restartRequested(), deck, fp)
     {
+        if (ft) {
+            GRIDSection gridSection(deck);
+            GridDims dims(deck);
+            auto faults = FaultCollection(gridSection, dims);
+            threshPres.readFaults(deck, faults);
+        }
     }
 
     explicit Setup(const std::string& input, const ParseContext& parseContextArg) :
@@ -297,14 +338,14 @@ BOOST_AUTO_TEST_CASE(ThresholdPressureThrowTest) {
 
     BOOST_CHECK_THROW(Setup sx(inputStr), std::runtime_error);
     ParseContext pc;
-    pc.update(ParseContext::UNSUPPORTED_INITIAL_THPRES, InputError::IGNORE);
+    pc.update(ParseContext::UNSUPPORTED_INITIAL_THPRES, InputErrorAction::IGNORE);
     BOOST_CHECK_NO_THROW(Setup sx(inputStrMissingPressure, pc));
 
     Setup s(inputStrMissingPressure, pc);
     BOOST_CHECK( s.threshPres.hasRegionBarrier(2, 3));
     BOOST_CHECK(!s.threshPres.hasThresholdPressure(2, 3));
 
-    pc.update(ParseContext::INTERNAL_ERROR_UNINITIALIZED_THPRES, InputError::THROW_EXCEPTION);
+    pc.update(ParseContext::INTERNAL_ERROR_UNINITIALIZED_THPRES, InputErrorAction::THROW_EXCEPTION);
     BOOST_CHECK_THROW(s.threshPres.getThresholdPressure(2, 3), std::invalid_argument);
 }
 
@@ -341,3 +382,9 @@ BOOST_AUTO_TEST_CASE(Irreversible) {
     const auto& thp = s.threshPres;
     BOOST_CHECK(thp.getThresholdPressure(1,2) == -thp.getThresholdPressure(2,1));
 }
+
+BOOST_AUTO_TEST_CASE(Faults) {
+    Setup s(inputStrft, true);
+    const auto& thp = s.threshPres;
+    BOOST_CHECK(thp.getThresholdPressureFault(0) == 100000.0);
+}
