@@ -34,7 +34,7 @@
 
 using namespace Opm;
 
-inline std::string prefix() {
+inline std::string pathprefix() {
 #if BOOST_VERSION / 100000 == 1 && BOOST_VERSION / 100 % 1000 < 71
     return boost::unit_test::framework::master_test_suite().argv[2];
 #else
@@ -44,7 +44,7 @@ inline std::string prefix() {
 
 BOOST_AUTO_TEST_CASE(CreateCPGrid) {
     Parser parser;
-    std::filesystem::path scheduleFile(prefix() + "GRID/CORNERPOINT.DATA");
+    std::filesystem::path scheduleFile(pathprefix() + "GRID/CORNERPOINT.DATA");
     auto deck =  parser.parseFile(scheduleFile.string());
     EclipseState es(deck);
     const auto& grid = es.getInputGrid();
@@ -58,7 +58,7 @@ BOOST_AUTO_TEST_CASE(CreateCPGrid) {
 
 BOOST_AUTO_TEST_CASE(CreateCPActnumGrid) {
     Parser parser;
-    std::filesystem::path scheduleFile(prefix() + "GRID/CORNERPOINT_ACTNUM.DATA");
+    std::filesystem::path scheduleFile(pathprefix() + "GRID/CORNERPOINT_ACTNUM.DATA");
     auto deck =  parser.parseFile(scheduleFile.string());
     EclipseState es(deck);
     const auto& grid = es.getInputGrid();
@@ -72,7 +72,7 @@ BOOST_AUTO_TEST_CASE(CreateCPActnumGrid) {
 
 BOOST_AUTO_TEST_CASE(ExportFromCPGridAllActive) {
     Parser parser;
-    std::filesystem::path scheduleFile(prefix() + "GRID/CORNERPOINT.DATA");
+    std::filesystem::path scheduleFile(pathprefix() + "GRID/CORNERPOINT.DATA");
     auto deck =  parser.parseFile(scheduleFile.string());
     EclipseState es(deck);
     const auto& grid = es.getInputGrid();
@@ -88,7 +88,7 @@ BOOST_AUTO_TEST_CASE(ExportFromCPGridAllActive) {
 
 BOOST_AUTO_TEST_CASE(ExportFromCPGridACTNUM) {
     Parser parser;
-    std::filesystem::path scheduleFile(prefix() + "GRID/CORNERPOINT_ACTNUM.DATA");
+    std::filesystem::path scheduleFile(pathprefix() + "GRID/CORNERPOINT_ACTNUM.DATA");
     auto deck =  parser.parseFile(scheduleFile.string());
     EclipseState es(deck);
     auto& grid = es.getInputGrid();
