@@ -28,7 +28,6 @@
 
 #include <algorithm>
 #include <fstream>
-#include <iomanip>
 #include <iostream>
 #include <math.h>
 #include <stdexcept>
@@ -39,6 +38,8 @@
 
 using namespace Opm::EclIO;
 
+namespace {
+
 template<typename InputIterator1, typename InputIterator2>
 bool
 range_equal(InputIterator1 first1, InputIterator1 last1,
@@ -66,13 +67,14 @@ bool compare_files(const std::string& filename1, const std::string& filename2)
     return range_equal(begin1, end, begin2, end);
 }
 
-
 template <typename T>
 bool operator==(const std::vector<T> & t1, const std::vector<T> & t2)
 {
     return std::equal(t1.begin(), t1.end(), t2.begin(), t2.end());
 }
 
+} // Anonymous namespace
+
 // test is using SPE1CASE1, with minor modifications in order to test API for EGrid class
 //  -> 6 cells made inactive, box: 5 7  5 6  1 1
 
@@ -173,19 +175,15 @@ BOOST_AUTO_TEST_CASE(TestERft_1) {
 }
 
 
-BOOST_AUTO_TEST_CASE(TestERft_2) {
-
-    std::string testFile = "SPE1CASE1.RFT";
-
-    std::string outFile = "TEST.RFT";
-
+BOOST_AUTO_TEST_CASE(TestERft_2)
+{
     {
         WorkArea work;
-        work.copyIn(testFile);
+        work.copyIn("SPE1CASE1.RFT");
         {
-            EclOutput eclTest(outFile, false);
+            EclOutput eclTest("TEST.RFT", false);
 
-            ERft rft1(testFile);
+            ERft rft1("SPE1CASE1.RFT");
 
             auto rftList = rft1.listOfRftReports();
 
@@ -224,6 +222,6 @@ BOOST_AUTO_TEST_CASE(TestERft_2) {
             }
         }
 
-        BOOST_CHECK_EQUAL(compare_files(testFile, outFile), true);
+        BOOST_CHECK_EQUAL(compare_files("SPE1CASE1.RFT", "TEST.RFT"), true);
     }
 }
