@@ -25,19 +25,24 @@
 #include <tests/WorkArea.hpp>
 #include <opm/common/utility/FileSystem.hpp>
 
-Opm::EclIO::ERsm create(const std::string& rsm_data) {
+namespace {
+
+Opm::EclIO::ERsm create(const std::string& rsm_data)
+{
     WorkArea work_area("test_ERsm");
+
     {
         std::ofstream os("TEST.RSM");
         os << rsm_data;
     }
+
     return Opm::EclIO::ERsm("TEST.RSM");
 }
 
 
 // Dont touch the string literals - it is all part of the format; down to the number of whitespace ... :-(
 
-std::string block1_days = R"(1                                                                                                                                  
+const std::string block1_days = R"(1                                                                                                                                  
  -------------------------------------------------------------------------------------------------------------------------------
  SUMMARY OF RUN nor01-temp01-rsm-all OPM FLOW VERSION 1910 ANYTHING CAN GO HERE: USER, MACHINE ETC.                                
  -------------------------------------------------------------------------------------------------------------------------------
@@ -52,7 +57,7 @@ std::string block1_days = R"(1
  8.000000     0.021903     4381.301            0            0     4381.301            0     4381.301            0            0     
 )";
 
-std::string block1_hours = R"(1                                                                                                                                  
+const std::string block1_hours = R"(1                                                                                                                                  
  -------------------------------------------------------------------------------------------------------------------------------
  SUMMARY OF RUN nor01-temp01-rsm-all OPM FLOW VERSION 1910 ANYTHING CAN GO HERE: USER, MACHINE ETC.                                
  -------------------------------------------------------------------------------------------------------------------------------
@@ -67,9 +72,7 @@ std::string block1_hours = R"(1
  8.000000     0.021903     4381.301            0            0     4381.301            0     4381.301            0            0     
 )";
 
-
-
-std::string block1_date = R"(1                                                                                                                                  
+const std::string block1_date = R"(1                                                                                                                                  
  -------------------------------------------------------------------------------------------------------------------------------
  SUMMARY OF RUN nor01-temp01-rsm-date OPM FLOW VERSION 1910 ANYTHING CAN GO HERE: USER, MACHINE ETC.                               
  -------------------------------------------------------------------------------------------------------------------------------
@@ -85,7 +88,7 @@ std::string block1_date = R"(1
   5-MAR-1999  249.3513            0     80016.70            0            0            0            0            0            0     
 )";
 
-std::string block2_date = R"(1                                                                                                                                  
+const std::string block2_date = R"(1                                                                                                                                  
  -------------------------------------------------------------------------------------------------------------------------------
  SUMMARY OF RUN nor01-temp01-rsm-date OPM FLOW VERSION 1910 ANYTHING CAN GO HERE: USER, MACHINE ETC.                               
  -------------------------------------------------------------------------------------------------------------------------------
@@ -100,7 +103,7 @@ std::string block2_date = R"(1
   4-MAR-1999  243.9480            0     71582.33            0            0            0            0            0            0     
 )";
 
-std::string missing_time = R"(1                                                                                                                                  
+const std::string missing_time = R"(1                                                                                                                                  
  -------------------------------------------------------------------------------------------------------------------------------
  SUMMARY OF RUN nor01-temp01-rsm-date OPM FLOW VERSION 1910 ANYTHING CAN GO HERE: USER, MACHINE ETC.                               
  -------------------------------------------------------------------------------------------------------------------------------
@@ -113,8 +116,7 @@ std::string missing_time = R"(1
   1-MAR-1999  227.7381            0     46279.20            0            0            0            0            0            0     
 )";
 
-
-std::string no_wgnames = R"(1                                                                                                                                  
+const std::string no_wgnames = R"(1                                                                                                                                  
  -------------------------------------------------------------------------------------------------------------------------------   
  SUMMARY OF RUN SPE1CASE1 OPM FLOW VERSION 1910 ANYTHING CAN GO HERE: USER, MACHINE ETC.                                           
  -------------------------------------------------------------------------------------------------------------------------------   
@@ -132,8 +134,7 @@ std::string no_wgnames = R"(1
        45           15            2         2015            0            0            0            0            0            0     
 )";
 
-
-std::string multiple_blocks = R"(1                                                                                                                                  
+const std::string multiple_blocks = R"(1                                                                                                                                  
  -------------------------------------------------------------------------------------------------------------------------------   
  SUMMARY OF RUN SPE1CASE1 OPM FLOW VERSION 1910 ANYTHING CAN GO HERE: USER, MACHINE ETC.                                           
  -------------------------------------------------------------------------------------------------------------------------------   
@@ -164,8 +165,10 @@ std::string multiple_blocks = R"(1
   1-FEB-2015         0            0           -0     2678.399            0            0            0            0            0     
 )";
 
+} // Anonymous namespace
 
-BOOST_AUTO_TEST_CASE(ERsm) {
+BOOST_AUTO_TEST_CASE(ERsm)
+{
     BOOST_CHECK_THROW(Opm::EclIO::ERsm("file/does/not/exist"), std::invalid_argument);
     BOOST_CHECK_THROW( create("SomeThingInvalid"), std::invalid_argument);
     BOOST_CHECK_THROW( create( block1_date + block2_date ) , std::invalid_argument );
