@@ -180,9 +180,8 @@ void checkInitFile(const Deck& deck, const data::Solution& simProps)
         const auto& expect = deck["PERMX"].back().getSIDoubleData();
         auto        permx  = initFile.get<float>("PERMX");
 
-        for (auto& kx : permx) {
-            kx *= 9.869233e-16;
-        }
+        std::transform(permx.begin(), permx.end(), permx.begin(),
+                       [](const auto& kx) { return kx * 9.869233e-16; });
 
         compareErtData(expect, permx, 1e-4);
     }
@@ -217,8 +216,10 @@ void checkRestartFile( int timeStepIdx ) {
 
         if (keywordExists(knownVec, "PRESSURE")) {
             const auto& press = rstFile.getRestartData<float>("PRESSURE", i, 0);
-            for( auto& x : sol.data<double>("PRESSURE") )
-                x /= Metric::Pressure;
+            std::transform(sol.data<double>("PRESSURE").begin(),
+                           sol.data<double>("PRESSURE").end(),
+                           sol.data<double>("PRESSURE").begin(),
+                           [](const auto& x) { return x / Metric::Pressure; });
 
             compareErtData( sol.data<double>("PRESSURE"), press, 1e-4 );
         }
@@ -308,7 +309,7 @@ WELSPECS
         const auto& eclGrid = es.getInputGrid();
         const Schedule schedule(deck, es, std::make_shared<Python>());
         const SummaryConfig summary_config( deck, schedule, es.fieldProps(), es.aquifer());
-        const SummaryState st(TimeService::now());
+        const SummaryState st(TimeService::now(), 0.0);
         es.getIOConfig().setBaseName( "FOO" );
 
         EclipseIO eclWriter( es, eclGrid , schedule, summary_config);
@@ -436,6 +437,8 @@ WELSPECS
     BOOST_CHECK_EQUAL(file_size, write_and_check(3, 5));
 }
 
+namespace {
+
 std::pair<std::string,std::array<std::array<std::vector<float>,2>,3>>
 createMULTXYZDECK(std::array<std::bitset<2>,3> doxyz, bool write_all_multminus)
 {
@@ -670,15 +673,17 @@ EQUALS
 )"};
          auto mult = edit_mult_x.begin();
          if (doxyz[0].test(0)) {
-             for (auto&& val: exspected_multipliers[0][0]) {
-                 val *= *(mult++);
-             }
+             std::transform(exspected_multipliers[0][0].begin(),
+                            exspected_multipliers[0][0].end(),
+                            exspected_multipliers[0][0].begin(),
+                            [&mult](const auto& val) { return val * (*mult++); });
          }
          mult = edit_mult_z.begin();
          if (doxyz[2].test(1)) {
-             for (auto&& val: exspected_multipliers[2][1]) {
-                 val *= *(mult++);
-             }
+             std::transform(exspected_multipliers[2][1].begin(),
+                            exspected_multipliers[2][1].end(),
+                            exspected_multipliers[2][1].begin(),
+                            [&mult](const auto& val) { return val * (*mult++); });
          }
      }
 
@@ -699,7 +704,7 @@ void testMultxyz(std::array<std::bitset<2>,3> doxyz, bool write_all_multminus =
     const auto& eclGrid = es.getInputGrid();
     const Schedule schedule(deck, es, std::make_shared<Python>());
     const SummaryConfig summary_config( deck, schedule, es.fieldProps(), es.aquifer());
-    const SummaryState st(TimeService::now());
+    const SummaryState st(TimeService::now(), 0.0);
     es.getIOConfig().setBaseName( "MULTXFOO" );
     EclipseIO eclWriter( es, eclGrid , schedule, summary_config);
     eclWriter.writeInitial( );
@@ -722,6 +727,7 @@ void testMultxyz(std::array<std::bitset<2>,3> doxyz, bool write_all_multminus =
     }
 }
 
+} // Anonymous namespace
 
 BOOST_AUTO_TEST_CASE(MULTXYZInit)
 {
@@ -739,3 +745,212 @@ BOOST_AUTO_TEST_CASE(MULTXYZInit)
     testMultxyz({ '3', '3' , '2'}, true);
     testMultxyz({ '3', '3' , '3'});
 }
+
+namespace {
+
+std::pair<std::string,std::vector<float>>
+createMULTPVDECK(bool edit)
+{
+    auto deckString = std::string { R"(RUNSPEC
+
+TITLE
+   1D OIL WATER
+
+
+
+DIMENS
+   100 1 1 /
+
+EQLDIMS
+/
+TABDIMS
+  2 1 100 /
+
+OIL
+WATER
+
+ENDSCALE
+/
+
+METRIC
+
+START
+   1 'JAN' 2024 /
+
+WELLDIMS
+   3 3 2 2 /
+
+UNIFIN
+UNIFOUT
+
+GRID
+
+INIT
+
+DX 
+  100*1 /
+DY
+        100*10 /
+DZ
+  100*1 /
+
+TOPS
+  100*2000 /
+
+PORO
+  100*0.3 /
+
+MULTPV
+  100*5 /  -- overwritten by the next MULTPV keyword
+
+MULTPV
+  100*10 /  -- partly overwritten by the next BOX statements
+
+BOX
+  69 75 1 1 1 1 /
+MULTPV
+  2*1 5*1.5 /
+ENDBOX
+
+BOX
+  76 85 1 1 1 1 /
+MULTPV
+  5*1.5 5*1 /
+ENDBOX
+
+EDIT
+)"};
+    std::vector<float> multpv(68, 10.);
+    multpv.insert(multpv.end(), 2, 1);
+    multpv.insert(multpv.end(), 10, 1.5);
+    multpv.insert(multpv.end(), 5, 1.);
+    multpv.insert(multpv.end(), 15, 10.);
+
+    if (edit) {
+        deckString += std::string { R"(MULTPV
+  75*10 25*10 / -- overwritten by the next
+MULTPV
+  75*0.8 25*1 /
+)"};
+        std::transform(multpv.begin(), multpv.begin() + 75, multpv.begin(),
+                       [](float f) { return f*.8; });
+    }
+    deckString += std::string { R"(PROPS
+
+SOLUTION
+SCHEDULE
+)"};
+    return { deckString, multpv };
+}
+
+void checkMULTPV(const std::pair<std::string, std::vector<float>>& deckAndValues)
+{
+    const auto [deckString, expectedMultPV] = deckAndValues;
+    const auto deck = Parser().parseString(deckString);
+    auto es = EclipseState( deck );
+    const auto& eclGrid = es.getInputGrid();
+    const Schedule schedule(deck, es, std::make_shared<Python>());
+    const SummaryConfig summary_config( deck, schedule, es.fieldProps(), es.aquifer());
+    const SummaryState st(TimeService::now(), 0.0);
+    es.getIOConfig().setBaseName( "MULTPVFOO" );
+    EclipseIO eclWriter( es, eclGrid , schedule, summary_config);
+    eclWriter.writeInitial( );
+
+    EclIO::EclFile initFile { "MULTPVFOO.INIT" };
+    BOOST_CHECK_MESSAGE( initFile.hasKey("MULTPV"),
+                         R"(INIT file must have MULTPV array)" );
+    const auto& multPVValues = initFile.get<float>("MULTPV");
+    auto expect = expectedMultPV.begin();
+    BOOST_CHECK(multPVValues.size() == expectedMultPV.size());
+
+    for (auto multVal = multPVValues.begin(); multVal != multPVValues.end();
+         ++multVal, ++expect) {
+        BOOST_CHECK_CLOSE(*multVal, *expect, 1e-8);
+    }
+}
+
+} // Anonymous namespace
+
+BOOST_AUTO_TEST_CASE(MULTPVInit)
+{
+    checkMULTPV(createMULTPVDECK(false));
+    checkMULTPV(createMULTPVDECK(true));
+    
+}
+
+namespace {
+
+std::pair<std::string,std::vector<float>>
+createMULTPVBOXDECK()
+{
+    auto deckString = std::string { R"(RUNSPEC
+
+TITLE
+   1D OIL WATER
+
+
+
+DIMENS
+   100 1 1 /
+
+EQLDIMS
+/
+TABDIMS
+  2 1 100 /
+
+OIL
+WATER
+
+ENDSCALE
+/
+
+METRIC
+
+START
+   1 'JAN' 2024 /
+
+WELLDIMS
+   3 3 2 2 /
+
+UNIFIN
+UNIFOUT
+
+GRID
+
+INIT
+
+DX 
+  100*1 /
+DY
+        100*10 /
+DZ
+  100*1 /
+
+TOPS
+  100*2000 /
+
+PORO
+  100*0.3 /
+
+BOX
+  11 100 1 1 1 1 /
+MULTPV
+  90*1.5 /
+ENDBOX
+
+EDIT
+PROPS
+SOLUTION
+SCHEDULE
+)"};
+    std::vector<float> expected(10, 1.);
+    expected.insert(expected.end(), 90, 1.5);
+    return { deckString, expected };
+}
+
+} // Anonymous namespace
+
+BOOST_AUTO_TEST_CASE(MULTPVBOXInit)
+{
+    checkMULTPV(createMULTPVBOXDECK());
+}
