@@ -64,10 +64,12 @@
 
 #include <opm/input/eclipse/Parser/Parser.hpp>
 
+#include <algorithm>
 #include <cstdlib>
 #include <ctime>
 #include <map>
 #include <memory>
+#include <numeric>
 #include <optional>
 #include <sstream>
 #include <stdexcept>
@@ -84,14 +86,15 @@ namespace {
     int ecl_file_get_num_named_kw(Opm::EclIO::ERst&  rst,
                                   const std::string& kw)
     {
-        int count = 0;
-        for (const auto& step : rst.listOfReportStepNumbers()) {
-            for (const auto& vec : rst.listOfRstArrays(step)) {
-                count += std::get<0>(vec) == kw;
-            }
-        }
-
-        return count;
+        return std::accumulate(rst.listOfReportStepNumbers().begin(),
+                               rst.listOfReportStepNumbers().end(), 0,
+                               [&kw, &rst](const auto count, const auto step)
+                               {
+                                   const auto list = rst.listOfRstArrays(step);
+                                   return count + std::count_if(list.begin(), list.end(),
+                                                                [&kw](const auto& vec)
+                                                                { return std::get<0>(vec) == kw; });
+                               });
     }
 
     EclIO::EclFile::EclEntry
@@ -99,10 +102,13 @@ namespace {
                            const std::string& kw,
                            const int          seqnum)
     {
-        for (const auto& vec : rst.listOfRstArrays(seqnum)) {
-            if (std::get<0>(vec) == kw) {
-                return vec;
-            }
+        const auto list = rst.listOfRstArrays(seqnum);
+        const auto it = std::find_if(list.begin(),
+                                     list.end(),
+                                     [&kw](const auto& vec)
+                                     { return std::get<0>(vec) == kw; });
+        if (it != list.end()) {
+            return *it;
         }
 
         return EclIO::EclFile::EclEntry{ "NoSuchKeyword", Opm::EclIO::eclArrType::MESS, 0 };
@@ -387,30 +393,31 @@ Opm::SummaryState sim_state(const Opm::Schedule& sched)
 
 struct Setup
 {
-    Deck deck;
     EclipseState es;
     const EclipseGrid& grid;
     Schedule schedule;
     SummaryConfig summary_config;
 
-    Setup(const char* path)
-        : deck          ( Parser().parseFile( path) )
-        , es            ( deck)
-        , grid          ( es.getInputGrid( ) )
-        , schedule      ( deck, es, std::make_shared<Python>() )
-        , summary_config( deck, schedule, es.fieldProps(), es.aquifer() )
+    explicit Setup(const char* path)
+        : Setup { Parser{}.parseFile(path) }
+    {}
+
+    explicit Setup(const Deck& deck)
+        : es             { deck }
+        , grid           { es.getInputGrid() }
+        , schedule       { deck, es, std::make_shared<Python>() }
+        , summary_config { deck, schedule, es.fieldProps(), es.aquifer() }
     {
-        auto& io_config = es.getIOConfig();
-        io_config.setEclCompatibleRST(false);
+        es.getIOConfig().setEclCompatibleRST(false);
     }
 };
 
 RestartValue
-first_sim(const Setup&   setup,
-          Action::State& action_state,
-          SummaryState&  st,
-          UDQState&      udq_state,
-          bool           write_double)
+first_sim(const Setup&         setup,
+          const Action::State& action_state,
+          SummaryState&        st,
+          UDQState&            udq_state,
+          bool                 write_double)
 {
     WellTestState wtest_state;
     EclipseIO eclWriter(setup.es, setup.grid, setup.schedule, setup.summary_config);
@@ -428,8 +435,8 @@ first_sim(const Setup&   setup,
     auto regionSetMatcherFactory = []() { return std::make_unique<RegionSetMatcher>(FIPRegionStatistics {}); };
 
     udq.eval(report_step,
-             setup.schedule,
              setup.schedule.wellMatcher(report_step),
+             setup.schedule[report_step].group_order(),
              segmentMatcherFactory,
              regionSetMatcherFactory,
              st, udq_state);
@@ -1081,6 +1088,7 @@ void init_st(SummaryState& st)
     st.update_group_var("GRP1", "GOPR", 100);
     st.update_group_var("WGRP1", "GOPR", 100);
     st.update_group_var("WGRP2", "GOPR", 100);
+
     st.update("FLPR", 100);
 }
 
@@ -1129,10 +1137,13 @@ BOOST_AUTO_TEST_CASE(UDQ_RESTART)
         for (const auto& def : udq.definitions(UDQVarType::GROUP_VAR)) {
             const auto& kw = def.keyword();
 
-            BOOST_CHECK_EQUAL( st1.has_group_var(group, kw), st2.has_group_var(group, kw));
+            BOOST_CHECK_EQUAL(st1.has_group_var(group, kw),
+                              st2.has_group_var(group, kw));
 
             if (st1.has_group_var(group, def.keyword())) {
-                BOOST_CHECK_EQUAL(st1.get_group_var(group, kw), st2.get_group_var(group, kw));
+                BOOST_TEST_MESSAGE("Group Vector " << kw << " for Group " << group);
+                BOOST_CHECK_EQUAL(st1.get_group_var(group, kw),
+                                  st2.get_group_var(group, kw));
             }
         }
     }
