@@ -818,7 +818,8 @@ BOOST_AUTO_TEST_CASE(Construct_Well_Guide_Rates_Group_Control_Object)
             rst_whistctl_cmode,
             Opm::TracerConfig{},
             Opm::UnitSystem::newMETRIC(),
-            1.0e+20
+            1.0e+20,
+            std::nullopt
         };
     };
 
@@ -905,7 +906,8 @@ BOOST_AUTO_TEST_CASE(Construct_Well_Explicit_THP_Control_Options_Object)
             rst_whistctl_cmode,
             Opm::TracerConfig{},
             Opm::UnitSystem::newMETRIC(),
-            1.0e+20
+            1.0e+20,
+            std::nullopt
         }.getWVFPEXP();
     };
 
@@ -963,7 +965,8 @@ BOOST_AUTO_TEST_CASE(Historic_Period)
             state.header.histctl_override,
             Opm::TracerConfig{},
             Opm::UnitSystem::newMETRIC(),
-            state.header.udq_undefined
+            state.header.udq_undefined,
+            std::nullopt
         };
 
         const auto ctrl = op1.productionControls(Opm::SummaryState { Opm::TimeService::now(), 0.0 });
@@ -979,7 +982,8 @@ BOOST_AUTO_TEST_CASE(Historic_Period)
             state.header.histctl_override,
             Opm::TracerConfig{},
             Opm::UnitSystem::newMETRIC(),
-            state.header.udq_undefined
+            state.header.udq_undefined,
+            std::nullopt
         };
 
         auto prop = op1.getProductionProperties();
@@ -999,7 +1003,8 @@ BOOST_AUTO_TEST_CASE(Historic_Period)
             1,                  // ORAT
             Opm::TracerConfig{},
             Opm::UnitSystem::newMETRIC(),
-            state.header.udq_undefined
+            state.header.udq_undefined,
+            std::nullopt
         };
 
         auto prop = op1.getProductionProperties();
@@ -1033,7 +1038,8 @@ BOOST_AUTO_TEST_CASE(Historic_Period_WHistCtl)
             state.header.histctl_override,
             Opm::TracerConfig{},
             Opm::UnitSystem::newMETRIC(),
-            state.header.udq_undefined
+            state.header.udq_undefined,
+            std::nullopt
         };
 
         const auto ctrl = op1.productionControls(Opm::SummaryState { Opm::TimeService::now(), state.header.udq_undefined });
@@ -1049,7 +1055,8 @@ BOOST_AUTO_TEST_CASE(Historic_Period_WHistCtl)
             state.header.histctl_override,
             Opm::TracerConfig{},
             Opm::UnitSystem::newMETRIC(),
-            state.header.udq_undefined
+            state.header.udq_undefined,
+            std::nullopt
         };
 
         auto prop = op1.getProductionProperties();
@@ -1069,7 +1076,8 @@ BOOST_AUTO_TEST_CASE(Historic_Period_WHistCtl)
             0,                  // NONE
             Opm::TracerConfig{},
             Opm::UnitSystem::newMETRIC(),
-            state.header.udq_undefined
+            state.header.udq_undefined,
+            std::nullopt
         };
 
         auto prop = op1.getProductionProperties();
